#!/usr/bin/perl

#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: configGen.pl,v 1.10 2014/07/31 18:59:17 sgupta2 Exp $
#

use strict;
use warnings;

use constant VMWARE_LINUX => "VMWARE_LINUX";
use constant LINUX => "linux";
use constant SOLARIS => "solaris";
use constant AIX => "aix";

my $abe_dir = shift;

if ( ! defined $abe_dir or $abe_dir eq "/" ) {
	$abe_dir = "";
}

### Check whether we can update namespace or not
my $xprtlconf = "$abe_dir/etc/opt/VRTSsfmh/xprtld.conf";

unless ( -e $xprtlconf ) {
	print "VRTSsfmh package does not seem to be installed. Execute $abe_dir/opt/VRTSvcs/portal/admin/conf/configGen.pl after installing VRTSsfmh package. This utility sets up necessary namespace for wizards to launch properly.\n";
	exit 0;
}

### Backup conf dir if it exists already
my $conf_dir = "$abe_dir/opt/VRTSvcs/portal/admin/conf";
my $backup_dir = "$abe_dir/opt/VRTSvcs/portal/admin/conf/backup/" . `date +%Y%m%d%H%M%S`;

#print "Conf dir: [$conf_dir]\n";
`mkdir -p $backup_dir`;

# Generate Supported Application cf file
my $appcf = "$conf_dir/supportedApp.conf";

# List of supported applications per operating system
my %supported_applications;
my $platform = "";

@{$supported_applications { ( VMWARE_LINUX ) }} = ( qw[ genericapp Oracle SAP webspheremq ] );
@{$supported_applications { ( LINUX ) }} = ( qw[ genericapp ] );
@{$supported_applications { ( SOLARIS ) }} = ( qw[ genericapp ] );
@{$supported_applications { ( AIX ) }} = ( qw[ genericapp ] );

if ( $^O =~ m/linux/i ) {
	system ("/opt/VRTSsfmh/bin/testVMware");
	if ( $? eq 0 ) { # vmware linux
		$platform = VMWARE_LINUX;
	} else { # non-vmware linux
		$platform = $^O;
	}
} else {
	$platform = $^O;
}

if ( -e $appcf ) {
	print "Saving existing $appcf\n";
	`mv $appcf $backup_dir`;
}

# Generate Descriptive Name String cf file
my $strcf = "$conf_dir/resName.mf";

if ( -e $strcf ) {
	print "Saving existing $strcf\n";
	`mv $strcf $backup_dir`;
}

foreach my $conf_file ( @{$supported_applications { $platform }} ) {

	`cat $conf_dir/app_config/$conf_file.conf >> $appcf`;
	`cat $conf_dir/app_config/$conf_file.mf >> $strcf`;
}

### Update vcs namespace if required
if ( `grep '^namespaces ' $xprtlconf | grep 'vcs='` eq "" ) {
        print "Adding vcs namespace\n";
        # Add a namespace for vcs
        system ("echo 'namespaces vcs=/opt/VRTSvcs/portal' >> $abe_dir/etc/opt/VRTSsfmh/xprtld.conf");
	if ( $abe_dir eq "" ) {
		# Reload xprtld configuration
		system ("/opt/VRTSsfmh/bin/xprtlc -l https://localhost/admin/lifecycle/stop?excludetree=1 &> /dev/null");
	}
}

if ( $abe_dir eq "" ) {
	if ($^O =~ m/aix/i) {
		system ("ODMDIR=/etc/objrepos /opt/VRTSsfmh/adm/xprtldctrl start &> /dev/null");
	} else {
		system ("/opt/VRTSsfmh/adm/xprtldctrl start &> /dev/null");
	}
	system ("/opt/VRTSperl/bin/perl /opt/VRTSvcs/portal/admin/genConfigDetails.pl > /dev/null");
}

system ("ln -sf $abe_dir/opt/VRTSvcs/bin/haappwizard /opt/VRTS/bin/haappwizard");

exit 0;
