#!/opt/VRTSsfmh/bin/perl

#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: config_guest_security.pl,v 1.4 2014/08/14 05:57:54 knagarka Exp $
#


#------------------------
# Common Header
#------------------------
use strict;
use warnings;

use Data::Dumper;
use CGI;
use Config;
use POSIX;
use Net::IP qw(:PROC);
use File::Copy;
use File::Path qw(mkpath);

# Push include paths!
my $LibPath1 = "";
my $notifysink_path;

BEGIN
{
	if ( $^O =~ m/win/i ) {
		require VRTS::Paths;

		my $xprtlInstallDir= VRTS::Paths::get_path("InstallDir");
		my $regquery = "\"$xprtlInstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\VERITAS\\VCS\\Base HomeDirectory";

		my $vcshome = `$regquery`;
		$LibPath1 = "$vcshome\\portal\\admin\\lib\\";
		$notifysink_path = "\"$vcshome"."\\bin\\notify_sink.exe\"";
	} else {
		my $APPDISCOVER_HOME = "/opt/VRTSvcs/portal/admin";
	
		$LibPath1 = "$APPDISCOVER_HOME/lib";
		$notifysink_path = "/opt/VRTSvcs/portal/admin/notify_sink";
	}
}

use lib $LibPath1;

#------------------------
# End of Common Header
#------------------------

use Constants;
use Utils;
use Logger;
use CmdUtils;
use MessageUtils;
use SFMUtils;
use ConfUtils;

use constant MSG_FILE => "sec.msg";

my $logDir = PlatUtils::getLogDir();
my $logFileName = "config_guest_security";
my $logFileSize = 5;
my ( $logger, $cmdutils ) = ();

eval
{
	$logger = new Logger($logFileName, $logDir, $logFileSize);
	$cmdutils = new CmdUtils( $logger );
};
if ( $@ ) {
	my $str = Utils::getCmdResponseXML("config_guest_security.pl", CONST_EXCEPTION, "<Message>$@</Message>", "");
	print "$str\n";
	exit CONST_EXCEPTION;
}


# Before doing anything else, let us set log config ...
# Params:
#   . FileName
#   . File size (in MBs)
Utils::setLogConfig($logFileName, $logFileSize);
$logger -> vxlog("INFO", ">>>> config_guest_security is invoked <<<<");
#------------------------
# End of Header
#------------------------


my ($eat_data_dir, $eat_home_dir, $sfmh_eat_data_dir) = ();
my ($xprtlc, $vssat) = "";
my ($BrokerHost, $VMHost, $ApplicationHAServerIP, $BrokerPort, $sGuestCredPack, $skey);


my $hConfRef = ConfUtils::read_guestConfig();
my $defaultApplicationHA_primary = $hConfRef -> { ApplicationHA_primary };
my $DefaultBrokerPort = "14153";
my $principleName = "appha_admin";
my $domainName = "appha_domain";
my $namespace = "vcs";
my $iExecGuestCred = 0;
my $sCalledFromSite = "Primary";


sub setup_env
{

	if ( $^O =~ /win/i ) {
		$eat_home_dir	 = VRTS::Paths::get_path("EAT_HOME_DIR");
		$eat_data_dir	 = VRTS::Paths::get_path("EAT_DATA_DIR");
		$sfmh_eat_data_dir = $eat_data_dir;
		$eat_data_dir    =~ s/sec$/ApplicationHA\\sec/;

		my $xprtlInstallDir= VRTS::Paths::get_path("InstallDir");
		$xprtlc = "\"$xprtlInstallDir\\bin\\xprtlc.exe\"";
		$vssat  = "\"$xprtlInstallDir\\bin\\vssat.bat\"";
	} else {
		$eat_data_dir = "/var/opt/VRTSsfmh/ApplicationHA/sec";
		$eat_home_dir = "/opt/VRTSsfmh";
		$sfmh_eat_data_dir = "/var/opt/VRTSsfmh/sec";

		$xprtlc = "/opt/VRTSsfmh/bin/xprtlc";
		$vssat = "/opt/VRTSsfmh/bin/vssat";
	}

	if ( not -d "$eat_data_dir" ) {
		mkpath $eat_data_dir;
	}

	$logger -> vxlog("INFO", "EAT_HOME_DIR=[$eat_home_dir], " .
	                         "EAT_DATA_DIR=[$eat_data_dir], " . 
	                         "sfmh_eat_data_dir=[$sfmh_eat_data_dir]");
	$logger -> vxlog("INFO", "vssat =[$vssat], xprtlc=[$xprtlc]");

	$ENV { EAT_HOME_DIR } = $eat_home_dir;
	$ENV { EAT_DATA_DIR } = $eat_data_dir;
}


#------------------------
# Misc subroutines
#------------------------

sub checkParams
{
	my $params = shift;
	my $query = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	$logger -> vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	
	if ( not defined $params -> { Broker } ) {
		($ret, $msg) = getMessage( MSG_FILE, "NO_BROKER_PARAM");
		goto END;
	}

	my $Broker = $params -> { Broker };
	$Broker =~ s/\s+$//;
	$Broker =~ s/^\s+//;
	if ( $Broker eq "" ) {
		($ret, $msg) = getMessage( MSG_FILE, "EMPTY_BROKER_PARAM");
		goto END;
	}

	if ( defined $params -> { VMHost } ) {
		my $VMHost = $params -> { VMHost };
		$VMHost =~ s/^\s+//;
		$VMHost =~ s/\s+$//;
		if ( $VMHost eq "" ) {
			($ret, $msg) = getMessage( MSG_FILE, "EMPTY_VMHOST_PARAM");
			goto END;
		}
	}

	if ( defined $params -> { BrokerPort } ) {
		my $Port = $params -> { BrokerPort };
		$Port =~ s/^\s+//;
		$Port =~ s/\s+$//;
		if ( $Port eq "" ) {
			($ret, $msg) = getMessage( MSG_FILE, "EMPTY_BROKERPORT_PARAM", 
			                           CONSOLE_PROD_NAME);
			goto END;
		}
	}

	if ( defined $params -> { Site } and
		$params -> { Site } =~ /Recovery/i ) {
		$sCalledFromSite = "Recovery";
	}

	if ( not defined $params -> { ApplicationHAServerIP } ) {
		($ret, $msg) = getMessage( MSG_FILE, "APPSERVER_IP_NOT_PROVIDED");
		goto END;
	} else {
		if ( $sCalledFromSite !~ /Recovery/i ) {
			my $IP = $params -> { ApplicationHAServerIP };
			my $version = ip_get_version( $IP );
			if ( not defined $version or ($version != 4 and $version != 6) ) {
				($ret, $msg) = getMessage( MSG_FILE, "INVALID_APPSERVER_IP",
				                           $IP);
				goto END;
			}
		}
	}

	if ( defined $params -> { AppHAGuestCred } and
		$params -> { AppHAGuestCred } ne "" ) {
		$iExecGuestCred = 1;
	}

	# check if file is passed as parameter; if filehandle found, then use the file as AppHAGuestCred
	my $filehandle = $query -> upload("data");
	if (defined $filehandle) {
		$params -> { AppHAGuestCred } = "";
		while ( <$filehandle> ) {
			$params -> { AppHAGuestCred } .= $_;
		}
		close $filehandle;
		$iExecGuestCred = 1;
	}

	$BrokerHost = $params -> { Broker };
	$VMHost = $params -> { VMHost } || "";
	$ApplicationHAServerIP = $params -> { ApplicationHAServerIP };
	$BrokerPort = $params -> { BrokerPort } || $DefaultBrokerPort;
	$sGuestCredPack = $params -> { AppHAGuestCred } || "";
	$skey = "ApplicationHA_primary";
	if ( $sCalledFromSite =~ /Recovery/i ) {
		$skey = "ApplicationHA_secondary";
	}
	$hConfRef -> { $skey } = $ApplicationHAServerIP;

END:
	return ( $ret, $msg );
}




#-------------------------
# Reverse look up test ...
#-------------------------

sub reverseLookup
{
    my $sMyName = ( caller 0 )[3];
    #---------
    # Announce
    #---------
    my $logStr = "Subroutine $sMyName called";
    $logger -> vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $op = "";

	if ( $VMHost eq "" ) {
		goto END;
	}

	my $cmd = "$xprtlc -l https://$ApplicationHAServerIP:5634/https://$VMHost:5634/world/getvitals";
	($ret, $op) = $cmdutils -> runCmd ( $cmd );
	if ( $ret != CONST_SUCCESS ) {
		($ret, $msg) = getMessage(MSG_FILE, "BROKER_VMHOST_REVERSE_LOOKUP_FAILED",
		                          CONSOLE_PROD_NAME, $ApplicationHAServerIP, $VMHost,SFMH_SERVICE_NAME,
			                           CONSOLE_TOMCAT_SEVICE_NAME, CONSOLE_AUTH_SERVICE);
		goto END;
	}

	$logger -> vxlog("INFO", "Reverse lookup successful");
END:
	return ($ret, $msg, $op);
}


sub setupTrust 
{
    my $sMyName = ( caller 0 )[3];
    #---------
    # Announce
    #---------
    my $logStr = "Subroutine $sMyName called";
    $logger -> vxlog("INFO", $logStr);

	###################################################
	# Setup trust with the broker:
	# vssat.bat setuptrust -b <CortechAppServ>:<14153> -s low
	###################################################
	
	my $ret = CONST_SUCCESS;
	my $msg = "";

	my $cmd = "$vssat setuptrust -b $ApplicationHAServerIP:$BrokerPort -s low";
	my ($CmdRet, $op) = $cmdutils -> runCmd ( $cmd );
	if ( $CmdRet != CONST_SUCCESS ) {
		($ret, $msg) = getMessage( MSG_FILE, "SETUP_TRUST_FAILED", 
		                           CONSOLE_PROD_NAME, SFMH_SERVICE_NAME,
                                   CONSOLE_TOMCAT_SEVICE_NAME, CONSOLE_AUTH_SERVICE,
                                   COMMUNICATION_PORT_LIST);
		goto END;
	}

	$logger -> vxlog("INFO", "Setup trust successful");

END:
	return ($ret, $msg, $op);	
}


sub checktimeskew
{
    my $sMyName = ( caller 0 )[3];
    #---------
    # Announce
    #---------
    my $logStr = "Subroutine $sMyName called";
    $logger -> vxlog("INFO", $logStr);

	my $cmd = "$vssat checkclockskew -b $ApplicationHAServerIP:$BrokerPort";
	my ($ret, $msg, $exitCode) = $cmdutils -> runCmd( $cmd );
	if ( $ret != CONST_SUCCESS ) {
		if ($exitCode == 91) {
			($ret, $msg) = getMessage( MSG_FILE, "TIME_SKEW_DETECTED",
			                           CONSOLE_PROD_NAME);
		} else {
			($ret, $msg) = getMessage( MSG_FILE, "CONSOLE_SERVICES_UNAVAILABLE",
			                           CONSOLE_PROD_NAME, SFMH_SERVICE_NAME,
			                           CONSOLE_TOMCAT_SEVICE_NAME, CONSOLE_AUTH_SERVICE, 
			                           COMMUNICATION_PORT_LIST);
		}
		goto END;
	}

	$logger -> vxlog("INFO", "time skew check is passed!");

END:
	return ($ret, $msg);
}


sub saveAndExecAppHAGuestCred
{
    my $sMyName = ( caller 0 )[3];
    #---------
    # Announce
    #---------
    my $logStr = "Subroutine $sMyName called";
    $logger -> vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $op = "";

	my $tmp_file = PlatUtils::dumpToTmpFile("");
	open  FH, ">$tmp_file";
	binmode FH;
	print FH $sGuestCredPack;
	close FH;

	#remove the appha_guest package for the given domain, broker if available.
	my $cmd = "$vssat deletecred -d vx:appha_domain -p appha_guest -b $ApplicationHAServerIP:$BrokerPort";
	($ret, $op) = $cmdutils -> runCmd( $cmd );

	# remove trust of APPHA_BROKER on given host if available.
	$cmd = "$vssat removetrust -b $BrokerHost -n APPHA_BROKER";
	($ret, $op) = $cmdutils -> runCmd( $cmd );
	
	# now deploy the package.
	$cmd = "$vssat execpkg -i \"$tmp_file\" -o";
	($ret, $op) = $cmdutils -> runCmd( $cmd );
	if ( $ret != CONST_SUCCESS ) {
		($ret, $msg) = getMessage( MSG_FILE, "EXEC_PKG_FAILED", CONSOLE_PROD_NAME,SFMH_SERVICE_NAME,
			                           CONSOLE_TOMCAT_SEVICE_NAME, CONSOLE_AUTH_SERVICE, 
			                           COMMUNICATION_PORT_LIST);
		goto END;
	}

	$logger -> vxlog("INFO", "Execute at package successful");

END:
	return ($ret, $msg, $op);
}


sub grantAccess
{
    my $sMyName = ( caller 0 )[3];
    #---------
    # Announce
    #---------
    my $logStr = "Subroutine $sMyName called";
    $logger -> vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $op = "";

	#--------------------------------------------------------------
	# Authorize the user:
	# xprtlc -l https://localhost:5634/admin/security/grant 
	#		 -d user=<vcs_admin>@<vcs_domain>@<CortechAppServ>.vx 
	# 		 -d type=user -d role=admin -n <vcs>
	#--------------------------------------------------------------
	
	my $cmd = "$xprtlc -l https://localhost:5634/admin/security/grant " . 
	       "-d user=$principleName\@$domainName\@$BrokerHost.vx " .
	       "-d type=user -d role=admin -n $namespace";

	($ret, $op) = $cmdutils -> runCmd ( $cmd );
	if ( $ret != CONST_SUCCESS ) {
		($ret, $msg) = getMessage( MSG_FILE, "USER_AUTHENTICATION_FAILED");
		goto END;
	}

	$logger -> vxlog("INFO", "Granting access is successful");

END:
	return ($ret, $msg, $op);
}


sub main
{
    my $sMyName = ( caller 0 )[3];
    #---------
    # Announce
    #---------
    my $logStr = "Subroutine $sMyName called";
    $logger -> vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $debug = "";
	my $op = "";
	
	my $query = CGI->new();
	print $query->header('text/xml');

	my $params = parseArguments($query);
	$logStr = Dumper( $params );
	$logger -> vxlog ("INFO", "params are:\n$logStr");
	
	($ret, $msg) = checkParams( $params, $query ); 
	if ( $ret != CONST_SUCCESS ) {
	    goto END;
	}
	
	setup_env();

	# create domain entry into smf_resolv.conf file.
	update_sfm_resolve_file($ApplicationHAServerIP, $eat_data_dir);

	# check for revese lookup
	($ret, $msg, $op) = reverseLookup();
	if ( $ret != CONST_SUCCESS ) {
		$logger -> vxlog("INFO", "Reverse lookup failed!");
		goto END;
	}

	# chanage the EAT_DATA_DIR.
	$logger -> vxlog ("INFO", "Settingup EAT_DATA_DIR to=[$sfmh_eat_data_dir]");
	$ENV { EAT_DATA_DIR } = $sfmh_eat_data_dir;
	($ret, $msg, $op) = setupTrust();
	$logger -> vxlog ("INFO", "Settingup EAT_DATA_DIR to=[$eat_data_dir]");
	$ENV { EAT_DATA_DIR } = $eat_data_dir;

	# according to params, execute steps
	if ( $iExecGuestCred != 0 ) {
		($ret, $msg, $op) = checktimeskew();
		if ( $ret == CONST_SUCCESS ) {
			($ret, $msg, $op) = saveAndExecAppHAGuestCred();
		}
	}

	if ( $ret != CONST_SUCCESS ) {
		$logger -> vxlog("INFO", "Setup trust / exec package failed!");
		goto END;
	}

	# grant access, so that xprtl(d/c) works fine
	($ret, $msg, $op) = grantAccess();
	if ( $ret != CONST_SUCCESS ) {
		$logger -> vxlog("INFO", "Grant access failed!");
		goto END;
	}

	# save the config file
	ConfUtils::dump_guestConfig($hConfRef);

        my $cmd = $notifysink_path . " -f";
        $logger -> vxlog("INFO", "Sending VCS service group information to console");
        my ($cmd_ret, $cmd_op) = $cmdutils -> runCmd( $cmd );
        if ( $cmd_ret != CONST_SUCCESS ) {
                $logger -> vxlog("WARNING", "Failed to send VCS service group information to console");
        }

END:
	$debug = "<Debug>$op</Debug>";
	my $str = Utils::getCmdResponseXML("config_guest_security.pl", $ret, $msg, $debug);
	print $str;

	return($ret, $msg);
}


#------------------------
# Call to main 
#------------------------
my ($ret, $msg) = main();
exit $ret;

