#!/opt/VRTSsfmh/bin/perl

#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: configureCluster.pl,v 1.14 2012/08/08 07:27:51 stripath Exp $
#

#------------------------
# Common Header
#------------------------
use strict;
use warnings;

use Data::Dumper;
use CGI;
use Config;
use POSIX;
use threads;
use threads::shared;

# Push include paths!
my $LibPath1 = "";

BEGIN
{
	if ( $^O =~ m/win/i ) {
		require VRTS::Paths;

		my $xprtlInstallDir= VRTS::Paths::get_path("InstallDir");
		my $regquery = "\"$xprtlInstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\VERITAS\\VCS\\Base HomeDirectory";
		my $vcshome = `$regquery`;
		$LibPath1 = "$vcshome\\portal\\admin\\lib\\";
		$ENV{'VCS_HOME'} = $vcshome;

	} else {
		my $APPDISCOVER_HOME = "/opt/VRTSvcs/portal/admin";
	
		$LibPath1 = "$APPDISCOVER_HOME/lib";
	}
}

use lib $LibPath1;

#------------------------
# End of Common Header
#------------------------

use Constants;
use Utils;
use VCS;
use MessageUtils;

use constant MSG_FILE => "clusterop.msg";

my $lockFile = PlatUtils::getLockDir() . "HVLock.l";

my $hOP = ();
$hOP -> { initiateCIDScan } 	= "initiateCIDScan";
$hOP -> { waitForCIDScan } 	= "waitForCIDScan";

my $thread_var = 0;
share ($thread_var);

#------------------------
# Subroutines
#------------------------

sub main
{
	my $subName = ( caller 0 )[3];
	Utils::vxlog("INFO", "Subroutine $subName called");

	my $ret   = CONST_SUCCESS;
	my $msg   = "";
	my $xml	  = "";
	my $op	  = "";
	my $hRet  = ();

	my $query = CGI->new();
	print $query->header('text/xml');

	Utils::vxlog("INFO", "Acquiring lock ...");
	my $lockfd = Utils::acquire_lock($lockFile, 1);
	Utils::vxlog("INFO", "Got lock, proceeding ...");

	my $params = Utils::parseArguments($query);

	my $OP = $params -> { OP } || "";

	my $tree = Utils::convertXMLTextToHash($params -> { params });
	Utils::vxlog("INFO", Dumper( $tree ));
	Utils::vxlog("DEBUG", "OP = [$OP]");

	if ($OP eq "") {

		my $thr = threads -> create('keepAliveThread');

		eval {

			($ret, $msg, $hRet) = VCS::CreateClusterInputFile($tree);
			if ( $ret != CONST_SUCCESS ) {
				$xml = "<Debug><![CDATA[" . ($hRet -> { op } || "") . "]]></Debug>";
				goto END;
			}
	
		
			($ret, $msg, $op) = VCS::LaunchClusterCreateTool($hRet);
			if ( $ret != CONST_SUCCESS ) {
				$xml = "<Debug><![CDATA[$op]]></Debug>";
				goto END;
			}
		};

		if ( $@ ) {
			$msg = $@;
			$ret = CONST_EXCEPTION;
			$xml = "";
			Utils::vxlog("EXCEPTION", $msg);
			goto END;
		}

		# set the thread variable, so that the thread can exit.
		$thread_var = 1;
		Utils::vxlog ("INFO", "$subName :: Waiting for the thread to return");
		$thr->join;
	
		VCS::sendFullSnap();
	}
	elsif ( $OP eq "unconfigureCluster" ) {
		Utils::vxlog("DEBUG", "Unconfiguring the cluster.");
		($ret, $msg, $op) = VCS::UnconfigureClusterViaProxy($tree);
		if ( $ret != CONST_SUCCESS ) {;
			$xml = "<Debug><![CDATA[$op]]></Debug>";
			goto END;
		}
	}
	else {
		if ( not defined $hOP -> { $OP } ) {
			($ret, $msg) = getMessage( MSG_FILE, "URL_SYNTAX_UNAVAILALE_OP", $OP);
			goto END;
		}
		
		my $vcsobj = new VCS();
		my $function_name  = $hOP -> { $OP };
		
		($ret, $msg, $xml) = $vcsobj -> $function_name( $tree );

		$msg = "" if (not defined $msg);
	}

END:

	my $xmlstr = Utils::getCmdResponseXML("configureCluster", $ret, $msg, $xml);
	Utils::vxlog("INFO", "$xmlstr");
	print "$xmlstr\n";

	Utils::vxlog("INFO", "Releasing lock ...");
	Utils::release_lock( $lockfd );

	return $ret;
}


sub keepAliveThread
{
	my $sMyName = ( caller 0 )[3];
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $sleepTime = 3;		# secs
	my $interval  = 4*60;	# 4 mins ( 4 * 60 )
	my $maxCnt    = $interval / $sleepTime;

	my $cnt = 0;
	while ($thread_var == 0) {
		while ($cnt < $maxCnt) {
			++ $cnt;
			sleep($sleepTime);
			if ($thread_var == 1) {
				last;
			}
		}

		$cnt = 0;
		if ($thread_var != 1) {
			Utils::vxlog("INFO", "$sMyName: keep alive msg printing");
			print " ";
			$|=1;
		}
	}
}


#---------------------------------------------
# main code
#---------------------------------------------

Utils::vxlog("INFO", "-------------------- Invoking configureCluster ------------------");
my $ret = main();
Utils::vxlog("INFO", "-------------------- Exiting from configureCluster --------------------");
exit $ret;

