#!/opt/VRTSperl/bin/perl

#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: configureSSO.pl,v 1.5 2014/08/14 05:57:54 knagarka Exp $
#

#------------------------
# Common Header
#------------------------
use strict;
use warnings;

use Data::Dumper;
use CGI;
use Config;
use POSIX;

# Push include paths!
my $LibPath1 = "";

my $notifysink_path;

BEGIN
{
	if ( $^O =~ m/win/i ) {
		require VRTS::Paths;

		my $xprtlInstallDir= VRTS::Paths::get_path("InstallDir");
		my $regquery = "\"$xprtlInstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\VERITAS\\VCS\\Base HomeDirectory";

		my $vcshome = `$regquery`;
		$LibPath1 = "$vcshome\\portal\\admin\\lib\\";
		$notifysink_path = "\"$vcshome\\bin\\notify_sink.exe\"";
	} else {
		my $APPDISCOVER_HOME = "/opt/VRTSvcs/portal/admin";
	
		$LibPath1 = "$APPDISCOVER_HOME/lib";
		$notifysink_path = "/opt/VRTSvcs/portal/admin/notify_sink";
	}
}

use lib $LibPath1;
#------------------------
# End of Common Header
#------------------------

use Logger;
use CmdUtils;
use Constants;
use Utils;
use SFMUtils;
use MessageUtils;

use Net::IP qw(:PROC);
use File::Path qw(mkpath);

my $logFileName = "configureSSO";
my $logDir = PlatUtils::getLogDir();
my $logFileSize = 5;
my ( $logger, $cmdutils ) = ();

use constant MSG_FILE => "sec.msg";

eval
{
	$logger = new Logger($logFileName, $logDir, $logFileSize);
	$cmdutils = new CmdUtils( $logger );
};
if ( $@ ) {
	print "$@\n";
	exit CONST_EXCEPTION;
}

# Before doing anything else, let us set log config ...
# Params:
#   . FileName
#   . File size (in MBs)
Utils::setLogConfig($logFileName, $logFileSize);
$logger -> vxlog("INFO", ">>>> configureSSO is invoked <<<<");
#------------------------
# End of Header
#------------------------


my ($EAT_DATA_DIR, $EAT_HOME_DIR, $xprtlc) = "";
sub setup_env
{

	if ( $^O =~ /win/i ) {
		my $xprtlInstallDir= VRTS::Paths::get_path("InstallDir");
		$EAT_HOME_DIR = $xprtlInstallDir;

		my $regquery = "\"$xprtlInstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\Symantec\\SFMH\\xprtl\\config var_dir";
		$EAT_DATA_DIR = `$regquery`;
		$EAT_DATA_DIR .= "\\ApplicationHA\\sec";

		$xprtlc = "\"$xprtlInstallDir\\bin\\xprtlc.exe\"";
	} else {
		$EAT_DATA_DIR = "/var/opt/VRTSsfmh/ApplicationHA/sec";
		$EAT_HOME_DIR = "/opt/VRTSsfmh";
		$xprtlc = "/opt/VRTSsfmh/bin/xprtlc";
	}

	if ( not -d "$EAT_DATA_DIR" ) {
		mkpath $EAT_DATA_DIR;
	}

	$logger -> vxlog("INFO", "EAT_HOME_DIR=[$EAT_HOME_DIR], EAT_DATA_DIR=[$EAT_DATA_DIR], xprtlc=[$xprtlc]");
	$ENV { EAT_HOME_DIR } = $EAT_HOME_DIR;
	$ENV { EAT_DATA_DIR } = $EAT_DATA_DIR;
}


#-------------
# Subroutines
#-------------
sub getLocalIPList_Linux
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	$logger -> vxlog("INFO", $logStr);

	my @IPList = ();
	my $cmd = "/sbin/ip addr show";

	my ($fret, $op) = $cmdutils -> runCmd( $cmd , { doNotLogCmd => 1} );
	my @lines = split(/\n/, $op);
	my @addrlines = grep (/inet/, @lines);

	$logger -> vxlog("INFO", "Matching lines are: \n" . Dumper( \@addrlines ));

=pod
	grep output sample:
    inet 127.0.0.1/8 scope host lo
    inet6 ::1/128 scope host
    inet 1.2.3.4/24 brd 10.209.199.255 scope global eth0
    inet6 fe80::250:56ff:feae:1b/64 scope link
    inet6 fe80::250:56ff:fe9d:64/64 scope link
    inet6 fe80::250:56ff:fe9d:65/64 scope link
=cut

	foreach my $addrline ( @addrlines ) {
		my $addr = "";
		if ( $addrline =~ m/\s+inet\s+([\d\.]*)\/.*/ ) {
			$addr = $1;
			if ( $addr eq "127.0.0.1" ) {
				next;
			}
			push(@IPList, $addr);
		}
	}

	$logger -> vxlog("INFO", "Found IP addresses are: \n" . Dumper(\@IPList));

END:
	return \@IPList;
}


sub getLocalIPList_Win
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	$logger -> vxlog("INFO", $logStr);
	my @IPList = ();

	my $cmd = "ipconfig";
	my ($fret, $op) = $cmdutils -> runCmd( $cmd , { doNotLogCmd => 1 });
	my @lines = split(/\n/, $op);
	my @addrlines = grep (/^\s+ip(v4)?\s* Address/i, @lines);

	$logger -> vxlog("INFO", "Matching lines are: \n" . Dumper( \@addrlines ));

=pod
	grep output sample:
   IPv4 Address. . . . . . . . . . . : 1.2.3.4
   IPv4 Address. . . . . . . . . . . : 2.3.4.5(Preferred)
=cut

	foreach my $addrline ( @addrlines ) {
		my $addr = "";
		if ( $addrline =~ m/.*:\s+(([\.[:alnum:]:])*)/ ) {
			$addr = $1;
			if ( $addr eq "127.0.0.1" ) {
				next;
			}
			push(@IPList, $addr);
		}
	}

	$logger -> vxlog("INFO", "Found IP addresses are: \n" . Dumper(\@IPList));

END:
	return \@IPList;

}

sub getLocalIPList
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	$logger -> vxlog("INFO", $logStr);

	my $IPList = ();
	if ( $^O =~ /win/i ) {
		$IPList = getLocalIPList_Win();
	} elsif ( $^O =~ /linux/i ) {
		$IPList = getLocalIPList_Linux();
	}

	# trim off IPv6 addresses.
	my @IPv4List = ();
	foreach my $IP ( @$IPList ) {
		if ( ip_is_ipv4($IP) and $IP !~ /^127\./ ) {
			push(@IPv4List, $IP);
		}
	}

	$logger -> vxlog("INFO", "Using IP addresses are: \n" . Dumper(\@IPv4List));
	# in case of non windows / linux, return empty list
	return \@IPv4List;
}


sub parseCmdLineParams
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	$logger -> vxlog("INFO", $logStr);

	my $ret    = CONST_SUCCESS;
	my $msg    = "";
	my $href   = ();

	my $PasswdHash = ();
	$PasswdHash -> { PlainPassword }  = "false";
	$PasswdHash -> { SecurePassword } = "true";

	my $ConsoleIP    = shift @ARGV;
	my $LocalUser    = shift @ARGV;
	my $LocalPasswd  = shift @ARGV;
	my $SecurePasswd = shift @ARGV || "PlainPassword";

	my $LocalIP     = getLocalIPList();

	if (not defined $ConsoleIP) {
		($ret, $msg) = getMessage( MSG_FILE, "SSO_INVALID_SYNTAX_CONSOLEIP");
		goto END;
	}

	if (not defined $LocalIP) {
		($ret, $msg) = getMessage( MSG_FILE, "SSO_INVALID_SYNTAX_LOCALIP");
		goto END;
	}

	if (not defined $LocalUser) {
		($ret, $msg) = getMessage( MSG_FILE, "SSO_INVALID_SYNTAX_LOCALUSER");
		goto END;
	}

	if (not defined $LocalPasswd) {
		($ret, $msg) = getMessage( MSG_FILE, "SSO_INVALID_SYNTAX_LOCALUSER");
		goto END;
	}

	if ($^O !~ /win/i ) {
		$SecurePasswd = "SecurePassword";
	}

	if (($SecurePasswd ne "SecurePassword") and 
	    ($SecurePasswd ne "PlainPassword")) {
		($ret, $msg) = getMessage( MSG_FILE, 
		                           "SSO_INVALID_SYNTAX_ENCRYPTEDPASSWD_WRONG_VALUE");
		goto END;
	}

	$href -> { ConsoleIP }    = $ConsoleIP;
	$href -> { LocalIP }      = $LocalIP;
	$href -> { LocalUser }    = $LocalUser;
	$href -> { LocalPasswd }  = $LocalPasswd;
	$href -> { SecurePasswd } = $PasswdHash -> { $SecurePasswd };
	
END:
	return ($ret, $msg, $href);
}


sub VirtualMachineToConsoleSSOConfigured
{
	my $href = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	$logger -> vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";

	my $cmd = "$xprtlc" .
		" -c appha://" . $href -> { ConsoleIP } . ":5634/" .
		" -l https://" . $href -> { ConsoleIP } . ":5634/appha_console/agent/verifySingleSignon.xml" . 
		" -u vxss://" . $href -> { ConsoleIP } . ":14153/appha_guest/appha_domain/vx";

	($ret, $msg) = $cmdutils -> runCmd( $cmd );

END:
	return ($ret, $msg);
}


sub triggerSSO
{
	my $href = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	$logger -> vxlog("INFO", $logStr);

	my $ret    = CONST_SUCCESS;
	my $msg    = "";

	my $LocalIP = "";
	my $IPListRef = $href -> { LocalIP };
	foreach my $GuestIP ( @$IPListRef ) {
		my $cmd = "$xprtlc -l https://" . $href -> { ConsoleIP } . ":5634/https://$GuestIP:5634/world/getvitals";
		my ($fret, $op, $CmdRet) = $cmdutils -> runCmd( $cmd );
		if ( $fret == CONST_SUCCESS ) {
			$LocalIP = $GuestIP;
			goto LOCALIP_FOUND;
		}
	}

	($ret, $msg) = getMessage( MSG_FILE, "LOCAL_IP_NOT_FOUND",  
	                           join (", ", @$IPListRef), CONSOLE_PROD_NAME,
	                           SFMH_SERVICE_NAME, CONSOLE_TOMCAT_SEVICE_NAME,
	                           CONSOLE_AUTH_SERVICE, COMMUNICATION_PORT_LIST);
	goto END;


LOCALIP_FOUND:
	my $cmd = "$xprtlc -l https://" . $href -> { ConsoleIP } . ":14152/ac/ConfigureSingleSignon" . 
			  " -d encrypted=" . $href -> { SecurePasswd } .
	          " -d vmIP=" . $LocalIP . 
	          " -d adminUserName=" . $href -> { LocalUser };

	$logger -> vxlog("INFO", "Executing cmd [$cmd -d adminPassword=XXX]");
	$cmd .=   " -d adminPassword=" . $href -> { LocalPasswd } .
	          " 2>&1";

	my ($fret, $op, $CmdRet) = $cmdutils -> runCmd( $cmd, { doNotLogCmd => 1} );

	#
	# check for cmd xml output as cmd exit code would be 0, but in reality cmd on server might have failed.
	# sample failure output:
	#
	# <?xml version="1.0" encoding="UTF-8"?>
	# <?xml-stylesheet type="text/xsl" href="/ac/css/errorPage.xsl"?>
	# <Cmd>
	# 	<ID>ConfigureSingleSignon</ID>
	#	<ReturnCode>-1</ReturnCode>
	#	<Message>Unexpected error occured</Message>
	# </Cmd>	
	#
	
	$logger -> vxlog("INFO", "Checking the output ...");
	my $tree = ();
	eval {
		$tree = Utils::convertXMLTextToHash( $op );
		$logger -> vxlog("INFO", "output is: \n" . Dumper( $tree ));
		if ( defined $tree and
		     $tree ne "" and
		     defined $tree -> { ReturnCode } and
		     $tree -> { ReturnCode } != 0 ) {

			$msg  = $tree -> { Message };
			($ret, $msg) = getMessage( MSG_FILE, "TRIGGER_SSO_FAILED", 
			               $msg, CONSOLE_PROD_NAME, SFMH_SERVICE_NAME,
			               CONSOLE_TOMCAT_SEVICE_NAME, CONSOLE_AUTH_SERVICE,
                           COMMUNICATION_PORT_LIST);
		}
	};
	if ( $@ ) {
		($ret, $msg) = getMessage( MSG_FILE, "TRIGGER_SSO_FAILED", 
		               $@, CONSOLE_PROD_NAME, SFMH_SERVICE_NAME,
		               CONSOLE_TOMCAT_SEVICE_NAME, CONSOLE_AUTH_SERVICE,
                       COMMUNICATION_PORT_LIST);
	}

END:
	return ($ret, $msg);
}


sub main
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	$logger -> vxlog("INFO", $logStr);

	my $ret    = CONST_SUCCESS;
	my $msg    = "";
	my $href   = ();

	($ret, $msg, $href) = parseCmdLineParams();
	if ( $ret != CONST_SUCCESS ) {
		usage();
		goto END;
	}

	# findout data dir, home dir etc
	setup_env();

    # create domain entry into smf_resolv.conf file.
	my $ConsoleIP = $href -> { ConsoleIP };
    update_sfm_resolve_file($ConsoleIP, $EAT_DATA_DIR);

	# pass-on information to console for configuring SSO
	my $cnt = 2;
	while ( $cnt > 0 ) {
		($ret, $msg) = triggerSSO( $href );		
		# check if really passed information did worked or not!
		if ( $ret == CONST_SUCCESS ) {
			($ret, $msg) = VirtualMachineToConsoleSSOConfigured( $href );
			if ( $ret == CONST_SUCCESS ) {
				goto END;
			}
		}
		sleep (2);
		$cnt --;
	}
END:
	return ($ret, $msg);
}


sub usage
{
	print "\n";
	print "Usage:\n";
	print "#perl configureSSO.pl <" . CONSOLE_PROD_NAME . " Address / Name> <Local Machine Username> <Local Machine Password> [PlainPassword|SecurePassword]";
	print "\n";
	print "\n";
}


#-----------
# Call main
#-----------
my ($ret, $msg) = main();
if ( $ret != CONST_SUCCESS ) {
	print $msg . "\n";
}
else {
	my $cmd = $notifysink_path . " -f";

	$logger -> vxlog("INFO", "Sending VCS service group information to console");
	my ($cmd_ret, $cmd_op) = $cmdutils -> runCmd( $cmd );
	if ( $cmd_ret != CONST_SUCCESS ) {
		$logger -> vxlog("WARNING", "Failed to send VCS service group information to console");
	}
}

$logger -> vxlog("INFO", "Exit code => " . $ret);
exit $ret;

