#!/opt/VRTSperl/bin/perl
#
# $Id $
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#

use CGI;
use Config;
use POSIX;
use XML::Simple;

# Push include paths!
my $LibPath = "";

BEGIN
{
    if ( $^O =~ m/win/i ) {
        require VRTS::Paths;

        my $xprtlInstallDir= VRTS::Paths::get_path("InstallDir");
        my $regquery = "\"$xprtlInstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\VERITAS\\VCS\\Base HomeDirectory";
        my $vcshome = `$regquery`;
        $LibPath = "$vcshome\\portal\\admin\\lib\\";
        $ENV{'VCS_HOME'} = $vcshome;

    } else {
        my $APPDISCOVER_HOME = "/opt/VRTSvcs/portal/admin";

        $LibPath = "$APPDISCOVER_HOME/lib/";
    }
}

use lib $LibPath;


use Utils;
use VCS;
use PlatUtils;

sub getESXStorage
{

	my ($diskid, $esxinfo) = @_;

	print "Getting ESX Storage information...\n";

	$params = "<Cmd>";
	
	$params .= "<ESXDetails>";
	foreach $esxelem (@$esxinfo) {
		@esx = split ':', $esxelem, 3;
		$esx[2] =~ s/"/\\"/g; # substitute all double quote characters as \" throughout
		$params .= "<ESX>";
		$params .= "<Name>$esx[0]</Name>";
		$params .= "<Port>443</Port>";
		$params .= "<Username>$esx[1]</Username>";
		$params .= "<Password><![CDATA[$esx[2]]]></Password>";
		$params .= "</ESX>";
	}
	$params .= "</ESXDetails>";

	$params .= "<StorageList>";
	foreach $disk (@$diskid) {
		$params .= "<DiskID>";
		$params .= $disk;
		$params .= "</DiskID>";
	}
	$params .= "</StorageList>";

	$params .= "</Cmd>";

	#print "Input command: $params\n";
	# Run xprtlc command
	my $output = `/opt/VRTSsfmh/bin/xprtlc -l https://localhost:5634/vcs/admin/configAppMon.pl -d ID=ESXStorage -d OP=discover -d params="$params"`;
	#print "output of ESXStorage: $output\n";
	
	my $tree = Utils::convertXMLTextToHash( $output );

	if ($tree->{ReturnCode} != 0) {
		# possible causes of failures:
		# (1) ESX passwords are correct
		# (2) VM is associated to the ESX systems
		# (3) Storage is connected to all ESX systems.
		# (4) Multiple SCSI controllers may be configured on the virtual machine.
		print "\nERROR: Failed to discover storage on all the input ESX Hosts.\n";
		print "\nLast command output is [$output]\n\n";
		print "Possible causes could be: \n";
		print "   - ESX passwords may be incorrect.\n";
		print "   - The virtual machine may not be associated to the ESX hosts.\n";
		print "   - The storage may not be connected or visible on each of the ESX hosts.\n";
		print "   - Multiple SCSI controllers may be configured on the virtual machine.\n";
		print "Please fix one or more of these problems and rerun the utility.\n";
		print "Exiting.\n";
		exit(1);
	}

	my $storagexml = "";
	if ( defined ($tree->{StorageDetails}) && ($tree->{StorageDetails} ne "")) {

		our %XMLOUT_SIMPLE_OPTIONS = (
				ContentKey => 'text',
				KeepRoot   => 1,
				NoAttr    => 1,
				KeyAttr    => []
				);	
		my $xs = new XML::Simple ();
		$storagexml = $xs->XMLout($tree->{StorageDetails},%XMLOUT_SIMPLE_OPTIONS);
		#print "Storage XML: $storagexml\n";
	}

	return ($ret, $storagexml);

}

sub discoverStorage
{
	my $mountpoint = shift;

	my $params = "";

	#print "Discovering storage for the mount point...\n";

	$params .= "<MountPointList>";

	foreach $mntpt (@$mountpoint) {
		$params .= "<MountPoint>";
		$params .= $mntpt;
		$params .= "</MountPoint>";
	}

	$params .= "</MountPointList>";

	#print "$params\n";
	# Run xprtlc command
	my $output = `/opt/VRTSsfmh/bin/xprtlc -l https://localhost:5634/vcs/admin/configAppMon.pl -d ID=VMwareDisk -d OP=discover -d params="$params"`;

	#print "OUTPUT of discoverStorage: $output\n";	
	my $tree = Utils::convertXMLTextToHash( $output );

	my (@inst) = ();
	if ( defined ($tree->{DiskIDList}{DiskID})) {
		@inst = ();
		my $iref = $tree->{DiskIDList}{DiskID};
		if ( ref($iref) =~ /^ARRAY/ ) {
			@inst = @$iref;
		} else {
			push( @inst, $iref );
		}
	} else {
		print "Internal Error: No Disk IDs found!\nLast command output is [$output]\n";
		exit(1);
	}

	my @diskid = ();
	foreach (@inst) {
		#print "DiskID: $_\n";
		push (@diskid, $_);
	}

	return ($ret, \@diskid);
}

sub commit 
{
	my $params = shift;

# Initialize return parameters
	my ( $ret, $msg, $xml, $insxml, $insResName) = (CONST_SUCCESS, "", "", "", "");
	my ( $hrInsResXMLs, $hrInsResNames, @inst ) = ();

	my $str = "";

	my $tree = Utils::convertXMLTextToHash( $params );

	#if ( $ret == CONST_FAILURE ) {
	#	Utils::vxlog("ERROR", "Commit input xml validation failed");
	#	goto END;

	#}

	my $href = PlatUtils::getVCSPaths();
	my $hares = $href -> { hares };
	my $haconf = $href -> { haconf };
	my $groupname = "";
	my ($retVal, $mntresname) = ();

# Get service group information
	my $vcsobj = VCS -> new();
	foreach my $mntpt (@mountpoints) {
		#print ("Mountpoint is $mntpt\n");
		my ($clusDetailsInputRef, $group) = ();
		$clusDetailsInputRef->{Resources}{AttributesVal}{MountPoint}{global} = "$mntpt";
		($retVal, $mntresname) = $vcsobj->isSimilarResourceExist($clusDetailsInputRef);
		if ($retVal) {
			#print("Resource Name: $mntresname\n");
			$groupname = `$hares -value $mntresname Group`;
			chomp ($groupname);
			#print("Group Name: $groupname\n");	
		} else {
			print "ERROR: No resource found with the mountpoint [$mntpt]. Exiting\n";
			exit(1);
		}
	}

	my ($resName, $vgstring, $vgdiskdep, $vgresname, $diskresname) = ();
	
	$vgstring = findVGDGResourceForMountResource_LinuxOnly ($mntresname);

	if ($vgstring eq "") {
                Utils::vxlog("ERROR", "Unable to locate DiskGroup or LVMVolumeGroup resource for Mount resource $mntresname. Please make sure that the Mount resource has a child Volume/LVMLogicalVolume resource and that resource has DiskGroup/LVMVolumeGroup resource.");
		print STDERR "ERROR: Unable to locate DiskGroup or LVMVolumeGroup resource for Mount resource $mntresname. Please make sure that the Mount resource has a child Volume/LVMLogicalVolume resource and that resource has DiskGroup/LVMVolumeGroup resource.\n";
                exit (1);
        }

	$vgstring =~ s/^\s*//;
	$vgstring =~ s/\s*$//;
	$vgdiskdep = `$hares -dep $vgstring`;
 	
	my ($line, @diskresname_var);		
	@vgdiskdep_var=split('\n', $vgdiskdep);
        foreach $line (@vgdiskdep_var) {
		
		if ($line =~ m/[ \t]+$vgstring[ \t]+(.*?)$/) {
			push(@diskresname_var, $1);
		}
	}
	$diskresname = $diskresname_var[0];
	# Make the configuration read write before editing VMwareDisks resource
	`$haconf -makerw &>/dev/null`;

	my $arrSize = @diskresname_var;
	if ($arrSize > 1) {
		Utils::vxlog("INFO",  "Consolidating VMwareDisks resources into a single resource $diskresname. " . $arrSize - 1 ." VMwareDisks resources will be deleted during consolidation process.");
		# Preserving the first resource and deleting the rest.
		foreach my $i (1..$#diskresname_var) {
			#print "\ni=<$i> <$vgstring> <$diskresname_var[$i]>\n";
			Utils::vxlog("INFO", "Removing resource dependency between $vgstring and $diskresname_var[$i].");
        		`hares -unlink $vgstring $diskresname_var[$i]`;
		 	Utils::vxlog("INFO", "Deleting $diskresname_var[$i] resource.");	
        		`hares -delete $diskresname_var[$i]`;
        	}	
	}

	if (!$diskresname) {
		print "Creating VMwareDisks resource...\n";

		# VMwareDisks resource NOT present!

		$resName = "VMwareDisks_$mntresname";
		Utils::vxlog("DEBUG", "resource Name:$resName");

		my $hares_add_output = `$hares -add $resName VMwareDisks $groupname 2>&1`;

		# If resource is successfully added, you see below message:
		# VCS NOTICE V-16-1-10242 Resource added. Enabled attribute must be set before agent monitors
		if ($? ne 0 or $hares_add_output !~ /V-16-1-10242/) {
			chomp ($hares_add_output);
			print "ERROR: Failed to create VMwareDisks resource [$resName].\n";
			print "VCS error string: [$hares_add_output].\n";
			print "Possible causes could be: \n";
			print "   - The VMwareDisks resource may already be present.\n";
			print "   - Some other resource with name same as [$resName] may be present.\n";
			print "Exiting.\n";
			exit (1);
		}
	} else {
		print "Updating existing VMwareDisks resource $diskresname ...\n";
		$resName = $diskresname;
		# Clear previous information about ESX servers and DiskPaths
		`$hares -modify $resName ESXDetails -delete -keys`;
		`$hares -modify $resName DiskPaths -delete -keys`;
	}


	if ( defined ($tree->{ESXDetails}{ESX})) {
		@inst = ();
		my $iref = $tree->{ESXDetails}{ESX};
		if ( ref($iref) =~ /^ARRAY/ ) {
			@inst = @$iref;
		} else {
			push( @inst, $iref );
		}
	}

	foreach (@inst) {
		my ($name, $user, $passwd) = ();
		while ( my ($attr, $val) = each %$_ ) {
# skip 'Name' special tag. It's not an attribute
			if ( $attr eq "Name" ) {
				$name = $val;
			} elsif ( $attr eq "Username" ) {
				$user = $val;
			} elsif ( $attr eq "Password" ) {
				$passwd = $val;
			}
		}
		if ($name && $user && $passwd) {
			`$hares -modify $resName ESXDetails -add $name $user=$passwd`;
		} else {
			Utils::vxlog("INFO", "Incorrect ESX credentials");
			$ret = 1;
		}
	}

	if ( defined ($tree->{Storage})) {
		@inst = ();
		my $iref = $tree->{Storage};
		if ( ref($iref) =~ /^ARRAY/ ) {
			@inst = @$iref;
		} else {
			push( @inst, $iref );
		}
	}

	my $cnt = 0;
	foreach (@inst) {

		my ($diskpath, $uuid, $rdm, $ctrlport) = ();
		while ( my ($attr, $val) = each %$_ ) {
# skip 'Name' special tag. It's not an attribute
			if ( $attr eq "DiskPath" ) {
			$diskpath = $val;
			}
			elsif ( $attr eq "DiskUUID" ) {
				$uuid = $val;
			}
			elsif ( $attr eq "RDM" ) {
				$rdm = $val;
			}
			elsif ( $attr eq "CtrlPort" ) {
				$ctrlport = $val;
			}
		}
		my $diskpathattr = "";
		if ($rdm == 1) {
			$diskpathattr .= "RDM:";
		}

		if ($uuid ne "") {
			$diskpathattr .= $uuid . ":";
		}
		$diskpathattr .= $diskpath;


		if (!$diskresname) {
			`$hares -modify $resName VMRegisterWait 5`;
			`$hares -modify $resName IsVMHAEnabled 1`;
		}

		`$hares -modify $resName DiskPaths -add "$diskpathattr" $ctrlport`;

		$cnt++;
	}

	chomp($groupname);

	# add dependency only if there is not an existing resource.
	if (($vgstring ne "") && (!$diskresname)) {
		#print "VG is: $vgstring\n";
		if ($vgstring =~ /^\s*(\S+)\s*/) {
			$vgres = $1;
			`$hares -link $vgres $resName`;
		}
	}
	`$hares -modify $resName Enabled 1`;
	`$haconf -dump -makero`;

	Utils::vxlog("INFO", $xml);
END:

	return 0;
}

sub findVGDGResourceForMountResource_LinuxOnly
{
	my $mntresname = shift;

        my $href = PlatUtils::getVCSPaths();
        my $hares = $href -> { hares };
	
	$mntvoldep = `$hares -dep $mntresname 2>/dev/null`;
 	if ( $? != 0 ) {
		$vgresname = "";
		goto RETURN;
	}	
	$mntvoldep =~ m/[ \t]+$mntresname[ \t]+(.*?)\n/;
	$volresname = $1;

	$volvgdep = `$hares -dep $volresname 2>/dev/null`;
	if ( $? != 0 ) {
                $vgresname = "";
		goto RETURN;
        }
	$volvgdep =~ m/[ \t]+$volresname[ \t]+(.*?)\n/;
	$vgresname = $1;

	RETURN:

	return $vgresname;
}

sub createResource
{
	my ($diskinfo, $esxinfo) = @_;
	my $vcsobj = VCS -> new();
	my $ret = CONST_SUCCESS;

	$params = "<Cmd>";
	
	$params .= "<ESXDetails>";
	foreach $esxelem (@$esxinfo) {
		@esx = split ':', $esxelem, 3;
		#$esx[2] =~ s/"/\\"/g; # substitute all double quote characters as \" throughout
		$params .= "<ESX>";
		$params .= "<Name>$esx[0]</Name>";
		$params .= "<Username>$esx[1]</Username>";
		my $passwd = "";
		my $CmdRet = "";
		if ( $^O =~ m/win/i ) {
			$passwd = Utils::encrypt_password($esx[2]);
		}
		else {
			($CmdRet, $passwd ) = $vcsobj->encrypt_password_aes($esx[2]);
			if ($CmdRet != CONST_SUCCESS ) {
				$ret = $CmdRet;
				print STDERR "Error:Failed while executing vcsencrypt\n";
				goto END;
			}
		}
		$params .= "<Password>$passwd</Password>";
		$params .= "</ESX>";
	}
	$params .= "</ESXDetails>";

	$params .= $diskinfo;

	$params .= "</Cmd>";

	#print "Commit Input: $params\n";
	# commit
	commit($params);

 END:
	return ($ret);
}

# main

my ($ret, $diskID, $diskInfo, $mountpoint, $ESXDetails) = ();

foreach my $arg (@ARGV) {
	if ($arg eq "-m") {
		$mntinfo = 1;
		$esxinfo = 0;
	} elsif ($arg eq "-e") {
		$esxinfo = 1;
		$mntinfo = 0;
	} elsif ($mntinfo == 1) {
		push(@mountpoints, $arg);
	} elsif ($esxinfo == 1) {
		push(@ESXDetails, $arg);
	}
	
}

if ((scalar @mountpoints == 0) || (scalar @ESXDetails == 0)) {
	print "ERROR: $0: Invalid arguments\n";
	#TODO: Add support for multiple mount points to this utility. --- print "Usage: $0 -m <mountpoint> [ mountpoint [..]] -e <esxhost:username:plaintext_password> [ esxhost:username:plaintext_password [..]]\n";
	print "Usage: $0 -m <mountpoint> -e <esxhost:username:plaintext_password> [ esxhost:username:plaintext_password [..]]\n";
	exit(1);
}

# strip trailing / from each mount point
foreach $mpref (@mountpoints) {
	$mpref =~ s/\/$//;
}

#print "Mount points : [@mountpoints]\n";

($ret, $diskID) = discoverStorage(\@mountpoints);

($ret, $diskInfo) = getESXStorage($diskID, \@ESXDetails);

($ret) = createResource($diskInfo, \@ESXDetails);

if ( $ret == CONST_SUCCESS ) {
	print "Success\n";
}
else {
	print STDERR "Failed\n";
}

exit(0);
