#!/opt/VRTSsfmh/bin/perl

#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: fetchClusterIDs.pl,v 1.5 2012/08/06 18:50:56 skambl2 Exp $
#

#------------------------
# Common Header
#------------------------
use strict;
use warnings;

use Data::Dumper;
use CGI;
use Config;
use POSIX;

# Push include paths!
my $LibPath1 = "";

BEGIN
{
	if ( $^O =~ m/win/i ) {
		require VRTS::Paths;

		my $xprtlInstallDir= VRTS::Paths::get_path("InstallDir");
		my $regquery = "\"$xprtlInstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\VERITAS\\VCS\\Base HomeDirectory";
		my $vcshome = `$regquery`;
		$LibPath1 = "$vcshome\\portal\\admin\\lib\\";
		$ENV{'VCS_HOME'} = $vcshome;

	} else {
		my $APPDISCOVER_HOME = "/opt/VRTSvcs/portal/admin";
	
		$LibPath1 = "$APPDISCOVER_HOME/lib";
	}
}

use lib $LibPath1;
#------------------------
# End of Common Header
#------------------------

use Constants;
use Utils;
use VCS;
use PlatUtils;


sub dumpClusterIDsToFile
{
	my ( $filename, $pidfile ) = @_;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	open (CLUSIDFILE, ">", $filename) or do {
		Utils::vxlog("ERROR", "Cannot write to file $filename ($!)");
		return CONST_FAILURE;
	};

	open (PIDLLTCONFIGFILE, ">", $pidfile) or do {
		Utils::vxlog("ERROR", "Cannot write to file $pidfile ($!)");
		return CONST_SUCCESS;
	};

	my $href = PlatUtils::getVCSPaths();
	my $lltconfig = $href -> { lltconfig };


	Utils::vxlog("INFO", "Executing lltconfig cmd [$lltconfig]");

	# TODO: move this to regular command line and check for return code of command.

	my $pidOfLLTConfigCmd = open ( LLTCONFIGFILE, "$lltconfig -N 2>&1 |" );

	my $cmdRet = CONST_SUCCESS;
	$cmdRet = CONST_FAILURE if ( not defined $pidOfLLTConfigCmd );
	if ( $cmdRet != CONST_SUCCESS ) {		
		Utils::vxlog("ERROR", "Failed to execute lltconfig command.");
		# TODO: handle error
	}

	Utils::vxlog("DEBUG", "PID of lltconfig cmd [$pidOfLLTConfigCmd]");
	print PIDLLTCONFIGFILE $pidOfLLTConfigCmd;
	close ( PIDLLTCONFIGFILE );

	while ( <LLTCONFIGFILE> ) {
		my @split_op = split ( /\s+|,/ );
		print CLUSIDFILE "$split_op[2]\n";
	}

	Utils::vxlog("INFO", "Successfully dumped discovered cluster IDs");
	close ( CLUSIDFILE );

	unlink ( $pidfile );
}

close ( STDIN );
close ( STDOUT );
close ( STDERR );

my $clusterIDsFile = PlatUtils::getLogDir() . "clusidfile";
my $pidFile = PlatUtils::getLockDir() . "lltconfig.pid";

if ( $^O =~ m/win/i )
{
	PlatVCS::initLLTService();
}
dumpClusterIDsToFile( $clusterIDsFile, $pidFile );
if ( $^O =~ m/win/i )
{
	PlatVCS::unInitLLTService();
	unlink ( PlatUtils::getLLTHostsFilePath() );
}

unlink ( PlatUtils::getLLTTabFilePath() );
