#!/opt/VRTSperl/bin/perl
#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: genConfigDetails.pl,v 1.2 2013/05/21 07:35:07 mtakwale Exp $
#

#------------------------
# Common Header
#------------------------
use strict;
use warnings;

use Data::Dumper;
use CGI;
use Config;
use POSIX;
use XML::Simple;

# Push include paths!
my $LibPath = "";

BEGIN
{
	if ( $^O =~ m/win/i ) {
		require VRTS::Paths;

		my $xprtlInstallDir= VRTS::Paths::get_path("InstallDir");
		my $regquery = "\"$xprtlInstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\VERITAS\\VCS\\Base HomeDirectory";
		my $vcshome = `$regquery`;
		$LibPath = "$vcshome\\portal\\admin\\lib\\";
		$ENV{'VCS_HOME'} = $vcshome;

	} else {
		my $APPDISCOVER_HOME = "/opt/VRTSvcs/portal/admin";
	
		$LibPath = "$APPDISCOVER_HOME/lib/";
	}
}

use lib $LibPath;
#------------------------
# End of Common Header
#------------------------

use Constants;
use Utils;
use VCS;
use MessageUtils;
use constant MSG_FILE => "appmon.msg";

#---------------------
# Subroutines
#---------------------
sub main
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $query = CGI->new();
	print $query->header('text/xml');

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

#	my $params = parseArguments($query);

	my $href = ();

	my $ConfigFile = "";

	eval {
		if ( $^O =~ /win/i ) {
			$ConfigFile = PlatUtils::getVCSHome() . "\\portal\\admin\\ConfigDetails.xml";
		} else {
			$ConfigFile = "/opt/VRTSvcs/portal/admin/ConfigDetails.xml";
		}

		$href->{'OS_DETAILS'} = Utils::getOSInfo();

		if ( $href->{'OS_DETAILS'}->{'VIRTUALIZATION_PLATFORM'} eq "VMware") {
			$href->{'TUNABLE_TASKS'}->{'CONFIG_NSSG'} = 1;
		}
		else {
			$href->{'TUNABLE_TASKS'}->{'CONFIG_NSSG'} = 0;
		}

   		my $xs = new XML::Simple ();
		my $xml = XMLout($href, NoAttr => 1, RootName => "ConfigDetails");

		open (CONFIGFILE,">$ConfigFile");
		print CONFIGFILE $xml;
		close (CONFIGFILE);

		$msg = "" if (not defined $msg);
		$xml = "" if (not defined $xml);
	};
	if ( $@ ) {
		$xml = "<Debug><![CDATA[$@]]></Debug>";
#		($ret, $msg) = getMessage( MSG_FILE, "CONFAPP_OP_EXECUTION_EXCEPTION", $OP, $ID);
		$ret = CONST_EXCEPTION;
	}
	
END:
	Utils::vxlog("INFO", Dumper( $xml ));	
	return $ret;	
}

my $ret = main();
exit $ret;

