
#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: CmdUtils.pm,v 1.1 2012/04/05 06:38:32 rjog Exp $
#

package CmdUtils;

use strict;
use warnings;

use Constants;

#---------------
# Constructor
#---------------
sub new 
{
	my $class = shift;
	my $logger = shift;

	my $self = {}; # allocate new hash for object
	bless($self, $class);

	$self -> { logger } = $logger;
	return $self;
}

#-------------------------------
# Command / File Run subroutines
#-------------------------------
sub strTrim
{
    my $str = shift;

    if (!defined $str) {
        return undef;
    }

    $str =~ s/\s+$//;
    $str =~ s/^\s+//;

    return $str;
}

sub runCmd
{
	my $self = shift;

	my $cmd = shift;
	my $hOpts = shift;

	my $logger         = $self -> { logger };
	my $doNotLogCmd    = $hOpts -> { doNotLogCmd } || 0;
	my $doNotLogOutput = $hOpts -> { doNotLogOutput } || 0;
	
	if ( !defined $cmd ) {
		return CONST_FAILURE;
	}

	$cmd = strTrim( $cmd );
	if ( $cmd eq "" ) {
		return CONST_FAILURE;
	}

	if ( $doNotLogCmd == 0 ) {
		my $logStr = "Executing [$cmd]";
		$logger -> vxlog("INFO", $logStr);
	}

	my $op = `$cmd 2>&1`;
	my $CmdRetCode = $? / 256;
	my $CmdRet = CONST_SUCCESS;

	$op = "" if (not defined $op);
	chomp $op;

	if ( $doNotLogOutput == 0 ) {
		my $logStr = "op=[$op], ret=[$CmdRetCode]";
		$logger -> vxlog("INFO", $logStr);
	}

	if ( $CmdRetCode != 0 ) {
		$CmdRet = CONST_FAILURE;
	}

	return ( $CmdRet, $op, $CmdRetCode );
}

1; # so that perl doesn't complain!
