#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Source: /project/vcs_extn/src_vcs/scripts/app_utils/applib/unix/GenericApplication.pm,v $
#
# $Id: GenericApplication.pm,v 1.41 2014/07/10 09:18:06 knagarka Exp $
#

package GenericApplication;

#use base qw(Application);
@ISA = ("UnixApplication");

use strict;
use warnings;
use Scalar::Util 'blessed';
use Data::Dumper;
use UnixApplication;
use Constants;
use Utils;
use PlatUtils;
use Storage;
use UnixNetwork;
use vcs_net_utils_inc;

sub validateAppInstances
{
        my $self = shift;
        my $params = shift;

        my $sMyName = ( caller 0 )[3];
        #---------
        # Announce
        #---------
        my $logStr = "Subroutine $sMyName called";
        Utils::vxlog("INFO", $logStr);

        my $ret = CONST_SUCCESS;
        my $msg = "Validation of application instances succeeded";
        my $xml = "<InvalidInstances>";

	my $tree = Utils::convertXMLTextToHash( $params );

	Utils::vxlog("INFO", Dumper( $tree ));

	my $hashSysPaths = PlatUtils::getSysPaths ();
	my $id = $hashSysPaths -> { id };
	my @validate_attrs = ( qw[ StartProgram StopProgram CleanProgram MonitorProgram ] );

	my @inst = ();
	if ( defined ($tree->{ApplicationInfo}{Instance})) {
		@inst = Utils::convertHashValuesToArray( $tree->{ApplicationInfo}{Instance} );
		foreach my $ins (@inst) {
			my $attr_xml = "";

			my $user_specified = 0;
			if ( defined $ins -> { User } && $ins -> { User } ne "" ) {
				$user_specified = 1;
			}

			foreach my $attr ( @validate_attrs ) {
				my $valid_param = CONST_SUCCESS;
				if ( defined $ins -> { $attr } && $ins -> { $attr } ne "" ) {
					my $val = $ins -> { $attr };
					$val =~ s/^\s*(\S+).*/$1/;
					if ( -f $val )
					{
						my $user = "";
						if ( $user_specified ) {
							$user = $ins -> { User };
						}
						$valid_param = PlatUtils::testUserFileExecPermission($val, $user);
					}
					else {
						$valid_param = CONST_FAILURE;
					}
					if ( $valid_param == CONST_FAILURE ) {				
						$attr_xml .= "<$attr>$val</$attr>";
					}
				}
			}
			if ( $user_specified ) {
				my $user = $ins -> { User };
				my $cmd = "$id $user";
				my ($CmdRet, $op) = Utils::runCmd($cmd);
				if ( $CmdRet != CONST_SUCCESS ) {
					$attr_xml .= "<User>$user</User>";
				}
			}
			if ( $attr_xml ne "" ) {
				$xml .= "<Instance>";
				$xml .= "<Name>";
				$xml .= $ins -> { Name };
				$xml .= "</Name>";
				$xml .= $attr_xml;
				$xml .= "</Instance>";

				$ret = CONST_FAILURE;
				$msg = "Validation of application instances failed";
			}
		}
	}
	$xml .= "</InvalidInstances>";
        Utils::vxlog("INFO", $xml);
        return ($ret, $msg, $xml);
}

sub discover
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "Generic App discovery successful";
	my $xml = "";

	($ret, $msg, $xml) = $self->doDiscovery();
	Utils::vxlog("INFO", "Discovered Generic App XML ...");
	Utils::vxlog("INFO", $xml);
	return ($ret, $msg, $xml);
}

sub getMounts
{
	my ($data, @mounts, @mount_data) = ();
	my ($mount_point, $BlkDev, $fstype) = ();
	my $sMyName = ( caller 0 )[3];
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);
	
	if ($^O =~ /sol/i) {
		# Use /etc/mnttab, as it has file system type info, 
		# which is not available in mount command output.
		$data = `cat /etc/mnttab`;
	}
	else {
		$data = `mount`;
	}

	@mount_data = split(/\n/, $data);

	foreach my $mount( @mount_data ) {
		chomp $mount;
		if($^O =~ /lin/i) {
			if ( $mount =~ m/^(\S+)\s(\S+)\s(\S+)\stype\s(\w+).*/ ) {
				$BlkDev = $1;
				$mount_point = $3;
				$fstype = $4;
			}
			if ( $BlkDev =~ m/^\/dev\/vx\/dsk\/([a-zA-Z0-9_-]+)\/([a-zA-Z0-9_-]+).*/  || 
					$BlkDev =~ m/^\/dev\/mapper\/([a-zA-Z0-9_+\.-]+)-([a-zA-Z0-9_+\.-]+).*/ ) {
				push(@mounts,$mount_point);
			}
		} # end if Linux
		elsif($^O =~ /aix/i) {
			# Example:
			#  node        mounted              mounted_over   vfs       date        options
			#              /dev/vx/dsk/dg1/vol1 /mnt           vxfs   Oct 10 00:43   rw,delaylog,suid
			#              /dev/fslv00          /vol1          jfs2   Oct 10 04:27   rw,log=/dev/loglv00
			#  nfsserver1  /re                  /re            nfs3   Oct 10 06:08   ro
			if ( $mount =~ m/^\s+(\S+)\s+(\S+)\s+(\S+)\s+\S+\s+\S+\s+\S+\s+(\S+)/ ) {
				$BlkDev = $1;
				$mount_point = $2;
				$fstype = $3;
				if ( $fstype =~ /vxfs|jfs/ ) {
					push(@mounts, $mount_point);
				}
			}
		} # end if AIX
		elsif($^O =~ /sol/i) {	
			# example:
			# /dev/vx/dsk/dg1/vol1 /mnt_pt1 vxfs  rw,suid,delaylog,dev=53c6d60 1323077240
			# nfsserver1:/re        /re      nfs   vers=3,xattr,dev=5bc001f     1354684713
			if ($mount =~ m/^(\S+)\s(\S+)\s(\S+)\s(\S*)(dev=)(\w+).*/ && $mount !~ m/(\S+)\s(\S+)\s(nfs).*/) {
				$BlkDev = $1;
				$mount_point = $2;
				$fstype = $3;
				if ( $BlkDev =~  m/^\/dev\/vx\/dsk\/([a-zA-Z0-9_]+)\/([a-zA-Z0-9_]+).*/ ) {
					push(@mounts, $mount_point);
				}
				elsif ($fstype =~ /zfs/) {
					push(@mounts, $mount_point);
				}
			}
		} # end if Solaris
	} # foreach

	Utils::vxlog("INFO", "Found mount points: [@mounts]");
	return @mounts;
}

sub doDiscovery
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "Generic App discovery successful";
	my $xml = "";

	my @Storageinsdata = ();
	my $SysDiskIDdata = ();

	my ($hrStorageDetails) = ();	
	my ($sDiskGroup, $sVolumeGroup) = ();
	my @mountpoints = ();
	@mountpoints = getMounts();
	my $Storage_ret = CONST_SUCCESS;
	my $arDiskIDs = ();
	my $iCnt =1;
	my (@StorageData) = ();
	my $resRef = $self->discoverExistingAppConfig();
	my $virt_plat = PlatUtils::getVirtualizationPlatform();

	foreach my $mountpoint( @mountpoints) { 
		$arDiskIDs = ();
		@StorageData = ();
		($Storage_ret, $hrStorageDetails) = $self->GetStorageDetails ($mountpoint);
		if ( CONST_SUCCESS == $Storage_ret ) {
			$hrStorageDetails -> { Instance } = "MountDetails" . "_" . "$iCnt";

			if ( $virt_plat eq "VMware" ) {
				if ( defined $hrStorageDetails -> { DiskGroup } ) {
					$sDiskGroup = $hrStorageDetails -> { DiskGroup };
					($Storage_ret, $msg, $arDiskIDs) = getVxVMDgDisks($sDiskGroup);
					if ($Storage_ret != CONST_SUCCESS) {
						$ret = $Storage_ret;
						goto END;
					}
				}
				if ( defined $hrStorageDetails -> { VolumeGroup } ) {
					$sVolumeGroup = $hrStorageDetails -> { VolumeGroup };
					($Storage_ret, $msg, $arDiskIDs) = getLVMVgDisks($sVolumeGroup);
					if ($Storage_ret != CONST_SUCCESS) {
						$ret = $Storage_ret;
						goto END;
					}
				}

				$hrStorageDetails -> { DiskID }  = $arDiskIDs;
				# Generate uniq DiskIDs
				foreach my $DiskID ( @$arDiskIDs ) {
					$SysDiskIDdata -> { $DiskID } = 1;
				}
			} # end if WMware

			push ( @StorageData, $hrStorageDetails);
			if (CONST_SUCCESS != $self->isStorageConfigured($resRef, \@StorageData)) {
				push ( @Storageinsdata, $hrStorageDetails);
			}
			else {
				Utils::vxlog("INFO", "Skipping mount point [$mountpoint] since there is already resource configured in VCS");
			}
		} else {
			Utils::vxlog("INFO", "Could not discover storage for [$mountpoint]");
		}
		$iCnt++;
	}

	$xml .= "<DiscoveryData>";
	$xml .= "<StorageList>";
	$xml .= $self->createApplicationXML( \@Storageinsdata, "Storage" );
        $xml .= "</StorageList>";
	$xml .= "\n<ConfiguredApplicationInfo>";

	######### discover existing config
	# In case of error, $resRef should be set to undef or "".
	my $resRef = $self->discoverExistingAppConfig();
	my ($start_program, $user) = ("", "");

	foreach my $resName (keys %$resRef) { 
		# If the resource is not of type Application, skip it
		if ($resRef->{$resName}{Type}{global} !~ /^Application$/) {
			next;
		}

		$start_program = $resRef -> { $resName } { 'StartProgram' } { 'global' };
		$user = $resRef -> { $resName } { 'User' } { 'global' };

		$xml .= "\n<Instance>";
		$xml .= "<Name>$resName</Name>";
		$xml .= "<StartProgram>$start_program</StartProgram>";
		$xml .= "<User>$user</User>";
		$xml .= "</Instance>";
	}

	$xml .= "</ConfiguredApplicationInfo>";

	$xml .= "</DiscoveryData>";

	Utils::vxlog("INFO", $xml);
END:
	return ($ret, $msg, $xml);
}

#---------------------------------------------
# Overriding the createCommitSGDetails method
#---------------------------------------------
sub createCommitSGDetails
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
#-------------------
# Announce ourselves
#-------------------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

# Initialize return parameters
	my ( $ret, $msg, $xml, $insxml, $insResName) = (CONST_SUCCESS, "", "", "", "");
	my ( $hrInsResXMLs, $hrInsResNames ) = ();

	my $SGName = "GenericApp_SG";
	my $str = "";

	my $tree = Utils::convertXMLTextToHash( $params );

	Utils::vxlog("INFO", Dumper( $tree ));

	($ret, $msg) = $self->validateXML($tree);

	if ( $ret == CONST_FAILURE ) {
		Utils::vxlog("ERROR", "Commit input xml validation failed");
		goto END;
		
	}

	if ( defined ($tree->{ApplicationInfo}{Instance})) {
		my @inst = ();
		my $iref = $tree->{ApplicationInfo}{Instance};
		if ( ref($iref) =~ /^ARRAY/ ) {
			@inst = @$iref;
		} else {
			push( @inst, $iref );
		}
		foreach (@inst) {
			$str = "$_->{Name}";
			$str =~ s/_Program$//;
			$SGName .= "_";
			$SGName .= $str;
		}
	}

	$SGName =  $self->GetUniqueGroupName("$SGName");
	Utils::vxlog("INFO", "Group Name:$SGName");

	$xml = "<xml>";

	if ( defined ($tree->{ApplicationInfo}{Instance})) {
		my @inst = ();
		my $iref = $tree->{ApplicationInfo}{Instance};
		if ( ref($iref) =~ /^ARRAY/ ) {
			@inst = @$iref;
		} else {
			push( @inst, $iref );
		}

		foreach (@inst) {
			$insResName =  $self->GetUniqueResName("$_->{Name}");
			Utils::vxlog("INFO", "Resource Name:$insResName");

			$self->addInstance($insResName);

			if (( defined ($_->{MountPointList})) and ($_ -> {MountPointList} ne "" ) and
                            ( defined ($_->{MountPointList}{MountPoint})) ) {
                                my @sMountPoints = Utils::convertHashValuesToArray ($_->{MountPointList}{MountPoint});
                                foreach my $sMountPoint (@sMountPoints) {
					my $selected = $sMountPoint->{ Selected };
                                        if ( $selected !~ /true/i ) {
                                                next;
                                        }
										$self->setMountDependency($insResName, $sMountPoint->{Value});
                                }
                        }

			$insxml = "<Resource>";
			$insxml .= "<Name>$insResName</Name>";
			$insxml .= "<Type>Application</Type>";

			$insxml .= "<OverrideAttributes>";
			#$xml .= "<Name>$_->{Name}</Name>";
			$insxml .= "<Name>IMF</Name>";
			$insxml .= "<attr>IMF</attr>";
			$insxml .= "</OverrideAttributes>";

			$insxml .= "<Attributes>";
			while ( my ($attr, $val) = each %$_ ) {
				# skip 'Name' special tag. It's not an attribute
				next if $attr eq "Name";
				next if $attr eq "IMF";
				next if $attr eq "MonitorProcesses";
				next if $attr eq "PidFiles";
				next if $attr eq "MountPointList";
				next if $attr eq "Storage";
				next if $attr eq "Selected";
				next if $attr eq "CleanProgramSelected";
				if ( $attr eq "MonitorProgram" ) {
					if ( $val =~ m!/etc/((init|(rc\.d/)?rc[0-6])\.d)/! ) {
						$val = "/opt/VRTSvcs/bin/Application/getServiceStatus $val";
					}
				}
				$insxml .= "<$attr><global>$val</global></$attr>";
			}
			if ( defined ($_->{IMF}) ) {
				my $mode = $_->{IMF}{Mode};
				my $monitor_freq = $_->{IMF}{MonitorFreq};
				my $reg_limit = $_->{IMF}{RegisterRetryLimit};
				$insxml .= "<IMF>";
				$insxml .="<global>Mode</global>";
				$insxml .="<global>$mode</global>";
				$insxml .="<global>MonitorFreq</global>";
				$insxml .="<global>$monitor_freq</global>";
				$insxml .="<global>RegisterRetryLimit</global>";
				$insxml .="<global>$reg_limit</global>";
				$insxml .= "</IMF>";
			}

			if ( defined ($_->{MonitorProcesses}) ) {
					if ( ref ($_->{MonitorProcesses}) =~ /^HASH/ ) {
						my @procs = ();
						my $pref = $_->{MonitorProcesses}{MonitorProcess};
						if ( ref($pref) =~ /^ARRAY/ ) {
							@procs = @$pref;
						} else {
							push ( @procs, $pref );
						}
						$insxml .= "<MonitorProcesses>";
						foreach ( @procs) {
							$insxml .="<global>$_</global>";
						}
						$insxml .= "</MonitorProcesses>";
					}
			}

			if ( defined ($_->{PidFiles}) ) {
				if ( ref ($_->{PidFiles}) =~ /^HASH/ ) {
					my @pids = ();
					my $pref = $_->{PidFiles}{PidFile};
					if ( ref($pref) =~ /^ARRAY/ ) {
						@pids = @$pref;
					} else {
						push ( @pids, $pref );
					}
					$insxml .= "<PidFiles>";
					foreach ( @pids) {
						$insxml .="<global>$_</global>";
					}
					$insxml .= "</PidFiles>";
				}
			}
			$insxml .= "</Attributes>";
			$insxml .= "</Resource>";
			$hrInsResXMLs -> { $insResName } = $insxml;
			$hrInsResNames -> { $insResName } = $insResName;

		}
	}

	if ( defined ($tree->{ApplicationInfo}{Dependency})) {
                my @dep = ();
                my $dref = $tree->{ApplicationInfo}{Dependency};
                if ( ref($dref) =~ /^ARRAY/ ) {
                        @dep = @$dref;
                } else {
                        push( @dep, $dref );
                }

                foreach (@dep) {
					$self->setInstanceDependency($_->{Parent}, $_->{Child});
                }
        }

        my $sSystemNICSGxml = "";
	($ret, $sSystemNICSGxml) = $self->getNetworkXML($tree);
        # TODO: check $ret
        $xml .= $sSystemNICSGxml;

	my $sgXML = $self->createSGXML ($tree, $hrInsResXMLs, $hrInsResNames, $SGName, "GenericApplication","Generic Application");
	$xml .= $sgXML;
	$xml .= "</xml>";
	Utils::vxlog("INFO", $xml);
END:
	my $retRef = ();

	$retRef -> { ret } = $ret;
	$retRef -> { msg } = $msg;
	$retRef -> { xml } = $xml;

	return $retRef;
}

sub findApplicationMounts
{
        my $self = shift;
        my $xmlstr = shift;

        my $ret = CONST_SUCCESS;
        my $sMyName = ( caller 0 )[3];
        #-------------------
        # Announce ourselves
        #-------------------
        my $logStr = "Subroutine $sMyName called";
        Utils::vxlog("INFO", $logStr);
        Utils::vxlog("INFO", "Input XML: $xmlstr ");

        my $tree = Utils::convertXMLTextToHash( $xmlstr );

	my $virt_plat = PlatUtils::getVirtualizationPlatform();

	if ( $virt_plat eq "Non-VMware" ) {
		Utils::vxlog("INFO", "Non VMware environment, returning from findApplicationMounts");
		return ( $ret , "" , "<StorageInfo/>");
	}

        my $retXML = "";
        my @aDiskIds = ();
        $retXML = "<StorageInfo>";
        #TODO: Uncomment below line, GUI changes required to pass <StorageInfo> to this function
        # Find the mount list array  for all applications
        # Find the diskID for each of the mountpoint in storage list
        my @arMountPointList = ();
        my @instances = Utils::convertHashValuesToArray($tree -> {ApplicationInfo}{ Instance });
        foreach my $ins (@instances) {
                if (($ins -> {Selected} =~ /true/ ) &&
                        (defined ($ins -> {MountPointList})) && ($ins -> {MountPointList} ne "" ) &&
                        (defined ($ins -> {MountPointList}{MountPoint})) ) {
                        my @selectedMountPoints = Utils::convertHashValuesToArray($ins -> {MountPointList}{MountPoint});
                        foreach(@selectedMountPoints) {
                                        if ( $_->{ Selected } !~ /true/i ) {
                                                next;
                                        }
                                        push(@arMountPointList, $_->{Value});
                        }
                }
        }
        my @arStorage = ();
        my @diskIds = ();
        my %uniqDisks = ();
        if ((defined ($tree -> {StorageList})) &&
                ($tree -> {StorageList} ne "") &&
                (defined ($tree -> {StorageList}{Storage}))) {
                @arStorage = Utils::convertHashValuesToArray($tree -> {StorageList}{Storage});
        }
        # find diskID of all the mount points
        foreach my $storage (@arStorage) {
                if ( (grep { $_ eq $storage->{MountPoint}{Value} } @arMountPointList)) {
                        if ((defined $storage->{DiskIDList}) &&
                                ($storage->{DiskIDList} ne "") &&
                                (defined $storage->{DiskIDList}{DiskID})) {
                                my @tmpDiskList = Utils::convertHashValuesToArray($storage->{DiskIDList}{DiskID});
                                foreach (@tmpDiskList) {
                                        if (!defined $uniqDisks{$_->{Value}}) {
                                                $retXML .= "<DiskID>$_->{Value}</DiskID>";
                                                $uniqDisks{$_->{Value}} = 1;
                                        }
                                }
                        }
                }
        }

        #$retXML .= "<DiskID>0:1</DiskID><DiskID>0:6</DiskID>";

        $retXML = $retXML . "</StorageInfo>";
        Utils::vxlog("INFO", "Output XML: $retXML");
        return ( $ret , "" , $retXML);
}

sub getIPParentInstance
{
    my $self = shift;
    my $resRef = shift;
    my $instResName= shift;
        if (defined $resRef-> { $instResName } {ResourceName} ) {
                return $resRef-> { $instResName } {ResourceName} {global};
        } else {
                return "";
        }
}

# This function would be called discoverExistingNetworkConfig to populate
# the instances to be shown on Network page
sub getDiscoveredInstances
{
    my $self = shift;
    my $resRef = shift;
    my @resList = ();
    foreach my $resName (keys %$resRef) {
        if ($resRef->{$resName}{Type}{global} !~ /^Application$/) {
            next;
        }
        if (defined $resRef->{ $resName }{ResourceName}) {
            push (@resList, $resRef->{ $resName }{ResourceName}{global});
        }
    }

    return \@resList;

}

sub validateXML
{
        my $self = shift;
	my $tree = shift;

	my $ret = CONST_SUCCESS;
	my $msg = "Valid commit input XML";
	my $sMyName = ( caller 0 )[3];

        my $logStr = "Subroutine $sMyName called";
        Utils::vxlog("INFO", $logStr);

	if( not defined $tree -> { ApplicationInfo } ) {
		$ret = CONST_FAILURE;
		$msg = "'ApplicationInfo' tag not provided in XML";
		Utils::vxlog("ERROR", $msg);
		goto END;
        }
        if ( not defined $tree -> { ApplicationInfo } { Instance } ) {
		$ret = CONST_FAILURE;
                $msg = "'Instance' tag inside 'ApplicationInfo' tag not provided in XML";
		Utils::vxlog("ERROR", $msg);
                goto END;
        }

	my @instances = ();
	@instances = Utils::convertHashValuesToArray( $tree->{ApplicationInfo}{Instance} );
	my @inst_attrs = qw( Name User StartProgram StopProgram CleanProgram MonitorProgram PidFiles MonitorProcesses IMF );
        foreach my $ins ( @instances ) {
                foreach my $attr ( @inst_attrs ) {
                        if (not defined $ins -> { $attr }) {
                                $msg = "$attr tag is not provided";
				$ret = CONST_FAILURE;
				Utils::vxlog("ERROR", $msg);
				goto END;
                        }
                        if ( $ins -> { $attr } eq "" and ( $attr eq "Name" or $attr eq "StartProgram" or $attr eq "StopProgram" )) {
                                $msg = "Value tag for $attr tag is not provided";
				$ret = CONST_FAILURE;
				Utils::vxlog("ERROR", $msg);
				goto END;
                        }
			if ( $attr eq "IMF" and ( ( not defined $ins -> {IMF}{RegisterRetryLimit} ) or ( not defined $ins -> {IMF}{MonitorFreq} ) or ( not defined $ins -> {IMF}{Mode} ) ) ) {
				$msg = "IMF tag not correctly set";
                                $ret = CONST_FAILURE;
				Utils::vxlog("ERROR", $msg);
                                goto END;
			}
                }
        }
END:
	return ($ret, $msg);
}

# This function is called to get description for service groups
# for displaying on UI
sub getSGDescription
{
	my $self = shift;
	my $sg_name = shift;
	my $resources = shift;
	my $msg = "Returning description for service group $sg_name : ";
	my $sg_desc = "";
	my $components = "";

	foreach my $res (@$resources) {
		my $res_xml = $self -> getResourceXML ( $res );
		my $res_details = Utils::convertXMLTextToHash( $res_xml );

		if ( defined ($res_details->{Type}) && $res_details->{Type} eq "Application" &&
			defined $res_details->{Name} ) {
				$components .= $res_details->{Name} . ", ";
			}
	}

	if ( $components ne "" ) {
		$components =~ s/(.*)\, /$1/;
		$sg_desc = "This service group contains the resources that relate to the following component(s): " . $components;
	}

	$msg .= $sg_desc;
	Utils::vxlog("DEBUG", $msg);

	return $sg_desc;
}

=comment

commit input xml data (params attribute value)
	<Cmd>
	<ID>LinuxGenericAppCommit</ID>
	<SystemList>...</SystemList>
	<ApplicationInfo>
    <Instance>
      <Name>cpuspeed_Program</Name>
      <StartProgram>/etc/init.d/cpuspeed start</StartProgram>
      <StopProgram>/etc/init.d/cpuspeed stop</StopProgram>
      <MonitorProgram>/etc/init.d/cpuspeed status</MonitorProgram>
      <CleanProgram>/etc/init.d/cpuspeed stop</CleanProgram>
      <User>root</User>
    </Instance>
    <Instance>
      <Name>cups_Program</Name>
      <StartProgram>/etc/init.d/cups start</StartProgram>
      <StopProgram>/etc/init.d/cups stop</StopProgram>
      <MonitorProgram>/etc/init.d/cups status</MonitorProgram>
      <CleanProgram>/etc/init.d/cups stop</CleanProgram>
      <User>root</User>
    </Instance>
    <Instance>
      <Name>sendmail_Program</Name>
      <StartProgram>/etc/init.d/sendmail start</StartProgram>
      <StopProgram>/etc/init.d/sendmail stop</StopProgram>
      <MonitorProgram>/etc/init.d/sendmail status</MonitorProgram>
      <CleanProgram>/etc/init.d/sendmail stop</CleanProgram>
      <User>root</User>
    </Instance>
    <Dependency>
      <Parent>cpuspeed_Program</Parent>
      <Child>cups_Program</Child>
    </Dependency>
    <Dependency>
      <Parent>cups_Program</Parent>
      <Child>sendmail_Program</Child>
    </Dependency>
  </ApplicationInfo>
</Cmd>

<Cmd>
  <ID>LinuxGenericAppCommit</ID>
  <SystemList/>
  <VirtualIPInfo/>
  <ApplicationInfo>
    <Instance>
      <Name>aaa_Program</Name>
      <StartProgram>aaa</StartProgram>
      <StopProgram>aaa</StopProgram>
      <MonitorProgram>aaa</MonitorProgram>
      <CleanProgram>aaa</CleanProgram>
      <User>root</User>
    </Instance>
    <Instance>
      <Name>bbb_Program</Name>
      <StartProgram>bbb</StartProgram>
      <StopProgram>bbb</StopProgram>
      <MonitorProgram>bbb</MonitorProgram>
      <CleanProgram>bbb</CleanProgram>
      <User>root</User>
    </Instance>
    <Instance>
      <Name>ccc_Program</Name>
      <StartProgram>ccc</StartProgram>
      <StopProgram>ccc</StopProgram>
      <MonitorProgram>ccc</MonitorProgram>
      <CleanProgram>ccc</CleanProgram>
      <User>root</User>
    </Instance>
    <Dependency>
      <Parent>aaa_Program</Parent>
      <Child>bbb_Program</Child>
    </Dependency>
    <Dependency>
      <Parent>bbb_Program</Parent>
      <Child>ccc_Program</Child>
    </Dependency>
  </ApplicationInfo>
  <VirtualIPInfo>
    <VirtualIP>
      <Type>ipv4</Type>
      <Address>1.1.1.1</Address>
      <NetMask>2.2.2.2</NetMask>
      <Lanman>
        <Name>3.3.3.3</Name>
        <ADUpdateRequired>1</ADUpdateRequired>
      </Lanman>
      <DNS>
        <Name>DNSName1</Name>
      </DNS>
      <NIC>
        <System>InguestVM01</System>
        <MAC>1</MAC>
      </NIC>
      <NIC>
        <System>InGuestVM02</System>
        <MAC>1</MAC>
      </NIC>
    </VirtualIP>
  </VirtualIPInfo>
  <SystemList>
    <System>
      <Name>InguestVM01</Name>
      <Priority>0</Priority>
    </System>
    <System>
      <Name>InGuestVM02</Name>
      <Priority>1</Priority>
    </System>
  </SystemList>
</Cmd>



commit xml (generic) schema
<xml>
	<ServiceGroup>
		<Name>sg1</Name>
		<Attributes>
			<attr1>
				<global>value</global>
				<!-- OR -->
				<sys1>value1</sys1>
				<sys2>value2</sys2>
				<sys3>value3</sys3>
			</attr1>
			<attr2>
				<global>value</global>
				<!-- OR -->
				<sys1>value1</sys1>
				<sys2>value2</sys2>
				<sys3>value3</sys3>
			</attr2>
		</Attributes>
		<Resources>
			<Resource>
				<Name>res1</Name>
				<Type>resType</Type>
				<SG>sg1</SG>
				<OverrideAttributes>
					<Name>res1</Name> <!-- resource name -->
					<attr>attr1<attr> <!-- name of attribute to override -->
					<Name>res2</Name>
					<attr>attr2<attr>
				</OverrideAttributes>
				<Attributes> <!-- Imp: Resource attributes -->
					<Attr1>
						<global>value</global>
						<!-- OR -->
						<sys1>value1</sys1>
						<sys2>value2</sys2>
						<sys3>value3</sys3>
					</Attr1>
					<Attr2>
						<global>value</global>
						<!-- OR -->
						<sys1>value1</sys1>
						<sys2>value2</sys2>
						<sys3>value3</sys3>
					</Attr2>
				</Attributes>
			</Resource>
		</Resources>
		<ResourceDependency>
			<Dependency>
				<Parent>parent_res</Parent>
				<Child>child_res</Child>
			</Dependency>
			<Dependency>
				<Parent>parent_res</Parent>
				<Child>child_res</Child>
			</Dependency>
		</ResourceDependency>
	</ServiceGroup>
	<ServiceGroup>
		<Name>sg2</Name>
		<Attributes>
			<attr1>
				<global>value</global>
				<!-- OR -->
				<sys1>value1</sys1>
				<sys2>value2</sys2>
				<sys3>value3</sys3>
			</attr1>
			<attr2>
				<global>value</global>
				<!-- OR -->
				<sys1>value1</sys1>
				<sys2>value2</sys2>
				<sys3>value3</sys3>
			</attr2>
		</Attributes>
		<Resources>
			<Resource>
				<Name>res1</Name>
				<Type>resType</Type>
				<SG>sg1</SG>
				<OverrideAttributes>
					<Name>res1</Name> <!-- resource name -->
					<attr>attr1<attr> <!-- name of attribute to override -->
				</OverrideAttributes>
				<Attributes> <!-- Imp: Resource attributes -->
					<Attr1>
						<global>value</global>
						<!-- OR -->
						<sys1>value1</sys1>
						<sys2>value2</sys2>
						<sys3>value3</sys3>
					</Attr1>
					<Attr2>
						<global>value</global>
						<!-- OR -->
						<sys1>value1</sys1>
						<sys2>value2</sys2>
						<sys3>value3</sys3>
					</Attr2>
				</Attributes>
			</Resource>
		</Resources>
		<ResourceDependency>
			<Dependency>
				<Parent>parent_res1</Parent>
				<Child>child_res1</Child>
			</Dependency>
			<Dependency>
				<Parent>parent_res2</Parent>
				<Child>child_res2</Child>
			</Dependency>
		</ResourceDependency>
	</ServiceGroup>
	<ServiceGroupDependency>
		<Dependency>
			<Parent>parent_sg1</Parent>
			<Child>child_sg1</Child>
			<Relationship>online local hard</Relationship>
		</Dependency>
		<Dependency>
			<Parent>parent_sg2</Parent>
			<Child>child_sg2</Child>
			<Relationship>online local firm</Relationship>
		</Dependency>
	</ServiceGroupDependency>
</xml>


=cut

1;
