
#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: LicenseManager.pm,v 1.3 2012/08/27 13:15:43 gadhaveg Exp $
#

package LicenseManager;

use base qw(Exporter);
our @EXPORT = qw(
	 );

use strict;
use warnings;

use Data::Dumper;

use Constants;
use Utils;
use MessageUtils;

use constant MSG_FILE => "clusterop.msg";

sub new
{
	my $class = shift;
	my $self = {}; # allocate new hash for object
	bless($self, $class);
	return $self;
}



#--------------------
# Sub routines ...
#--------------------
sub getLicInfo
{
	my $self = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $xmlstr = "";
	my $LicInfo = "";

	($ret, $xmlstr, $LicInfo) = $self -> getInstalledLicenses();
	if ( $ret != CONST_SUCCESS ) {
		goto END;
	}


	my @appha_lics = ();
		
	foreach my $lic ( keys %$LicInfo ) {		
			push(@appha_lics, $lic);		
	}
	

	$xmlstr = "<Licenses>";
	foreach my $lic ( @appha_lics ) {
		$xmlstr .= "<License>";
		$xmlstr .= "<LicenseKey>" . $lic . "</LicenseKey>";
		$xmlstr .= "<Type>" . $LicInfo -> { $lic } { LicenseType } . "</Type>";
		$xmlstr .= "<ExpiryDate><![CDATA[" . ($LicInfo -> { $lic } { ExpiryDate } || "") . "]]></ExpiryDate>";
		$xmlstr .= "<ApplicationHAEnabled>" . 
		            $LicInfo -> { $lic } { ApplicationHAFeature } . 
		           "</ApplicationHAEnabled>";
   		if ($LicInfo -> { $lic } { ApplicationHAFeature } =~ m/1|yes/i) 
		{
			$xmlstr .= "<Product>" . "ApplicationHA" . "</Product>";
		}
		else
		{
			$xmlstr .= "<Product>" . $LicInfo -> { $lic } { Product } . "</Product>";
		}
		$xmlstr .= "<Keyless>" . $LicInfo -> { $lic } { Keyless } . "</Keyless>";
		$xmlstr .= "</License>";
	}

	$xmlstr .= "</Licenses>";

END:
	return ($ret, $xmlstr)
}


sub installLic
{
	my $self = shift;
	my $lic = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $op = "";
	my $xmlstr = "";

	my $href = PlatUtils::getVCSPaths();
	my $vxlicinst = $href -> { vxlicinst };

	Utils::vxlog("INFO", "Passed data is: [$lic]" );
	$lic =~ s/"/\\"/g;
	my $cmd = "$vxlicinst -k \"$lic\"";
	($ret, $op) = Utils::runCmd( $cmd );
	if ( $ret != CONST_SUCCESS ) {
		( $ret, $xmlstr ) = getMessage( MSG_FILE, "LIC_INSTALL_FAILED");
	}

	my ($fret, $licInfo) = $self -> getLicInfo();

END:
	return ($ret, $xmlstr, $op, $licInfo)
}


sub getInstalledLicenses
{
	my $self = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my ($ret, $msg) = (CONST_SUCCESS, "");
	my $LicInfo = ();

	my $href = PlatUtils::getVCSPaths();
	my $vxlicrep = $href -> { vxlicrep };
	my ($CmdRet, $op) = runCmd ("$vxlicrep -e");
	if ( $CmdRet != CONST_SUCCESS ) {
		($ret, $msg) = getMessage( MSG_FILE, "LIC_COLLECTINFOFAILED" );
		goto END;
	}

	my @lines = split(/\n/, $op);
	my @lics = ();
	my $idx = -1;
	while ( $idx < $#lines ) {
		++ $idx;
		if ( $lines [$idx] =~ /-----------------/ ) {
			goto AFTER_INITIAL_LINES;
		}
	}

AFTER_INITIAL_LINES:
	++ $idx;  # skip --------******------- line

	while ($idx <= $#lines) {
		my $op = "";
		do {
			my $l = $lines [ $idx ];
			$l =~ s/^\s+//;
			$l =~ s/\s+$//;

			if ( $l ne "" ) {
				$op .= "$l\n";
			}

			++ $idx;
		}
		while ( $idx <= $#lines and $lines [$idx] !~ /License Key\s*=\s*/ );

		if ($op eq "") {
			next;
		}

		my @liclines = split(/\n/, $op);
		my @matchinglines = grep(/License Key/, @liclines);
		my $l = $matchinglines[0];
		my @fields = split(/\s+/, $l);
		my $lic = $fields[-1];

		$LicInfo -> { $lic } = ();
		$LicInfo -> { $lic } { CmdOutput } = $op;
		$LicInfo -> { $lic } { Expired	 } = 0;
		$LicInfo -> { $lic } { ApplicationHAFeature } = 0;
		$LicInfo -> { $lic } { LicenseType } = "";
		$LicInfo -> { $lic } { ExpiryDate }  = "";
		$LicInfo -> { $lic } { Product }  = "";
		$LicInfo -> { $lic } { Keyless }  = "NA";

		Utils::vxlog("INFO", "Processing license: [$lic]");
		Utils::vxlog("INFO", Dumper($LicInfo -> { $lic }));

		# check for Expired lic!
		if ( $op =~ /Invalid|EXPIRED/i ) {
			Utils::vxlog("INFO", "License is Invalid or Expired.");
			$LicInfo -> { $lic } { Expired } = 1;
		}

		if ( $op =~ /ApplicationHA\s+=\s+Enabled/i ) {
			$LicInfo -> { $lic } { ApplicationHAFeature } = 1;
		}

		if ( $op =~ /License Type\s*=\s*(.*)\n/i ) {
			$LicInfo -> { $lic } { LicenseType } = $1;
		}

		if ( $op =~ /End Date\s*=\s*(.*)\n/i ) {
			$LicInfo -> { $lic } { ExpiryDate } = $1;
		}

		if ( $LicInfo -> { $lic } { LicenseType } =~ /perm/i ) {
			$LicInfo -> { $lic } { ExpiryDate } = "-NA-";
		}
		
		if ( $op =~ /Keyless\s*=\s*(.*)\n/i  ) {
			$LicInfo -> { $lic } { Keyless } = $1;
		}
		
		# get the products
		my $prods = ();
		foreach my $prod ( grep (/Product Name/, @liclines) ) {
			$prod =~ s/\s+/ /;
			@fields = split(/=/, $prod);
			$fields[-1] =~ s/^\s+//;
			$fields[-1] =~ s/\s+$//;
			$prods -> { $fields[-1] } = 1;
		}

		$LicInfo -> { $lic } { Product } = join (", ", sort keys %$prods );

		Utils::vxlog("INFO", "License=[$lic], " . 
									"License Type=[" . $LicInfo -> { $lic } { LicenseType } . "], " . 
									"Expired=[" . $LicInfo -> { $lic } { Expired } . "], " . 
									"ApplicationHA Feature=[" . $LicInfo -> { $lic } { ApplicationHAFeature } . "]" .
									"Products=[" . $LicInfo -> { $lic } { Product } . "]");
	}

END:
	return ($ret, $msg, $LicInfo);
}


1; # so that perl doesn't complain
