
#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: Logger.pm,v 1.3 2014/05/29 12:43:59 sgupta2 Exp $
#

package Logger;

use strict;
use warnings;

use POSIX;
use File::Path;
use File::Basename;
use Fcntl qw(:DEFAULT :flock);

use PlatUtils;

#------------------
# Sub routines ...
#------------------
sub new
{
	my $class = shift;

	my $sLogFileName = shift;
	my $sLogDir		 = shift;
	my $iLogFileSize = shift;

	my $self = {}; # allocate new hash for object
	bless($self, $class);

	$self -> { logfileName } = $sLogFileName;
	$self -> { logDir }		 = $sLogDir;
	$self -> { logFileSize } = $iLogFileSize;

	die "LogFileName not passed" if (not defined $sLogFileName);
	die "LogDir not passed"		 if (not defined $sLogDir);
	die "LogFileSize not passed" if (not defined $iLogFileSize);

	if ( $^O =~ /win/ ) {
		$sLogDir =~ s/\\\\/\\/g;
		$sLogDir .= "\\" if ( $sLogDir !~ m@\$@);
		$sLogDir =~ s/\\/\\\\/g;
	} else {
		$sLogDir .= "/" if ( $sLogDir !~ m@/$@ );
	}

	my $logfile = $sLogDir . $sLogFileName;
	$self -> { logfile_without_suffix } = $logfile;

	$logfile .= "_A.log";
	$self -> { logfile } = $logfile;

	# create directory if not present
	mkpath $sLogDir if ( not -d $sLogDir );

	# create log file if not present
	if ( not -e $logfile ) {
		open  FH, ">$logfile";
		close FH;
	}

	$self -> rotateLogFile();
	return $self;
}


#-----------------------------
# Logging related subriutines
#-----------------------------
sub vxlog
{
	my $self = shift;

	my $level = shift || "UNSPECIFIED";
	my $log   = shift || "";

        if ($log =~ m/'Password' => (.*)/) {
                $log =~ s/'Password' => (.*)/'Password' => '********'/g;
        }

        if($log =~ m/TargetPassword=(.*)/) {
                $log =~ s/TargetPassword=(.*)/TargetPassword=********/g;
        }

        if ($log =~ m/'DomainPassword' => (.*)/) {
  		$log =~ s/'DomainPassword' => (.*)/'DomainPassword' => '********'/g;
	}

	if ($log =~ m/<Password>(.*)<\/Password>/) {
  		$log =~ s/<Password>(.*?)<\/Password>/<Password>********<\/Password>/g;
	}

=comment - This is required only if ESX password in ESXDetails tag is not encrypted.
	if ($log =~ m/<ESXDetails>(.*)<\/ESXDetails>/s) {
		my $esxDetails = $1;
		$esxDetails =~ s/<global>(.*?=)(.*?)(<\/global>)/<global>\1********\3/g;
		$log =~ s/<ESXDetails>(.*)<\/ESXDetails>/<ESXDetails>$esxDetails<\/ESXDetails>/gs;
	}

	if ($log =~ m/'ESXDetails' => {\s*'global' => \[(.*?)\]/s) {
		my $esxDetails = $1;
		$esxDetails =~ s/'(.*?)',(\s*)'(.*?)=(.*?)'/'\1',\2'\3=********'/g;
		$log =~ s/'ESXDetails' => {(\s*)'global' => \[(.*?)\]/'ESXDetails' => {\1'global' => \[$esxDetails\]/gs;
	}
=cut

	my $fileFullSpec = shift;
	my $line = shift;

	if ( !defined $fileFullSpec or !defined $line ) {
		my $package = ();
		( $package, $fileFullSpec, $line ) = caller;
	}

	my $filename = fileparse($fileFullSpec);
	my $timestamp = strftime '%Y-%m-%d %H:%M:%S', localtime;
	my $logfile = $self -> { logfile };

	open  FH, ">>$logfile";
	print FH  "[$timestamp] [PID:$$] [$level] [$filename, $line]: $log\n";
	close FH;

	return 0;
}


sub rotateLogFile
{
	my $self = shift;

	my $logDir		= $self -> { logDir };
	my $logfile		= $self -> { logfile_without_suffix };
	my $logFileName	= $self -> { logfile };
	my $logFileSize = $self -> { logFileSize };

	my $lockDir = PlatUtils::getLockDir();
	my $lockfile = $lockDir . "rotateLog.l";
	if ( not -f $lockfile ) {
		open FH, ">$lockfile";
		print FH "\n";
		close FH;
	}

	open (FILE, ">$lockfile") || return;
	flock( FILE, 2 ) || return;

	my $fileSize = -s $logFileName;
	my $bytes = $logFileSize * 1024 * 1024;

	if ( $fileSize > $bytes ) {
		my $AlphaH = ();

		$AlphaH -> { A } = "B";
		$AlphaH -> { B } = "C";
		$AlphaH -> { C } = "D";
		$AlphaH -> { D } = "E";

		my @array = reverse sort keys %$AlphaH;
		foreach my $Alpha ( @array ) {
			my $BackupFile = $logfile . "_$Alpha" . ".log";
			my $NBackupFile =  $logfile . "_" . $AlphaH -> { $Alpha } . ".log";
			if ( -f $BackupFile ) {
				my $cmd = "";
				if ( $^O =~ /win/i ) {
					$cmd = "move /Y \"$BackupFile\" \"$NBackupFile\"";
				} else {
					$cmd = "mv $BackupFile $NBackupFile 2>&1 > /dev/null";
				}
				`$cmd`;
			}
		}
	}

	close FILE;
}


1; # so that perl doesn't complain!
