
package Network;
=comment
XML Output Format:

<Result>
        <Host>
                <Name>10.209.144.195</Name>
                <AlreadyInUse>1</AlreadyInUse>
        </Host>
        <Host>
                <Name>10.209.144.194</Name>
                <AlreadyInUse>0</AlreadyInUse>
        </Host>
</Result>
=cut
use base qw(Application);

use strict;
use warnings;

use Constants;
use Utils;
use PlatUtils;
use Data::Dumper;
use Net::Ping;
use constant PING_TIMEOUT => 5;
use constant HOST_EXISTS => 1;
use constant HOST_UNREACHABLE => 0;

sub convertHashtoIPDetailsXml
{
	my $hostList = shift;
	my $xml;
	$xml = "<Result>\n";
	foreach my $host (keys %$hostList)
	{
		$xml .= "\t<Host>\n";
		$xml .= "\t\t<Name>$host</Name>\n";
		$xml .= "\t\t<AlreadyInUse>$hostList->{$host}</AlreadyInUse>\n";
		$xml .= "\t</Host>\n";
	}
	$xml .= "</Result>\n";
	return $xml;
}


sub validate
{
	my $self = shift;
	my $params = shift;
	my $ret = CONST_SUCCESS;
	my $msg = "";
	
	my $sMyName = ( caller 0 )[3];
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);
	
	my @hostsArr;
	@hostsArr = split(",",$params) if defined $params;
	
    my $pingObj = Net::Ping->new("icmp");
	if(!defined $pingObj)
	{
		$ret = CONST_FAILURE;
		$msg = "Failed to create object of ping.";
		goto EXIT;
	}
	
	my $hostList;
	foreach my $hostname (@hostsArr)
	{
		my $ping_host = $hostname;
		if ($^O =~ /aix/i) {
			$ping_host =~ s/:/\\:/g;
		}
		if ($pingObj->ping($ping_host))
		{
			$hostList->{$hostname} = HOST_EXISTS ;
		}
		else
		{
			$hostList->{$hostname} = HOST_UNREACHABLE ;
		}
	}	
    $pingObj->close();
	
	my $xml = convertHashtoIPDetailsXml($hostList);	
EXIT:
	return ($ret, $msg, $xml);
}

1;
