
#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: LinuxNetwork.pm,v 1.1 2013/01/30 17:01:29 sgupta2 Exp $
#

package PlatNetwork;

use strict;
use warnings;

use Net::IP qw(:PROC);

use PlatUtils;

sub getNICList
{
	my $nicListHash = {};
	my $nicID = 0;

	my $nic = "";
	my $nicHash = {};
	my $ipCount = 0;
	my $IPaddressHash = {};
	my %nicPrefixeHash = {};

	my $hashSysPaths = PlatUtils::getSysPaths ();
	my $ip = $hashSysPaths -> { ip };

	open (IPA, "$ip a |");

	while ( <IPA> ) {
		if ( /^\S/ ) {
			my @nicInfo = split ( /: / );
			$nic = $nicInfo[1];
			next if ( $nic =~ /lo|sit/ );

			$nicHash -> { IPAddresses } = $IPaddressHash if defined ( (keys %$IPaddressHash)[0] );
			$nicListHash -> { $nicID++ } = $nicHash if defined ( (keys %$nicHash)[0] );
			$nicHash = {};
			$nicHash -> { Caption } = $nic;
			$nicHash -> { DHCPEnabled } = "";
			$ipCount = 0;
			$IPaddressHash = {};
		}
		else {
			next if ( $nic =~ /lo|sit/ );

			s/^\s+//;

			my %IPDetailsHash;
			my @nicInfo = split /\s+/;
			if ( $nicInfo[0] =~ /link\/ether/ ) {
				$nicHash -> { MACAddress } = $nicInfo[1];
			}
			elsif ( $nicInfo[0] =~ /inet6/ ) {
				my @ipInfo = split ( /\//, $nicInfo[1]);
				$IPDetailsHash {IPAddress} = $ipInfo[0];
				$IPDetailsHash {NetMask} = "";
				$IPDetailsHash {PrefixLen} = $ipInfo[1];
				$IPDetailsHash {IPVersion} = "IPv6";
				$IPaddressHash -> { $ipCount++ } = \%IPDetailsHash;
				my $prefix = PlatUtils::getIPV6Prefix($ipInfo[0], $ipInfo[1]);
				$prefix = "$prefix/$ipInfo[1]";
				push(@{$nicPrefixeHash{$nicID}}, $prefix);
			}
			elsif ( $nicInfo[0] =~ /inet/ ) {
				my @ipInfo = split ( /\//, $nicInfo[1]) ;
				$IPDetailsHash {IPAddress} = $ipInfo[0];
				$IPDetailsHash {NetMask} = ip_bintoip ( ip_get_mask ( $ipInfo[1], 4 ), 4 );
				$IPDetailsHash {PrefixLen} = "";
				$IPDetailsHash {IPVersion} = "IPv4";
				$IPaddressHash -> { $ipCount++ } = \%IPDetailsHash;
			}
		}
	}

	$nicHash -> { IPAddresses } = $IPaddressHash;
	$nicListHash -> { $nicID++ } = $nicHash;
	foreach my $nicid (keys %$nicListHash) {
		$nicListHash->{$nicid}->{Prefixes} = $nicPrefixeHash{$nicid} if $nicPrefixeHash{$nicid};
	}

	return $nicListHash;
}

sub setPlatFilterAttrs
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $clusDetailsInputRef = shift;

	my $mii = "0";
	my $virt_plat = PlatUtils::getVirtualizationPlatform();

	# Set Mii to 0 for filtering in case of VMWare environment and linux platform
	if ( $virt_plat eq "VMware" ) {
		push(@{$clusDetailsInputRef->{Resources}{Attributes}},"Mii");
		$clusDetailsInputRef->{Resources}{AttributesVal}{Mii}{global} = "$mii";
	}
}

sub setPlatAttrsXML
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $hrVirtualIP = shift;

	my $mii = "0";
	my $virt_plat = PlatUtils::getVirtualizationPlatform();
	my $attrXML = "";

	# Set Mii attribute for NIC resource to 0 in case of VMWare environment and linux platform
	if ( $virt_plat eq "VMware" ) {
		$attrXML = "<Mii><global>$mii</global></Mii>";
	}

	return $attrXML;
}

1; # so that perl doesn't complain!
