
#------------------------------------------------------------------------------
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: SFMUtils.pm,v 1.2 2012/04/13 10:12:56 rjog Exp $
#------------------------------------------------------------------------------

package SFMUtils;

use base qw(Exporter);
our @EXPORT = qw(update_sfm_resolve_file
	get_domains
	print_domains
	add_domain
	save_domains
);

use strict;
use warnings;

use Constants;
use File::Copy;
use Data::Dumper;


#---------------
# Globals
#---------------
my $sfm_resolv	  = "";
my $sfm_fileName = "sfm_resolv.conf";

if ( $^O =~ /win/i ) {
	# windows section.
	my $sysDrive = $ENV { SystemDrive };
	$sfm_resolv = "$sysDrive\\Windows\\System32\\drivers\\etc\\$sfm_fileName";
} else {
	# unix section.
	$sfm_resolv = "/etc/default/$sfm_fileName";
}

#------------------------------------
# Function to update sfm_resolv file.
#------------------------------------

sub get_domains
{
	my $hDomains = ();

	if ( not -f $sfm_resolv ) {
		goto END;
	}

	open  FH, "<$sfm_resolv";

	my $in_domains_section = 0;
	my $current_domain = "";
	while ( <FH> ) {
		my $line = $_;
		chomp $line;

		if ($line =~ /^\[domains\]$/) {
			$in_domains_section = 1;
			next;
		}

		# this is the domain name
		if ( $in_domains_section == 1 and $line =~ /^(.*) = (.*);$/) {
			my $domain = $1;
			my $domain_name = $2;

			if ( not defined $hDomains -> { $domain } ) {
				$hDomains -> { $domain } = ();
			}

			$hDomains -> { $domain } { domain_name } = $domain_name;
			next;
		}

		$in_domains_section = 0;

		# domain details ...
		if ( $line =~ /^\[(.*)\]$/ ) {
			my $domain = $1;
			if ( not defined $hDomains -> { $domain } ) {
				$hDomains -> { $domain } = ();
			}

			$current_domain = $domain;
			next;
		}

		my @fields = split(/=/, $line);
		my $key = $fields[0];
		my $val = $fields[1];

		$key =~ s/^\s*//;  $key =~ s/\s*$//;
		$val =~ s/^\s*//;  $val =~ s/\s*;$//;

		$hDomains -> { $current_domain } { $key } = $val;
	}

	close FH;
END:
	return $hDomains;
}


sub remove_domains
{
	my $AppHAIPs = shift;

	my $hDomains = get_domains();
	foreach my $AppHAIPs ( @$AppHAIPs ) {
		my $domain = "appha://$AppHAIPs:5634/";
		if (defined $hDomains -> { $domain } and $hDomains -> { $domain } { domain_name } eq $AppHAIPs ) {
			delete $hDomains -> { $domain };
		}
	}
	save_domains();
}


sub remove_empty_domains
{
	my $hDomains = shift;

	if (not defined $hDomains) {
		goto END;
	}
 
	my @domains = sort keys %$hDomains;
	foreach my $domain ( @domains ) {
		my $href = $hDomains -> { $domain };
		my $domain_name = $href -> { domain_name };
		my @keys = keys %$href;

		my $sPrimaryBroker = "primary_broker";
		my $sEatDataDir    = "eat_data_dir";

		if (not grep /$sPrimaryBroker/, @keys and 
		    not grep /$sEatDataDir/, @keys) {
			delete $hDomains -> { $domain };
			next;
		}
	}

END:
	return $hDomains;
}


sub add_domain
{
	my $AppHAConsoleIP = shift;
	my $eat_data_dir = shift;
	my $component_name = shift || "";

	if ($component_name ne "" and $component_name !~ /^-/) {
		$component_name = "-$component_name";
	}

	my $hDomains = get_domains();

	if (not defined $AppHAConsoleIP or $AppHAConsoleIP eq "") {
		goto END;
	}

	my $domain = "appha://$AppHAConsoleIP" . $component_name . ":5634/";
	my $domain_name = $AppHAConsoleIP;
	my $primary_broker = "vxss://$AppHAConsoleIP:14153/appha_admin/appha_domain/vx";

	if ( not defined $hDomains -> { $domain } ) {
		$hDomains -> { $domain } = ();
	}

	$hDomains -> { $domain } { domain_name } = $domain_name;
	$hDomains -> { $domain } { eat_data_dir } = $eat_data_dir;
	$hDomains -> { $domain } { primary_broker } = $primary_broker;

END:
	save_domains($hDomains);
}


sub save_domains
{
	my $hDomains = shift;

	my $sfm_resolv_bk = $sfm_resolv . ".applicationha.save";

	if ( -f $sfm_resolv ) {
		copy($sfm_resolv, $sfm_resolv_bk);
	}

	# remove empty domains
	$hDomains = remove_empty_domains($hDomains);

	open  FH, ">$sfm_resolv";
	if ( scalar keys %$hDomains <= 0 ) {
		goto CLOSE_FILE;
	}

	print FH "[domains]\n";
	foreach my $domain ( sort keys %$hDomains ) {
		my $domain_name = $hDomains -> { $domain } { domain_name };
		if ($domain_name !~ /^"(.*)"$/) {
			$domain_name = "\"$domain_name\"";
		}

		print FH "$domain = $domain_name;\n";
		delete $hDomains -> { $domain } { domain_name };
	}

	foreach my $domain ( sort keys %$hDomains ) {
		my $href = $hDomains -> { $domain };
		print FH "[$domain]\n";

		foreach my $key ( sort keys %$href ) {
			my $val = $href -> { $key };

			$val =~ s/\\\\/\\/g;
			$val =~ s/\\/\\\\/g;

			print FH "$key = $val;\n";
		}
	}

CLOSE_FILE:
	close FH;
}


sub print_domains
{
	my $ret = CONST_SUCCESS;
	my $msg = "";

	my $hDomains = get_domains();
	foreach my $domain ( sort keys %$hDomains ) {
		print "Domain: [$domain] {\n";
		my $href = $hDomains -> { $domain };
		foreach my $key ( sort keys %$href ) {
			my $val = $href -> { $key };
			print "\t[$key]: [$val]\n";
		}
		print "}\n";
	}

	print "\n";

END:
	return($ret, $msg);
}


sub update_sfm_resolve_file
{
	my $ApplicationHAServerIP = shift;
	my $eat_data_dir = shift;
	my $component_name = shift || "";

	my $ret = CONST_SUCCESS;
	my $msg = "";

	if ($eat_data_dir !~ /^"/) {
		$eat_data_dir = "\"$eat_data_dir\"";
	}

	add_domain($ApplicationHAServerIP, $eat_data_dir, $component_name);
	start_xprtd_service();

END:
	return ($ret, $msg);
}


sub start_xprtd_service
{
	if ($^O =~ /win/i ) {
		`net start xprtld`;
	} else {
		`/etc/init.d/xprtld start`;
	}
}


1; # so that perl doesn't complain!
