#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: UnixApplication.pm,v 1.68 2014/05/29 12:44:00 sgupta2 Exp $
#

package UnixApplication;

use Application;
@ISA = ('Application');

use strict;
use warnings;

use Data::Dumper;
use Scalar::Util 'blessed';
use Constants;
use Utils;
# use PlatUtils;
use VCS;
use PlatNetwork;

use Storage;
use vcs_net_utils_inc;

my $hrUniqueResList = {};
my $hrUniqueGrpList = {};

my $vcsobj = new VCS();
$hrUniqueResList = $vcsobj -> getAllResources();
$hrUniqueGrpList = $vcsobj -> getAllGroups();

# my $lockFile = PlatUtils::getLockDir() . "HVLock.l"; 

#--------------------------------------------
# common functions for all Unix Applications
#--------------------------------------------

sub discoverMountPoint
{
	return undef;
}


sub createMountResXML
{
	return undef;
}

sub GetStorageDetails 
{

# . Following optional attributes of Mount agent are not configured by wizard
#	. VxFSMountLock
#	. RecursiveMnt
# . Volume set is not handled

	my $self = shift;
	my $sPath  = shift;
	my $sMyName = ( caller 0 )[3];
	my $isLVMDevice = 0;
	my ($vgName, $volName, $cmd, $vgMajorNumber) = ();
	my ($ZpoolName, $Legacy) = ();

	#---------
	# Announce
	#---------
	my $logStr = "Subroutine [$sMyName] called with Path [$sPath]";
	Utils::vxlog("INFO", $logStr);

	my %hStorageDetails = ();
	my %hValidatedStorageDetails = ();
	my ($lsOp) = ();

	if ($^O =~ /sol/i) {
		# Use /etc/mnttab, as it has file system type info, 
		# which is not available in mount command output.
		$cmd = "cat /etc/mnttab";
	}
	else {
		$cmd = "mount";
	}

	$lsOp = `$cmd 2>&1`;
	my $CmdRetCode = $? / 256;
	my $StorageRet = CONST_FAILURE; 

	$lsOp = "" if (not defined $lsOp);
	chomp $lsOp;

	if ( $CmdRetCode != 0 ) {
		$StorageRet = CONST_FAILURE;
		return ( $StorageRet, \%hStorageDetails);
	}

	my @lsMountPaths = split(/\n/, $lsOp);
	my ($sMountPoint,$sBlockDevice,$sFSType,$sVolType,$sMountOpt) = ();
	my ($itmpLength, $iLength) = (0,0);

	# This loop parses all the lines of mount output and checks for requried mount points.
	# Reverse the mount output so that in case of recursive mounts, given path matches
	# with the biggest portion of the mount point.
	foreach my $sMountPath (reverse @lsMountPaths) {
		chomp $sMountPath;
		%hStorageDetails = ();

		if ($^O =~ /lin/i) {
			# Examples:
			# /dev/vx/dsk/vcsdg/vcsvol on /mnt/dir1 type vxfs (rw,delaylog,largefiles,ioerror=mwdisable)
			# /dev/mapper/vg01-lvol01 on /mnt/myos type ext3 (rw)

			if ( $sMountPath =~ m/^(\S+)\s(\S+)\s(\S+)\stype\s(\w+).*/ ) {
				$sBlockDevice = $1;
				$sMountPoint = $3;
				$sFSType = $4;
				$sMountOpt = "";
				if ( $sMountPath =~ m/^(\S+)\s(\S+)\s(\S+)\stype\s(\w+)\s\((\S*)\).*/ ) {
					$sMountOpt = $5;
				}
			}
		} # Linux
		elsif ($^O =~ /aix/i) {
			# Examples:
			# node       mounted              mntd_over  vfs    date           options
			#            /dev/vx/dsk/dg1/vol1 /mnt       vxfs   Oct 10 00:43   rw,delaylog,suid
			#            /dev/fslv00          /vol1      jfs2   Oct 10 04:27   rw,log=/dev/loglv00
			# punevault  /re                  /re        nfs3   Oct 10 06:08   ro
			if($sMountPath =~ m/^\s+(\S+)\s+(\S+)\s+(\S+)\s+\S+\s+\S+\s+\S+\s+(\S+)/) {
				$sBlockDevice = $1;
				$sMountPoint = $2;
				$sFSType = $3;
				$sMountOpt = $4;
			}
		} # AIX
		elsif ($^O =~ /sol/i) {
			# Example:
			# /dev/vx/dsk/dg1/vol1 /mnt_pt1 vxfs  rw,suid,delaylog,dev=53c6d60 1323077240
			# nfsserver1:/re        /re      nfs   vers=3,xattr,dev=5bc001f     1354684713
			if ($sMountPath =~ m/^(\S+)\s(\S+)\s(\S+)\s(\S*dev=\S*)\s(\w+).*/) {
				$sBlockDevice = $1;
				$sMountPoint = $2;
				$sFSType = $3;
				$sMountOpt = $4;
			}
		}

		if ( $sPath =~ /^$sMountPoint\b/ ) {
			$itmpLength = length($sMountPoint);
			if ( "/" eq $sMountPoint ) {
				next;
			}
			Utils::vxlog("INFO", "Found MountPoint [$sMountPoint] ");

			if ($itmpLength > $iLength) {

				if ( $sBlockDevice =~ m/^\/dev\/vx\/dsk\/([a-zA-Z0-9_+\.-]+)\/([a-zA-Z0-9_+\.-]+).*/ ) {
					# VxVM block device
					$hStorageDetails{DiskGroup} = $1;
					$hStorageDetails{Volume} = $2;
				}

				# LVM block device
				if ($^O =~ /lin/i) {
					if ( $sBlockDevice =~ m/^\/dev\/mapper\// ) {
						# We got the substring. However, we can not separate the VG name and Volume name 
						# from available information.
						$isLVMDevice = 1;
						($StorageRet, $vgName, $volName) = $self->matchLVMVolume_Linux($sBlockDevice);
					}
				}
				elsif ($^O =~ /aix/i) {
					if ( $sFSType =~ /jfs/ ) {
						$isLVMDevice = 1;
						($StorageRet, $vgName, $volName, $vgMajorNumber) = $self->matchLVMVolume_AIX($sBlockDevice);
					}
				}
				elsif ($^O =~ /sol/i) {
					($StorageRet, $ZpoolName, $Legacy) = $self->matchZpool_Sol($sBlockDevice);
					if ($StorageRet == CONST_SUCCESS) {
						$isLVMDevice = 1;
					}
					else {
						Utils::vxlog("INFO", "Skipping the file system [$sMountPoint] because it is not of ZFS type.");
					}
				}

				if($isLVMDevice) {
					goto END if ($StorageRet != CONST_SUCCESS);
					if ($^O =~ /sol/i) {
						$hStorageDetails{PoolName} = $ZpoolName;
						$hStorageDetails{Legacy} = $Legacy;
					}
					else {
						$hStorageDetails{VolumeGroup} = $vgName;
						$hStorageDetails{LogicalVolume} = $volName;
						if ($^O =~ /aix/i) {
							$hStorageDetails{MajorNumber} = $vgMajorNumber;
						}
					}
				}

				if ( ( not defined $hStorageDetails{DiskGroup} ) and
						( not defined $hStorageDetails{VolumeGroup} ) and 
							( not defined $hStorageDetails{PoolName} ) ) {
						Utils::vxlog("INFO", "Skipping the file system [$sMountPoint] from the mounted file system list because it is not mounted from VxVM, LVM or ZFS storage pool.");
						next;
				} else {
					$hStorageDetails{MountPoint} = $sMountPoint;
					$hStorageDetails{BlockDevice} = $sBlockDevice;
					$hStorageDetails{FSType} = $sFSType;
					$hStorageDetails{MountOpt} = $sMountOpt;

					$iLength = $itmpLength;
					if ( $iLength > 1 ) {
						$StorageRet = CONST_SUCCESS;
						%hValidatedStorageDetails = %hStorageDetails;
						last;
					}
				}
			}
		}
	}
END:
       	return ( $StorageRet, \%hValidatedStorageDetails);
}

# Syntax:
# sgXML = createSGXML ($SystemListxml);

sub createSGXML
{
	my ($self, $tree, $hrInsResXMLs, $hrInsResNames, $sSGNamePrefix,$sTypeId,$sDisplayName) = (@_);
	my $SystemListxml = "";
	my $resDepXml = "";
	my $ret = CONST_SUCCESS;
	my $xml = "";
	my ($msg);

	my $sMySubName = ( caller 0 )[3];
	my $announceStr = "Subroutine $sMySubName called";
	Utils::vxlog("INFO", $announceStr);

	# Get SystemListxml
	($ret, $SystemListxml) = Application::getSystemListXML($tree->{SystemList});
	if ( CONST_FAILURE == $ret ) {
		$msg = "SystemList tag not found in input XML to createCommitSGDetails";
		goto END;
	}

	# set global hashes($hrDgMounts, $hrMountDg, $hrResXMLs, $hrResNames, $hrDepMountVol, $hrDepVolDg, $hrDepDgDisks) 
	$ret = $self->createStorageSGTables ($tree->{StorageList}, $tree->{VMwareStorageInfo}); 
	if ( CONST_FAILURE == $ret ) {
		$msg = "Error while creating storage xml."; goto END;
	}
	$self->defineInstanceGroupings ( $sSGNamePrefix ); 
	my $logStr = Dumper( $hrSGs );
    	Utils::vxlog("INFO", $logStr);

	@{$hrResXMLs}{keys %$hrInsResXMLs} = values %$hrInsResXMLs;
	@{$hrResNames}{keys %$hrInsResNames} = values %$hrInsResNames;

	Utils::vxlog("INFO", " [[hrResNames]]"); Utils::vxlog("INFO", Dumper( $hrResNames ));
	Utils::vxlog("INFO", " [[hrResXMLs]]"); Utils::vxlog("INFO", Dumper( $hrResXMLs ));

	foreach my $SGName ( keys %{$hrSGs} ) {
		#### print "\n $SGName: ";
		$xml .= "<ServiceGroup>";
		$xml .=	"<Name>";
		$xml .=	$SGName;
		$xml .= "</Name>";
		$xml .= "<Description>";
		$xml .= $self -> getSGDescription( $SGName, $hrSGs->{$SGName} );
		$xml .= "</Description>";
		$xml .= "<Attributes>";
		$xml .= $SystemListxml;
                if ( ( defined $sTypeId ) && ( defined $sDisplayName ) ) {
                        $xml .= "<UserAssoc>";
                        $xml .= "<global>Type</global>";
                        $xml .= "<global>$sTypeId</global>";
                        $xml .= "<global>Name</global>";
                        $xml .= "<global>$sDisplayName</global>";
                        $xml .="</UserAssoc>";
                }

		$xml .= "</Attributes>";
		$xml .= "<Resources>";
		$resDepXml = "<ResourceDependency>";
		foreach my $sResourceValue ( @{$hrSGs->{$SGName}} ){
			#### print "Processing sResourceValue = [$sResourceValue] \n";
			my $sResourceName = "";
			if ( not defined $hrResNames->{$sResourceValue} ) {
				#### print " [$sResourceValue] not found in hrResNames . "; #We should not come here
				$xml .= "<!-- Internal Error -->";
				$xml .= "</Resources>";
				goto END;
			} 
			$sResourceName = $hrResNames->{$sResourceValue}  ;
			#### print   $sResourceName  ; print  " , ";
			# Append resource definition to XML
			$xml .= $hrResXMLs->{$sResourceValue};
			#### print "xml = $xml \n";

			# For VxVM storage, the $hrDepMountVol->{$sResourceName} will contain
			# mapping from Mount to volume resource.

			# Similarly for LVM storage on Linux platform, the $hrDepMountVol->{$sResourceName} will contain
			# mapping from Mount to volume resource.

			# For AIX LVM storage , $hrDepMountVol->{$sResourceName}  will contain 
			# mapping from Mount to VG resource

			if ( defined $hrDepMountVol->{$sResourceName}) {

				$resDepXml .= "<Dependency>";
				$resDepXml .= "<Parent>" . $sResourceName 			. "</Parent>";
				$resDepXml .= "<Child>"  . $hrDepMountVol->{$sResourceName} 	. "</Child>";
				$resDepXml .= "</Dependency>";

				# For VxVM storage,$hrDepVolDg->{$hrDepMountVol->{$sResourceName}} will contain
				# mapping from Volume to DG resource.

				# Similarly for Linux LVM storage,$hrDepVolDg->{$hrDepMountVol->{$sResourceName}} will contain
				# mapping from Volume to VG resource.

				# For AIX native LVM,$hrDepVolDg->{$hrDepMountVol->{$sResourceName}} will not be defined

				if ( defined $hrDepVolDg->{$hrDepMountVol->{$sResourceName}} ) {
					$resDepXml .= "<Dependency>";
					$resDepXml .= "<Parent>" . $hrDepMountVol->{$sResourceName} 	. "</Parent>";
					$resDepXml .="<Child>".$hrDepVolDg->{$hrDepMountVol->{$sResourceName}}."</Child>";
					$resDepXml .= "</Dependency>";
				}
			}
				
			if ( defined $hrInstanceMounts->{$sResourceValue}) { 
			    my $sMntResName = "";
			    my @tmpMntResList = ();
			    @tmpMntResList = Utils::convertHashValuesToArray ($hrInstanceMounts->{$sResourceValue});
			    @tmpMntResList = removeDuplicates(\@tmpMntResList); 

        		    foreach my $tmpMntResValue (@tmpMntResList){
				$resDepXml .= "<Dependency>";
				$resDepXml .= "<Parent>" . $sResourceName 			. "</Parent>";
				$resDepXml .= "<Child>"  . $hrResNames->{$tmpMntResValue} 	. "</Child>";
				$resDepXml .= "</Dependency>";
			    }	
			}
			if ( defined $hrInsChildParents->{$sResourceValue}) { 
			    my @tmpParentResList = ();
			    @tmpParentResList = Utils::convertHashValuesToArray ($hrInsChildParents->{$sResourceValue});
			    @tmpParentResList = removeDuplicates(\@tmpParentResList); 
        		    foreach my $sParentResValue (@tmpParentResList){
				$resDepXml .= "<Dependency>";
				$resDepXml .= "<Parent>" . $hrResNames->{$sParentResValue}	. "</Parent>";
				$resDepXml .= "<Child>"  . $sResourceName 		 	. "</Child>";
				$resDepXml .= "</Dependency>";
			    }	
			}	
			if ( defined $hrDepDgDisks ->{$sResourceName}) { 
			    my $sDiskResName = "";
			    my @tmpDiskResList = ();
			    @tmpDiskResList = Utils::convertHashValuesToArray ($hrDepDgDisks->{$sResourceName});
			    @tmpDiskResList = removeDuplicates(\@tmpDiskResList); 
        		    foreach my $tmpDiskResName (@tmpDiskResList){
				$resDepXml .= "<Dependency>";
				$resDepXml .= "<Parent>" . $sResourceName 			. "</Parent>";
				$resDepXml .= "<Child>"  . $tmpDiskResName		 	. "</Child>";
				$resDepXml .= "</Dependency>";
			    }	
			}
			if ( defined $hrIPProxy ->{$sResourceValue}) { 
				$resDepXml .= "<Dependency>";
				$resDepXml .= "<Parent>" . $sResourceName		 	. "</Parent>";
				$resDepXml .="<Child>". $hrResNames->{$hrIPProxy ->{$sResourceValue}}		 ."</Child>";
				$resDepXml .= "</Dependency>";
			}
			if ( defined $hrInstanceIP ->{$sResourceValue}) { 
				$resDepXml .= "<Dependency>";
				$resDepXml .= "<Parent>" . $sResourceName		 	. "</Parent>";
				$resDepXml .="<Child>". $hrResNames->{$hrInstanceIP ->{$sResourceValue}}		 ."</Child>";
				$resDepXml .= "</Dependency>";
			}
		}
		$xml .= "</Resources>";
		if ( not ( $resDepXml eq "<ResourceDependency>" ) ) {
			$resDepXml .= "</ResourceDependency>";
                	$xml .= $resDepXml;
		} 
		$xml .= "</ServiceGroup>";
	}

END:
	return $xml;
}

#
# Function: createApplicationXML
# Objective:
#   The function reads the application specific information from in-memory hash
#   and creates Discovery xml out of that!
#
#   The in-memory structure should look like as follow
#      $VAR1 = [
#          {
#             'Instance' => '<Instance Name>'
#             'attr1' => 'val1'
#             'attr2' => [
#                          'val21',
#                          'val22' 
#                        ],
#             'attr3' => 'val3'
#          }
#        ];
#
#   For SAP, the following structure is passed
#      $VAR1 = [
#          {
#            'Instance' => 'ASCS40',
#            'StartProfile' => '/sapmnt/ER1/profile/START_ASCS40_rhel5vm4',
#            'SAPHome' => '',
#            'ProcMon' => 'cs',
#            'SAPSID' => 'ER1',
#            'SAPAdmin' => '',
#            'InstType' => 'CENTRAL',
#            'InstName' => 'ASCS40'
#          }
#        ];
#
#	User can pass the some other xml content the second parameter,
#		if passed, then after creating xml for first parameter, function appends the passed xml.
#
#
sub createApplicationXML
{
	my $self = shift;
	my $insdataRef = shift;
	my $discoveredEntity = shift;
	my $extraTags = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	$logStr = Dumper( $insdataRef );
    	Utils::vxlog("INFO", $logStr);

	my $xmlstr = "";

	foreach my $data ( @$insdataRef ) {

		my $Name = $data -> { Instance };
		$xmlstr .= "<$discoveredEntity>";
		$xmlstr .= "<Name>$Name</Name>";

		foreach my $key ( keys %$data ) {
			if ($key =~ /Instance/i ) {
				next;
			}

			my $ref = $data -> { $key };
			my $str = "$data->{$key}";

			my $outerKey = 0;
			my @values = ();

			if ($str =~ /^ARRAY/) {
				@values = @$ref;
				$outerKey = 1;
			} else {
				push (@values, $ref);
			}

			if ( $str =~ /ARRAY/ ) {
				$xmlstr .= "<$key"  . "List>";
			}

			foreach my $val ( @values ) {
				$xmlstr .= "<$key><Value>$val</Value></$key>";
			}

			if ( $str =~ /ARRAY/ ) {
				$xmlstr .= "</$key"  . "List>";
			}
		}
		
		$xmlstr .= "</$discoveredEntity>";
    }

	# Extra Tags provision
	my $TagsToAdd = $extraTags || "";
	Utils::vxlog("INFO", "adding extra tags [$TagsToAdd]");
	$xmlstr .= $TagsToAdd;

	chomp ($xmlstr);
	return $xmlstr;
}

sub findApplicationMounts
{
        my $self = shift;
        my $xmlstr = shift;

        my $ret = CONST_SUCCESS;
        my $sMyName = ( caller 0 )[3];
        #-------------------
        # Announce ourselves
        #-------------------
        my $logStr = "Subroutine $sMyName called";
        Utils::vxlog("INFO", $logStr);
        Utils::vxlog("INFO", "Input XML: $xmlstr ");

        my $tree = Utils::convertXMLTextToHash( $xmlstr );

        my $retXML = "";
        my @aDiskIds = ();
        $retXML = "<StorageInfo>";
        # Find the mount list array  for all applications
        # Find the diskID for each of the mountpoint in storage list
        my @arMountPointList = ();
	my @instances = Utils::convertHashValuesToArray($tree -> { Instance });
	foreach my $ins (@instances) {
		if (($ins -> {Selected} =~ /true/ ) &&
			(defined ($ins -> {MountPointList})) && ($ins -> {MountPointList} ne "") &&
			(defined ($ins -> {MountPointList}{MountPoint})) ) {
			my @selectedMountPoints = Utils::convertHashValuesToArray($ins -> {MountPointList}{MountPoint});
			foreach(@selectedMountPoints) {
				push(@arMountPointList, $_->{Value});
			}
		}
	}
	my @arStorage = ();
	my @diskIds = ();
	my %uniqDisks = ();
	if ((defined ($tree -> {StorageList})) && 
		($tree -> {StorageList} ne "") &&
		(defined ($tree -> {StorageList}{Storage}))) {
		@arStorage = Utils::convertHashValuesToArray($tree -> {StorageList}{Storage});
	}

	# find diskID of all the mount points
	foreach my $storage (@arStorage) {
		if ( (grep { $_ eq $storage->{MountPoint}{Value} } @arMountPointList)) {
			if ((defined $storage->{DiskIDList}) &&
				($storage->{DiskIDList} ne "") &&
				(defined $storage->{DiskIDList}{DiskID})) {
				my @tmpDiskList = Utils::convertHashValuesToArray($storage->{DiskIDList}{DiskID});
				foreach (@tmpDiskList) {
					if (!defined $uniqDisks{$_->{Value}}) {
        					$retXML .= "<DiskID>$_->{Value}</DiskID>";
						$uniqDisks{$_->{Value}} = 1;
					}
				}
			}	
		}	
	}
	
        #$retXML .= "<DiskID>0:1</DiskID><DiskID>0:6</DiskID>";

        $retXML = $retXML . "</StorageInfo>";
        Utils::vxlog("INFO", "Output XML: $retXML");
        return ( $ret , "" , $retXML);
}


sub createUpdateResDetails
{
	my $self = shift;
	my $tree = shift;
	my $sgName = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	my %systemList = ();
	if($tree->{SystemList} && $tree->{SystemList}{System})
	{
		my @sysList = Utils::convertHashValuesToArray($tree->{SystemList}{System});
		foreach my $sys ( @sysList ) {
			$systemList{$sys->{Name}} = $sys->{SGConfigured};
		}
	}

	my $deviceXml = ();
	my $proxyXml = ();

	if($tree->{VirtualIPInfo} && $tree->{VirtualIPInfo}{VirtualIP}) {
		my @lVirtualIPs = Utils::convertHashValuesToArray($tree -> { VirtualIPInfo }{ VirtualIP });

		foreach my $vip ( @lVirtualIPs ) {
			my $sVIPAddress = $vip -> { Address };
			$sVIPAddress =~ s/\.|:/_/g;

			$deviceXml -> { $sVIPAddress } = "";
			$proxyXml -> { $sVIPAddress } = "";

			if ($vip->{NIC}) {
				$deviceXml -> { $sVIPAddress } .= "<Device>";
				$proxyXml -> { $sVIPAddress } .= "<TargetResName>";

				my @nicList = Utils::convertHashValuesToArray( $vip -> { NIC } );

				foreach my $nic ( @nicList ) {
					my $sysName = $nic->{System};
					if (exists($systemList{$sysName}) && $systemList{$sysName} eq "false") {
						$deviceXml -> { $sVIPAddress } .= "<$sysName>$nic->{Device}</$sysName>";
						my $sDeviceName = $nic->{Device};
						my $clusDetailsInputRef = ();
						my $vcsobj = new VCS();
						$clusDetailsInputRef->{Resources}{Types} = ["NIC"];
						$clusDetailsInputRef->{Resources}{Attributes} = [ "Device" ];
						$clusDetailsInputRef->{Resources}{AttributesVal}{Device}{$sysName} = "$sDeviceName";
						PlatNetwork::setPlatFilterAttrs($clusDetailsInputRef);
						my ($retVal, $MatchedRes) = $vcsobj->isSimilarResourceExist($clusDetailsInputRef);
						my $sNICResName = "NIC_" . $sysName . "_" . $nic->{Device};
						if ($retVal) {
							Utils::vxlog("DEBUG", "resource Name:$MatchedRes");
							$proxyXml -> {$sVIPAddress} .= "<$sysName>$MatchedRes</$sysName>";
						} else {
							$sNICResName =  $self->GetUniqueResName($sNICResName);
							Utils::vxlog("DEBUG", "resource Name:$sNICResName");
							$proxyXml -> { $sVIPAddress } .= "<$sysName>$sNICResName</$sysName>";

						}
					}
				}

				$deviceXml -> { $sVIPAddress } .= "</Device>";
				$proxyXml -> { $sVIPAddress } .= "</TargetResName>";
			}
		}
	}
	else {
		return ($ret, $msg, $xml);
	}

	my $vcsobj = VCS -> new();
	my @resList = ();
	my ($cmdRet, $cmdMsg, $clusRef);

	my $clusDetailsInputRef = ();

	$clusDetailsInputRef -> { Resources }{ Types      } = [ "IP", "Proxy" ];
	$clusDetailsInputRef -> { Resources }{ Attributes } = [ "Address", "Type" ];
	$clusDetailsInputRef -> { Resources }{ Groups     } = [ "$sgName" ];
	
	($cmdRet, $cmdMsg, $clusRef) = $vcsobj -> getClusterDetails($clusDetailsInputRef);	
	if ( $cmdRet == CONST_FAILURE ) {
		Utils::vxlog("ERROR", "Error occurred while fetching resource details- [$cmdMsg]");
	}
	else {
		my $resRef = $clusRef -> { ResRef };

		foreach my $res ( keys %$resRef ) {
			if ($resRef -> { $res }{ Type }{ global } =~ /^IP$/ ) {
				my $address = $resRef -> { $res }{ Address }{ global };
				$address =~ s/\.|:/_/g;

				if ( defined $deviceXml -> { $address } ) {  
					Utils::vxlog("INFO", "Editing attributes for the resource : $res");
					$xml .= addEditModeResourceAttr ($res, "IP", $deviceXml -> { $address } );
				}
			}
			elsif ($resRef -> { $res }{ Type }{ global } =~ /^Proxy$/ ) {
				foreach my $address ( keys %$proxyXml) {
					Utils::vxlog("INFO", " Address : $address Res : $res");
					if ( $res =~ /$address/ ) {
						Utils::vxlog("INFO", "Editing attributes for the resource : $res");
						$xml .= addEditModeResourceAttr ($res, "Proxy", $proxyXml -> { $address } );
						last;
					}
				}
			}
		}
	}
	
	return ($ret, $msg, $xml);
}


sub addEditModeResourceAttr
{
	my $resName = shift;
	my $resType = shift;
	my $attrValues = shift;

	my $sMySubName = ( caller 0 )[3];
	my $announceStr = "Subroutine $sMySubName called";
	Utils::vxlog("INFO", $announceStr);

	my $xml = "";
	$xml .= "<Resource>";
	$xml .= "<Name>$resName</Name>";
	$xml .= "<Type>$resType</Type>";
	$xml .= "<Mode>edit</Mode>";
	$xml .= "<Attributes>$attrValues</Attributes>";
	$xml .= "</Resource>";

	return $xml;
}

sub updateNetworkInstanceInfo
{
	my $sMyName = ( caller 0 )[3];
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	return $ret;
}
sub discoverExistingConfig
{
        my $self = shift;
        my $params = shift;

        my $sMyName = ( caller 0 )[3];
        my $logStr = "Subroutine $sMyName called";
        Utils::vxlog("INFO", $logStr);
        Utils::vxlog("INFO", " Input params: [$params]");

        my $ret = CONST_SUCCESS;
        my $msg = "";
        my $xml = "";
        my @paramsArr;

        if(defined $params)
        {
            @paramsArr = split ",",$params;
        }
	($ret, $msg, my $retXML) = $self->discoverExistingNetworkConfig(\@paramsArr);
    goto END if($ret ne CONST_SUCCESS);
	$xml .= $retXML;

	#$xml .= "<StorageInfo><DiskID>00:10.0:3</DiskID><DiskID>00:10.0:4</DiskID></StorageInfo>";
	# Output of discoverExistingStorageConfig  will be used by GUI to decide
	# whether to show the ESX page during add node wizard
	# Therefore the output only needs to define the
	# "<StorageInfo><DiskID></DiskID></StorageInfo>" and does not need the
	# actual resource details.
	$xml .= $self->discoverExistingStorageConfig(\@paramsArr);
END:
        return ($ret, $msg, $xml);
}

sub discoverInstanceIPMapping
{
    my $self = shift;
    my $clusRef = shift;
    my $sMySubName = ( caller 0 )[3];
    my $announceStr = "Subroutine $sMySubName called";
    Utils::vxlog("INFO", $announceStr);

    my $resRef = $clusRef -> { ResRef };
    my $xmlstr = "";
    my @resList = keys %$resRef;
    my @arIPResName = map { $resRef->{$_}{Type}{global} =~ /^IP$/; $_ } keys %$resRef; 
    my @arIPResDep = Utils::convertHashValuesToArray($clusRef -> { ResDepRef });
    foreach my $resIP (@arIPResName) {
		# Check if child is IP
		my @arResDep = grep { $_->{ Child } eq $resIP } @arIPResDep; 
		foreach my $resDep (@arResDep) {
		my $instList = "";
			if (grep ($resDep->{Parent}, @resList)) { 
				my $instName = $self->getIPParentInstance($resRef, $resDep ->{ Parent });
				if ($instName ne "") {
				    $instList .= "<Instance>" . $instName . "</Instance>";
				}
			}
		    if (defined $arResDep[0] and defined $resRef-> { $resIP }{ Address }{ global } and $instList ne "") {
			$xmlstr .= "<InstanceIPMapping>";
			$xmlstr .= $instList;
			$xmlstr .= "<Address>" . $resRef-> { $resIP }{ Address }{ global } . "</Address>";
			$xmlstr .= "</InstanceIPMapping>";
		}

		}
    }
    return $xmlstr;
}

sub discoverExistingNetworkConfig
{
    my $self = shift;
    my $sgNamesArr  = shift;
    my $sMySubName = ( caller 0 )[3];
    my $announceStr = "Subroutine $sMySubName called";
    Utils::vxlog("INFO", $announceStr);
    my $ret         = CONST_SUCCESS;
    my $msg = "";
    my $xmlstr      = "";
    my $clusDiscoveryInputRef   = ();
	my $retXML = "";
    $clusDiscoveryInputRef->{Resources}{Types}          = ();
    $clusDiscoveryInputRef->{Resources}{Groups}         = $sgNamesArr;
    $clusDiscoveryInputRef->{ServiceGroup}{Name}		= $sgNamesArr;
    $clusDiscoveryInputRef->{ServiceGroup}{Attributes}  = [ "Parallel", "SystemList" ];
    $clusDiscoveryInputRef->{ResourcesDep} 				= "";
  	my $vcsobj = VCS -> new();
    ($ret, $msg, my $clusRef ) = $vcsobj -> getClusterDetails($clusDiscoveryInputRef);
    goto END if($ret ne CONST_SUCCESS);

    my $resRef = $clusRef -> { ResRef };
    my $groupRef = $clusRef -> { GroupRef };
    my ($device, $address, $netmask, $type, $system) = ();

    # Create group information 
    $xmlstr = "<ServiceGroups>";

    foreach my $grpName ( sort keys %$groupRef )
    {
        Utils::vxlog("INFO", " ServiceGroup Name: $grpName");
        $xmlstr .= "<ServiceGroup>";
        $xmlstr .= "<IsExistingConfig>1</IsExistingConfig>";
        $xmlstr .= "<Name>" . $grpName . "</Name>";
        $xmlstr .= "<Parallel>" . $groupRef->{$grpName}{Parallel}{global} . "</Parallel>";
		my $systemList = $groupRef->{$grpName}{SystemList}{global};
		my %sysPriority = split (/\s+/, $systemList);
        $xmlstr .= "<SystemList>";
		foreach my $sysName (keys %sysPriority) {
            $xmlstr .= "<System>";
            $xmlstr .= "<SysName>" . $sysName . "</SysName>";
            $xmlstr .= "<Priority>" . $sysPriority{ $sysName } . "</Priority>";
            $xmlstr .= "</System>";
		}
        $xmlstr .= "</SystemList>";
        $xmlstr .= "</ServiceGroup>";
    }
    $xmlstr .= "</ServiceGroups>";

    # Create group information

    $xmlstr .= "<ApplicationInfo>";
    my $instList = ();
    $instList = $self->getDiscoveredInstances($resRef) if defined $resRef;
    foreach my $instName ( @$instList )
    {
        Utils::vxlog("INFO", " Instance Name: $instName");
        $xmlstr .= "<Instance>";
        $xmlstr .= "<InstanceName>" . $instName . "</InstanceName>";
        $xmlstr .= "</Instance>";
    }
    ($ret, $msg, my $validtionInfoXML) = $self->getValidationInput($resRef, $clusRef) if defined $resRef;
    goto END if($ret ne CONST_SUCCESS);
	$xmlstr .= $validtionInfoXML;
    my $sInstanceIPMapping .= $self->discoverInstanceIPMapping($clusRef); 
	if ($sInstanceIPMapping ne "")
	{
		$xmlstr .= "<isIPMappingPresent>true</isIPMappingPresent>";
	} else {
		$xmlstr .= "<isIPMappingPresent>false</isIPMappingPresent>";
	}
	$xmlstr .= "</ApplicationInfo>";

    # Create virtual IP  tag
    $xmlstr .= "<VirtualIPInfo>";
    if ( defined ($resRef)) {
	    foreach my $resName ( sort keys %$resRef )
	    {
		    if ($resRef->{$resName}{Type}{global} !~ /^IP$/) {
			    next;
		    }
		    $xmlstr .= "<VirtualIP>";
		    $xmlstr .= "<ExistingResource>1</ExistingResource>";
		    my $ipType = "";
		    if (defined $resRef->{$resName}{Address}{global}) {
			    $address = $resRef->{$resName}{Address}{global};
			    $xmlstr .= "<Address>" . $resRef->{$resName}{Address}{global} . "</Address>";
			    $ipType =  VCSNetGetIPVersion($resRef->{$resName}{Address}{global});
			    if ($ipType == -1) {
				$ret = CONST_FAILURE; 
				$msg = "Could not find the IP version."; 
    				Utils::vxlog("ERROR", "Could not find the IP version for address [" . $resRef->{$resName}{Address}{global} . "].");
				goto END;
			    }
			    $xmlstr .= "<Type>ipv" . $ipType . "</Type>";
			
		    }
		    if("$ipType" eq "4") {
			    if (defined $resRef->{$resName}{NetMask}{global}) {
				    $netmask = $resRef->{$resName}{NetMask}{global};
				    $xmlstr .= "<NetMask>" . $resRef->{$resName}{NetMask}{global} . "</NetMask>";
			    }
		    }
		    elsif("$ipType" eq "6") {
			    if (defined $resRef->{$resName}{PrefixLen}{global} &&
				defined $resRef->{$resName}{Address}{global}) { 
				my $prefix = PlatUtils::getIPV6Prefix(
					$resRef->{$resName}{Address}{global},
					$resRef->{$resName}{PrefixLen}{global});
				    $xmlstr .= "<Prefix>$prefix</Prefix>";
			    }
		    }

		    if (ref ($resRef->{$resName}{Device}) =~ /HASH/) {
			    foreach my $sysname (keys %{$resRef->{$resName}{Device}}) {
				    $xmlstr .= "<NIC>";
				    $xmlstr .= "<System>" . $sysname . "</System>";
				    $xmlstr .= "<Device>" . $resRef->{$resName}{Device}{$sysname} . "</Device>";
				    $xmlstr .= "</NIC>";
			    }
		    } 
		    $xmlstr .= "</VirtualIP>";
	    }
    }
	$xmlstr .= $sInstanceIPMapping;
    $xmlstr .= "</VirtualIPInfo>";
	$retXML = $xmlstr;
    Utils::vxlog("INFO", "Existing config discovered xml: [" . Dumper($xmlstr). "]");
END:
    return ($ret, $msg, $xmlstr);
}

sub discoverExistingStorageConfig
{
    my $self = shift;
    my $sgNamesArr  = shift;
    my $sMySubName = ( caller 0 )[3];
    my $announceStr = "Subroutine $sMySubName called";
    Utils::vxlog("INFO", $announceStr);
    my $ret         = CONST_SUCCESS;
    my $msg = "";
    my $xmlstr      = "";
    my $clusDiscoveryInputRef   = ();
    $clusDiscoveryInputRef->{Resources}{Types}          = [ "VMwareDisks" ] ;
    $clusDiscoveryInputRef->{Resources}{Groups}         = $sgNamesArr;
    #$clusDiscoveryInputRef->{ServiceGroup}{Name}		= $sgNamesArr;
    #$clusDiscoveryInputRef->{ServiceGroup}{Attributes}  = [ "Parallel", "SystemList" ];
    my $vcsobj = VCS -> new();
    ($ret, $msg, my $clusRef ) = $vcsobj -> getClusterDetails($clusDiscoveryInputRef);
    goto END if($ret ne CONST_SUCCESS);
    my $resRef = $clusRef -> { ResRef };
#    my $groupRef = $clusRef -> { GroupRef };
    my ($device, $address, $netmask, $type, $system) = ();

    if ( defined ($resRef)) {
	    foreach my $resName ( sort keys %$resRef )
	    {
		    if ($resRef->{$resName}{Type}{global} =~ /^VMwareDisks$/) {
			$xmlstr = "<StorageInfo><DiskID></DiskID></StorageInfo>";
			last;
		    }
    	    }
    }
END:
    return $xmlstr;
}

# Following function will be used to get the cluster details so that we can handle the case in which VCS
# is already configured forthe type of resource.
sub discoverExistingAppConfig
{
    my $self = shift;
    my $sMySubName = ( caller 0 )[3];
    my $announceStr = "Subroutine $sMySubName called";
    Utils::vxlog("INFO", $announceStr);
    my $ret  = CONST_SUCCESS;
    my $msg  = "";
    my $xmlstr      = "";
    my $clusDiscoveryInputRef   = ();

	# We need information about all the service groups which contain any of these resources
	# therfore we do not initialize $clusDiscoveryInputRef->{ServiceGroup} 
    $clusDiscoveryInputRef->{Resources}{Attributes} 	= [];
    $clusDiscoveryInputRef->{Resources}{Types}		= [];
    my $vcsobj = VCS -> new();
    ($ret, $msg, my $clusRef ) = $vcsobj -> getClusterDetails($clusDiscoveryInputRef);
    goto END if($ret ne CONST_SUCCESS);

    my $resRef = $clusRef -> { ResRef };
    my $groupRef = $clusRef -> { GroupRef };
END:
	return ($resRef);
}

# Following function will be called by each application to check if any 
# component of the discovered storage are already configured.
sub isStorageConfigured
{
    my $self = shift;
    my $sMyName = ( caller 0 )[3];
    my $resRef = shift;
    my $storageDetails = shift;
    my $ret = CONST_FAILURE;

	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

    if ((ref $resRef) !~ /HASH/) {
		return $ret;
    }

	my $res_type = (); # Volume group resource type is "LVMVolumeGroup" for linux or LVMVG for aix
	$res_type = "LVMVolumeGroup" if ( $^O eq "linux" );
	$res_type = "LVMVG" if ( $^O eq "aix" );

	# For Solaris, check for Zpool resource
	$res_type = "Zpool" if ( $^O eq "solaris" );

	# Why check volumes if the checking of DGs or VGs can give you same result.
	# Therefore we do not check for volumes at all !
    my @storageTypes = ( "Mount", $res_type, "DiskGroup" );
    foreach my $resName (keys %$resRef) { 
		if (! grep (/^$resRef->{$resName}{Type}{global}$/,@storageTypes)) {
	            next;
	        }
		foreach my $mountDetail (@$storageDetails) {
		    # if resource type is "Mount" 
		    if ($resRef->{$resName}{Type}{global} =~ /^Mount$/) {
				if ((defined $resRef->{$resName}{MountPoint}) and 
				    ($resRef->{$resName}{MountPoint}{global} =~ /^$mountDetail->{MountPoint}$/)) {
					$logStr = "Resource for MountPoint " . $mountDetail->{MountPoint} . " is already available.";
					Utils::vxlog("INFO", $logStr);
				    $ret = CONST_SUCCESS;
				    return $ret; 
				}
		    }
		    # If the two resources share the LVMVolume, they will also have to share the volume group.
		    # Volume group resource type is "LVMVolumeGroup" for linux or LVMVG for aix
		    # For Solaris, resource type will be Zpool
		    if ($resRef->{$resName}{Type}{global} =~ /^$res_type$/) {
			if ( $^O eq "solaris" ) {
				if ((defined $resRef->{$resName}{PoolName}) and
				($resRef->{$resName}{PoolName}{global} =~ /^$mountDetail->{PoolName}$/)) {
					$logStr = "Resource for Zpool " . $mountDetail->{PoolName} . " is already created.";
					Utils::vxlog("INFO", $logStr);
				    $ret = CONST_SUCCESS;
				    return $ret;
				}
			}
			else {
				if ((defined $resRef->{$resName}{VolumeGroup}) and 
			    	($resRef->{$resName}{VolumeGroup}{global} =~ /^$mountDetail->{VolumeGroup}$/)) {
					$logStr = "Resource for VolumeGroup " . $mountDetail->{VolumeGroup} . " is already created.";
					Utils::vxlog("INFO", $logStr);
				    $ret = CONST_SUCCESS;
				    return $ret;
				}
			}
		    }
		    # if resource type is "DiskGroup"
		    if ($resRef->{$resName}{Type}{global} =~ /^DiskGroup$/) {
				if ((defined $resRef->{$resName}{DiskGroup}) and 
			    	($resRef->{$resName}{DiskGroup}{global} =~ /^$mountDetail->{DiskGroup}$/)) {
					$logStr = "Resource for DiskGroup " . $mountDetail->{DiskGroup} . " is already created.";
					Utils::vxlog("INFO", $logStr);
					$ret = CONST_SUCCESS;
			  		return $ret; 
				}
		    }
    	}
    }
    return $ret;
}

#---------------------------------------------
# Overriding the validate method
#---------------------------------------------
# Input for validate is same as commit 
sub validate
{	
	my $retCode = CONST_SUCCESS;
	my $retMsg = "Successful completion of operation.";
	my $retXML = "";
	my $applicationInfoHash ;
	
	my $self = shift;
	my $xmlParams = shift;
	my $targetParams = shift;

	my $sMyName = ( caller 0 )[3];
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);
	LOOP: 
	{
		if (!defined $xmlParams) 
		{
			$retCode = CONST_FAILURE;
			$retMsg = "Input xml parameters are not provided.";
		   Utils::vxlog("INFO", "Input xml parameters are not provided.");
		   last;
		}
		my $node = { Name => $targetParams -> { TargetHost }, UserName => $targetParams -> { TargetUser }, Password => $targetParams -> { TargetPassword }};
		my $className = blessed ($self);
		($retCode, $retMsg) = $self -> performShallowDiscovery($node, $className, $xmlParams);
	}
	return ($retCode, $retMsg, $retXML);
}


sub performShallowDiscovery
{
	my $self = shift;
	my $sMyName = ( caller 0 )[3];
	my $node = shift;
	my $appName = shift;
	my $params = shift;
	$params =~ s/\n//g;
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);
	my $nodeName = $node -> { Name };
	my $userName = $node -> { UserName };
	my $passwd = $node -> { Password };

	$passwd =~ s/'/'"'"'/g;

	my $xprtlc = "/opt/VRTSsfmh/bin/xprtlc";
	my $targetUrl = "https://$nodeName:5634/vcs/admin/configAppMon.pl";
	my $targetArgs = "-d ID=$appName -d OP=shallowDiscover -d params=\"'$params'\"";
	my $args = "-d TargetURL=$targetUrl -d TargetUser=\'$userName\' -d TargetPassword=\'$passwd\' " . $targetArgs;

	my $proxyCmd = $xprtlc . " -l https://localhost:5634/vcs/admin/proxy.pl $args";

	Utils::vxlog("INFO", "Going to run shallow discovery on node [$nodeName] ");
	my ($fret, $msg, $ret) = Utils::runCmd($proxyCmd);
	$logStr = "Output of the command: $ret, ==$msg==";
	my $tree = ();
	if ($fret == CONST_SUCCESS) {
		$tree = Utils::convertXMLTextToHash($msg);
		if (defined $tree-> { ReturnCode } and $tree-> { ReturnCode } != 0) {
			$fret = $tree-> { ReturnCode };
			$msg = $tree-> { Message };
		} else {
			$msg = "";
		}
	} else {
		$msg = "Could not add the node as a failover target node for this application because some requirements were not met. You can find the details in the log file.";
	}
	return ($fret, $msg);
}

# Get the list of mount points which are required by the current instance.
sub getDependentMountPointList
{
    my $self = shift;
    my $sMySubName = ( caller 0 )[3];
    my $announceStr = "Subroutine $sMySubName called";
    Utils::vxlog("INFO", $announceStr);
    my $resName = shift;
    my $resRef = shift;
    my $clusRef = shift;
	my @mntList = ();
    my $ret = CONST_FAILURE;
    my @arResDep = Utils::convertHashValuesToArray($clusRef -> { ResDepRef });
	my @arOraResDep = grep { $_->{ Parent } eq $resName } @arResDep; 
	my @arOraMntResDep = grep { $resRef->{ $_->{ Child } }{Type}{global} =~ /^Mount$/} @arOraResDep; 
	foreach my $resDep (@arOraMntResDep) {
		push (@mntList, $resRef->{ $resDep->{ Child } }{MountPoint}{global});
		$ret = CONST_SUCCESS;
	}
	return ($ret, \@mntList);
}

# Set dependency between Parent Instance and Child Instance
sub setInstanceDependency
{

	my $sMySubName = ( caller 0 )[3];
	my $announceStr = "Subroutine $sMySubName called";
	Utils::vxlog("INFO", $announceStr);

	my ($self, $parent, $child) = @_;

	push ( @{$hrInsParentChildren->{ $parent }}, $child );
	push ( @{$hrInsChildParents->{ $child }}, $parent );

}

# Add current instance to global Instance list
sub addInstance
{
	my $sMySubName = ( caller 0 )[3];
	my $announceStr = "Subroutine $sMySubName called";
	Utils::vxlog("INFO", $announceStr);

	my ($self, $name) = @_;

	push ( @gInstances, $name);
}

# Set Mount dependency with Instance
sub setMountDependency
{

	my $sMySubName = ( caller 0 )[3];
	my $announceStr = "Subroutine $sMySubName called";
	Utils::vxlog("INFO", $announceStr);

	my ($self, $instance, $mountpoint) = @_;

	push ( @{$hrInstanceMounts->{ $instance }} , $mountpoint );
	push ( @{$hrMountInstances->{ $mountpoint }} , $instance );
}

sub GetUniqueResName
{
	my $self = shift;
        my $resNameSeed = shift;
        if ( $resNameSeed !~ /^[A-Za-z].*/ ) {
                $resNameSeed = "res_" . $resNameSeed;
        }
        $resNameSeed =~ s/\W+/_/g; 
	my %hNamesRes = reverse %{$hrResNames} if (defined $hrResNames);
        return $resNameSeed unless defined ( $hrUniqueResList->{$resNameSeed} or $hNamesRes{$resNameSeed} );
        my $resName = ();

	$resNameSeed =~ s/(\S+)_(\d+)$/$1/;
        for(my $index = 1; $index < 10000; $index++)
        {
                $resName = $resNameSeed . "_" . $index;
                return $resName unless defined ( $hrUniqueResList->{$resName} or $hNamesRes{$resName} );
        }
        return "";
}

sub GetUniqueGroupName
{
	my $self = shift;
        my $grpNameSeed = shift;
        return $grpNameSeed unless defined ( $hrUniqueGrpList->{$grpNameSeed} or $hrSGs->{$grpNameSeed} ); 
        my $grpName = ();
	$grpNameSeed =~ s/(\S+)_(\d+)$/$1/;
        for(my $index = 1; $index < 10000; $index++)
        {
                $grpName = $grpNameSeed . "_" . $index;
                return $grpName unless defined ( $hrUniqueGrpList->{$grpName} or $hrSGs->{$grpName});
        }

        return "";
}

# Get resource xml information from $hrResXMLs
sub getResourceXML
{
	my $sMySubName = ( caller 0 )[3];
	my $announceStr = "Subroutine $sMySubName called";
	Utils::vxlog("INFO", $announceStr);
	my ($self, $sResXMLKey) = @_;

	my $sResXMLValue = $hrResXMLs -> { $sResXMLKey };
	return $sResXMLValue;
}
# Set resource xml information in $hrResXMLs
sub setResourceXML
{
	my $sMySubName = ( caller 0 )[3];
	my $announceStr = "Subroutine $sMySubName called";
	Utils::vxlog("INFO", $announceStr);
	my ($self, $sResXMLKey, $sResXMLValue) = @_;
 
	$hrResXMLs -> { $sResXMLKey } = $sResXMLValue;
	Utils::vxlog("DEBUG", "key:[$sResXMLKey] value:[$sResXMLValue]");
}

# Set instance xml and resource name information in $hrResXMLs and $hrResNames
sub setInstanceInfo
{
	my $sMySubName = ( caller 0 )[3];
	my $announceStr = "Subroutine $sMySubName called";
	Utils::vxlog("INFO", $announceStr);
	my ($self, $hrInsResXMLs, $hrInsResNames) = @_;
 
	@{$hrResXMLs}{keys %$hrInsResXMLs} = values %$hrInsResXMLs;
	@{$hrResNames}{keys %$hrInsResNames} = values %$hrInsResNames;
}



# This function is called by discoverExistingNetowrkConfig during add node wizard.
# it returns the application specific information to the wizard.
# The format of data is similar to the discovery wizard
# We also simulate selection of instances so that there is no changes required in 
# the shallowDiscovery subroutine.

sub getValidationInput
{
    my $self = shift;
    my $sMySubName = ( caller 0 )[3];
    my $announceStr = "Subroutine $sMySubName called";
    Utils::vxlog("INFO", $announceStr);
    my $resRef = shift;
    my $clusRef = shift;
    my $ret = CONST_SUCCESS;
	my $outXML = "";
	$outXML = "<ValidationInfo>";
    $outXML .= "</ValidationInfo>";
    return ($ret, "", $outXML);
}

sub matchLVMVolume_AIX
{
	my $self = shift;
	my $blockDevice = shift;
	my $sMySubName = ( caller 0 )[3];
	my $announceStr = "Subroutine $sMySubName called";
	Utils::vxlog("INFO", $announceStr);
	my $ret = CONST_FAILURE;
	my %hLVMVGs = ();
	my ($tmp, $vg_major, $vg, $bd_major, $vol) = ();

	# Major number of block device
	$bd_major = (split (/\s+/, `ls -l $blockDevice`))[4];
	$bd_major =~ s/,$//;

	# Volume name: /dev/vol
	$vol = (split (/\//, $blockDevice))[2];

	# Make a hash of all volume groups and their major numbers
	# vg_major => vg_name
	open (LSVG, "lsvg |");
	while (<LSVG>) {
		chomp;
		$vg = $_;
		$vg_major = (split (/\s+/, `ls -l /dev/$vg`))[4];
		$vg_major =~ s/,$//;

		$hLVMVGs { $vg_major } = $vg;
	}

	if (defined ($hLVMVGs {$bd_major})) {
		$vg = $hLVMVGs {$bd_major};
		$ret = 0;
	}

	return ($ret, $vg, $vol, $bd_major);
}

sub matchLVMVolume_Linux
{
	my $self = shift;
	my $blockDevice = shift;
	my $sMySubName = ( caller 0 )[3];
	my $announceStr = "Subroutine $sMySubName called";
	Utils::vxlog("INFO", $announceStr);
	my $ret = CONST_FAILURE;
	my $Cmd = "lvs";
	my $vgName = "";
	my $volName = "";
	my $retcode = "";
	($ret, my $op, $retcode) = Utils::runCmd($Cmd);
	goto END if ($ret != CONST_SUCCESS && $retcode != LVS_VG_EXPORTED ) ;

	$ret = CONST_FAILURE;
	# Linux appends '-' as escape character for every '-' in LVM volume name.
	# however the lvs output does not contain extra dashes due to which the 
	# volume names obtained from lvs and mount command does not match.
	# we remove the extra '-' to match the blockdevice name with LVM information
	$blockDevice =~ s/--/-/g;
	my @listVol = split('\n', $op);
	splice (@listVol, 0, 1);
	for my $volLine (@listVol) {
		$volLine =~ s/^\s*//;
		my @volEntries = split ('\s+', $volLine);
		$vgName  = $volEntries[1];
		$volName = $volEntries[0];
		if ($blockDevice =~ m/\/dev\/mapper\/$vgName-$volName$/ ) {
			$ret = CONST_SUCCESS;
			last;
		}
	}
END: 
	return ($ret, $vgName, $volName);
}

sub removeDuplicates
{
	my $sMySubName = ( caller 0 )[3];
	my $announceStr = "Subroutine $sMySubName called";
	Utils::vxlog("INFO", $announceStr);
	my $arResList = shift;
	my %unique = ();
	foreach ( @{$arResList} ) 
	{
		$unique{$_} = 1;
	}
	return(keys %unique); 
}

sub matchZpool_Sol
{
	my $self = shift;
	my ($ZpoolBlkDev) = shift;
	my ($zpool_cmd, $output, $ret) = ();
	my ($ZpoolName, $isLegacyMntPt) = ();
	my $sMySubName = ( caller 0 )[3];
	my $announceStr = "Subroutine $sMySubName called";
	Utils::vxlog("INFO", $announceStr);

	$isLegacyMntPt = 0;
	$ret = CONST_SUCCESS;

	# Zpool block device doesn't start with '/'
	if ($ZpoolBlkDev =~ /^\//) {
		return CONST_FAILURE;
	}

	# Run "zfs get -H -o value mountpoint $ZpoolBlkDev" command to check if it is
	# a ZFS file system or not. If command succedes, then it is ZFS file system.
	# The "value" field will print "legacy" if it is a legacy mount point.
	# "-H" option is to hide the header from getting printed.

	$zpool_cmd = "zfs get -H -o value mountpoint $ZpoolBlkDev";
	$output = `$zpool_cmd 2>&1`;
	$ret = $? >> 8;

	if ($ret != CONST_SUCCESS) {
		return CONST_FAILURE;
	}

	# Example: mypool/abc/xyz
	$ZpoolName = (split (/\//, $ZpoolBlkDev))[0];

	chomp($output);
	if ($output eq "legacy") {
		$isLegacyMntPt = 1;
	}

	return ($ret, $ZpoolName, $isLegacyMntPt);
}

# This function needs to be over-ridden by each application to provide appropriate description of SG.
sub getSGDescription
{
	my $self = shift;
	my $sgName = shift;
	my $lResources = shift;

	Utils::vxlog("DEBUG", "SG [$sgName]");

	foreach my $res ( @$lResources ) {
		Utils::vxlog("DEBUG", "Resource in $sgName: [$res]");
	}

	return "Contains resources related to the following service group: $sgName";
}

1;

