#
# $Id $
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#

package VMwareDisk;

#use base qw(Application);
@ISA = ("UnixApplication");

use strict;
use warnings;
use Scalar::Util 'blessed';
use Data::Dumper;
use UnixApplication;
use Constants;
use Utils;
use PlatUtils;
use Storage;
use UnixNetwork;
use vcs_net_utils_inc;


sub createDiskIDXML
{

	my $diskDetails = shift;
	##########################
	# Create XML
	##########################
	my $xmlDisks = "";
	my $xmlOut = "";
	my $ret = 0;
	my $msg = "";
	
	# Create XML for disk info
	$xmlOut = $xmlOut . "<DiskIDList>" ;

	if ($ret == 0) {
		foreach my $_disk ( @$diskDetails ) {
			if ($_disk =~ /(\d+):(\d+)$/) {
				$xmlOut = $xmlOut . "<DiskID>" ;
				#$xmlOut = $xmlOut . $1 . ":" . $2;
				$xmlOut = $xmlOut . $_disk;
				$xmlOut = $xmlOut . "</DiskID>" ;
			}
		}
	}

	$xmlOut = $xmlOut . "</DiskIDList>" ;
	
	return ($xmlOut);
}

sub discover
{
	my $self = shift;
	my $params = shift;

	my $sMyName = ( caller 0 )[3];
#-------------------
# Announce ourselves
#-------------------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

# Initialize return parameters
	my ( $ret, $msg, $xml, $insxml, $insResName) = (CONST_SUCCESS, "", "", "", "");
	my ( $hrInsResXMLs, $hrInsResNames ) = ();

	my $SGName = "GenericApp_SG";
	my $str = "";

	my $tree = Utils::convertXMLTextToHash( $params );

	Utils::vxlog("INFO", Dumper( $tree ));

	#($ret, $msg) = $self->validateXML($tree);

	if ( $ret == CONST_FAILURE ) {
		Utils::vxlog("ERROR", "Commit input xml validation failed");
		goto END;
		
	}

	my $arDiskIDs = ();
	my $iCnt =1;
	my (@StorageData) = ();
	my $Storage_ret = CONST_SUCCESS;
	my ($hrStorageDetails) = ();	
	my ($sDiskGroup, $sVolumeGroup) = ();
	my $mountpoint = $tree->{MountPoint};
	$msg .= "MountPoint is: $mountpoint, params is: $params\n";
	($Storage_ret, $hrStorageDetails) = $self->GetStorageDetails ($mountpoint);
	$msg .= "Return of GetStorageDetails: $Storage_ret\n";
	if ( CONST_SUCCESS == $Storage_ret ) {
			$hrStorageDetails -> { Instance } = "MountDetails" . "_" . "0";

			if ( defined $hrStorageDetails -> { DiskGroup } ) {
				$sDiskGroup = $hrStorageDetails -> { DiskGroup };
				($Storage_ret, $msg, $arDiskIDs) = getVxVMDgDisks($sDiskGroup);
				if ($Storage_ret != CONST_SUCCESS) {
					$ret = $Storage_ret;
					goto END;
				}
			}
			if ( defined $hrStorageDetails -> { VolumeGroup } ) {
				$sVolumeGroup = $hrStorageDetails -> { VolumeGroup };
				($Storage_ret, $msg, $arDiskIDs) = getLVMVgDisks($sVolumeGroup);
				if ($Storage_ret != CONST_SUCCESS) {
					$ret = $Storage_ret;
					goto END;
				}
			}

			$hrStorageDetails -> { DiskID }  = $arDiskIDs;

			($xml) = createDiskIDXML($arDiskIDs);
		} else {
			Utils::vxlog("INFO", "Could not discover storage for [$mountpoint]");
		}

	Utils::vxlog("INFO", $xml);
END:
	my $retRef = ();

	return ($ret, $msg, $xml);
}


1;
