#!/opt/VRTSsfmh/bin/perl
#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: performOperation.pl,v 1.4 2012/07/10 13:46:12 stripath Exp $
#

#------------------------
# Common Header
#------------------------
use strict;
use warnings;

use Data::Dumper;
use CGI;
use Config;
use POSIX;

# Push include paths!
my $LibPath = "";

BEGIN
{
	if ( $^O =~ m/win/i ) {
		require VRTS::Paths;

		my $xprtlInstallDir= VRTS::Paths::get_path("InstallDir");
		my $regquery = "\"$xprtlInstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\VERITAS\\VCS\\Base HomeDirectory";
		my $vcshome = `$regquery`;
		$LibPath = "$vcshome\\portal\\admin\\lib\\";
		$ENV{'VCS_HOME'} = $vcshome;

	} else {
		my $APPDISCOVER_HOME = "/opt/VRTSvcs/portal/admin";
	
		$LibPath = "$APPDISCOVER_HOME/lib/";
	}
}

use lib $LibPath;
#------------------------
# End of Common Header
#------------------------

use Constants;
use Utils;
use VCS;
use MessageUtils;
use constant MSG_FILE => "appmon.msg";


my $hOP = ();
$hOP -> { onlineAppSG }		= "onlineAppSG";
$hOP -> { offlineAppSG }	= "offlineAppSG";
$hOP -> { switchAppSG }		= "switchAppSG";
$hOP -> { clearAppSG }		= "clearAppSG";
$hOP -> { freezeAppSG }		= "freezeAppSG";
$hOP -> { unFreezeAppSG }	= "unFreezeAppSG";
$hOP -> { flushAppSG }		= "flushAppSG";
$hOP -> { deleteSystemAppSG }	= "deleteSystemAppSG";
$hOP -> { unconfigureAppSG }	= "unconfigureAppSG";
$hOP -> { probeAppSG }		= "probeAppSG";
$hOP -> { unconfigureAllSG }	= "unconfigureAllSG";


#---------------------
# Subroutines
#---------------------
sub main
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $query = CGI->new();
	print $query->header('text/xml');

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	my $params = parseArguments($query);

	my $OP = $params -> { OP } || "";
	my $ID = $params -> { ID } || "";

	if ( $OP eq "" ) {
		($ret, $msg) = getMessage( MSG_FILE, "URL_SYNTAX_INVALID_OP" );
		goto END;
	}

	if ( $ID eq "" ) {
		($ret, $msg) = getMessage( MSG_FILE, "URL_SYNTAX_INVALID_ID" );
		goto END;
	}

	eval {
		my $appobj = Utils::getAppObjFactory($ID);

		if ( not defined $hOP -> { $OP } ) {
			($ret, $msg) = getMessage( MSG_FILE, "URL_SYNTAX_UNAVAILALE_OP", $OP);
			goto END;
		}

		my $function_name  = $hOP -> { $OP };
		my $function_param = $params -> { params };

		($ret, $msg, $xml) = $appobj -> $function_name ( $function_param );

		$msg = "" if (not defined $msg);
		$xml = "" if (not defined $xml);
	};
	if ( $@ ) {
		$xml = "<Debug><![CDATA[$@]]></Debug>";
		($ret, $msg) = getMessage( MSG_FILE, "CONFAPP_OP_EXECUTION_EXCEPTION", $OP, $ID);
	}

END:
	my $xmlstr = Utils::getCmdResponseXML($ID, $ret, $msg, $xml);
	print "$xmlstr\n";
	return $ret;	
}



my $ret = main();
exit $ret;

