#!/opt/VRTSsfmh/bin/perl -w

# $Id: push_file.pl,v 1.2 2014/09/11 17:42:27 sjinturk Exp $
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $

use strict;
use CGI;
use CGI::Carp qw(fatalsToBrowser);
use VRTS::Whitelist;
use VRTS::Paths;
use VRTS::CMD;
use File::Temp qw/ tempfile /;
use File::Copy;
use VRTS::Logger;
use JSON;

my $query;
my $tmpfh;
my $tmpfile;

sub test
{

my $heredoc =<<END;
  <p>
  <table width="100%" cellpadding=6 cellspacing=0>
  <tr bgcolor="#000088"><td><font color="#ffffff">
  Push a file</td></tr></table>
  <p><table width="100%" cellpadding=6 cellspacing=0>
     <form method="post" action="/admin/cgi-bin/push_file.pl/push" enctype="multipart/form-data">

    <tr><td>Local file:       <td><input type="file" name="data"/>    </tr>
    <tr><td>Destination file: <td><input type="text" name="destfile"/>
            <td>(may be \$TMPFILE, \$TMPDIR/filename, \$VARDIR/filename or exact path)</tr>
    <tr><td>Permission:       <td><input type="text" name="permission"/>
            <td>(e.g. 755, 644)</tr>
    <tr><td>Force:            <td><input type="checkbox" name="force" value="force"/>
            <td>overwrite if exists</tr>
    <tr><td>White list name:  <td><input type="text" name="whitename"/>
            <td><a href="/admin/cgi-bin/whitelist.pl/test">whitelist</a></tr>
    <tr><td>Exec:             <td><input type="checkbox" name="exec" value="exec"/>
            <td>run after upload</tr>
    <tr><td>Argv:             <td><input type="text" name="argv"/>     <td>
        example: ["arg1", "arg2"]</tr>
    <tr><td>Delete:           <td><input type="checkbox" name="delete" value="delete"/>
            <td>clean up after execute</tr>
    <tr><td>Json output:      <td><input type="checkbox" name="format" value="json"/> </tr>
    <tr><td><input type="hidden" name="test" value="test"  />
    <input type="submit" name="push" value="push" />
  </tr>
  </form>
  </table>

END
    print $query->header("text/html");
    print $heredoc;
    print $query->end_html();
    exit(0);
}

sub remove_file
{
   my $filename = shift;

   if ($^O =~ /Win32/i) {
        my $RM = &VRTS::CMD::RM;
        my $tmp = "\"".$filename."\"";
        my $rc = `$RM $tmp`;
    } else {
        unlink($filename);
    }
}

sub safe_exec
{
    my $cmd_in = shift;


    if(ref $cmd_in ne "ARRAY") {
       return (-1, "", "internal error, strings not supported by safeexec");
    }

    my $tmp_out=undef;
    my $tmp_err=undef;
    if ($^O =~ /Win32/i) {
        $tmp_out=VRTS::Paths::get_path("AppDir")."pushfile-tmp-out-$$";
        $tmp_err=VRTS::Paths::get_path("AppDir")."pushfile-tmp-err-$$";

        if(@$cmd_in[0] =~ /\.pl$/) {
            # perl script, use sfmh's perl
            my $pcmd; # e.g c:\\Program Files\\VERITAS\\VRTSsfmh\\bin\\perl.exe
            $pcmd = VRTS::Paths::get_path("InstallDir") . "bin\\perl.exe";
            unshift @$cmd_in, $pcmd;
        }
    } else {
        (undef, $tmp_out) = tempfile(DIR=>"/tmp", SUFFIX=>".pushfile.out");
        (undef, $tmp_err) = tempfile(DIR=>"/tmp", SUFFIX=>".pushfile.err");
    }

    # take copies of the file descriptors
    open(OLDOUT, ">&STDOUT") || return (-1, "", "file error (1)");
    open(OLDERR, ">&STDERR") || return (-1, "", "file error (2)");

    # redirect stdout and stderr
    open(STDOUT, "> $tmp_out") || return (-1, "", "file error (3)");
    open(STDERR, "> $tmp_err") || return (-1, "", "file error (4)");

    VRTS::Logger::log('debug', "running @$cmd_in");

    # run the program
    my $ret = system (@$cmd_in);

    # close the redirected filehandles
    close(STDOUT);
    close(STDERR);

    # restore stdout and stderr
    open(STDOUT, ">&OLDOUT");
    open(STDERR, ">&OLDERR");

    # avoid leaks by closing the independent copies
    close(OLDOUT);
    close(OLDERR);

    open (OUT, $tmp_out);
    my $complete_out = join("", <OUT>);
    close(OUT);

    open (ERR, $tmp_err);
    my $complete_err = join("", <ERR>);
    close(ERR);

    remove_file($tmp_err);
    remove_file($tmp_out);

    # log first 100 chars of output
    if(length($complete_out)) {
      $complete_out =~ /(.{0,100})/;
      VRTS::Logger::log('debug', "stdout=".$1."...");
    }
    if(length($complete_err)) {
      $complete_err =~ /(.{0,100})/;
      VRTS::Logger::log('debug', "stderr=".$1."...");
    }

    return ($ret, $complete_out, $complete_err);
}

sub error
{
    my $status = shift;
    VRTS::Logger::log('error', $status);
    print $query->header("text/plain", "500 ".$status);
    print "Error: ". $status ;
    exit(1);
}

sub check_destfile
{
    my $destfile = shift;
    my $force = shift;

    my $vardir = VRTS::Paths::get_path("AppDir");
    my $tmpdir = File::Spec->catdir($vardir, "tmp");
    my @DIRS;
    if ($^O =~ /Win32/i) {
      @DIRS = ( $tmpdir, $vardir );
      eval { File::Path::mkpath($tmpdir) };
    } else {
      @DIRS = ( $tmpdir, $vardir, 
        "/var/opt/VRTSsfmcs/", "/etc/default/");
    }

    my $filecreated;
    my $match = 0;

    $destfile =~ s#\$TMPDIR#$tmpdir#;
    $destfile =~ s#\$VARDIR#$vardir#;

    if($destfile =~ /\$TMPFILE/) {
        my (undef, $tmpfile) = tempfile( DIR => $tmpdir, SUFFIX=>".tmp" );
        $destfile = $tmpfile;
        $filecreated = 1;
        $match = 1;
    }

    $destfile =~ s/^'//;
    $destfile =~ s/'$//;

    for my $dir (@DIRS)
    {
      $dir =~ s#\\#\\\\#g;  # adjust regex for windows backslashes
      if($destfile =~ /^$dir/)
      {
        $match = 1;
      }
    }

    if($match == 0) {
     error("Invalid destination file path ($destfile)");
    }

    if(-e $destfile)
    {
      if(-l $destfile) {
          error("destfile exists as a link ($destfile)");
      }
      if(!$filecreated && not defined $force) {
          error("destfile exists ($destfile)");
      }
    }
    if($^O =~ /Win32/i) {
        $destfile =~ s#/#\\#g;  # convert slashes for windows
    }
    return $destfile;
}

sub temp_write_error
{
   my $filename = shift;
   remove_file($filename);
   error("Cannot write to file $filename");
}

sub main
{
  my ($head, $body, $tail);

  $query = new CGI;

  if($query->path_info() =~ /\/test$/) {
      test();
  }

  my $log = new VRTS::Logger(Module => 'pushfile',
                  Logdir => VRTS::Paths::get_path('LogDir'),
                  Level => 'debug',
                  Logfile => 'push_file.log',
                  Logsize => 1024*1024,
                  UmiString => 'V-0000-0000');


  my $destfile_in = $query->param("destfile") || $query->param("dest-file");
  my $perm = $query->param("permission");
  my $force = $query->param("force");
  my $whitename = $query->param("whitename");
  my $argv_in = $query->param("argv");
  my $exec_cmd = $query->param("exec");
  my $delete_cmd = $query->param("delete");
  my $format = $query->param("format");
  my $sep = $query->param('sep');
  my $argv = [];

# debug...
#  print $query->header("text/plain");
#  print "destf=$destfile_in\n";
#  print "perm=$perm\n";
#  print "force=$force\n";
#  print "white=$whitename\n";
#  print "argv=$argv_in\n";
#  print "exec=$exec_cmd\n";
#  print "delete=$delete\n";
#  print "format=$format\n";
#  exit(0);

  VRTS::Logger::log('debug', "====");

  if(not defined $destfile_in or $destfile_in eq '') {
    error("Invalid parameter destfile");
  }

  my $destfile = check_destfile($destfile_in, $force);
  VRTS::Logger::log('debug', "$destfile_in  resolves to:  $destfile");

  if ($whitename) {
    my $whitecmd = VRTS::Whitelist::get($whitename);
    if($whitecmd && $whitecmd ne $destfile) {
        error("A command is already in whitelist by the name $whitename");
    }
  }

  if($argv_in) {
     $argv = JSON::jsonToObj($argv_in);
     if(ref($argv) ne "ARRAY") {
         error("Incorrect parameter type, argv not an array");
     }
  }

  $tmpfile = $destfile.".$$.xfer";

  my $upload_filehandle = $query->param("data");
  open UPLOADFILE, ">$tmpfile";
  binmode UPLOADFILE;
  while ( <$upload_filehandle> )
  {
      print UPLOADFILE or temp_write_error($tmpfile);
  }
  close(UPLOADFILE);

  if(!move($tmpfile, $destfile)) {
    remove_file($tmpfile);
    error("move failed $! d=$destfile");
  }

  if(defined $perm && $perm ne '')
  {
      chmod oct($perm), $destfile;
  }

  if ($whitename) {
     VRTS::Whitelist::insert($whitename, $destfile);
  }

  if ($exec_cmd) {
     unshift(@$argv, $destfile);
     my ($rc, $cmdout, $cmderr) = safe_exec($argv);
     if($delete_cmd) {
         remove_file($destfile);
     }

     if($format && $format eq "json") {
       my $obj = {
         RESULT => {
             ARGV => $argv,
             RETURNCODE => $rc,
             RESULT => { out => $cmdout, err => $cmderr },
             FILENAME => $destfile,
             UMI => "",
           }
       };
       print $query->header("application/json", "200 OK");
       print JSON::objToJson($obj, {pretty=>1});
     } else {
       print $query->header("text/plain", "200 OK");
       if (defined($sep) && ($sep ne '')) {
            print $cmdout . $sep . $cmderr;
       } else {
            print $cmdout . $cmderr;
       }
     }
     exit(0);
  }

  print $query->header("text/plain", "200 OK");
  print "SUCCESS\n";
  print "$destfile\n";
  exit(0);
}
 
main();

