#!/opt/VRTSsfmh/bin/perl
#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
# $Id: setupSSH.pl,v 1.2 2012/08/17 22:03:14 sgupta2 Exp $
#

#------------------------
# Common Header
#------------------------
use strict;
use warnings;

use Data::Dumper;
use CGI;
use Config;
use POSIX;

# Push include paths!
my $LibPath1 = "";

BEGIN
{
# This file is shipped only on linux. This common header is kept for consistency.
	if ( $^O =~ m/win/i ) {
		require VRTS::Paths;

		my $xprtlInstallDir= VRTS::Paths::get_path("InstallDir");
		my $regquery = "\"$xprtlInstallDir"."\\bin\\regquery.exe\" query SOFTWARE\\VERITAS\\VCS\\Base HomeDirectory";
		my $vcshome = `$regquery`;
		$LibPath1 = "$vcshome\\portal\\admin\\lib\\";
		$ENV{'VCS_HOME'} = $vcshome;

	} else {
		my $APPDISCOVER_HOME = "/opt/VRTSvcs/portal/admin";
	
		$LibPath1 = "$APPDISCOVER_HOME/lib";
	}
}

use lib $LibPath1;
#------------------------
# End of Common Header
#------------------------

use Constants;
use Utils;
use VCS;
use PlatUtils;
use MessageUtils;

use constant MSG_FILE => "appmon.msg";

sub configureSSH
{
	my $paramsStr = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $msg = "";
	my $ret = CONST_SUCCESS;

	if ( $paramsStr eq "" ) {
		($ret, $msg) = getMessage( MSG_FILE, "INVALID_PARAMS_XML_SCHEMA" );
		goto END;
	}

	my $tree = Utils::convertXMLTextToHash($paramsStr);

	$logStr = Dumper($tree);
	Utils::vxlog("DEBUG", $logStr);

	# validates
	if (not defined $tree or $tree eq "") {
		($ret, $msg) = getMessage( MSG_FILE, "INVALID_PARAMS_XML_SCHEMA" );
		goto END;
	}

	my $authFile = AUTHORIZED_KEYS_FILE;
	my $pubkey = $tree -> { PublicKey };

	if ( ! -d SSH_DIR ) {
		mkdir ( SSH_DIR, 0700 );
	}

	my $cmdRet = open ( AUTHFILE, ">>$authFile" );
	if ( not defined $cmdRet ) {
		$ret = CONST_FAILURE;
		$msg = "Could not open $authFile.";
		Utils::vxlog("ERROR", $msg);
		goto END;
	}

	# $pubkey already contains new line
	print AUTHFILE $pubkey;

	close (AUTHFILE);

END:
	return ( $ret, $msg );
}


sub unconfigureSSH
{
	my $paramsStr = shift;

	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	if ( $paramsStr eq "" ) {
		($ret, $msg) = getMessage( MSG_FILE, "INVALID_PARAMS_XML_SCHEMA" );
		goto END;
	}

	my $tree = Utils::convertXMLTextToHash($paramsStr);

	$logStr = Dumper($tree);
	Utils::vxlog("DEBUG", $logStr);

	# validates
	if (not defined $tree or $tree eq "") {
		($ret, $msg) = getMessage( MSG_FILE, "INVALID_PARAMS_XML_SCHEMA" );
		goto END;
	}

	my $authFile = AUTHORIZED_KEYS_FILE;
	my $tmpAuthFile = $authFile . ".tmp";

	my $pubkey = $tree -> { PublicKey };
	chomp ( $pubkey );

	# The file is first copied to preserve permissions.
	($ret, $msg) = Utils::runCmd("cp -p $authFile $tmpAuthFile; grep -v \"$pubkey\" $tmpAuthFile > $authFile; rm -f $tmpAuthFile");

END:
	return ( $ret, $msg );
}


sub main
{
	my $sMyName = ( caller 0 )[3];
	#---------
	# Announce
	#---------
	my $logStr = "Subroutine $sMyName called";
	Utils::vxlog("INFO", $logStr);

	my $query = CGI->new();
	print $query->header('text/xml');

	my $ret = CONST_SUCCESS;
	my $msg = "";
	my $xml = "";

	my $params = parseArguments($query);

	my $OP = $params -> { OP } || "";
	my $ID = $params -> { ID } || "";

	if ( $OP eq "" ) {
		($ret, $msg) = getMessage( MSG_FILE, "URL_SYNTAX_INVALID_OP" );
		goto END;
	}

	eval {
		if ( $OP =~ /^configureSSH$/i ) {
			($ret, $msg) = configureSSH($params -> { params } || "");
		} elsif ( $OP =~ /^unconfigureSSH$/ ) {
			($ret, $msg) = unconfigureSSH($params -> { params } || "");
		} else {
			($ret, $msg) = getMessage( MSG_FILE, "URL_SYNTAX_UNAVAILALE_OP");
		}
	};
	if ( $@ ) {
		$xml = "<Debug><![CDATA[$@]]></Debug>";
		($ret, $msg) = getMessage( MSG_FILE, "CONFAPP_OP_EXECUTION_EXCEPTION", $OP, $ID);
	}

END:
	my $xmlstr = Utils::getCmdResponseXML("setupSSH", $ret, $msg, $xml);
	print "$xmlstr\n";
	return $ret;

}


my $ret = main();
exit $ret;

