#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package CPIC;
# require libs that are CPIC package and use modules that are unique objects
use CPIC::initiation;
use CPIC::options;
use CPIC::systems;
use CPIC::pkgs;
use CPIC::shutdown;
use CPIC::uninstall;
use CPIC::install;
use CPIC::configure;
use CPIC::startup;
use CPIC::postcheck;
use CPIC::completion;
use CPIC::rollback;
use CPIC::version;

use CPIP;
use Rel;
use Prod;
use Pkg;
use Proc;
use Patch;
use Hotfix;
use MR;
use Obj;
use strict;

@CPIC::ISA = qw(Obj);

# define core cpic things here
sub init {
    my $cpic=shift;
    $cpic->{falsefailurefile} = '/tmp/cpifalsefailure';
    $cpic->{threadlimit}=8;
    $cpic->{hotfixdir}='/opt/VRTS/install/.cpihotfix';
    return;
}

sub main {
    my ($rel,$padv,$mediapath,$task,$prod) = @_;
    my (@subs,$cpic,@init);
    $cpic=CPIC->new();
    # initiation, systems must be run first to parse args and determine upgrade
    $cpic->initiation($rel, $padv, $mediapath, $task, $prod);
    @init=qw(install_useragreement options systems upgrade_useragreement);
    $cpic->run_subs(@init);
    @subs = (Cfg::opt('precheck')) ? qw() :
       (Cfg::opt('postcheck')) ? qw(postcheck) :
       (Cfg::opt('install')) ? qw(set_pkgs shutdown uninstall install licensing
                                  prestart_config shutdown startup poststart_config) :
       (Cfg::opt('patchupgrade')) ? qw(set_pkgs shutdown uninstall install licensing
                                       prestart_config startup poststart_config) :
       (Cfg::opt('hotfixupgrade')) ? (Cfg::opt('vom') ? qw(set_pkgs pre_setup shutdown uninstall install startup post_setup) : qw(set_pkgs shutdown uninstall install startup)) :
       (Cfg::opt('upgrade')) ? qw(set_pkgs upgrade_prestop_config shutdown uninstall install licensing prestart_config startup poststart_config) :
       (Cfg::opt('uninstall')) ? qw(set_pkgs shutdown uninstall) :
       (Cfg::opt('rollback')) ? qw(set_pkgs shutdown rollback licensing prestart_config startup poststart_config) :
       (Cfg::opt('configure')) ? qw(licensing prestart_config shutdown startup poststart_config) :
       (Cfg::opt('license')) ? qw(licensing) :
       (Cfg::opt("hotfix")) ? qw(hotfix) :
       (Cfg::opt("restore")) ? qw(restore_hotfix) :
       (Cfg::opt('start')) ? qw(licensing startup) :
       (Cfg::opt('startstoptest')) ? qw(startstoptest) :
       (Cfg::opt('stop')) ? qw(shutdown) : EDR::die ('no task');
    $cpic->run_subs(@subs,'completion');
    return;
}

sub deploy_sfha {
    my ($rel,$padv,$mediapath,$task,$prod) = (@_);
    my ($cpic,$deploy,$edr,$titlemsg);
    require Deploy;
    $cpic=CPIC->new();
    $deploy=Deploy->new();
    $cpic->edr_initiation($rel, $padv, $mediapath, $task, $prod);
    $edr=Obj::edr();
    # init cpip object
    $cpic->create_cpip_objects($cpic->{padv});
    $cpic->process_deploy_sfha_args();
    $edr->create_local_tempdir();
    $titlemsg=Msg::new("Deployment Server", 30, 1488);
    $cpic->cli_set_title($titlemsg);
    $edr->cli_display_copyright();

    $rel=$cpic->rel();
    $deploy->init_preference_hash();
    $rel->set_matrix_path();
    $rel->copy_releasematrix();
    $rel->init_releasematrix($rel->{matrix_path});
    $deploy->cli_deploy_sfha_menu();
    $edr->exit_noexitfile();
    return;
}

sub run_subs {
    my ($cpic,@subs) = @_;
    my ($i,$sub,$timer,$elapsed);

    $i = 0;
    while($i <= $#subs) {
        $sub=$subs[$i];
        Metrics::set_phase($sub);
        $timer=Timer->new("main::sub");
        $timer->start();
        $cpic->$sub();
        $elapsed=$timer->stop();
        if (Cfg::opt('back')) {
            $i-- if ($i>0);
            Cfg::unset_opt('back');
        } else {
            $i++;
            Metrics::add_phase_summary_metrics('','Total,Execute_Time',$elapsed);
            $elapsed=$timer->hms('short_hires');
            Msg::log("Phase '$sub' took $elapsed");
        }
    }
    return;
}

sub set_value {
    my $obj=shift;
    Obj::set_value($obj->{pool}, @_);
    Thr::setq($obj->{pool}, @_) if (EDR::thread_support());
    return;
}

sub set_task {
    my ($cpic,$opt) = @_;
    my ($rel,$cfg,$task);

    $rel=$cpic->rel;
    return unless EDRu::inarr($opt, @{$rel->{args_task}});

    $cfg=Obj::cfg();
    for my $task (@{$rel->{args_task}}) {
        if ($cfg->{opt}{$task}) {
            $cfg->set_value("opt,$task",undef);
        }
    }
    $cfg->set_value("opt,$opt",1);
    return;
}

# Pkg, Patch, and Proc objects MUST be associated with a Padv, however
# CPIC will make some Rel and Prod calls that are not associated with a
# system, and therefore a padv, because the system loop and operation will
# be done within the subroutine.  For single-padv, this is fine, but for any
# to many, there is no single padv to correctly map the multi-system call to.
# Since we do not ever create a Common Rel or Prod object itself, we will pass
# back the object of the first padv, and assume that all such multi-system
# multi-padv Rel and Prod calls will be correctly defined at the Common level

sub padvisa {
    my ($cpic,$padv)=@_;
    # override if $cpic->{padvisa}{$padv} is set
    $padv=$cpic->{padvisa}{$padv} if ($padv && $cpic->{padvisa}{$padv});
    $padv||=$cpic->{padv};
    return $padv;
}

# release must be passed in the hypothetical multi-release situation
sub rel {
    my($cpic,$padv,$donotdie)= @_;
    return Obj::rel($cpic->padvisa($padv),$donotdie);
}

sub prod {
    my($cpic,$prod,$padv)= @_;
    $prod||=$cpic->{prod};
    EDR::die ("Call to \$cpic->prod made without passing prod or \$cpic->{prod} defined") if (!$prod);
    return Obj::prod($prod, $cpic->padvisa($padv));
}

sub pkg {
    my($cpic,$pkg,$padv)= @_;
    return Obj::pkg($pkg, $cpic->padvisa($padv));
}

sub proc {
    my($cpic,$proc,$padv)= @_;
    return Obj::proc($proc, $cpic->padvisa($padv));
}

sub patch {
    my($cpic,$patch,$padv)= @_;
    return Obj::patch($patch, $cpic->padvisa($padv));
}

sub mr {
    my($cpic,$mr,$padv)= @_;
    return Obj::mr($mr, $cpic->padvisa($padv));
}
sub hotfix {
    my($cpic,$hotfix,$padv)= @_;
    return Obj::hotfix($hotfix, $cpic->padvisa($padv));
}

sub eeb {
    my($cpic,$eeb,$padv)= @_;
    return Obj::eeb($eeb, $cpic->padvisa($padv));
}

sub nsystems {
    my $cpic=shift;
    $cpic||=Obj::cpic();
    return scalar(@{$cpic->{systems}});
}

sub systems_need_reboot {
    my (@reboot_sys,$cpic,$sys,$check_tunables);
    $cpic=shift;
    $check_tunables=shift;
    if ($check_tunables) {
        for my $sys (@{$cpic->{systems}}) {
            push(@reboot_sys,$sys->{sys}) if ($sys->{proc_reboot} || $sys->{reboot} ||
                                              (Cfg::opt('upgrade') && (!$sys->{zru_supported})) ||
                                              (defined $sys->{requirerebootpkgs}) ||
                                              (defined $sys->{requirerebootpkgspatches}) ||
                                              (defined $sys->{requirerebootpatches}) ||
                                              (defined $sys->{requirereboottunables}));
        }
    } else {
        for my $sys (@{$cpic->{systems}}) {
            push(@reboot_sys,$sys->{sys}) if ($sys->{proc_reboot} || $sys->{reboot} ||
                                              (Cfg::opt('upgrade') && (!$sys->{zru_supported})) ||
                                              (defined $sys->{requirerebootpkgs}) ||
                                              (defined $sys->{requirerebootpkgspatches}) ||
                                              (defined $sys->{requirerebootpatches}));
        }
    }
    return \@reboot_sys;
}

sub upgrade_sub {
    my $sub=shift;
    return (Cfg::opt(qw(rollback upgrade patchupgrade hotfixupgrade))) ? "upgrade_$sub" : $sub;
}

sub false_failure {
    my $cpic=shift;
    return 1 if (($cpic->{falsefailurefile}) &&
                 (-f $cpic->{falsefailurefile}) &&
                 (++$cpic->{falsefailures}==1));
    return '';
}

sub get {
    my $cpic=Obj::cpic();
    return $cpic->{$_[0]};
}

sub get2 {
    my $cpic=Obj::cpic();
    return $cpic->{$_[0]}{$_[1]};
}

# main sub for product licensing
# moving lib/CPIC/v60/CPIC/licensing.pm to
#        lib/CPIP/uxrt60/Rel/UXRTLicensing60.pl
sub licensing {
    my ($cpic,$back_from_key_menu) = @_;
    my $rel = $cpic->rel();

    if ($rel->can("licensing")) {
        $rel->licensing($back_from_key_menu);
    }
    return;
}

sub show_ui_form {
    my $cpic=shift;
    my $form_name=shift;

    if (Obj::webui()) {
        my $web = Obj::web();
        $web->web_script_form($form_name, @_);
    }
    return 1;
}

sub store_obj {
    my ($cpic, $argv, $tmpdir) = @_;

    $tmpdir ||= "/tmp/serialized_obj";

    #refresh the serialized dir
    if (-d $tmpdir) {
        EDRu::rmdir_local_nosys($tmpdir);
    }
    EDRu::mkdir_local_nosys($tmpdir);

    #store Obj::pool
    for my $class (keys %Obj::pool) {
        if (!Obj->store($Obj::pool{$class}, "$tmpdir/$class")){
            print "failed to store $class in $tmpdir/$class\n";
            return;
        }
    }

   if (!Obj->store(\%ENV, "$tmpdir/ENV")) {
       print "failed to store ENV in $tmpdir/ENV\n";
       return;
   }
   if ($argv && !Obj->store($argv, "$tmpdir/argv")) {
       print "failed to store argv in $tmpdir/argv\n";
       return;
   }

    print "store objs in $tmpdir successfully\n";
    return;
}

sub load_obj {
    my ($cpic, $tmpdir) = @_;
    my $basename;
    my @argv;
    my $var;

    $tmpdir ||= "/tmp/serialized_obj";
    if (!-d $tmpdir) {
        print "cannot find $tmpdir in local system\n";
        return;
    }

    for my $file (glob("$tmpdir/*")) {
        $basename = EDRu::basename($file);
        $var = Obj->load($file);
        if (!$var) {
            print "failed to load variable from $file\n";
            return;
        }
        if ($basename eq "argv") {
            @argv = @{$var};
        } elsif ($basename eq "ENV") {
            %ENV = %{$var};
        } else {
            $Obj::pool{$basename} = $var;
        }
    }

    #reset %SIG
    if (defined $Obj::pool{EDR}) {
        my $edr = Obj::edr();
        $edr->set_interrupt();
    }

    #reset @ISA
    $cpic = Obj::cpic();
    $cpic->rebuild_inherit();


    print "load objs from $tmpdir successfully\n";
    return @argv;
}

sub create_cpip_inherit {
    my ($cpic, $class,$padv, $is_patch) = @_;
    my ($plat,$inherit_padv,$codes,$baseclass);

    # in case some OS packages name have -.+
    $class=~s/[-\=\,\.\+]/_/mg if($is_patch);

    $plat=Padv::plat($padv);
    $inherit_padv=$cpic->{padv};
    if (!$inherit_padv || $inherit_padv eq 'Common' || $inherit_padv eq $padv) {
        $inherit_padv=Padv::inherit_uplevel($padv);
    }

    if ($inherit_padv && UNIVERSAL::can("$class\::$inherit_padv", 'new')) {
        $codes = "package $class\::$padv; \@$class\::$padv\::ISA = qw($class\::$inherit_padv);";
    } elsif (UNIVERSAL::can("$class\::$plat", 'new')) {
        $codes = "package $class\::$padv; \@$class\::$padv\::ISA = qw($class\::$plat);";
    } elsif (UNIVERSAL::can("$class\::Common", 'new') && $is_patch) {
        $codes = "package $class\::$padv; \@$class\::$padv\::ISA = qw($class\::Common);";
    } elsif ($is_patch) {
        $baseclass=$class;
        $baseclass=~s/\::.*$//m;
        $codes = "package $class\::$padv; \@$class\::$padv\::ISA = qw($baseclass);";
    }
    eval $codes if ($codes);

    return "$class\::$padv"->new();
}

sub rebuild_inherit {
    my $cpic = shift;
    my ($rel, $padv, $prod, $pkg, $patch, @pkgs, $pkglist);
    $padv = $cpic->{padv};
    $cpic->create_cpip_inherit("Rel::$cpic->{release}", $padv);
    $rel = $cpic->{rel}{$padv};
    push(@pkgs, @{$rel->allpkgs});
    for my $prodv (@{$rel->{prods}}) {
        $prod = $cpic->create_cpip_inherit("Prod::$prodv", $padv);
        push(@pkgs, @{$prod->allpkgs_on_media});
        for my $subprodv (@{$prod->{subprods}}) {
            my $subprod = $cpic->create_cpip_inherit("Prod::$subprodv", $padv);
            push(@pkgs, @{$subprod->allpkgs_on_media});
        }
    }
    $pkglist = EDRu::arruniq(@pkgs);
    for my $pkgi (@{$pkglist}) {
        $pkg = $cpic->create_cpip_inherit("Pkg::$pkgi", $padv);
        for my $proc (@{$pkg->allprocs}) {
            $cpic->create_cpip_inherit("Proc::$proc",$padv);
        }
    }
    if ($cpic->{mr}) {
        my $mr = $cpic->create_cpip_inherit("MR::$cpic->{mr}", $padv); 
    }
    for my $hfi(@{$cpic->{hotfixes}}) {
        my $hotfix= $cpic->create_cpip_inherit("Hotfix::$hfi",$padv);
    }
    for my $pkgi (@{$pkglist}) {
        $pkg=Obj::pkg($pkgi, $padv);
        for my $patchi (@{$pkg->patches_allvers}) {
            $cpic->create_cpip_inherit("Patch::$patchi",$padv,1);
        }
    }
}



sub base_release { }
sub base_requires { }
sub maintenance_release { }
sub maintenance_version { }
sub maintenance_requires { }
sub hotfix_release { }

1;
