#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package CPIC;
use strict;

# no stubs for the following configuration subs:
#     cli_prestart_config_questions
#     cli_poststart_config_questions
#     configure_sys (handles preinstall and postinstall)
# in Prod.pm as their declaration are step counters

sub prestart_config {
    my $cpic=shift;
    my ($ayn,$cfg,$edr,$prod,$sys,$reboot_systems);
    $prod=$cpic->prod;
    $edr=Obj::edr();
    $cfg=Obj::cfg();

    if (Cfg::opt(qw(upgrade))) {
        ($cpic->ask_upgrade_configure($prod));
        return 1;
    }

    if (Cfg::opt('install')) {
        $reboot_systems=$cpic->systems_need_reboot();
        if (Cfg::opt('rootpath') || @{$reboot_systems}) {
            $cpic->printmsg_skip_setting_tunables('reboot',$prod);
            $cpic->completion();
        }
        if ($prod->disableconfig()) {
            $cpic->printmsg_skip_setting_tunables('disableconfig',$prod);
            $cpic->completion();
        }
        if ($prod->hasconfig()) {
            if ($cpic->ask_configure()) {
                $cpic->set_task('configure');
            } else {
                $cpic->printmsg_skip_setting_tunables('skipconfig',$prod);
                $cpic->completion();
            }
        }
        $cpic->set_task('configure');
        $cpic->{installconfig}=1;
    }

    Cfg::set_opt('silent') if ($prod->{silent_installconfig});

    $cpic->validate_tunables();
    $cpic->set_tunables_any();
    $cpic->set_tunables_prestart();

    $cpic->{prod_running}=1;
    for my $sys (@{$cpic->{systems}}) {
        unless ($sys->{prod_running}) {
            $cpic->{prod_running}='';
            last;
        }
    }
    $cpic->{reuse_config}=$cpic->{prod_running};

    if (Cfg::opt('responsefile')) {
        $cpic->{reuse_config}=$cfg->{reuse_config};
        $prod->responsefile_prestart_config() unless (Cfg::opt(qw(patchupgrade rollback)) && $cfg->{reuse_config});
        if (Cfg::opt(qw(patchupgrade rollback))) {
            # complete responsefile patch upgrade if no configuration task is set
            $cpic->completion() unless ($cpic->{reuse_config} || Cfg::opt('patchconfigure'));
        }
    }

    if (Cfg::opt(qw(patchupgrade rollback))) {
        ($cpic->ask_patchupgrade_configure($prod)) ? Cfg::set_opt('patchconfigure',1) : return '';
    }

    return '' if (Cfg::opt('responsefile') && ($edr->{exitfile} eq ''));
    $prod->cli_prestart_config_questions() if
        ($prod->can('cli_prestart_config_questions'));
    $cpic->web_prestart_config($prod);
    return 1;
}

sub web_prestart_config {
    my ($cpic, $prod) = @_;
    my $web = Obj::web();
    $web->{to_config} = 1;

    $prod->web_prestart_config_questions() if
        ($prod->can('web_prestart_config_questions'));
    return 1;
}

sub ask_configure {
    my $cpic=shift;
    my ($cfg,$prod,$systems,$msg,$ayn);

    $prod=$cpic->prod;
    return 1 if (Cfg::opt('responsefile') && Cfg::opt('configure'));
    # do not ask configure if not using responsefile and silent_installconfig is set
    return 1 if ($prod->{silent_installconfig} && (!Cfg::opt('responsefile')));
    # do not ask configure if any pkgs or patches failed to be installed on any systems
    for my $sys (@{$cpic->{systems}}) {
        return 0 if ( $#{$sys->{pkginstallfail}}>=0 || $#{$sys->{patchinstallfail}}>=0 );
    }

    $cfg=Obj::cfg();
    $systems = join(' ',@{$cfg->{systems}});
    $msg=Msg::new("Would you like to configure $prod->{abbr} on $systems?", 30, 1029, "$prod->{abbr}", "$systems");
    $ayn=$msg->aynn;
    Msg::n();
    return 0 if ($ayn eq 'N');
    return 1;
}

sub ask_patchupgrade_configure {
    my ($cpic,$prod) = @_;
    my ($ayn,$cfg,$edr,$help,$msg);
    $edr=Obj::edr();
    $cfg=Obj::cfg();

    # ask_patchupgrade_configure must be called for rollback/patchupgrade task.
    $prod->ask_patchupgrade_configure() if ($prod->can('ask_patchupgrade_configure'));
    
    return '' if (Cfg::opt('responsefile') && ($edr->{exitfile} eq ''));
    $cfg->{reuse_config} = $cpic->{prod_running};
    return '' if ($cpic->{prod_running});
    return '' unless (($prod->can('cli_prestart_config_questions')) || ($prod->can('web_prestart_config_questions')));
    # if product doesn't have config, no need to prompt user to re-use previous configure.
    return '' unless ($prod->hasconfig);

    if ($prod->check_config) {
        $help = Msg::new("If you are upgrading $prod->{abbr} and want to use the previous copy of the $prod->{abbr} configuration, enter 'y'. If you desire to do new configuration of $prod->{abbr}, enter 'n'.", 30, 1030, "$prod->{abbr}", "$prod->{abbr}", "$prod->{abbr}");
        $msg = Msg::new("Do you want to restore and reuse the previous $prod->{abbr} configuration on all systems", 30, 1031, "$prod->{abbr}");
        $ayn = $msg->ayny($help,0);
        if ($ayn eq 'Y') {
            $cpic->{reuse_config}=1;
            $cfg->{reuse_config}=1;
            return '';
        }
    }
    $cpic->completion() unless $cpic->ask_configure();
    return 1;
}

sub ask_upgrade_configure {
    my ($cpic,$prod) = @_;

    $prod->ask_upgrade_configure() if ($prod->can('ask_upgrade_configure'));
    return 1;
}

sub upgrade_prestop_config {
    my $cpic=shift;
    my ($prod);

    $prod=$cpic->prod;
    $prod->upgrade_prestop_config() if ($prod->can('upgrade_prestop_config'));
    return 1;
} 

sub poststart_config {
    my $cpic=shift;
    my $edr=Obj::edr();
    my $prod=$cpic->prod;

    $cpic->set_tunables_poststart();
    $cpic->print_tunable_failures();

    return '' if ($cpic->{reuse_config} && Cfg::opt('patchupgrade'));
    return '' if (Cfg::opt('upgrade'));
    $prod->responsefile_poststart_config() if (Cfg::opt('responsefile'));
    return '' if (Cfg::opt('responsefile') && ($edr->{exitfile} eq ''));

    my $reboot_systems=$cpic->systems_need_reboot();
    return '' if (@{$reboot_systems});

    $prod->cli_poststart_config_questions() if
        ($prod->can('cli_poststart_config_questions'));
    $prod->web_poststart_config_questions() if
        ($prod->can('web_poststart_config_questions'));

    # poststart config tasks done silently and serially
    # no need for new one step web screen right before completion
    if ($prod->can('poststart_configure_sys')) {
        for my $sys(@{$cpic->{systems}}) {
            $prod->poststart_configure_sys($sys);
        }
    }
    return '';
}

sub prod_prestart_sys {
    my ($cpic,$sys)= @_;
    return 1 if (Cfg::opt('makeresponsefile'));
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $sys->prod->prestart_sys($sys) if ($sys->prod->can('prestart_sys'));
    return 1;
}

sub prod_configure_sys {
    my ($cpic,$sys)= @_;
    return 1 if (Cfg::opt('makeresponsefile'));
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $sys->prod->configure_sys($sys);
    return 1;
}

sub prod_upgrade_configure_sys {
    my ($cpic,$sys)= @_;
    return 1 if (Cfg::opt('makeresponsefile'));
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $sys->prod->upgrade_configure_sys($sys);
    return 1;
}

sub prod_patchupgrade_configure_sys {
    my ($cpic,$sys)= @_;
    return 1 if (Cfg::opt('makeresponsefile'));
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $sys->prod->patchupgrade_configure_sys($sys);
    return 1;
}

sub validate_tunables {
    my ($cpic) = @_;
    my ($msg,$ayn,$cfg,$edr,$rel,$val,$sys,$sysi,$tunable,$tunable_info,$systems,$inputsystems,$all_supported_tunables,@reboot_tunables,$reboot_tunables_msg,$nr_line,$max_nr_line,$supported_tunables,$t_prodname,$i_prodname,$msgyes,$msgno,$msg_title,$current_value,$reboot);
    $cfg = Obj::cfg();
    $edr = Obj::edr();
    $rel = Obj::rel();

    if (Cfg::opt(qw(install configure upgrade patchupgrade settunables)) && Cfg::opt(qw(tunablesfile responsefile))) {

        if (Cfg::opt(qw(responsefile)) && !$cfg->{tunable}) {
            return;
        }

        if (Cfg::opt(qw(upgrade_kernelpkgs upgrade_nonkernelpkgs))) {
            Msg::log("Skip setting tunable parameters for rolling upgrade.");
            return;
        }

        $systems = $cpic->{systems};
        if (!$systems) {

            $systems = $cfg->{systems};

            if (!$systems) {

                if ($cfg->{opt}{hostfile}){
                    $inputsystems = EDRu::readfile($cfg->{opt}{hostfile});
                } else {
                    Msg::n();
                    $msg = Msg::new("Enter the system names to set tunable parameters: ", 30, 1032);
                    $inputsystems = $msg->ask($edr->{localsys}->{sys}, '', '');
                }
                @{$systems} = @{EDRu::arruniq(split(/\s+/m, $inputsystems))};
            }

            $edr->create_local_tempdir;
            $edr->{savelog} = 1;
            $cpic->register_logfiles();

            for my $sysname (@{$systems}) {
                $sys = Sys->new($sysname);
                if ($edr->check_and_setup_transport_sys($sys)==-1) {
                    $systems = EDRu::arrdel($systems, $sys->{sys});
                    next;
                }
                push @{$cpic->{systems}},$sys;
                push @{$edr->{systems}},$sys;
            }
            $systems = $cpic->{systems};
        }

        if (!$systems) {
            $cpic->{donotsettunables} =1;
            return;
        }

        if (Cfg::opt(qw(settunables))){
            Msg::n();
        } else {
            Msg::title();
        }

        $msg = Msg::new("Validating tunable parameters to be set", 30, 1033);
        $msg->bold;

        $all_supported_tunables = [];
        if ($cpic->all_tunables) {
            @$all_supported_tunables = keys %{$cpic->all_tunables};
        }

        $supported_tunables = [];
        if ($cpic->supported_tunables) {
            @$supported_tunables = keys %{$cpic->supported_tunables};
        }

        for my $tunable (sort keys %{$cfg->{tunable}}) {
            if (! EDRu::inarr($tunable, @{$all_supported_tunables})){
                $msg = Msg::new("$tunable is not a supported tunable parameter", 30, 1034, "$tunable");
                $msg->print;
                next;
            }
            $tunable_info = $cpic->get_tunable_info($tunable);
            if (!Cfg::opt(qw/settunables/) && !EDRu::inarr($tunable, @{$supported_tunables})){
                $t_prodname = $tunable_info->{define_object}->{name};
                $i_prodname = $cpic->prod->{name};

                $msg = Msg::new("$tunable is for $t_prodname but $i_prodname is being installed or configured", 30, 1035, "$tunable", "$t_prodname", "$i_prodname");
                $msg->print;
                next;
            }
            for my $sys (@$systems) {
                $sysi = $sys->{sys};
                if (EDRu::inarr($sysi, keys %{$cfg->{tunable}{$tunable}})) {
                    $val = $cfg->{tunable}{$tunable}{"$sysi"};
                } elsif (EDRu::inarr("*", keys %{$cfg->{tunable}{$tunable}}) ) {
                    $val = $cfg->{tunable}{$tunable}{"*"};
                } else {
                    $val = undef;
                }

                if (defined($val)){
                    if (!$cpic->validate_tunable_sys($sys,$tunable,$val)) {
                        if ($tunable_info->{when_to_set} == 0) {
                            push @{$cpic->{valid_tunables_prestart}}, [ $tunable,$val,$sys ];
                        } elsif ($tunable_info->{when_to_set} == 1) {
                            push @{$cpic->{valid_tunables_poststart}}, [ $tunable,$val,$sys ];
                        } elsif ($tunable_info->{when_to_set} == 2) {
                            push @{$cpic->{valid_tunables_any}}, [ $tunable,$val,$sys ];
                        } else {
                            push @{$cpic->{valid_tunables_prestart}}, [ $tunable,$val,$sys ];
                        }
                        #if ($tunable_info->{reboot}) {
                        #    push @reboot_tunables, $tunable if (!EDRu::inarr($tunable, @reboot_tunables));
                        #}
                    }
                } else {
                    Msg::n();
                    $msg = Msg::new("Value of $tunable is not specified for $sysi", 30, 1036, "$tunable", "$sysi");
                    $msg->print;
                }
            }
        }

        Msg::n();

        if (($cpic->{valid_tunables_prestart} && @{$cpic->{valid_tunables_prestart}}) ||
            ($cpic->{valid_tunables_poststart} && @{$cpic->{valid_tunables_poststart}}) ||
            ($cpic->{valid_tunables_any} && @{$cpic->{valid_tunables_any}})
           ) {
            $msg = Msg::new("The following tunable parameters will be set:", 30, 1037);
            $msg->bold;
            Msg::n();
            my $format_string = "%-32s %-10s %-10s %-16s %-8s";
            $msg_title = Msg::string_sprintf($format_string,
                                       Msg::new("Tunable", 30, 1038)->{msg},
                                       Msg::new("Current", 30, 1039)->{msg},
                                       Msg::new("New", 30, 1040)->{msg},
                                       Msg::new("System", 30, 1041)->{msg},
                                       Msg::new("Reboot", 30, 1042)->{msg},
                                       );
            Msg::bold($msg_title);
            print '=============================================================================='."\n";
            $nr_line = 0;
            $max_nr_line = 20;
            for my $tun_val_sys (@{$cpic->{valid_tunables_any}}, @{$cpic->{valid_tunables_prestart}}, @{$cpic->{valid_tunables_poststart}}) {
                if ( $nr_line>=$max_nr_line ) {
                    $nr_line = 0;
                    Msg::prtc();
                    Msg::n();
                    Msg::bold($msg_title);
                    print '============================================================================'."\n";
                }
                ($tunable,$val,$sys) = @$tun_val_sys;
                $tunable_info = $cpic->get_tunable_info($tunable);
                $sysi = $sys->{sys};
                $msgyes = Msg::new("Yes", 30, 1043);
                $msgno = Msg::new("No", 30, 1044);
                $current_value = $tunable_info->{define_object}->get_tunable_value_sys($sys,$tunable);
                if ($current_value eq "" || $current_value eq undef) {
                    $current_value = Msg::new("unset", 30, 1045);
                    $current_value = '('.$current_value->{msg}.')';
                }
                $rel->check_tunable_info_sys($sys,$tunable_info);
                if ($tunable_info->{reboot} && $current_value ne $val ) {
                    push @reboot_tunables, $tunable if (!EDRu::inarr($tunable, @reboot_tunables));
                    $reboot = $msgyes->{msg};
                } else {
                    $reboot = $msgno->{msg};
                }
                $msg = Msg::string_sprintf($format_string,$tunable,$current_value,$val,$sysi,$reboot);
                Msg::print($msg);
                $nr_line++;
            }

            # for my $tun_val_sys (@{$cpic->{valid_tunables_prestart}}) {
            #     ($tunable,$val,$sys) = @$tun_val_sys;
            #     $tunable_info = $cpic->get_tunable_info($tunable);
            #     $sysi = $sys->{sys};
            #     $msg = Msg::string_sprintf($format_string,$tunable,$val,$sysi,$tunable_info->{reboot}?Msg::new("Yes")->{msg}:Msg::new("No")->{msg});
            #     Msg::print($msg);
            # }

            # for my $tun_val_sys (@{$cpic->{valid_tunables_poststart}}) {
            #     ($tunable,$val,$sys) = @$tun_val_sys;
            #     $tunable_info = $cpic->get_tunable_info($tunable);
            #     $sysi = $sys->{sys};
            #     $msg = Msg::string_sprintf($format_string,$tunable,$val,$sysi,$tunable_info->{reboot}?Msg::new("Yes")->{msg}:Msg::new("No")->{msg});
            #     Msg::print($msg);
            # }

            Msg::n();

            if (@reboot_tunables) {
                $reboot_tunables_msg = join("\n\t",sort @reboot_tunables);
                $msg = Msg::new("System reboot is required for the following tunable parameter(s) to take effect:\n\t$reboot_tunables_msg", 30, 1046, "$reboot_tunables_msg");
                $msg->print;
                Msg::n();
            }

            if (Cfg::opt("responsefile silent")) {
                $ayn = "Y";
            } else {
                $msg = Msg::new("Do you want installer to set the tunable parameter(s) listed above?", 30, 1047);
                $ayn = $msg->ayny;
            }

            if ( $ayn eq 'N') {
                $cpic->{donotsettunables} = 1;
                $msg = Msg::new("Skip setting tunable parameters", 30, 1048);
                $msg->print;
                return;
            } else {
                $cpic->{donotsettunables} = 0;
            }

        } else {
            $cpic->{donotsettunables} = 1;
            $msg = Msg::new("No tunable parameters will be set", 30, 1049);
            $msg->print;
            Msg::n();
        }
    }
    return;
}

sub set_tunables_prestart {
    my ($cpic) = @_;
    return $cpic->set_tunables("prestart");
}

sub set_tunables_poststart {
    my ($cpic) = @_;
    $cpic->set_tunables("poststart");
    return;
}

sub set_tunables_any {
    my ($cpic) = @_;
    return $cpic->set_tunables("any");
}

sub set_tunables {
    my ($cpic, $when) = @_;
    my ($tunable,$val,$sys,$key);
    if ( !$when || ! EDRu::inarr($when, qw/prestart poststart any/)) {
        return;
    }

    if (!$cpic->{donotsettunables}) {
        $key = "valid_tunables_$when";
        if ($cpic->{$key} && @{$cpic->{$key}}) {
            for my $tun_val_sys (@{$cpic->{$key}}) {
                ($tunable,$val,$sys) = @$tun_val_sys;
                $cpic->set_tunable_sys($sys,$tunable,$val);
            }
        }
    }
    return;
}

sub print_tunable_failures {
    my ($cpic) = @_;
    my ($msg,$sys,$tunable,$value,$out,$sysi);
    if ($cpic->{donotsettunables}) {
        return;
    }
    if ( $cpic->{tunable_failures} && @{$cpic->{tunable_failures}}) {
        Msg::n();
        $msg = Msg::new("The following errors were found when setting tunable parameters:", 30, 1050);
        $msg->bold;
        Msg::n();
        for my $error (@{$cpic->{tunable_failures}}) {
            ($sys, $tunable, $value, $out) = @{$error};
            $sysi = $sys->{sys};
            $msg = Msg::new("Failed to set $tunable to $value on $sysi", 30, 1051, "$tunable", "$value", "$sysi");
            $msg->print;
            $msg = Msg::new("\t$out", 30, 1052, "$out");
            $msg->print;
            Msg::n();
        }
    }
    return;
}

sub get_tunable_info {
    my ($cpic, $tunable) =@_;
    return $cpic->{tunables}{$tunable};
}

sub validate_tunable_sys {
    my ($cpic,$sys,$tunable,$value) =@_;
    my ($msg,$name,$defobj,$sysi,$prodi,$tunable_info,$retval);
    $tunable_info = $cpic->get_tunable_info($tunable);
    $sysi = $sys->{sys};
    $prodi = $cpic->{prod};
    $name = $tunable_info->{name};
    $defobj = $tunable_info->{define_object};

    ($retval,$msg) = $defobj->validate_tunable_sys($sys,$tunable,$value);

    # 0 valid
    # 1 Padv is not supported
    # 2 Prod(Ver) is not installed
    # 3 value is invalid
    # 4 Release not supported

    if ($retval) {
        Msg::n();
        $msg->print;
    }
    return $retval;
}

sub set_tunable_sys {
    my ($cpic,$sys,$tunable,$value) =@_;
    my ($msg,$name,$sysi,$tunable_info,$defobj,$retval, $out);
    $sysi = $sys->{sys};
    $tunable_info = $cpic->get_tunable_info($tunable);
    $name = $tunable_info->{name};
    $msg = Msg::new("Setting $tunable to $value on $sysi", 30, 1053, "$tunable", "$value", "$sysi");
    $msg->left;
    $defobj = $tunable_info->{define_object};
    ($retval,$out) = $defobj->set_tunable_sys($sys,$tunable,$value);

    # 0 Failed to set
    # 1 Done
    # 2 Not changed

    if ( $retval ) {
        Msg::right_done();
        if ($tunable_info->{reboot} && $retval == 1) {
            push @{$sys->{requirereboottunables}}, $tunable;
        }
    } else {
        Msg::right_failed();
        Msg::log($out);
        push @{$cpic->{tunable_failures}}, [$sys, $tunable, $value, $out];
    }
    return $retval;
}

sub all_tunables {
    my ($cpic) = @_;
    my ($tunables, $prod, $rel);

    if ($cpic->{tunables}) {
        return $cpic->{tunables};
    }

    $tunables = {};
    $rel = $cpic->rel;

    # release tunables
    if ($rel->get_tunables) {
        for my $tunable (@{$rel->get_tunables}) {
            $tunables->{$tunable->{name}} = $tunable;
        }
    }

    # each product's tunables
    for my $prodi (@{$rel->{prods}}) {
        $prod = $cpic->prod($prodi);
        if ($prod->get_tunables) {
            for my $tunable (@{$prod->get_tunables}) {
                $tunables->{$tunable->{name}} = $tunable;
            }
        }
    }

    $cpic->{tunables} = $tunables;
    return $tunables;
}

sub supported_tunables {
    my ($cpic) = @_;
    my ($tunables, $prod, $rel);

    if ($cpic->{supported_tunables}) {
        return $cpic->{supported_tunables};
    }

    $tunables = {};
    $rel = $cpic->rel;

    # release tunables
    if ($rel->get_tunables) {
        for my $tunable (@{$rel->get_tunables}) {
            $tunables->{$tunable->{name}} = $tunable;
        }
    }

    # each product's tunables
    if ($cpic->{prod}) {
        $prod = $cpic->prod;
        if ($prod->get_supported_tunables) {
            for my $tunable (@{$prod->get_supported_tunables}) {
                $tunables->{$tunable->{name}} = $tunable;
            }
        }
    }

    $cpic->{supported_tunables} = $tunables;
    return $tunables;
}

sub printmsg_skip_setting_tunables {
    my ($cpic,$reason,$prod) = @_;
    my ($cfg,$msg);
    $cfg = Obj::cfg();

    if (Cfg::opt(qw(tunablesfile)) || (Cfg::opt(qw(responsefile)) && $cfg->{tunable}) ) {
        if ( !$reason || $reason =~ /reboot/mxi ) {
            $msg = Msg::new("Skip setting tunable parameters because one or more systems require reboot prior to configuration", 30, 1054);
        } elsif ($reason =~ /disableconfig/mxi ) {
            $msg = Msg::new("Skip setting tunable parameters because configuration of $prod->{prod} is disabled", 30, 1055, "$prod->{prod}");
        } elsif ($reason =~ /skipconfig/mxi) {
            $msg = Msg::new("Skip setting tunable parameters because $prod->{prod} is not configured", 30, 1056, "$prod->{prod}");
        } else {
            $msg = Msg::new("Skip setting tunable parameters", 30, 1048);
        }
        $msg->print;
        Msg::n();
    }
    return;
}

1;
