#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package CPIC;
use strict;

sub options {
    my $cpic=shift;
    my $rel=$cpic->rel;
    if (!Cfg::opt(qw(hotfixupgrade patchupgrade upgrade)) && $cpic->{prod}) {
        $cpic->prod->cli_initiation_questions();
        $cpic->prod->web_initiation_questions();
        $rel->cli_prod_option();
        $rel->web_prod_option();
    }
    $rel->adjust_options();
    $cpic->select_install_pkgs() if (Cfg::opt('install'));
    $rel->cli_language_menu();
    return '';
}

# on the system based on user selection [minimal, rec, or all]. No upsell.
# Lists required space for each install pkgset.
sub select_install_pkgs {
    my ($cpic,$cfg,$rel,$prod,@space,$pkgspace,$prodspace,$recspace,$minspace,$pkg,$ret,$web,$ayn);
    $cfg=Obj::cfg();
    $web=Obj::web();
    $cpic=shift;
    $rel=$cpic->rel;
    $prod=$cpic->prod;
    if ($prod->{lp}) {
        Cfg::set_opt('installallpkgs',1);
        return 0;
    }

    # skip this menu if allpkgs,recpkgs,minpkgs are the same or if recpkgs or minpkgs are undefined
    if ((($#{$prod->allpkgs}==$#{$prod->recpkgs}) && ($#{$prod->recpkgs}==$#{$prod->minpkgs})) ||
         ($#{$prod->recpkgs}<0) || ($#{$prod->minpkgs}<0)) {
        Msg::log("$prod->{prod} does not offer all/rec/min package option");
        Cfg::set_opt('installallpkgs',1);
        return 0;
    }

    if (Cfg::opt('installminpkgs')&&(!$web->{browserid}) && (!$prod->{not_prompt_minpkgs_warning})) {
        $ayn=$rel->minpkgs_nag();
        Cfg::unset_opt('installminpkgs') if ($ayn eq 'N');
    }

    # Determine the space requirement for the product
    for my $pkgi (@{$prod->allpkgs}) {
        # not sure what we do here for Common?
        $pkg=$cpic->pkg($pkgi);
        $pkg->{space}||=[100,100,100,100];
        @space=@{$pkg->{space}};
        $pkgspace=$space[0]+$space[1]+$space[2]+$space[3];
        $minspace+=$pkgspace if (EDRu::inarr($pkgi,@{$prod->minpkgs}));
        $recspace+=$pkgspace if (EDRu::inarr($pkgi,@{$prod->recpkgs}));
        $prodspace+=$pkgspace;
    }

    $ret=$cpic->cli_select_install_menu($minspace,$recspace,$prodspace);
    Cfg::set_opt('installminpkgs',1) if ($ret==1);
    Cfg::set_opt('installrecpkgs',1) if ($ret==2);
    Cfg::set_opt('installallpkgs',1) if ($ret==3);
    if (Obj::webui()) {
        if ($prod->{silent_installconfig}) {
            Cfg::set_opt('installrecpkgs',1);
        } else {
            $web->web_script_form('selectpkgs', $minspace, $recspace, $prodspace);
        }
    }
    Cfg::opt('installminpkgs') ? Msg::log('Minimum Package Set selected for installation') :
        Cfg::opt('installrecpkgs') ? Msg::log('Recommended Package Set selected for installation') :
            Cfg::opt('installallpkgs') ? Msg::log('All Package Set selected for installation') : 1;

    Msg::n();
    return 0;
}

# Menu for package set selection
sub cli_select_install_menu {
    my ($cpic,$minspace,$recspace,$prodspace) = @_;
    my ($m,$all,$edr,$ret,$ps,$ayn,$pdfrs,$help,$cs,$ms,$display,$ret_default,$rec,@menu,$min);
    return 1 if ((Cfg::opt(qw(installminpkgs))) || ($cpic->rel->{installminpkgs}) ||
                                                   ($cpic->prod->{installminpkgs}));
    return 3 if ((Cfg::opt(qw(installallpkgs))) || ($cpic->rel->{installallpkgs}) ||
                                                   ($cpic->prod->{installallpkgs}));
    return 2 if ((Cfg::opt(qw(installrecpkgs))) || ($cpic->rel->{installrecpkgs}) ||
                                                   ($cpic->prod->{installrecpkgs}));
    $edr=Obj::edr();

    $pdfrs=Msg::get('pdfrs');
    $ms=int($minspace/1024);
    $cs=int($recspace/1024);
    $ps=int($prodspace/1024);

    $min=Msg::new("Install minimal required $pdfrs - $ms MB required", 30, 1290, "$pdfrs", "$ms");
    $rec=Msg::new("Install recommended $pdfrs - $cs MB required", 30, 1291, "$pdfrs", "$cs");
    $all=Msg::new("Install all $pdfrs - $ps MB required", 30, 1292, "$pdfrs", "$ps");
    $display=Msg::new("Display $pdfrs to be installed for each option", 30, 1293, "$pdfrs");
    $help=Msg::new("Users can select a minimal set of $pdfrs required for the basic product functionality, a recommended set of $pdfrs that constitute some advanced functionality, or all of the $pdfrs offered by the product. When the set of recommended $pdfrs is equal to all the $pdfrs, only recommended will be offered in the menu.", 30, 1294, "$pdfrs", "$pdfrs", "$pdfrs", "$pdfrs", "$pdfrs");
    $m=Msg::new("Select the $pdfrs to be installed on all systems?", 30, 1295, "$pdfrs");
    # code not to show all packages if rec and all are same
    ($recspace==$prodspace) ?  push(@menu,$min->{msg},$rec->{msg},$display->{msg}) && ($ret_default = 3) :
                               push(@menu,$min->{msg},$rec->{msg},$all->{msg},$display->{msg}) && ($ret_default = 4);
    while(1) {
        Msg::title();
        $ret=$m->menu(\@menu,2,$help);
        if ($ret==$ret_default) {
            $cpic->display_pkgs();
            Msg::prtc();
            next;
        } elsif ($ret == 1) {
            $ayn=$cpic->rel->minpkgs_nag();
            next if ($ayn eq 'N');
        }
        return $ret;
    }
    return;
}

1;
