#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package CPIC;
use strict;

sub set_pkgs {
    my $cpic=shift;
    my $prod=$cpic->prod;

    Cfg::set_opt('silent',1) if (($prod->{silent_installconfig}) && (Cfg::opt('install')));
    $cpic->set_uninstall_language_pkgs() if (Cfg::opt(qw(install patchupgrade hotfixupgrade uninstall upgrade)));
    $cpic->set_uninstall_pkgs() if (Cfg::opt(qw(install patchupgrade hotfixupgrade uninstall rollback upgrade)));
    $cpic->set_install_pkgs() if (Cfg::opt(qw(precheck install patchupgrade hotfixupgrade upgrade)));
    $cpic->analyse_and_display_pkgs();

    return '';
}

sub set_install_pkgs {
    my $cpic=shift;
    my ($pkg,$installpkgs,$prod,$pkgs,$plat,$sys0,$prodvers,$rel);
    # for makeresponsefile upgrade where sometimes fake systems are specified and product cannot be detected
    return '' unless ($cpic->{prod});
    # some attribute needs to be empty before setting install packages in rolling upgrade phase II.
    $cpic->{installpatchpkgs}=[];
    $cpic->{installpatchpkgslast}=[];
    $cpic->{installpatcheslast}=[];
    $rel=$cpic->rel;
    for my $sys (@{$cpic->{systems}}) {
        $prod=$sys->prod($cpic->{prod});
        $prodvers=$prod->version_sys($sys);
        $sys->{upgrade_prior_pkgset}=1 if (EDRu::compvers($prodvers,$rel->{pkgset_version},2) != 2);
        if ($prod->{lp}) {
            $installpkgs=$prod->getlangpkg();
        } else {
            if (Cfg::opt('upgrade')) {
                # Upgrading from any release All package set is used to check against
                $installpkgs=$prod->allpkgs;
            } else {
                $installpkgs=$prod->get_pkgs;
            }
        }
        # add dependency (co/pre-requisite) on each pkg only on HPUX platform
        if (EDRu::plat($cpic->{padv}) =~ /^HPUX/m) {
            $plat=EDRu::plat($cpic->{padv});
            $plat='Padv::'.$plat;
            $sys0=$cpic->{systems}->[0];
            $plat->pkg_deps_sys($sys0,@{$installpkgs});
        }
        for my $pkgi (@{$installpkgs}) {
            $pkg=$sys->pkg($pkgi);
            if ($prod->{lp}) {
                # install language packages for all installed packages
                $pkg->languagepkgs_sys($sys);
            } else {
                # determine whether to install/upgrade pkg and install patches
                $pkg->set_install_sys($sys, $prod);
            }
        }
        push @{$cpic->{installpkgs}},@{$installpkgs};

        # To upgrade those obsoleted pkgs which need refresh to new version when they are present
        $pkgs=$prod->obsoleted_but_need_refresh_when_upgrade_pkgs;
        if (Cfg::opt('upgrade') && @{$pkgs}) {
            for my $pkgi (@{$pkgs}) {
                $pkg=Pkg->new_pkg($pkgi, $sys->{padv});
                $pkg=Pkg->new_pkg($pkgi, $cpic->{padv});
                if ($pkg->version_sys($sys)) {
                    $pkg->set_install_sys($sys, $prod);
                }
            }
            push @{$cpic->{installpkgs}},@{$pkgs};
        }
    }
    $cpic->{installpkgs} = EDRu::arruniq(@{$cpic->{installpkgs}});
    $rel->adjust_pkgs();
    $cpic->{installpkgs} = $rel->adjust_pkg_sequence($cpic->{installpkgs});
    return '';
}

# @{$cpic->{uninstallbundles}}: to record the sequence of uninstall bundles (For HPUX).
# @{$cpic->{uninstallpkgs}}: to record the sequence of uninstall pkgs.
# @upkgs:  to record the pkgs really need be uninstalled.
#
# For fresh install
# @{$cpic->{uninstallpkgs}}:   L10Npkgs + obsoletedpkgs
# @upkgs:                      obsoletedpkgs
#
# For patch upgrade. need add reverse($prod->[pkgs}) in sequence list since we need remove some product pkgs with older version.
# @{$cpic->{uninstallpkgs}}:   obsoletedpkgs + reverse($prod->allpkgs)
# @upkgs:                      obsoletedpkgs
#
# For full upgrade, need add obsoleted_but_need_refresh_when_upgrade_pkgs in sequence list since we may need upgrade them.
# @{$cpic->{uninstallpkgs}}:   L10Npkgs + obsoletedpkgs + $prod->{obsoleted_but_need_refresh_when_upgrade_pkgs} + reverse($prod->allpkgs)
# @upkgs:                      obsoletedpkgs
#
# For uninstall
# @{$cpic->{uninstallpkgs}}:   L10Npkgs + obsoletedpkgs + reverse($prod->allpkgs)
# @upkgs:                      obsoletedpkgs + reverse($prod->allpkgs)
sub set_uninstall_pkgs {
    my $cpic=shift;
    my ($prod,$prodpkgs,$obsoletedpkgs,$pkgs,$pkg,$rel);
    my (@upkgs,$upkgs_expanded,@upkgs_seq,$upkgs_seq_expanded);
    my ($msg, $msg_flag);

    #e3490263:define a flag to recored if message has been printed. 
    $msg_flag = 1;

    # for makeresponsefile upgrade where sometimes fake systems are specified and product cannot be detected
    return '' unless ($cpic->{prod});
    $rel=$cpic->rel;

    $cpic->{uninstallbundles}=$cpic->obsoletedbundles();
    $obsoletedpkgs=$cpic->obsoletedpkgs();

    for my $sys (@{$cpic->{systems}}) {
        $prod=$sys->prod;
        $prodpkgs=$prod->allpkgs;
        if ($rel->{inplaceupgrade}) {
            for my $pkgi(@{$prodpkgs}) {
                $pkg=$cpic->pkg($pkgi,$cpic->{padv});
                if (!defined $pkg->{donotrmonupgrade}) {
                    $pkg->{donotrmonupgrade}=1;
                }
            }
        }

        @upkgs_seq=@upkgs=@{$sys->{obsoletedpkgs}};
        if (Cfg::opt('install')) {
            # if upsell install, need remove pkgs that should not be in the new product.
            if($sys->{can_upsell} == 1 && ${$sys->{upgradeprod}}[0] ne '' && $msg_flag){
               $msg=Msg::new("You are trying to install over an existing product, no package will be removed for this installation.", 30, 1296);
               $msg->printn;
               Msg::prtc();
               $msg_flag = 0;
            }
        } elsif (Cfg::opt('upgrade')) {

            # if upgrade, need remove all obsoleted pkgs first.
            $pkgs=$prod->obsoleted_but_need_refresh_when_upgrade_pkgs;
            push(@upkgs_seq, @{$pkgs}) if (defined $pkgs);
            push(@upkgs_seq, reverse(@{$prodpkgs}));

        } elsif (Cfg::opt(qw(patchupgrade hotfixupgrade))) {

            # if patchupgrade, need check all reversed product pkgs.
            push(@upkgs_seq, reverse(@{$prodpkgs}));
            #$rel->determine_rmonpatchupgrade_pkgs($cpic->{systems},$prodpkgs);

        } elsif (Cfg::opt('uninstall')) {

            # if uninstall, need remove all obsoleted pkgs and reversed product pkgs.
            push(@upkgs, reverse(@{$prodpkgs}));
            push(@upkgs_seq, reverse(@{$prodpkgs}));
        }  elsif (Cfg::opt('rollback')) {
            # if rollback, remove all patches associated with the release
            for my $pkgi(reverse(@{$prodpkgs})) {
                $pkg=$cpic->pkg($pkgi,$cpic->{padv});
                $pkg->set_uninstall_sys($sys);
                $rel->new_pkg_init($pkg);
                for my $patchi(@{$pkg->patches_allvers($rel->{currentrelease})}) {
                    push(@{$cpic->{uninstallpatches}},$patchi)
                        unless EDRu::inarr($patchi, @{$cpic->{uninstallpatches}});;
                }
            }
        }

        $upkgs_expanded=$cpic->expand_uninstallpkgs_sys($sys, \@upkgs);
        for my $pkgi (@{$upkgs_expanded}) {
            $pkg=Pkg->new_pkg($pkgi, $sys->{padv});
            $pkg=Pkg->new_pkg($pkgi, $cpic->{padv});
            if (EDRu::inarr($pkgi,@{$sys->{obsoleted_thirdparty_pkgs}})) {
                $pkg->{force_uninstall}=1;
            }
            $pkg->set_uninstall_sys($sys,$upkgs_expanded);
        }

        $upkgs_seq_expanded=$cpic->expand_uninstallpkgs_sys($sys, \@upkgs_seq);
        push @{$cpic->{uninstallpkgs}}, @{$upkgs_seq_expanded};
    }

    $cpic->{uninstallpkgs} = EDRu::arruniq(@{$cpic->{uninstallpkgs}});
    return '';
}

sub set_uninstall_language_pkgs {
    my $cpic=shift;
    my($rel,@upkgs,$pkg);
    for my $sys (@{$cpic->{systems}}) {
        $rel=$sys->rel;
        # do not uninstall lp pkgs during rp install.
        # uninstall lp pkgs if base release has been upgraded to rp release and then uninstalled.
        next if (($rel->{type}=~/^M/) && (!Cfg::opt('uninstall')));
        if($rel->{langpkgs}){
            @upkgs=@{$rel->{langpkgs}};
        }else{
            next;
        }
        next if ($#upkgs < 0);

        push @{$cpic->{uninstallpkgs}},@{$rel->{langpkgs}};

        # do not uninstall language packages during install if product is installed
        next if (Cfg::opt('install') && (!$cpic->{lp}) && ($sys->{upgradeprod}[0]));
        for my $pkgi (@upkgs) {
            $pkg=Pkg->new_pkg($pkgi, $sys->{padv});
            $pkg=Pkg->new_pkg($pkgi, $cpic->{padv});
            $pkg->set_uninstall_sys($sys);
        }
    }
    return '';
}

sub expand_uninstallpkgs_sys {
    my ($cpic,$sys,$upkgs)=@_;
    my ($pkg,@upkgs_expanded,$all_instances);

    @upkgs_expanded=();
    for my $pkgi (@{$upkgs}) {
        # get all other instances of this pkg and its pkgs with previous names
        $pkg=Pkg->new_pkg($pkgi, $sys->{padv});
        $pkg=Pkg->new_pkg($pkgi, $cpic->{padv});

        $all_instances=$sys->padv->pkg_allinstances_sys($sys,$pkg);
        push @upkgs_expanded,@{$all_instances};

        for my $pkgname (@{$pkg->{previouspkgnames}}) {
            $pkg=Pkg->new_pkg($pkgname, $sys->{padv});
            $pkg=Pkg->new_pkg($pkgname, $cpic->{padv});

            $all_instances=$sys->padv->pkg_allinstances_sys($sys,$pkg);
            push @upkgs_expanded,@{$all_instances};
        }
    }
    return EDRu::arruniq(@upkgs_expanded);
}

sub analyse_and_display_pkgs {
    my ($cpic) = @_;
    my ($patch,$pkg);

    return if (!defined $cpic->{uninstallpkgs});

    # Save the sequence of uninstallpkgs since it would be merged with systems' uninstallpkgs later.
    $cpic->{uninstallpkgs_seq} = [ @{$cpic->{uninstallpkgs}} ];

    for my $sys (@{$cpic->{systems}}) {
        for my $pkgi (@{$sys->{installpkgs}}) {
            $pkg=$sys->pkg($pkgi);
            if ($sys->padv->is_pkg_require_reboot_sys($sys,$pkg,0)) {
                $sys->set_value('requirerebootpkgspatches', 'push', $pkg->{pkg});
            }
        }
        for my $patchi (@{$sys->{installpatches}}) {
            $patch=$sys->patch($patchi);
            if ($sys->padv->is_pkg_require_reboot_sys($sys,$patch,0)) {
                $sys->set_value('requirerebootpkgspatches', 'push', $patch->{patchname});
            }
        }
        for my $pkgi (@{$sys->{uninstallpkgs}}) {
            $pkg=$sys->pkg($pkgi);
            if ($sys->padv->is_pkg_require_reboot_sys($sys,$pkg,1)) {
                $sys->set_value('requirerebootpkgspatches', 'push', $pkg->{pkg});
            }
        }
        for my $patchi (@{$sys->{uninstallpatches}}) {
            $patch=$sys->patch($patchi);
            if ($sys->padv->is_pkg_require_reboot_sys($sys,$patch,1)) {
                $sys->set_value('requirerebootpkgspatches', 'push', $patch->{patchname});
            }
        }
    }

    # Call of fix for Etrack 3512592 & 3512210, part 2
    $cpic->patch_release_rollback_sanity_check();

    if (Cfg::opt(qw(install patchupgrade hotfixupgrade upgrade uninstall rollback))) {
        if (Obj::webui()) {
            $cpic->web_display_uninstpkgs();
        } else {
            $cpic->cli_display_uninstpkgs();
        }
    }
    if (Cfg::opt(qw(install patchupgrade hotfixupgrade upgrade))) {
        if (Obj::webui()) {
            $cpic->web_display_instpkgs();
        } else {
            $cpic->cli_display_instpkgs();
        }
    }
    return;
}

# Fix for Etrack 3512592 & 3512210, part 2
# The part 1 of 3512592 is in Pkg::set_uninstall_patches_sys 
# The part 1 of 3512210 is in EDR::Padv::SunOS::patches_sys
#
# For MR/HF, we should stop further actions for rollback if
# 1. no patches defined for the target system
# 2. defined patches are not the latest installed
#
sub patch_release_rollback_sanity_check {
    my $cpic=shift;
    my ($errors, $msg, $rel, @patchreqs, $patch, @reqs, $dep);

    $rel=$cpic->rel;
    return unless (Cfg::opt("rollback") && ($rel->{type} =~ /M|H/));
    $errors=0;
    for my $sys (@{$cpic->{systems}}) {
        next if ($sys->{padv} =~ /Sol11/i);
        next if (EDRu::plat($sys->{padv}) =~ /Linux/i);
        unless (@{$sys->{uninstallpatches}}) {
            $msg=Msg::new("The rollback patch list is empty for $sys->{sys}. No further actions recommended.", 30, 1297, "$sys->{sys}");
            $sys->push_error($msg);
            $errors++;
            next;
        }
        my @patchreqs=(sort keys %{$sys->{patchreqs}});
        for my $patchi (@{$sys->{uninstallpatches}}) {
            $patch=$sys->patch($patchi);
            next unless (EDRu::inarr($patch->{patchid},@patchreqs));
            my @reqs=();
            for my $req (@{$sys->{patchreqs}{$patch->{patchid}}}) {
                my $req_name=join "_", (split /-/, $req);
                next if (EDRu::inarr($req_name, @{$sys->{uninstallpatches}}));
                push (@reqs, $req) unless (EDRu::inarr($req,@reqs));
            }
            if (@reqs) {
                $dep=join ", ", @reqs;
                $msg=Msg::new("Patch $patch->{patchid} is required by $dep on $sys->{sys}. Remove them first before retrying $rel->{currentrelease} Rollback.", 30, 1298, "$patch->{patchid}", "$dep", "$sys->{sys}", "$rel->{currentrelease}");
                $sys->push_error($msg);
                $errors++;
            }
        }
    }
    if ($errors) {
        $msg=Msg::new("The following errors were discovered on the systems:", 30, 1299);
        $msg->print;
        $msg->add_summary(1);
        $msg->addError($msg->{msg});

        for my $sys (@{$cpic->{systems}}) {
            for my $errmsg ((@{$sys->{errors}})) {
                Msg::print($errmsg);
                Msg::add_summary($errmsg);
                $msg->addError($errmsg);
            }
        }
        Msg::n();
        EDR::exit_exitfile(EDR->EXITPROCESSFAILURE);
    }

    return '';
}

sub web_display_uninstpkgs {
    my $cpic=shift;
    my ($sys0,$pdfrs,$prod,$prodname,$ref_allsystem_pkgs,$ref_remaining,$msg);
    my $web=Obj::web();

    $prod=$cpic->prod;
    $sys0=$cpic->{systems}->[0];
    $pdfrs=Msg::get('pdfrs');
    $prodname=$prod->{name};
    $prodname=$prod->{upgrade_from} if (defined $prod->{upgrade_from} && Cfg::opt(qw(upgrade patchupgrade)));
    for my $pkgi (@{$cpic->{uninstallpkgs}}) {
        push @{$ref_allsystem_pkgs},$pkgi;
        for my $sys (@{$cpic->{systems}}) {
            $ref_allsystem_pkgs=EDRu::arrdel($ref_allsystem_pkgs,$pkgi)
                  unless (EDRu::inarr($pkgi,@{$sys->{uninstallpkgs}}));
        }
    }
    if ($#$ref_allsystem_pkgs >= 0) {
        if (Cfg::opt(qw(upgrade_kernelpkgs))) {
            my $syslist = join(' ',@{$web->{rup1list}});
            $msg .= Msg::new("<B>The following $prodname $pdfrs will be uninstalled on $syslist:</B>\\n", 30, 1300, "$prodname", "$pdfrs", "$syslist")->{msg};
            $msg .= $web->list_packages($sys0, 1, @{$ref_allsystem_pkgs});
        } else {
            $msg .= Msg::new("<B>The following $prodname $pdfrs will be uninstalled on all systems:</B>\\n", 30, 1301, "$prodname", "$pdfrs")->{msg};
            $msg .= $web->list_packages($sys0, 1, @{$ref_allsystem_pkgs});
        }
    }

    for my $sys (@{$cpic->{systems}}) {
        $ref_remaining=EDRu::arrdel($sys->{uninstallpkgs},@{$ref_allsystem_pkgs});
        if ($#$ref_remaining >= 0) {
            $msg .= Msg::new("<B>The following $prodname $pdfrs will be uninstalled on $sys->{sys}:</B>\\n", 30, 1300, "$prodname", "$pdfrs", "$sys->{sys}")->{msg};
            $msg .= $web->list_packages($sys, 1, @{$ref_remaining});
        }
    }
    $web->web_script_form('alert', $msg) if ($msg);

    return '';
}

sub web_display_instpkgs {
    my $cpic=shift;
    my ($sys0,$pdfrs,$prod,$prodname,$ref_allsystem_pkgs,$ref_allsystem_patches,$ref_remaining,$msg,$remaining);
    my $web=Obj::web();

    $prod=$cpic->prod;
    $sys0=$cpic->{systems}->[0];
    $pdfrs=Msg::get('pdfrs');
    $prodname=$prod->{name};
    for my $pkgi (@{$cpic->{installpkgs}}) {
        push @{$ref_allsystem_pkgs},$pkgi;
        for my $sys (@{$cpic->{systems}}) {
            $ref_allsystem_pkgs=EDRu::arrdel($ref_allsystem_pkgs,$pkgi)
                  unless (EDRu::inarr($pkgi,@{$sys->{installpkgs}}));
        }
    }
    for my $patchi (@{$cpic->{installpatches}}) {
        push @{$ref_allsystem_patches},$patchi;
        for my $sys (@{$cpic->{systems}}) {
            $ref_allsystem_patches=EDRu::arrdel($ref_allsystem_pkgs,$patchi)
                  unless (EDRu::inarr($patchi,@{$sys->{installpatches}}));
        }
    }
    $remaining=0;
    if ($#$ref_allsystem_pkgs >= 0) {
        if (Cfg::opt('upgrade')) {
            if (Cfg::opt(qw(upgrade_kernelpkgs))) {
                my $syslist = join(' ',@{$web->{rup1list}});
                $msg .= Msg::new("<B>The following $prodname $pdfrs will be installed or upgraded on $syslist:</B>\\n", 30, 1302, "$prodname", "$pdfrs", "$syslist")->{msg};
            } else {
                $msg .= Msg::new("<B>The following $prodname $pdfrs will be installed or upgraded on all systems:</B>\\n", 30, 1303, "$prodname", "$pdfrs")->{msg};
            }
        } else {
            $msg .= Msg::new("<B>The following $prodname $pdfrs will be installed on all systems:</B>\\n", 30, 1304, "$prodname", "$pdfrs")->{msg};
        }
        $msg .= $web->list_packages($sys0, 0, @{$ref_allsystem_pkgs});
    }
    if ($#$ref_allsystem_patches >= 0) {
        $msg .= Msg::new("<B>The following $prodname patches will be installed on all systems:</B>\\n", 30, 1305, "$prodname")->{msg};
        $msg .= $web->list_patches(@{$ref_allsystem_patches});
    }


    for my $sys (@{$cpic->{systems}}) {
        $ref_remaining=EDRu::arrdel($sys->{installpkgs},@{$ref_allsystem_pkgs});
        if ($#$ref_remaining >= 0) {
            $remaining=1;
            if (Cfg::opt('upgrade')) {
                $msg .= Msg::new("<B>The following $prodname $pdfrs will be installed or upgraded on $sys->{sys}:</B>\\n", 30, 1302, "$prodname", "$pdfrs", "$sys->{sys}")->{msg};
            } else {
                $msg .= Msg::new("<B>The following $prodname $pdfrs will be installed on $sys->{sys}:</B>\\n", 30, 1306, "$prodname", "$pdfrs", "$sys->{sys}")->{msg};
            }
            $msg .= $web->list_packages($sys, 0, @{$ref_remaining});
       }

       $ref_remaining=EDRu::arrdel($sys->{installpatches},@{$ref_allsystem_patches});
       if ($#$ref_remaining >= 0) {
           $remaining=1;
           if (Cfg::opt('upgrade')) {
               $msg .= Msg::new("<B>The following $prodname patches will be installed or upgraded on $sys->{sys}:</B>\\n", 30, 1307, "$prodname", "$sys->{sys}")->{msg};
           } else {
               $msg .= Msg::new("<B>The following $prodname patches will be installed on $sys->{sys}:</B>\\n", 30, 1308, "$prodname", "$sys->{sys}")->{msg};
           }
           $msg .= $web->list_patches(@{$ref_remaining});
       }
    }
    if (($#$ref_allsystem_pkgs < 0) && ($#$ref_allsystem_patches < 0)) {
        # product is already installed, do not continue if not resumed from previous installer process.
        if (!$remaining && Cfg::opt("responsefile") !~ /exitfile/ && !Cfg::opt("makeresponsefile")) {
            $cpic->completion();
        }
    }
    $web->web_script_form('alert', $msg) if($msg);

    return '';
}

# Display uninstall pkg name and description
sub cli_display_uninstpkgs {
    my $cpic=shift;
    my ($sys0,$n,$pdfrs,$prod,$prodname,$ref_allsystem_pkgs,$ref_allsystem_patches,$ref_remaining,$msg,$title_shown);

    $prod=$cpic->prod;
    $n=0;
    $sys0=$cpic->{systems}->[0];
    $pdfrs=Msg::get('pdfrs');
    $prodname=$prod->{name};
    $prodname=$prod->{upgrade_from} if (defined $prod->{upgrade_from} && Cfg::opt(qw(upgrade patchupgrade)));
    for my $pkgi (@{$cpic->{uninstallpkgs}}) {
        push @{$ref_allsystem_pkgs},$pkgi;
        for my $sys (@{$cpic->{systems}}) {
            $ref_allsystem_pkgs=EDRu::arrdel($ref_allsystem_pkgs,$pkgi)
                  unless (EDRu::inarr($pkgi,@{$sys->{uninstallpkgs}}));
        }
    }
    for my $patchi (@{$cpic->{uninstallpatches}}) {
        push(@{$ref_allsystem_patches},$patchi);
        for my $sys (@{$cpic->{systems}}) {
            $ref_allsystem_patches=EDRu::arrdel($ref_allsystem_patches,$patchi)
                unless (EDRu::inarr($patchi,@{$sys->{uninstallpatches}}));
        }
    }

    $title_shown=0;
    if ($#$ref_allsystem_pkgs >= 0) {
        Msg::title();
        $title_shown=1;
        $msg=Msg::new("The following $prodname $pdfrs will be uninstalled on all systems:", 30, 1309, "$prodname", "$pdfrs");
        $msg->printn;
        $n=$cpic->list_packages($sys0,$n,$ref_allsystem_pkgs,1);
    }
    unless ($#$ref_allsystem_patches < 0) {
        Msg::title();
        $title_shown=1;
        $msg=Msg::new("The following $prodname patches will be uninstalled on all systems:", 30, 1310, "$prodname");
        $msg->printn;
        $n=$cpic->list_patches($sys0,$n,$ref_allsystem_patches);
    }

    for my $sys (@{$cpic->{systems}}) {
        $ref_remaining=EDRu::arrdel($sys->{uninstallpkgs},@{$ref_allsystem_pkgs});
        if ($#$ref_remaining >= 0) {
            if (!$title_shown) {
                Msg::title();
                $title_shown=1;
            }
            Msg::n();
            $msg=Msg::new("The following $prodname $pdfrs will be uninstalled on $sys->{sys}:", 30, 1311, "$prodname", "$pdfrs", "$sys->{sys}");
            $msg->printn;
            $n=$cpic->list_packages($sys,$n,$ref_remaining,1) if (@{$ref_remaining});
        }
    
        $ref_remaining=EDRu::arrdel($sys->{uninstallpatches},@{$ref_allsystem_patches});
        unless ($#$ref_remaining < 0) {
            if (!$title_shown) {
                Msg::title();
                $title_shown=1;
            }
            Msg::n();
            $msg=Msg::new("The following $prodname patches will be uninstalled on $sys->{sys}:", 30, 1312, "$prodname", "$sys->{sys}");
            $msg->printn;
            $n=$cpic->list_patches($sys,$n,$ref_remaining) if (@{$ref_remaining});
        }
    }

    if ($title_shown) {
        Msg::prtc();
        Msg::n();
    }
    return '';
}

# Display pkg name and description
sub cli_display_instpkgs {
    my $cpic=shift;
    my (@rebootsys,$sys0,$string_rebootsys,$ref_allsystem_pkgs,$ref_allsystem_patches,$ref_remaining,$sys,$cfg,$prod,$prodname,$pdfrs,$msg,$pkg,$pkgi,$patchi,$patch,$n,$rel,$remaining,$installscript);
    $prod=$cpic->prod;
    $rel=$cpic->rel;
    $n=0;
    $cfg=Obj::cfg();
    $sys0=$cpic->{systems}->[0];
    $pdfrs=Msg::get('pdfrs');
    $prodname=$prod->{name};
    for my $pkgi (@{$cpic->{installpkgs}}) {
        push @{$ref_allsystem_pkgs},$pkgi;
        for my $sys (@{$cpic->{systems}}) {
            $ref_allsystem_pkgs=EDRu::arrdel($ref_allsystem_pkgs,$pkgi)
                  unless (EDRu::inarr($pkgi,@{$sys->{installpkgs}}));
        }
    }
    for my $patchi (@{$cpic->{installpatches}}) {
        push @{$ref_allsystem_patches},$patchi;
        for my $sys (@{$cpic->{systems}}) {
            $ref_allsystem_patches=EDRu::arrdel($ref_allsystem_patches,$patchi)
                unless (EDRu::inarr($patchi,@{$sys->{installpatches}}));
        }
    }
    Msg::title();
    if ($#$ref_allsystem_pkgs >= 0) {
        if (Cfg::opt('upgrade')) {
             $msg=Msg::new("The following $prodname $pdfrs will be installed or upgraded on all systems:", 30, 1313, "$prodname", "$pdfrs");
        } else {
             $msg=Msg::new("The following $prodname $pdfrs will be installed on all systems:", 30, 1314, "$prodname", "$pdfrs");
        }
        $msg->printn;
        $n=$cpic->list_packages($sys0,$n,$ref_allsystem_pkgs);
    }

    if ($#$ref_allsystem_patches >= 0) {
        Msg::n();
        $msg=Msg::new("The following $prodname patches will be installed on all systems:\n", 30, 1315, "$prodname");
        $msg->print;
        $n=$cpic->list_patches($sys0,$n,$ref_allsystem_patches);
    }

    if (($#$ref_allsystem_pkgs < 0) && ($#$ref_allsystem_patches < 0)) {
        $msg=Msg::new("No $pdfrs or patches will be installed on all systems\n", 30, 1316, "$pdfrs");
        $msg->print;
    }

    $remaining=0;
    for my $sys (@{$cpic->{systems}}) {
        $ref_remaining=EDRu::arrdel($sys->{installpkgs},@{$ref_allsystem_pkgs});
        if ($#$ref_remaining >= 0) {
            $remaining=1;
            Msg::n();
            if (Cfg::opt('upgrade')) {
                $msg=Msg::new("The following $prodname $pdfrs will be installed or upgraded on $sys->{sys}:", 30, 1317, "$prodname", "$pdfrs", "$sys->{sys}");
            } else {
                $msg=Msg::new("The following $prodname $pdfrs will be installed on $sys->{sys}:", 30, 1318, "$prodname", "$pdfrs", "$sys->{sys}");
            }
            $msg->printn;
            $n=$cpic->list_packages($sys,$n,$ref_remaining) if (@{$ref_remaining});
        }
        $ref_remaining=EDRu::arrdel($sys->{installpatches},@{$ref_allsystem_patches});
        if ($#$ref_remaining >= 0) {
            $remaining=1;
            Msg::n();
            if (Cfg::opt('upgrade')) {
                $msg=Msg::new("The following $prodname patches will be installed or upgraded on $sys->{sys}:", 30, 1319, "$prodname", "$sys->{sys}");
            } else {
                $msg=Msg::new("The following $prodname patches will be installed on $sys->{sys}:", 30, 1320, "$prodname", "$sys->{sys}");
            }
            $msg->printn;
            $n=$cpic->list_patches($sys,$n,$ref_remaining) if (@{$ref_remaining});
        }
        if (defined $sys->{requirerebootpkgspatches}) {
            push @rebootsys,$sys->{sys};
        }
    }

    if (($#$ref_allsystem_pkgs < 0) && ($#$ref_allsystem_patches < 0)) {
        # product is already installed, do not continue if not resumed from previous installer process.
        if (!$remaining && Cfg::opt("responsefile") !~ /exitfile/ && !Cfg::opt(qw(makeresponsefile addnode))) {
            $cpic->completion();
        }
    }

    if(@rebootsys) {
        $string_rebootsys=join(' ',@rebootsys);
        Msg::n();
        if (!EDRu::inarr('configure',@{$rel->{args_task}}) || Cfg::opt(qw(upgrade patchupgrade hotfixupgrade))) {
             $msg=Msg::new("At least one package or patch will require a reboot on systems $string_rebootsys after installation. Reboot after $cpic->{script} has completed.", 30, 1321, "$string_rebootsys", "$cpic->{script}");
        } else {
            $installscript = "$cpic->{installpath}/install" . lc($prod->{installscript_name} || $prod->{prod}) . $rel->get_local_script_version_suffix();
             $msg=Msg::new("At least one package will require a reboot on systems $string_rebootsys prior to configuration. Reboot and run '$installscript -configure' after $cpic->{script} has completed.", 30, 1322, "$string_rebootsys", "$installscript", "$cpic->{script}");
        }
        $msg->print;
    }
    Msg::prtc();
    Msg::n();
    return '';
}

# Package name for display
sub list_packages {
    my ($cpic,$sys,$n,$ref_pkgarray,$uninstall) = @_;
    my ($msg,$pdfr,$pdfr_desc,$pl,$edr,$msg_pdfr_desc,$format,$pkgi,$termy,$padv,$pkg,$prod,$pkgname,$pkgvers,$rel);
    $padv = ($sys->{padv}) ? $sys->{padv} : $cpic->{padv};
    $prod=$cpic->prod;
    $edr=Obj::edr();
    $rel=$cpic->rel;
    $termy=$edr->{tput}{termy}-7;
    $termy=20 if ($termy<2);
    $format = '%-17s %-20s %s';
    $pdfr=ucfirst(Msg::get('pdfr'));
    $pdfr_desc= Msg::new("$pdfr Description", 30, 1323, "$pdfr");
    my $vers_desc= Msg::new("Version", 30, 1324);
    $msg_pdfr_desc = $pdfr_desc->{msg};
    my $msg_vers_desc = $vers_desc->{msg};
    $pl=Msg::string_sprintf($format,$pdfr,$msg_vers_desc,$msg_pdfr_desc);
    Msg::bold($pl);
    for my $pkgi (@{$ref_pkgarray}) {
        $pkg=$cpic->pkg($pkgi,$padv);
        $pkgname=$pkg->{name}||'';
        $pkgname=$rel->pkgdesc($pkg) || $sys->pkgdesc($pkg) if ($uninstall && !$pkgname);
        # will be slightly off on uninstall if systems have the same package set but different versions
        $pkgvers = ($uninstall) ? $sys->{pkgvers}{$pkg->{pkg}} : $pkg->{vers};
        $msg=sprintf($format, $pkg->{pkg}, $pkgvers, $pkgname);
        $edr->{spacelinebreak}=' ' x 39;
        Msg::print($msg);
        $edr->{spacelinebreak}='';
        $n++;
        Msg::prtc() if (($n!=($#$ref_pkgarray+1)) && ($n%($termy-1)==0));
    }
    return $n;
}

sub list_patches {
    my ($cpic,$sys,$n,$ref_patcharray) = @_;
    my ($patch,$msg,$pdfr,$pl,$n_patch,$edr,$format,$termy,$padv,$pkg,$patchi,$msg_patch,$prod);
    $n_patch = 0;
    $padv=$sys->{padv}?$sys->{padv}:$cpic->{padv};
    $prod=$cpic->prod;
    $edr=Obj::edr();
    $termy=$edr->{tput}{termy}-7;
    $termy=20 if ($termy<2);
    $format = '%-20s %s';
    $pdfr=ucfirst(Msg::get('pdfr'));
    $msg_patch=Msg::new("Patch", 30, 1204)->{msg};
    $pl=Msg::string_sprintf($format,$msg_patch,$pdfr);
    Msg::bold($pl);
    for my $patchi (@{$ref_patcharray}) {
        $patch=$cpic->patch($patchi,$padv);
        $pkg=$cpic->pkg($patch->{pkg},$padv);
        $msg=sprintf('%-20s %s', $patch->{showform}, $pkg->{pkg});
        Msg::print($msg);
        $n++;
        $n_patch++;
        Msg::prtc() if (($n_patch!=($#$ref_patcharray+1)) && ($n%($termy-1)==0));
    }
    return $n;
}

sub obsoletedbundles {
    my ($cpic) = @_;
    my ($bundle,$sys,$obsoletedbundles);

    $obsoletedbundles=[];
    for my $sys (@{$cpic->{systems}}) {
        for my $bundle (@{$sys->{uninstallbundles}}) {
            push(@{$obsoletedbundles}, $bundle) unless (EDRu::inarr($bundle, @{$obsoletedbundles}));
        }
    }
    return $obsoletedbundles;
}

sub obsoletedpkgs {
    my ($cpic) = @_;
    my ($prod,$obsoletedpkgs,@all_obsoletedpkgs,@pkgs);

    $prod=$cpic->prod;
    @all_obsoletedpkgs=();

    #
    #   ----------------------------------------------------------------------------------------------------------
    #   | Categories            |   Fresh install  |     Upgrade        |   Patchupgrade     |  Uninstall        |
    #   ----------------------------------------------------------------------------------------------------------
    #   | obsoleted thirdparty  |  need uninstall  |   need uninstall   |   -                |  need uninstall   |
    #   | ASL/APM pkgs          |                  |                    |                    |                   |
    #   ----------------------------------------------------------------------------------------------------------
    #   | obsoleted bundled     |  need uninstall  |   need uninstall   |   -                |  -                |
    #   | pkgs (HPUX)           |                  |                    |                    |                   |
    #   ----------------------------------------------------------------------------------------------------------
    #   | obsoleted pkgs of     |  -               |   need uninstall   |   need uninstall   |  -                |
    #   | GA release            |                  |                    |                    |                   |
    #   | (like VRTScutil)      |                  |                    |                    |                   |
    #   ----------------------------------------------------------------------------------------------------------
    #   | obsoleted pkgs of     |  need uninstall  |   need uninstall   |   -                |  -                |
    #   | previous releases     |  when HPUX       |                    |                    |                   |
    #   | (known as UOU pkgs)   |                  |                    |                    |                   |
    #   ----------------------------------------------------------------------------------------------------------
    #   | obsoleted pkgs but    |  -               |   -                |   -                |  need uninstall   |
    #   | still support in      |                  |                    |                    |                   |
    #   | current release       |                  |                    |                    |                   |
    #   | (like VRTSdbms3)      |                  |                    |                    |                   |
    #   ----------------------------------------------------------------------------------------------------------
    #   | obsoleted pkgs but    |  -               |   if present then  |   -                |  -                |
    #   | still support and     |                  |   need uninstall   |                    |                   |
    #   | need upgrade to       |                  |   old version and  |                    |                   |
    #   | latest version        |                  |   install new      |                    |                   |
    #   | in current release    |                  |   version          |                    |                   |
    #   | (like VRTSacclib)     |                  |                    |                    |                   |
    #   ----------------------------------------------------------------------------------------------------------
    #
    #
    # In precheck phase,  the following variables could be set:
    #     $sys->{obsoleted_bundles}       - For HPUX
    #     $sys->{obsoleted_thirdparty_pkgs} - Mainly for obsoleted ASL/APM pkgs
    #     $sys->{obsoleted_bundled_pkgs}  - For HPUX
    #     $prod->{obsoleted_ga_release_pkgs}  - For obsoleted pkgs of GA release, such as VRTScutil(5.1)
    #     $prod->{obsoleted_previous_releases_pkgs}  - For obsoleted pkgs of previous releases
    #     $prod->{obsoleted_but_still_support_pkgs}  - For pkgs which obsoleted but still support in current release,
    #                                                  such as VRTdbms3
    #     $prod->{obsoleted_but_need_refresh_when_upgrade_pkgs}  - For pkgs which obsoleted but need be upgraded when upgrade,
    #                                                              such as VRTSacclib
    #
    # For fresh install
    # obsoletedpkgs:    obsoleted_thirdparty_pkgs + obsoleted_bundled_pkgs(HPUX)
    #
    # For full upgrade
    # obsoletedpkgs:    obsoleted_thirdparty_pkgs + obsoleted_ga_release_pkgs +
    #                   obsoleted_previous_releases_pkgs + obsoleted_but_need_refresh_when_upgrade_pkgs
    #
    # For patch upgrade.
    # obsoletedpkgs:    obsoleted_ga_release_pkgs
    #
    # For uninstall
    # obsoletedpkgs:    obsoleted_thirdparty_pkgs + obsoleted_but_still_support_pkgs
    #

    for my $sys (@{$cpic->{systems}}) {
        @pkgs=();
        if (Cfg::opt('install')) {
            $obsoletedpkgs=$sys->{obsoleted_thirdparty_pkgs};
            push(@pkgs, @{$obsoletedpkgs}) if (defined $obsoletedpkgs);

            $obsoletedpkgs=$sys->{obsoleted_bundled_pkgs};
            push(@pkgs, @{$obsoletedpkgs}) if (defined $obsoletedpkgs);

            # on HP-UX, need push the pkgs with old version into obsoletedpkgs list when fresh install.
            # For fresh install, need to uninstall all the obsoletedpkgs on every platforms,e2016325,e1984814
            #if ($sys->{padv} =~ /HPUX/) {
                # To get obsoleted_previous_releases_pkgs
                $obsoletedpkgs=$prod->obsoleted_previous_releases_pkgs;
                push(@pkgs, @{$obsoletedpkgs}) if (defined $obsoletedpkgs);
            #}
        } elsif (Cfg::opt('upgrade')) {
            $obsoletedpkgs=$sys->{obsoleted_thirdparty_pkgs};
            push(@pkgs, @{$obsoletedpkgs}) if (defined $obsoletedpkgs);

            $obsoletedpkgs=$sys->{obsoleted_bundled_pkgs};
            push(@pkgs, @{$obsoletedpkgs}) if (defined $obsoletedpkgs);

            # To get obsoleted_previous_releases_pkgs
            $obsoletedpkgs=$prod->obsoleted_previous_releases_pkgs;
            push(@pkgs, @{$obsoletedpkgs}) if (defined $obsoletedpkgs);

            $obsoletedpkgs=$prod->obsoleted_ga_release_pkgs;
            push(@pkgs, @{$obsoletedpkgs}) if (defined $obsoletedpkgs);

        } elsif (Cfg::opt(qw(patchupgrade hotfixupgrade))) {

            $obsoletedpkgs=$sys->{obsoleted_bundled_pkgs};
            push(@pkgs, @{$obsoletedpkgs}) if (defined $obsoletedpkgs);

            $obsoletedpkgs=$prod->obsoleted_ga_release_pkgs;
            push(@pkgs, @{$obsoletedpkgs}) if (defined $obsoletedpkgs);

        } elsif (Cfg::opt('uninstall')) {
            $obsoletedpkgs=$sys->{obsoleted_thirdparty_pkgs};
            push(@pkgs, @{$obsoletedpkgs}) if (defined $obsoletedpkgs);

            $obsoletedpkgs=$prod->obsoleted_but_still_support_pkgs;
            push(@pkgs, @{$obsoletedpkgs}) if (defined $obsoletedpkgs);
        }
        $sys->{obsoletedpkgs} = EDRu::arruniq(@pkgs);
        push @all_obsoletedpkgs, @{$sys->{obsoletedpkgs}};
    }

    return EDRu::arruniq(@all_obsoletedpkgs);
}

1;
