#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package CPIC;
use strict;

sub postcheck {
    my $cpic=shift;
    my($edr,$phase,$msg,$failmsg,$prod,$web);
    return 1 if (Cfg::opt(qw(makeresponsefile rootpath)) && !($cpic->{prod}));
    $prod=$cpic->prod;
    return 1 if ($prod->{lp});

    $cpic->postcheck_register_items();

    $web=Obj::web();
    $web->web_script_form('postcheck') if (Obj::webui());

    $phase=$cpic->setup_postcheck_phase();
    $phase->execute_tasks();

    $msg=Msg::new("$prod->{name} Postcheck completed successfully", 30, 1325, "$prod->{name}");
    $failmsg=Msg::new("$prod->{name} Postcheck did not complete successfully", 30, 1326, "$prod->{name}");

    $edr=Obj::edr();
    $edr->display_errors_warnings($msg,$failmsg);
    $web->{complete_failed}=1 if $web->{errors};
    #my $failures=$cpic->failures("postcheckfailmsg");
    #$web->display_completion($failures, $msg, $failmsg);

    $cpic->update_summary_postcheck();

    return 1;
}

sub postcheck_register_items {
    my $cpic=shift;
    my $prod=$cpic->prod;
    if ($prod->can('register_postchecks')) {
        # For CPIP version after uxrt61
        $prod->register_postchecks();
    } elsif ($prod->can('register_postchecks_per_system')) {
        # For CPIP version before uxrt61
        $prod->register_postchecks_per_system();
        $prod->register_postchecks_per_cluster();
    }
    return 1;
}

sub setup_postcheck_phase {
    my $cpic=shift;
    my ($phase,$desc,$prod,$task,$task_name,%task_params,$check,$cprod,$handler,$ifpersystem,$id);

    $cpic->{falsefailures}=0;

    $prod=$cpic->prod;
    $desc = Msg::new("Performing post-install checking for $prod->{abbr}", 30, 1327, "$prod->{abbr}");
    $phase=Phase->new('CPIC_postcheck');
    $phase->set_description($desc);
    $phase->initialize_tasks();

    # Checking status of packages
    $id=10;
    $task_name = "product_postcheck_packages";
    %task_params=(
        'sequence_id' => $id,
        'description' => Msg::new("Checking status of packages", 30, 1328),
        'description_sys' => Msg::new("Checking status of packages on #{SYS}", 30, 1329),
        'if_per_system' => 1,
        'handler' => \&CPIC::prod_postcheck_pkgs_sys,
        'handler_object_arg' => $cpic,
        'handler_args' => [ $cpic->{prod} ],
    );
    $task=Task->new($task_name, %task_params);
    $phase->add_task($task);

    if ($cpic->rel->{type}=~/^M/) {
        # Checking status of patches
        $id+=100;
        $task_name = "product_postcheck_patches";
        %task_params=(
            'sequence_id' => $id,
            'description' => Msg::new("Checking status of patches", 30, 1330),
            'description_sys' => Msg::new("Checking status of patches on #{SYS}", 30, 1331),
            'if_per_system' => 1,
            'handler' => \&CPIC::prod_postcheck_patches_sys,
            'handler_object_arg' => $cpic,
            'handler_args' => [ $cpic->{prod} ],
        );
        $task=Task->new($task_name, %task_params);
        $phase->add_task($task);
    }
    $id+=100;
    $task_name = "product_postcheck_processes";
    %task_params=(
        'sequence_id' => $id,
        'description' => Msg::new("Checking status of processes and drivers", 30, 1332),
        'description_sys' => Msg::new("Checking status of processes and drivers on #{SYS}", 30, 1333),
        'if_per_system' => 1,
        'handler' => \&CPIC::prod_postcheck_procs_sys,
        'handler_object_arg' => $cpic,
        'handler_args' => [ $cpic->{prod} ],
    );
    $task=Task->new($task_name, %task_params);
    $phase->add_task($task);

    $id+=100;
    for my $checki (sort keys %{$cpic->{postchecks}}) {
        $check=$cpic->{postchecks}{$checki};
        $desc=$check->{description};
        $cprod=$check->{product};
        $handler=$check->{handler};
        $ifpersystem=$check->{if_per_system};
        $task_name = "product_postcheck_$checki";
        %task_params=(
            'sequence_id' => $id,
            'description' => Msg::new("Checking $desc", 30, 1334, "$desc"),
            'description_sys' => Msg::new("Checking $desc on #{SYS}", 30, 1335, "$desc"),
            'if_per_system' => $ifpersystem,
            'handler' => \&$handler,
            'handler_object_arg' => $cprod,
        );
        $task=Task->new($task_name, %task_params);
        $phase->add_task($task);
        $id++;
    }

    return $phase;
}

sub prod_postcheck_procs_sys {
    my ($cpic,$sys,$prod) = @_;
    my ($msg,$proci,@failed_procs,$running,$startprocs,$proc,$procs);

    # scalars must be passed to threaded call
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $prod=$sys->prod($prod) unless (ref($prod));

    return 1 if (Cfg::opt('makeresponsefile'));
    return '' if ($sys->{stop_checks});

    $startprocs=$prod->startprocs_sys($sys);
    #3491953:skip the postcheck if there has no started processes.
    return if (!$startprocs || !@{$startprocs}); 

    for my $proci (@{$startprocs}) {
        $proc=$sys->proc($proci);
        next if ($proc->{donotstart}||$proc->{reboot_donotstart});
        $running=$proc->check_sys($sys, 'poststart');
        if (!$running) {
            push(@failed_procs, $proc->{proc});
        }
    }

    if (@failed_procs) {
        $sys->set_value('postcheckfailprocs','list',@failed_procs);
        $procs=join("\n\t", @failed_procs);
        $msg=Msg::new("The following processes or drivers are not started on $sys->{sys}:\n\t$procs", 30, 1336, "$sys->{sys}", "$procs");
        $sys->push_error($msg);
        return 0;
    }
    return 1;
}

sub prod_postcheck_pkgs_sys{
    my ($cpic,$sys,$prod) = @_;
    my ($msg,$pkg,$pkgi,$pkgs,$reqpkgs,$pkgvers);
    my (%invalidpkgs,@notinstalledpkgs,@notinstalledreqpkgs,@partialpkgs,$vers,$rel,$licensed);
    my ($msg_missing_pkgs, $msg_invalid_pkgs, $msg_partial_pkgs);
    # scalars must be passed to threaded call
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $prod=$sys->prod($prod) unless (ref($prod));

    return 1 if (Cfg::opt('makeresponsefile'));
    return '' if ($sys->{stop_checks});

    $rel = Obj::rel();
    $reqpkgs = $prod->minpkgs();
    $pkgs = $prod->allpkgs();
    for my $pkgi (@{$pkgs}) {
        $cpic->create_cpip_objects($sys->{padv});
        $pkg = $sys->pkg($pkgi);
        next if ($pkg->donotinstall_sys($sys));
        $vers = $pkg->version_sys($sys);
        if (!$vers) {
            if (EDRu::inarr($pkgi, @$reqpkgs)) {
                push @notinstalledreqpkgs, $pkg->{pkg};
            } else {
                push @notinstalledpkgs, $pkg->{pkg} unless ($pkg->{mpok});
            }
        } else {
            # only need validate pkg version for required packages
            next unless EDRu::inarr($pkgi, @$reqpkgs);

            next if ($prod->{skip_pkgverify_postcheck});
            # todo verify pkg
            if ($sys->padv->pkg_verify_sys($sys, $pkg->{pkg})) {
                $msg=$sys->{pkg_verify}{$pkg->{pkg}};
                $sys->push_check_note($msg);
                push @partialpkgs, $pkg->{pkg};
            }
            next if ($prod->{skip_pkgvers_postcheck});
            # If it's RP/MR release, we don't check package version
            next if ($cpic->rel->{type}=~/^M/);
            if ($cpic->{fromdisk}) {
                $pkgvers = $pkg->{gavers} || $rel->{vers};
            } else {
                $pkgvers = $pkg->{vers};
            }
            if ($pkg->{gavers}){
                if (EDRu::compvers($vers,$pkgvers,4) == 2) {
                    $invalidpkgs{$pkg->{pkg}}="$vers|$pkgvers";
                    next;
                }
            } else {
                if (EDRu::compvers($vers,$pkgvers,3) != 0) {
                    $invalidpkgs{$pkg->{pkg}}="$vers|$pkgvers";
                    next;
                }
            }
        }
    }

    if (@notinstalledreqpkgs) {
        $msg_missing_pkgs = join("\n\t", @notinstalledreqpkgs);
        $msg=Msg::new("The following required packages are not installed on $sys->{sys}:\n\t$msg_missing_pkgs", 30, 1337, "$sys->{sys}", "$msg_missing_pkgs");
        $sys->push_error($msg);
    }

    if (@notinstalledpkgs) {
        $msg_missing_pkgs = join("\n\t", @notinstalledpkgs);
        $msg=Msg::new("The following packages are not installed on $sys->{sys}, these packages are required only for additional functions:\n\t$msg_missing_pkgs", 30, 1338, "$sys->{sys}", "$msg_missing_pkgs");
        $sys->push_warning($msg);
    }

    if (keys %invalidpkgs ) {
        $msg_invalid_pkgs = "";
        for my $pkg (sort keys %invalidpkgs) {
            ($vers,$pkgvers) = split(/\|/,$invalidpkgs{$pkg});
            $msg_invalid_pkgs .= sprintf("\n\t%-12s  installed: %-16s  expected: %-16s", $pkg, $vers, $pkgvers);
        }
        $msg=Msg::new("The following required packages are not installed with the correct version on $sys->{sys}:$msg_invalid_pkgs", 30, 1339, "$sys->{sys}", "$msg_invalid_pkgs");
        $sys->push_error($msg);
    }

    if (@partialpkgs) {
        $msg_partial_pkgs = join("\n\t", @partialpkgs);
        $msg=Msg::new("The following packages are not passed package verification on $sys->{sys}:\n\t$msg_partial_pkgs\nDetailed information can be found in the log files or the summary file.", 30, 1340, "$sys->{sys}", "$msg_partial_pkgs");
        $sys->push_warning($msg);
    }

    if (!@notinstalledreqpkgs) {
        $licensed=$rel->prod_licensed_sys($sys,$prod->{prodi});
        if (!$licensed) {
            $msg=Msg::new("$prod->{abbr} $prod->{vers} does not appear to be licensed on $sys->{sys}", 30, 1341, "$prod->{abbr}", "$prod->{vers}", "$sys->{sys}");
            $sys->push_warning($msg);
        }
    }

    if ( %invalidpkgs || @notinstalledpkgs || @partialpkgs) {
        return 0;
    } else {
        return 1;
    }
}

sub prod_postcheck_patches_sys {
    my ($cpic,$sys,$prod) = @_;
    my ($pkg,$patch,$padv,$rel);
    my (@notinstalledpatches, $msg_missing_patches, $msg);
    # scalars must be passed to threaded call
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $prod=$sys->prod($prod) unless (ref($prod));
    $rel=$prod->rel;
    $padv=$sys->padv;

    return 1 if (Cfg::opt('makeresponsefile'));
    return '' if ($sys->{stop_checks}) || (!$rel->{patches});

    for my $pkgi (@{$prod->allpkgs}) {
        $pkg=$prod->pkg($pkgi);
        next unless ($pkg->version_sys($sys));
        for my $patchi (@{$pkg->patches_sys($sys)})  {
            $patch=$sys->patch($patchi) unless (ref($patchi));
            if ($patch->{vers}) {
                if (EDRu::compvers($padv->patch_version_sys($sys, $patch),$patch->{vers}) != 0) {
                    # Patch not installed. Need to record.
                    push @notinstalledpatches, $patch->{showform};
                }
            }
        }
    }

    if( @notinstalledpatches ) {
        $msg_missing_patches=join("\n\t", @notinstalledpatches);
        $msg=Msg::new("The following patches are not installed on $sys->{sys}:\n\t$msg_missing_patches", 30, 1342, "$sys->{sys}", "$msg_missing_patches");
        $sys->push_error($msg);
        return 0;
    }

    return 1;
}

sub update_summary_postcheck {
    my $cpic=shift;

    my $edr=Obj::edr();
    my $msg=Msg::new("The following checks and actions were performed during postcheck on systems:", 30, 1343);
    $edr->update_check_action_summary($msg);

    return 1;
}

1;
