#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package CPIC;
use strict;

sub pre_setup {
    my $cpic = shift;
    my $web = Obj::web();
    my $config = Web::json_to_obj(EDRu::readfile("$web->{vom_patch_store}/patches/config.json"));
    return 1 unless ($config->{pre_setup});
    my $phase=$cpic->pre_setup_phase($config);
    $phase->execute_tasks();
    return 1;
}

sub exec_pre_setup_sys {
    my ($config,$sys) = @_;
    my @rtn;
    my $trans = $sys->{transport_obj};
    my $mh_installdir = $trans->get_MH_InstallDir($sys);
    my $mh_appdir = $trans->get_MH_AppDir($sys);
    my $web=Obj::web();
    my $separator = ($^O =~ /Win32/i)? '\\':'/';
    my $localsys=EDR::get('localsys');
    @rtn = $trans->cmd_sys($sys,"mkdir $mh_appdir/$web->{scriptid}");
    $trans->copy_to_sys($localsys,$web->{vom_patch_store}.$separator."patches".$separator."scripts".$separator.$config->{pre_setup},$sys, "$mh_appdir/$web->{scriptid}");
    @rtn = $trans->cmd_sys($sys,"chmod 755 $mh_appdir/$web->{scriptid}/$config->{pre_setup}");
    @rtn = $trans->cmd_sys($sys,"$mh_installdir/bin/perl $mh_appdir/$web->{scriptid}/$config->{pre_setup}");

    return $rtn[2]?0:1;
}
sub pre_setup_phase {
    my ($cpic,$config)=@_;
    my ($phase,$phase_name,$phase_desc,$task,$task_name,%task_params,$prod,$proc,$id);

    $phase_name='VOM_pre_setup';
    $phase_desc=$config->{pre_setup_desc};

    $phase= Phase->new($phase_name);
    $phase->set_description($phase_desc);
    $phase->initialize_tasks();

    $id=10;

    $task_name="VOM_pre_setup";
    %task_params=(
        'sequence_id' => $id,
        'description' => Msg::new("Performing pre set up tasks", 30, 1355),
        'description_sys' => Msg::new("Performing pre set up tasks on #{SYS}", 30, 1356),
        'if_per_system' => 1,
        'handler' => \&CPIC::exec_pre_setup_sys,
        'handler_object_arg' => $config,
    );
    $task=Task->new($task_name, %task_params);
    $phase->add_task($task);

    return $phase;
}

sub shutdown {
    my $cpic=shift;
    my ($phase,$rtn,$prod,$upgrade_from);

    $prod=$cpic->prod;
    if (defined $prod->{upgrade_from} && Cfg::opt(qw(patchupgrade upgrade))) {
        $upgrade_from=$cpic->rel->match_prod($prod->{upgrade_from});
        $cpic->set_prod($upgrade_from) if ($upgrade_from);
        $prod=$cpic->rel->match_prod($prod->{prod});
    }
    $rtn=$cpic->preshutdown();
    if ($rtn) {
        $phase=$cpic->setup_shutdown_phase();
        $phase->execute_tasks();
        $rtn=$cpic->postshutdown();
    }
    if ($upgrade_from) {
        $cpic->set_prod($prod);
    }
    return $rtn;
}

# Return 1, continue shutdown processes.
# Return 0, skip whole shutdown phase.
sub preshutdown {
    my $cpic=shift;
    my ($prod,$steps,$edr);

    # If -makeresponsefile but no prod specified, skip whole shutdown phase.
    return 0 if (Cfg::opt('makeresponsefile') && !($cpic->{prod}));

    $prod=$cpic->prod;

    # If localization product, skip whole shutdown phase.
    return 0 if ($prod->{lp});

    # If do not re-configure product, skip whole shutdown phase.
    return 0 if ($prod->donotreconfig());

    if (Cfg::opt('rootpath')) {
        # need run prestop for Live upgrade case.
        $steps=1;
    } else {
        $cpic->{stopprocs}=$prod->stopprocs();
        for my $sys (@{$cpic->{systems}}) {
            $sys->{stopprocs}=$prod->stopprocs_sys($sys);
        }
        $cpic->{stopprocs}=$cpic->verify_list('stopprocs');

        $steps=scalar(@{$cpic->{stopprocs}});
    }

    # If no shutdown tasks, skip whole shutdown phase.
    return 0 if (!$steps);

    $cpic->{falsefailures}=0;

    $prod->pre_shutdown_question();

    if (!Cfg::opt('rootpath')) {
        $edr=Obj::edr();
        if(Cfg::opt('upgrade_kernelpkgs')) {
            $cpic->rel->ru_switch_groups($cpic->{systems});
        }
        $edr->exit_noexitfile() if ($cpic->cli_ask_stop_processes($prod));
    }

    # Show stop processes UI form
    $cpic->show_ui_form('stopprocess');

    return 1;
}

sub setup_shutdown_phase {
    my $cpic=shift;
    my ($phase,$phase_name,$phase_desc,$task,$task_name,%task_params,$prod,$proc,$id);

    $prod=$cpic->prod;

    $phase_name=$cpic->{shutdown_phase_name} || 'CPIC_shutdown';
    $phase_desc=$cpic->{shutdown_phase_desc};
    if (Cfg::opt('rootpath')) {
        $phase_desc||=Msg::new("Backing up $prod->{abbr} configuration", 30, 1357, "$prod->{abbr}");
    } else {
        $phase_desc||=Msg::new("Stopping $prod->{abbr}", 30, 1358, "$prod->{abbr}");
    }

    $phase= Phase->new($phase_name);
    $phase->set_description($phase_desc);
    $phase->initialize_tasks();

    $id=10;
    if ($prod->can(upgrade_sub('prestop_sys'))) {
        $task_name="product_prestop";
        %task_params=(
            'sequence_id' => $id,
            'description' => Msg::new("Performing $prod->{abbr} prestop tasks", 30, 1359, "$prod->{abbr}"),
            'description_sys' => Msg::new("Performing $prod->{abbr} prestop tasks on #{SYS}", 30, 1360, "$prod->{abbr}"),
            'if_per_system' => 1,
            'handler' => \&CPIC::prod_prestop_sys,
            'handler_object_arg' => $cpic,
            'post_handler' => \&CPIC::prod_prestop_errors,
            'post_handler_object_arg' => $cpic,
        );
        $task=Task->new($task_name, %task_params);
        $phase->add_task($task);
    }

    return $phase if (Cfg::opt('rootpath'));

    $id+=100;
    for my $proci (@{$cpic->{stopprocs}}) {
        $proc=$cpic->proc($proci);

        $task_name = "stop_$proc->{proc}";
        %task_params=(
            'sequence_id' => $id,
            'description' => Msg::new("Stopping $proc->{proc}", 30, 1358, "$proc->{proc}"),
            'description_sys' => Msg::new("Stopping $proc->{proc} on #{SYS}", 30, 1361, "$proc->{proc}"),
            'if_per_system' => 1,
            'handler' => \&CPIC::proc_stop_sys,
            'handler_object_arg' => $cpic,
            'handler_args' => [ $proci, 1 ],
            'rc_handler' => \&CPIC::proc_stop_rc_check_sys,
            'rc_handler_object_arg' => $cpic,
            'rc_handler_args' => [ $proci ],
        );
        $task=Task->new($task_name, %task_params);
        $phase->add_task($task);

        # Skip the task on some systems for serial mode
        for my $sys (@{$cpic->{systems}}) {
            if (!EDRu::inarr($proci, @{$sys->{stopprocs}}) ||
                EDRu::inarr($proci,@{$sys->{donotstopprocs}})) {
                $task->set_skip_sys($sys,1);
            }
        }
        $id++;
    }

    $id+=100;
    if ($prod->can(upgrade_sub('poststop_sys'))) {
        $task_name="product_poststop";
        %task_params=(
            'sequence_id' => $id,
            'description' => Msg::new("Performing $prod->{abbr} poststop tasks", 30, 1362, "$prod->{abbr}"),
            'description_sys' => Msg::new("Performing $prod->{abbr} poststop tasks on #{SYS}", 30, 1363, "$prod->{abbr}"),
            'if_per_system' => 1,
            'handler' => \&CPIC::prod_poststop_sys,
            'handler_object_arg' => $cpic,
            'post_handler' => \&CPIC::prod_poststop_errors,
            'post_handler_object_arg' => $cpic,
        );
        $task=Task->new($task_name, %task_params);
        $phase->add_task($task);
    }

    return $phase;
}

sub postshutdown {
    my $cpic=shift;
    my ($prod,$msg,$failmsg,$failures);

    $prod=$cpic->prod;

    # do we need $cpic->reboot_messages here?
    if (Cfg::opt('rootpath')) {
        $msg=Msg::new("$prod->{name} configuration backup completed successfully", 30, 1364, "$prod->{name}");
        $failmsg=Msg::new("$prod->{name} configuration backup did not complete successfully", 30, 1365, "$prod->{name}");
    } else {
        $msg=Msg::new("$prod->{name} Shutdown completed successfully", 30, 1366, "$prod->{name}");
        $failmsg=Msg::new("$prod->{name} Shutdown did not complete successfully", 30, 1367, "$prod->{name}");
    }
    $failures=$cpic->failures('stopfailmsg');
    $msg->display_completion($failures, $msg, $failmsg);
    if ($#$failures>=0) {
        EDR::die('stop failures') if (Cfg::opt('startstoptest'));
        $cpic->ask_confirmation_if_fatal_stop_failures();
    }

    $cpic->update_summary_stop();
    return 1;
}

sub cli_ask_stop_processes {
    my ($cpic, $prod) = @_;
    return ($cpic->ask_stop_processes($prod));
}

sub ask_stop_processes {
    my ($cpic,$prod) = @_;
    my ($stop_prod,$ayn,$help,$msg,$cfg);
    return '' if ($prod->{autostop} || Cfg::opt(qw(stop responsefile startstoptest)));
    $stop_prod=$prod->{stop_prod} || $prod->{abbr};
    $msg=Msg::new("All $stop_prod processes that are currently running must be stopped", 30, 1368, "$stop_prod");
    $msg->print;
    $msg=Msg::new("Do you want to stop $stop_prod processes now?", 30, 1369, "$stop_prod");
    $help=Msg::new("Enter 'y' if you want to stop processes, otherwise, enter 'n' and installer exits", 30, 1370);
    Msg::n();
    $ayn=$msg->ayny($help,0);
    Msg::n();
    $cfg = Obj::cfg();
    if ($ayn eq 'N') {
       if ($cfg->{reconfigurevcs}) {
           $msg=Msg::new("If you do not stop $stop_prod processes, you will lose all the changed configuration and installer will exit. Do you want to stop the processes now?", 30, 1371, "$stop_prod");
           $ayn=$msg->ayny($help,0);
           Msg::n();
           if ($ayn eq 'Y') {
               return '';
           }
       }
       if (Cfg::opt('uninstall')) {
           $msg=Msg::new("You have to stop processes if you want to uninstall $stop_prod.", 30, 1372, "$stop_prod");
           $msg->print;
           Msg::n();
       }
       return 1;
    }
    return '';
}

sub ask_confirmation_if_fatal_stop_failures {
    my $cpic=shift;
    my (@systems,$systems,$msg,$help,$ayn);

    # check whether any systems have fatal stop failures
    @systems=();
    for my $sys (@{$cpic->{systems}}) {
        if (defined $sys->{fatalstopfailprocs}) {
            push @systems, $sys->{sys};
        }
    }

    if (@systems && Cfg::opt(qw(hotfixupgrade patchupgrade upgrade rollback))) {
        $systems=join(' ', @systems);
        $msg=Msg::new("It is recommended to reboot the systems $systems to resolve the failures and then retry. If issues persist after reboot, contact Symantec technical support or refer to installation guide for further troubleshooting.\nDo you want to continue?", 30, 1373, "$systems");
        $ayn=$msg->aynn();
        $cpic->edr_completion() if ($ayn eq 'N');
    } elsif (Cfg::opt(qw(uninstall))) {
        $msg=Msg::new("Some processes failed to stop. Do you want to continue?", 30, 1374);
        $help=Msg::new("Enter 'y' if you want to continue, otherwise, enter 'n' and then the installer will exit", 30, 1375);
        $ayn=$msg->ayny($help,0);
        $cpic->edr_completion() if ($ayn eq 'N');
    }
    return 1;
}

sub prod_prestop_sys {
    my ($cpic,$sys) = @_;
    my ($tmpsub,$mr);
    # scalars must be passed to threaded call
    return 1 if (Cfg::opt('makeresponsefile'));
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $tmpsub = upgrade_sub('prestop_sys');
    $sys->prod->$tmpsub($sys);
    if ($mr = $cpic->{mrobj}) {
        $mr->prestop_sys($sys);
    }
    for my $hotfix(@{$cpic->{hotfixobj}}) {
        $hotfix->prestop_sys($sys);
    }
    return 1;
}

sub prod_prestop_errors {
    my ($cpic)=@_;

    my $prod=$cpic->prod;
    if ($cpic->sys_error) {
        $prod->prestop_errors($cpic);
        Msg::title();
    }

    EDR::reset_errors_warnings();
    return 1;
}

sub proc_stop_sys {
    my ($cpic,$sys,$proc,$checkproclist) = @_;
    my ($running,$rtn,$sleep,$edr);
    $edr=Obj::edr();
    # scalars must be passed to threaded call
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $proc=$sys->proc($proc) unless (ref($proc));
    return 1 if (Cfg::opt('makeresponsefile'));
    # proc stop is conditional
    return if ($checkproclist &&
        (!EDRu::inarr($proc->{proci}, @{$sys->{stopprocs}}) ||
         EDRu::inarr($proc->{proci},@{$sys->{donotstopprocs}})));

    $edr->{logfile2}="$edr->{tmpdir}/stop."."$proc->{proc}."."$sys->{sys}";
    Msg::log("## Stopping $proc->{proc} on $sys->{sys}");
    # initial check
    Msg::log("## Begin prestop checking for $proc->{proc} on $sys->{sys}");
    $running=$proc->check_sys($sys, 'prestop');
    Msg::log("## The prestop checking return '$running'");
    if ($running) {
        # save process status
        $proc->set_value("running,$sys->{sys}",1);
        Msg::log("## $proc->{proc} is running on $sys->{sys}");
        Msg::log("## Begin stopping $proc->{proc} on $sys->{sys}");
        $rtn=$proc->stop_sys($sys);
        Msg::log("## The stop_sys() return '$rtn'");
        if ($rtn) {
            # confirm shutdown, may retry check for a defined period
            Msg::log("## Begin poststop checking for $proc->{proc} on $sys->{sys}");
            $running=$proc->check_sys($sys, 'poststop');
            Msg::log("## The poststop checking return '$running'");
            if ($running) {
                $sleep=0;
                while ($sleep<$proc->{stop_period}) {
                    $sleep+=10;
                    sleep 10;
                    Msg::log('## Sleep 10, and do poststop checking again');
                    if ($proc->{retry_stop}) {
                        Msg::log("## The previous stop failed, retry stopping $proc->{proc}");
                        $proc->stop_sys($sys);
                    }
                    $running=$proc->check_sys($sys, 'poststop');
                    Msg::log("## The poststop checking return '$running'");
                    last if (!$running);
                }
                if ($running && $proc->can('force_stop_sys')) {
                    Msg::log("## Begin force stop $proc->{proc} on $sys->{sys}");
                    $proc->force_stop_sys($sys);
                    $running=$proc->check_sys($sys, 'poststop');
                    Msg::log("## The poststop checking after force stop return '$running'");
                }
            }
            if ($running) {
                Msg::log("## $proc->{proc} is not successfully stopped on $sys->{sys}");
                # To show debug informations in stop_failed_sys().
                $proc->stop_failed_sys($sys);
                $sys->push_fatal_stop_proc($proc);
                $rtn=0;
            } else {
                Msg::log("## $proc->{proc} is successfully stopped on $sys->{sys}");
                $rtn=1;
            }
        } else {
            Msg::log('## The stop_sys() directly failed');
            $rtn=0;
        }
    } else {
        Msg::log("## $proc->{proc} is not running on $sys->{sys}");
        $rtn=1;
    }
    Msg::log("## The proc_stop_sys() return '$rtn'");
    $edr->{logfile2}='';

    # adding temp failures to test the web script
    return 0 if ($cpic->false_failure);
    return $rtn;
}

# $cpic ref not passed when called through $thr->mq_read_rtns
sub proc_stop_failed_sys {
    my($sys,$proc)= @_;
    $proc=$sys->proc($proc) unless (ref($proc));
    push(@{$sys->{stopfail}}, $proc->{proc});
    $sys->{proc_reboot}=1 if (!$proc->{noreboot_failstop});
    my $msg=Msg::new("$proc->{proc} failed to stop on $sys->{sys}", 30, 1376, "$proc->{proc}", "$sys->{sys}");
    $msg->log;
    push(@{$sys->{stopfailmsg}}, $msg);
    EDRu::appendfile($proc->{proc}, EDR::tmpdir()."/stop_fail.$sys->{sys}")
        if (Cfg::opt('bait'));
    return 1;
}

sub proc_stop_passed_sys {
    my($sys,$proc,$checkproclist)= @_;
    $proc=$sys->proc($proc) unless (ref($proc));
    # proc stop is conditional
    return 1 if ($checkproclist &&
        (!EDRu::inarr($proc->{proci}, @{$sys->{stopprocs}}) ||
        EDRu::inarr($proc->{proci},@{$sys->{donotstopprocs}})));
    push(@{$sys->{stoppass}}, $proc->{proc});
    Msg::log("$proc->{proc} stopped successfully on $sys->{sys}");
    EDRu::appendfile($proc->{proc}, EDR::tmpdir()."/stop_pass.$sys->{sys}")
        if (Cfg::opt('bait'));
    return 1;
}

sub proc_stop_rc_check_sys {
    my ($cpic,$sys,$proc,$rc) = @_;
    $proc=$sys->proc($proc) unless (ref($proc));
    if ($rc) {
        proc_stop_passed_sys($sys, $proc);
    } elsif (defined $rc) {
        proc_stop_failed_sys($sys, $proc);
    }
    return $rc;
}

sub prod_poststop_sys {
    my ($cpic,$sys) = @_;
    my ($tmpsub);
    # scalars must be passed to threaded call
    return 1 if (Cfg::opt('makeresponsefile'));
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $tmpsub = upgrade_sub('poststop_sys');
    $sys->prod->$tmpsub($sys);
    return 1;
}

sub prod_poststop_errors {
    my ($cpic)=@_;

    my $prod=$cpic->prod;
    $prod->poststop_errors($cpic) if ($cpic->sys_error);

    EDR::reset_errors_warnings();
    return 1;
}

sub update_summary_stop {
    my $cpic=shift;
    my $msg;

    for my $sys (@{$cpic->{systems}}) {
        if ($#{$sys->{stoppass}}>=0) {
            $msg=Msg::new("The following processes were stopped on $sys->{sys}:", 30, 1377, "$sys->{sys}");
            $cpic->update_summary($msg, @{$sys->{stoppass}});
        }
        if ($#{$sys->{stopfail}}>=0) {
            $msg=Msg::new("The following processes failed to stop on $sys->{sys}:", 30, 1378, "$sys->{sys}");
            $cpic->update_summary($msg, @{$sys->{stopfail}});
        }
    }
    return 1;
}

1;
