#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package CPIC;
use strict;

sub post_setup {
    my $cpic = shift;
    my $web = Obj::web();
    my $config = Web::json_to_obj(EDRu::readfile("$web->{vom_patch_store}/patches/config.json"));
    return 1 unless ($config->{post_setup});
    my $phase=$cpic->post_setup_phase($config);
    $phase->execute_tasks();
    return 1;
}

sub exec_post_setup_sys {
    my ($config,$sys) = @_;
    my @rtn;
    my $trans = $sys->{transport_obj};
    my $mh_installdir = $trans->get_MH_InstallDir($sys);
    my $mh_appdir = $trans->get_MH_AppDir($sys);
    my $web=Obj::web();
    my $separator = ($^O =~ /Win32/i)? '\\':'/';
    my $localsys=EDR::get('localsys');
    @rtn = $trans->cmd_sys($sys,"mkdir $mh_appdir/$web->{scriptid}");
    $trans->copy_to_sys($localsys,$web->{vom_patch_store}.$separator."patches".$separator."scripts".$separator.$config->{post_setup},$sys, "$mh_appdir/$web->{scriptid}/");
    @rtn = $trans->cmd_sys($sys,"chmod 755 $mh_appdir/$web->{scriptid}/$config->{post_setup}");
    @rtn = $trans->cmd_sys($sys,"$mh_installdir/bin/perl $mh_appdir/$web->{scriptid}/$config->{post_setup}");
    return $rtn[2]?0:1;
}

sub post_setup_phase {
    my ($cpic,$config)=@_;
    my ($phase,$phase_name,$phase_desc,$task,$task_name,%task_params,$prod,$proc,$id);

    $phase_name='VOM_pre_setup';
    $phase_desc=$config->{post_setup_desc};

    $phase= Phase->new($phase_name);
    $phase->set_description($phase_desc);
    $phase->initialize_tasks();

    $id=10;

    $task_name="VOM_pre_setup";
    %task_params=(
        'sequence_id' => $id,
        'description' => Msg::new("Performing post set up tasks", 30, 1379),
        'description_sys' => Msg::new("Performing post set up tasks on #{SYS}", 30, 1380),
        'if_per_system' => 1,
        'handler' => \&CPIC::exec_post_setup_sys,
        'handler_object_arg' => $config,
    );
    $task=Task->new($task_name, %task_params);
    $phase->add_task($task);

    return $phase;
}

sub startup {
    my $cpic=shift;
    my ($phase,$rtn);

    $rtn=$cpic->prestartup();
    return 0 if (!$rtn);

    $phase=$cpic->setup_startup_phase();
    return 1 if (!$phase->tasks_steps());

    $cpic->show_ui_form('startprocess');
    $phase->execute_tasks();

    return $cpic->poststartup();
}

# Return 1, continue startup processes.
# Return 0, skip whole startup phase.
sub prestartup {
    my $cpic=shift;
    my ($prod);

    # If -makeresponsefile but no prod specified, skip whole startup phase.
    return 0 if (Cfg::opt('makeresponsefile') && !($cpic->{prod}));

    $prod=$cpic->prod;

    # If localization product, skip whole startup phase.
    return 0 if ($prod->{lp});

    # e3544923, if before patchupgrade sfrac is not configured, need to generate /etc/vxfenmode
    if (Cfg::opt('configure') || (Cfg::opt('patchupgrade') && Cfg::opt('patchconfigure'))) {
        for my $sys (@{$cpic->{systems}}) {
            $cpic->prod_prestart_sys($sys);
        }
    }
    return 1;
}

sub setup_startup_phase {
    my $cpic=shift;
    my ($phase,$phase_name,$phase_desc,$task,$task_name,%task_params,$prod,$proc,$id,$reboot_systems,$taskmsg);

    $cpic->{falsefailures}=0;
    $prod=$cpic->prod;

    $reboot_systems=$cpic->systems_need_reboot();
    $cpic->{reboot}=1 if (@{$reboot_systems} || Cfg::opt('rootpath'));

    $phase_name=$cpic->{startup_phase_name} || 'CPIC_startup';
    $phase_desc=$cpic->{startup_phase_desc};
    if (($cpic->{reboot}) || (Cfg::opt(qw(rootpath nostart))) ||
        (Cfg::opt(qw(upgrade patchupgrade hotfixupgrade)) && ($cpic->{prod_not_enabled_before_upgrade} || $cpic->{prod_not_running_before_upgrade}))) {
        $phase_desc||=Msg::new("Configuring $prod->{abbr}", 30, 1381, "$prod->{abbr}");
    } else {
        $phase_desc||=Msg::new("Starting $prod->{abbr}", 30, 1382, "$prod->{abbr}");
    }
    $phase= Phase->new($phase_name);
    $phase->set_description($phase_desc);
    $phase->initialize_tasks();

    $id=10;
    if (Cfg::opt(qw(upgrade patchupgrade rollback))) {
        if ((Cfg::opt('upgrade') || $cpic->{reuse_config})
            && ($prod->can(upgrade_sub('configure_sys')))) {
            if (Cfg::opt(qw(rollback))) {
                $taskmsg="rollback";
            } else {
                $taskmsg="upgrade";
            }
            $task_name="product_upgrade_configuration";
            %task_params=(
                'sequence_id' => $id,
                'description' => Msg::new("Performing $prod->{abbr} $taskmsg configuration", 30, 1383, "$prod->{abbr}", "$taskmsg"),
                'description_sys' => Msg::new("Performing $prod->{abbr} $taskmsg configuration on #{SYS}", 30, 1384, "$prod->{abbr}", "$taskmsg"),
                'if_per_system' => 1,
                'handler' => \&CPIC::prod_upgrade_configure_sys,
                'handler_object_arg' => $cpic,
                'post_handler' => \&CPIC::prod_configure_errors,
                'post_handler_object_arg' => $cpic,
            );
        } elsif ($prod->can('configure_sys')) {
            $task_name="product_configuration";
            %task_params=(
                'sequence_id' => $id,
                'description' => Msg::new("Performing $prod->{abbr} configuration", 30, 1385, "$prod->{abbr}"),
                'description_sys' => Msg::new("Performing $prod->{abbr} configuration on #{SYS}", 30, 1386, "$prod->{abbr}"),
                'if_per_system' => 1,
                'handler' => \&CPIC::prod_configure_sys,
                'handler_object_arg' => $cpic,
                'post_handler' => \&CPIC::prod_configure_errors,
                'post_handler_object_arg' => $cpic,
            );
        }
        $task=Task->new($task_name, %task_params);
        $phase->add_task($task);
    }

    $id+=100;
    if ((Cfg::opt('configure')) && ($prod->can('configure_sys'))) {
        $task_name="product_configuration";
        %task_params=(
            'sequence_id' => $id,
            'description' => Msg::new("Performing $prod->{abbr} configuration", 30, 1385, "$prod->{abbr}"),
            'description_sys' => Msg::new("Performing $prod->{abbr} configuration on #{SYS}", 30, 1386, "$prod->{abbr}"),
            'if_per_system' => 1,
            'handler' => \&CPIC::prod_configure_sys,
            'handler_object_arg' => $cpic,
            'post_handler' => \&CPIC::prod_configure_errors,
            'post_handler_object_arg' => $cpic,
        );
        $task=Task->new($task_name, %task_params);
        $phase->add_task($task);
    }

    return $phase if (Cfg::opt(qw(rootpath)));

    unless ($cpic->{reboot} || (Cfg::opt("nostart")) ||
            (Cfg::opt(qw(upgrade patchupgrade hotfixupgrade)) &&
             ($cpic->{prod_not_enabled_before_upgrade} ||
              $cpic->{prod_not_running_before_upgrade}))) {

        # define the product start proc lists for all systems and per-system
        $cpic->{startprocs}=$prod->startprocs();
        for my $sys (@{$cpic->{systems}}) {
            $sys->{startprocs}=$prod->startprocs_sys($sys);
        }
        $cpic->{startprocs}=$cpic->verify_list('startprocs');

        $id+=100;
        for my $proci (@{$cpic->{startprocs}}) {
            $proc=$cpic->proc($proci);
            next if $proc->{donotstart};

            $task_name = "startup_$proc->{proc}";
            %task_params=(
                'sequence_id' => $id,
                'description' => Msg::new("Starting $proc->{proc}", 30, 1382, "$proc->{proc}"),
                'description_sys' => Msg::new("Starting $proc->{proc} on #{SYS}", 30, 1387, "$proc->{proc}"),
                'if_per_system' => 1,
                'handler' => \&CPIC::proc_start_sys,
                'handler_object_arg' => $cpic,
                'handler_args' => [ $proci, 1 ],
                'rc_handler' => \&CPIC::proc_start_rc_check_sys,
                'rc_handler_object_arg' => $cpic,
                'rc_handler_args' => [ $proci ],
            );
            $task=Task->new($task_name, %task_params);
            $phase->add_task($task);

            if (($#{$cpic->{systems}}>0) && ($proc->{multisystemserialstart})) {
                $task->set_serial(1);
            }
            for my $sys (@{$cpic->{systems}}) {
                if (!EDRu::inarr($proci,@{$sys->{startprocs}}) ||
                    EDRu::inarr($proci,@{$sys->{donotstartprocs}})) {
                    $task->set_skip_sys($sys,1);
                }
            }
            $id++;
        }

        $id+=100;
        $reboot_systems=$cpic->systems_need_reboot();
        unless (@{$reboot_systems}) {
            if ($prod->can(upgrade_sub('poststart_sys'))) {
                $task_name="product_poststart";
                %task_params=(
                    'sequence_id' => $id,
                    'description' => Msg::new("Performing $prod->{abbr} poststart tasks", 30, 1388, "$prod->{abbr}"),
                    'description_sys' => Msg::new("Performing $prod->{abbr} poststart tasks on #{SYS}", 30, 1389, "$prod->{abbr}"),
                    'if_per_system' => 1,
                    'handler' => \&CPIC::prod_poststart_sys,
                    'handler_object_arg' => $cpic,
                );
                $task=Task->new($task_name, %task_params);
                $phase->add_task($task);
                if (($#{$cpic->{systems}}>0) && ($prod->{multisystemserialpoststart})) {
                    $task->set_serial(1);
                }
            }
        }
    }

    return $phase;
}

sub poststartup {
    my $cpic=shift;
    my ($prod,$msg,$failmsg,$failures);

    $prod=$cpic->prod;

    if ($cpic->{reboot} || (Cfg::opt("nostart")) ||
        (Cfg::opt(qw(upgrade patchupgrade hotfixupgrade)) &&
         ($cpic->{prod_not_enabled_before_upgrade} ||
          $cpic->{prod_not_running_before_upgrade}))) {
        $msg=Msg::new("$prod->{name} Configure completed successfully", 30, 1390, "$prod->{name}");
        $failmsg=Msg::new("$prod->{name} Configure did not complete successfully", 30, 1391, "$prod->{name}");
        $msg=$failmsg if ($prod->{has_poststart_config} && (Cfg::opt('configure')));
    } else {
        $msg=Msg::new("$prod->{name} Startup completed successfully", 30, 1392, "$prod->{name}");
        $failmsg=Msg::new("$prod->{name} Startup did not complete successfully", 30, 1393, "$prod->{name}");
    }
    $failures=$cpic->failures('startfailmsg');
    $msg->display_completion($failures, $msg, $failmsg);

    $cpic->prod_poststart_errors();

    EDR::die('start failures') if ((Cfg::opt('startstoptest')) &&
                                   ($#$failures>=0));

    $cpic->update_summary_start();

    return 1;
}

sub prod_configure_errors {
    my $cpic=shift;
    my $prod=$cpic->prod;
    $prod->config_errors($cpic) if ($cpic->sys_error);
    EDR::reset_errors_warnings();
    return 1;
}

#sub ask_start_processes {
#    my($ayn,$cpic,$prod,$help,$msg);
#    ($cpic,$prod)=@_;
#    return "" if ($prod->{autostart} || Cfg::opt(qw(start responsefile)));
#    $msg=Msg::new("Do you want to start $prod->{abbr} processes now?");
#    $help=Msg::new("Enter 'y' if want to start processes, otherwise, enter 'n'");
#    Msg::n();
#    $ayn=$msg->ayny($help,0);
#    return 1 if ($ayn eq "N");
#    return "";
#}

sub prod_poststart_sys {
    my ($cpic,$sys) = @_;
    my ($tmpsub,$mr);
    return 1 if (Cfg::opt('makeresponsefile'));
    # scalars must be passed to threaded call
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $tmpsub = upgrade_sub('poststart_sys');
    $sys->prod->$tmpsub($sys);
    if ($mr = $cpic->{mrobj}) {
        $mr->poststart_sys($sys);
    }
    for my $hotfix(@{$cpic->{hotfixobj}}) {
        $hotfix->poststart_sys($sys);
    }
    return 1;
}

# not using OO call, $cpic not needed and for thread/serial consistency
sub proc_start_sys {
    my ($cpic,$sys,$proc,$checkproclist) = @_;
    my ($sleep,$edr,$running,$rtn,$msg);
    $edr=Obj::edr();
    # scalars must be passed to threaded call
    $sys=Obj::sys($sys) unless (ref($sys) eq 'Sys');
    $proc=$sys->proc($proc) unless (ref($proc));
    return 1 if (Cfg::opt('makeresponsefile'));
    # proc start is conditional
    return if ($checkproclist &&
        (!EDRu::inarr($proc->{proci}, @{$sys->{startprocs}}) ||
         EDRu::inarr($proc->{proci},@{$sys->{donotstartprocs}})));

    $edr->{logfile2}="$edr->{tmpdir}/start."."$proc->{proc}."."$sys->{sys}";

    if ($sys->{abort_startup} ||
        EDRu::inarr($proc->{proc}, @{$sys->{abort_procs}})) {
        Msg::log("## Aborted to start $proc->{proc} on $sys->{sys}");
        return;
    }

    Msg::log("## Starting $proc->{proc} on $sys->{sys}");
    # initial check
    Msg::log("## Begin prestart checking for $proc->{proc} on $sys->{sys}");
    $running=$proc->check_sys($sys, 'prestart');
    Msg::log("## The prestart checking return '$running'");
    if ($running) {
        Msg::log("## $proc->{proc} is already running on $sys->{sys}");
        $rtn=1;
    } else {
        # start the process
        Msg::log("## Begin starting $proc->{proc} on $sys->{sys}");
        $rtn=$proc->start_sys($sys);
        Msg::log("## The start_sys() return '$rtn'");
        if ($rtn) {
            # confirm startup, may retry check for a defined period
            Msg::log("## Begin poststart checking for $proc->{proc} on $sys->{sys}");
            $running=$proc->check_sys($sys, 'poststart');
            Msg::log("## The poststart checking return '$running'");
            if (!$running) {
                $sleep=0;
                while ($sleep<$proc->{start_period}) {
                    $sleep+=10;
                    sleep 10;
                    Msg::log('## Sleep 10, and do poststart checking again');
                    $running=$proc->check_sys($sys, 'poststart');
                    Msg::log("## The poststart checking return '$running'");
                    last if ($running);
                }
            }
            if ($running) {
                Msg::log("## $proc->{proc} is successfully started on $sys->{sys}");
                $rtn=1;
            } else {
                Msg::log("## $proc->{proc} is not successfully started on $sys->{sys}");
                $rtn=0;
            }
        } else {
            Msg::log('## The start_sys() directly failed');
            $rtn=0;
        }

        if ($rtn==0) {
            # Start failed.
            # To show debug informations in start_failed_sys().
            $proc->start_failed_sys($sys);

            # abort those processes which depend on this proc.
            $sys->set_value('abort_startup', 1) if ($proc->{abort_startup});
            if (defined $proc->{abort_procs}) {
                $sys->set_value('abort_procs', 'push', @{$proc->{abort_procs}});
            }
        }
    }

    Msg::log("## The proc_start_sys() return '$rtn'");
    $edr->{logfile2}='';

    # adding temp failures to test the web script
    return 0 if ($cpic->false_failure);
    return $rtn;
}

# $cpic ref not passed when called through $thr->mq_read_rtns
sub proc_start_failed_sys {
    my($sys,$proc)= @_;
    $proc=$sys->proc($proc) unless (ref($proc));
    push(@{$sys->{startfail}}, $proc->{proc});
    $sys->{proc_reboot}=1 if (!$proc->{noreboot_failstart});
    my $msg=Msg::new("$proc->{proc} failed to start on $sys->{sys}", 30, 1394, "$proc->{proc}", "$sys->{sys}");
    $msg->log;
    push(@{$sys->{startfailmsg}}, $msg);
    EDRu::appendfile($proc->{proc}, EDR::tmpdir()."/start_fail.$sys->{sys}")
        if (Cfg::opt('bait'));
    return '';
}

sub proc_start_passed_sys {
    my($sys,$proc,$checkproclist)= @_;
    $proc=$sys->proc($proc) unless (ref($proc));
    # proc start is conditional
    return '' if ($checkproclist &&
        (!EDRu::inarr($proc->{proci}, @{$sys->{startprocs}}) ||
        EDRu::inarr($proc->{proci},@{$sys->{donotstartprocs}})));
    push(@{$sys->{startpass}}, $proc->{proc});
    Msg::log("$proc->{proc} started successfully on $sys->{sys}");
    EDRu::appendfile($proc->{proc}, EDR::tmpdir()."/start_pass.$sys->{sys}")
        if (Cfg::opt('bait'));
    return '';
}

sub proc_start_aborted_sys {
    my($sys,$proc)= @_;
    $proc=$sys->proc($proc) unless (ref($proc));
    return '' if (!EDRu::inarr($proc->{proc}, @{$sys->{abort_procs}}));
    push(@{$sys->{startfail}}, $proc->{proc});
    $sys->{proc_reboot}=1 if (!$proc->{noreboot_failstart});
    my $msg=Msg::new("$proc->{proc} aborted to start on $sys->{sys}", 30, 1395, "$proc->{proc}", "$sys->{sys}");
    $msg->log;
    push(@{$sys->{startfailmsg}}, $msg);
    EDRu::appendfile($proc->{proc}, EDR::tmpdir()."/start_fail.$sys->{sys}")
        if (Cfg::opt('bait'));
    return '';
}

sub proc_start_rc_check_sys {
    my ($cpic,$sys,$proc,$rc) = @_;
    $proc=$sys->proc($proc) unless (ref($proc));
    if ($rc) {
        proc_start_passed_sys($sys, $proc);
    } elsif (defined $rc) {
        proc_start_failed_sys($sys, $proc);
    } else {
        proc_start_aborted_sys($sys, $proc);
    }
    return $rc;
}

sub startstoptest {
    my $cpic=shift;
    my $n=0;
    while(1) {
        $n++;
        Msg::print("startstoptest $n:");
        $cpic->startup();
        $cpic->shutdown();
    }
    return;
}

# Return 0 meaning no abort.
# Return 1 meaning the proc is partially aborted on some systems.
# Return 2 meaning the proc is aborted on all systems.
sub proc_abort_startup_status {
    my ($cpic,$proc)=@_;
    my ($num,$nsystems);

    $num=0;
    $nsystems=scalar @{$cpic->{systems}};
    for my $sys (@{$cpic->{systems}}) {
        if ($sys->{abort_startup} ||
            EDRu::inarr($proc->{proc}, @{$sys->{abort_procs}})) {
            $num++;
        }
    }
    if ($num) {
        return 1 if ($num < $nsystems);
        return 2 if ($num == $nsystems);
    }
    return 0;
}

sub prod_poststart_errors {
    my $cpic=shift;
    my ($prod,$edr);
    $prod=$cpic->prod;
    $edr=Obj::edr();
    $prod->poststart_errors($cpic) if ($cpic->sys_error);
    $edr->display_errors_warnings('','');
    EDR::reset_errors_warnings();
    return 1;
}

sub update_summary_start {
    my $cpic=shift;
    my $msg;

    for my $sys (@{$cpic->{systems}}) {
        if ($#{$sys->{startpass}}>=0) {
            $msg=Msg::new("The following processes were started on $sys->{sys}:", 30, 1396, "$sys->{sys}");
            $cpic->update_summary($msg, @{$sys->{startpass}});
        }
        if ($#{$sys->{startfail}}>=0) {
            $msg=Msg::new("The following processes failed to start on $sys->{sys}:", 30, 1397, "$sys->{sys}");
            $cpic->update_summary($msg, @{$sys->{startfail}});
        }
    }
    return '';
}

1;
