#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package CPIC;
use strict;

# main sub for version checker
sub version_check {
    my ($cpic, $deploy_sfha) = @_;
    my ($edr, $cfg, $msg, $padv, $rel);
    my ($sys, $localsys, $systems, $inputsystems, $sysname, $localsysname, $localsys_backup);
    my ($pseudo_hotfix, $noipc, $sortready, $ayn, $rootpath, %updated_padvs, $ident_systems, $rtn);

    $edr = Obj::edr();
    $cfg = Obj::cfg();
    $rel = $cpic->rel;
    $edr->create_local_tempdir();
    $localsys=$edr->{localsys};
    $localsys_backup = $edr->{localsys};
    $localsysname = $localsys->{sys} if ($edr->{script}!~/^deploy/);
    $rel->require_versionmatrix_lib();

    $rootpath=Cfg::opt('rootpath');

    $noipc = $cfg->{opt}{noipc};
    # if -noipc option is not used, check network connection to SORT server for Patch/Hotfix check; Otherwise, we ignore this check
    if ($noipc == 0) {
        if (EDRu::is_port_connectable("sort.symantec.com",443,0,1)) {
            $sortready = 1;
        } else {
            if (Cfg::opt('history')) {
                $sortready = 0;
            } else {
                Msg::n();
                $msg = Msg::new("The version checker cannot connect to the SORT website. Due to lack of access to release matrix files on the SORT website, the results returned from the version checker may not be up to date.", 30, 1480);
                $msg->warning();
                Msg::n();
                $msg = Msg::new("Do you want to continue?", 30, 1481);
                $ayn = $msg->aynn();
                return '' if ($ayn eq 'N');
                $sortready = 0;
            }
        }
    } else {
        Msg::n();
        $msg = Msg::new("With the -noipc option, the version checker disables all attempts to make outbound networking calls to the SORT website. Due to lack of access to release matrix files on the SORT website, the results returned from the version checker may not be up to date.", 30, 1482);
        $msg->print;
        Msg::prtc();
        $sortready = 0;
    }


    # check local system if -history specified for version history
    if (Cfg::opt('history')) {
        if ($cfg->{systems} && @{$cfg->{systems}}) {
            $sys = $cfg->{systems}[0];
            $sys = Sys->new($sys);
            if (! $edr->transport_sys($sys)) {
                Msg::n();
                $msg = Msg::new("Communication error occurred for $sys->{sys}", 30, 1196, "$sys->{sys}");
                $msg->warning();
                return;
            }
        } else {
            $sys = $localsys;
        }

        $sys->pkgs_patches();
        $padv=$rel->get_padv_for_vc($sys);
        $rel->{padv_managed_host_for_vc} = $padv;
        $rel->init_releasematrix_vc();
        if ( $rel->can('output_installed_pkgs_patches_for_version_history')) {
            $rel->output_installed_pkgs_patches_for_version_history($sys);
        } elsif ($cpic->can('output_installed_pkgs_patches_for_version_history')) {
            $cpic->output_installed_pkgs_patches_for_version_history($sys);
        }
        return '';
    }

    while (1) {
        $rel->{sort_ready_for_vc} = $sortready;
        $pseudo_hotfix = ($cfg->{opt}{pseudo_hotfix}) ? 1 : 0;

        if ($deploy_sfha) {
            my $deploy=Obj::deploy();
            if (defined $deploy->{preference}{Repository}{value} && $deploy->{preference}{Repository}{value}) {
                $rel->{repos_set_for_vc} = 1;
                $rel->{repository_path_for_vc} = $deploy->{preference}{Repository}{value};
            } else {
                $rel->{repos_set_for_vc} = 0;
            }
        } else {
            $rel->{repos_set_for_vc} = 1;
            $rel->{repository_path_for_vc} = '/opt/VRTS/install/repository';
        }

        if ($rel->{sort_ready_for_vc}) {
            if ($deploy_sfha) {
                if ($rel->{repos_set_for_vc} == 1) {
                    $rel->{vc_matrix_path_update} = $rel->{repository_path_for_vc}.'/.matrix/';
                    $rel->{matrix_path_in_installpath} = $rootpath.$edr->{installpath}.'/.matrix/';
                } else {
                    $rel->{vc_matrix_path_update} = $rootpath.$edr->{installpath}.'/.matrix/';
                }
            } else {
                $rel->{vc_matrix_path_update} = $rootpath.$edr->{installpath}.'/.matrix/';
            }
        } else {
            #Cloud installer is invoking this sub
            if ($deploy_sfha) {
                if ($rel->{matrix_path}) {
                    #Cloud installer set the $rel->{matrix_path} value, so we do not have to set this value again.
                } else {
                    if ($rel->{repos_set_for_vc} == 1) {
                        $rel->{matrix_path} = $rel->{repository_path_for_vc}.'/.matrix/';
                    }
                }
                $rel->{matrix_path_in_installpath} = $rootpath.$edr->{installpath}.'/.matrix/';
                $rel->{matrix_path_in_prod} = $rootpath.$cpic->{perlpath}.'/lib/site_perl/'.$cpic->{release}.'/CPIP/Rel/Matrix/';
            } else {
                #installer -version is invoking this and we have to decide where the original json files
                if ($edr->{script} eq "installer" || $edr->{script} eq "installmr"|| $edr->{script} eq "installrp" || $edr->{script}=~/install.*HF/) {
                    $rel->{matrix_path} = $rootpath.$edr->{scriptdir}.'/scripts/CPIP/Rel/Matrix/';
                } else {
                    if ($edr->{scriptdir} =~ /\/opt\/VRTS\/install$/) {
                        $rel->{matrix_path} = $rootpath.$cpic->{perlpath}.'/lib/site_perl/'.$cpic->{release}.'/CPIP/Rel/Matrix/';
                    } else {
                        $rel->{matrix_path} = $rootpath.$edr->{scriptdir}.'/../scripts/CPIP/Rel/Matrix/';
                    }
                }
            }
        }

        $systems = $cfg->{systems} if (!$deploy_sfha);

        if (!$systems || !@$systems) {
            if ($cfg->{opt}{hostfile}) {
                $inputsystems = EDRu::readfile($cfg->{opt}{hostfile});
            } else {
                $msg = Msg::new("\nEnter the system names separated by spaces for version checking:", 30, 1483);
                $inputsystems = $msg->ask($localsysname, '');
            }
            @{$systems} = @{EDRu::arruniq(split(/\s+/m, $inputsystems))};
        }
        while ($edr->validate_systemnames(@{$systems})) {
            $msg = Msg::new("\nEnter the system names separated by spaces for version checking:", 30, 1483);
            $inputsystems = $msg->ask($localsysname, '', '');
            @{$systems} = @{EDRu::arruniq(split(/\s+/m, $inputsystems))};
        }

        $rel->{vc_sys} = [];
        $cfg->{systems} = \@{$systems};

        $edr->init_sys_objects();
        Msg::n();
        $rtn=$edr->check_and_setup_transport($edr->{systems});
        return if ($rtn == -1);

        my @ill_systems = ();
        for my $sysname (@{$systems}) {
            $sys = Obj::sys($sysname);
            $msg = Msg::new("Checking installed products on $sysname", 30, 1484, "$sysname");
            $msg->left();
            $sys->pkgs_patches();

            $padv=$rel->get_padv_for_vc($sys);
            $rel->{padv_managed_host_for_vc} = $padv;
            $rel->{vc_host_padv}{$sysname} = $padv;
            $rel->{vc_update_status}{$sysname}{available_ga_update}{id_set} = [];
            $rel->{vc_update_status}{$sysname}{available_patch_update}{id_set} = [];
            $rel->{vc_update_status}{$sysname}{available_hotfix_update}{id_set} = [];

            $rel->{vc_update_status}{$sysname}{available_update}{id_set} = [];

            push @{$rel->{vc_sys}}, $sysname;

            #one padv only need to be updated for one time.
            if (!$updated_padvs{$padv}) {
                $rel->update_matrix_data($pseudo_hotfix, $noipc);
                $updated_padvs{$padv} = 1;
            }

            $rel->init_releasematrix_vc();
            $rel->set_inst_vrtsprods_sys($sys);
            $rel->check_prods_install_status_sys($sys);
            $rel->check_available_update_status_sys($sys);
            $msg->right_done();
        }

        $systems = EDRu::arrdel($systems, @ill_systems);

        for my $sysname (@{$systems}) {
            $sys = Sys->new($sysname);
            $rel->output_prods_install_status_sys($sys);
        }

        $rel->reorganize_systems();

        if (scalar @{$systems}) {
            Msg::n();
            $msg = Msg::new("Would you like to view Available Upgrade Options?", 30, 1485);
            if ($msg->ayny() eq 'Y') {
                for my $prodinfo (keys %{$rel->{prodsys}}) {
                    $sys = undef;
                    $ident_systems= join(' ', @{$rel->{prodsys}{$prodinfo}});

                    Msg::n();
                    $msg = Msg::new("Upgrade options on $ident_systems:", 30, 1486, "$ident_systems");
                    $msg->bold();

                    $sys = Sys->new($rel->{prodsys}{$prodinfo}[0]);
                    $rel->output_upgrade_options_sys($sys);
                }

                $rel->vc_download_patches();
            }
        }
        # Do com cleanup here
        $edr->cleanup_com_setup();

        for my $sysname (@{$systems}) {
            delete $Obj::pool{"Sys::$sysname"};
        }
        $cfg->{systems} = undef;
        $systems = undef if ($deploy_sfha);
        # Clean up the systems that has SSH/RSH established
        for my $key (qw/prodsys relname_to_vers_level relid_to_relname relname_to_relid hotfixname_to_vers_level hotfixid_to_hotfixname hotfixname_to_hotfixid/) {
            delete $rel->{$key};
        }
        $msg = Msg::new("Do you want to version check additional systems?", 30, 1487);
        $ayn=$msg->ayny();
        if ($ayn eq "N") {
            last;
        }
    }

    if (!$Obj::pool{"Sys::$localsysname"}) {
        $Obj::pool{"Sys::$localsysname"} = $localsys_backup;
    }

    Msg::n();
    $msg = Msg::new("Please visit https://sort.symantec.com for more information.", 30, 1260);
    $msg->bold();
    Msg::n();
    return '';
}

1;
