#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#
package Patch;
use strict;
@Patch::ISA = qw(CPIP);

sub pre_init {
    my $patch=shift;
    (undef,$patch->{patch},$patch->{padv})=split(/::/m,$patch->{class},3);
    ($patch->{patchname},undef)=split(/_/m,$patch->{patch});
    # Solaris
    $patch->{patchid}=$patch->{patch};
    $patch->{patchid}=~s/_/-/;
    return '';
}

sub new_patch {
    my ($cpic,$patch,$class,$patchname,$patchvers,$padv,$pool,$refpkg);
    ($class,$patchname,$patchvers,$padv,$refpkg)=(@_);
    $pool="$class\::$patchname\::$padv";
    return $Obj::pool{$pool} if (defined($Obj::pool{$pool}));
    $patch=EDR::create_cpip_object("$class\::$patchname",$padv);
    $patch->{pool}=$pool;
    $patch->{patch}=$patchname unless (defined $patch->{patch});
    ($patch->{patchname},$patch->{patchvers})=split(/_/,$patchname,2);
    $patch->{vers}=$patchvers unless (defined $patch->{vers});
    $patch->{patchi}=$patchname;  # use patchname as patchi for patches not built-in defined
    $patch->{pkg}=$refpkg;
    $cpic=Obj::cpic();
    $cpic->rel->new_patch_init($patch);
    $Obj::pool{$pool}=$patch;
    return $patch;
}

sub verify_media {
    my ($patch,$pkg)=@_;
    my ($cpic,$padv,$patchpath,$rel,$edr,$msg,$patchi,$release,$mediapath);
    $cpic = Obj::cpic();
    $edr = Obj::edr();
    $padv=$patch->padv;
    $rel = $patch->rel;
    $release = $rel->{currentrelease};
    # Added block for patch only release
    $patch->{patchvers}||=$pkg->{patchvers}{$release}{$patch->{patch}} if($pkg->{patchvers}{$release}{$patch->{patch}});
    $patch->{patchvers}||=$rel->{vers};
    $patch->{patchvers}=~s/_/\./g; # In definition version is having _ instead of . due to object creation cli
    $patch->{patchname} = $pkg->{patchname}{$release}{$patch->{patch}};
    $patch->{showform} = $patch->{patchname}."-".$patch->{patchvers};
    $patch->{patch_vers} = (($padv->{plat} eq "SunOS") && ($padv->{vers} eq '5.10')) ?
                    $patch->{patchname}."-".$patch->{patchvers} : $patch->{patchname};
    $patch->{vers}||=$pkg->{patchvers}{$release}{$patch->{patch}} if($pkg->{patchvers}{$release}{$patch->{patch}});
    $patch->{vers}||=$rel->{vers};
    # In definition version is having _ instead of . due to object creation cliche
    $patch->{vers}=~s/_/\./g;

    if ($rel->{type} eq 'H') {
        push (@{$rel->{hf_packages}{$pkg->{pkgi}}}, $patch->{patchvers});
    }


    # initiate the padv release object to define file, vers, size, and space
    if ($cpic->{fromdisk} || $patch->{file}) {
        return '';
    }
    $patchpath=Cfg::opt('patchpath');
    $mediapath=Cfg::opt($patch->{arg}) if (defined $patch->{arg});
    $mediapath||=$cpic->{mediapath};
    $patchpath||=$rel->pkgs_patches_dir('patchpath', $mediapath);
    $patch->{file}=$padv->media_patch_file($patch, $patchpath);
    if ($patch->{file}) {
        $patch->{vers} = ($cpic->common) ? $patch->version :
            $padv->media_patch_version($patch,$patchpath);
        EDR::die("Cannot determine version of $patch->{padv} $patch->{patchname}")
            if (!$patch->{vers});
        $patch->{size}=$cpic->pkg_patch_size($patch->{file});
        $patch->{space}=$patch->space();
    } elsif (Cfg::opt('mpok')) {
        $patch->{mpok}=1;
    } elsif (!$rel->{verify_media}) {
        Msg::log("Cannot find $patch->{patchname} for padv $patch->{padv} in $patchpath");
    } elsif (Cfg::opt('patchpath')) {
        $patchi = $patch->{patch};
        $msg = Msg::new("Cannot find patch $patchi in $patchpath", 30, 1489, "$patchi", "$patchpath");
        $msg->die;
    } else {
        EDR::die("Cannot find patch $patch->{patchname} for padv $patch->{padv} in $patchpath");
    }
    return;
}

# get a defined space from $rel->patch_space
sub space {
    my ($patch,$rel,$line,$space,$opt,$usr,$var,$root);
    $patch=shift;
    $rel=$patch->rel;
    $space=$rel->patch_space || $rel->{patchesspace};
    for my $line (split(/\n/,$space)){
        next unless ($line=~/^$patch->{patch_vers}:/mx);
        (undef,$opt,$usr,$var,$root,undef)=split(/[:,]/m,$line,6);
        return [$opt,$usr,$var,$root];
    }
    Msg::log("No space defined for $patch->{patchname} in $rel->{vers} patch_space\n$space");
    return ([0,0,0,0]);
}

# common Patch subs and trace-only stubs for optional Patch subs

# add a patch to a systems install list
sub add_install_sys {
    my ($patch,$sys)= @_;
    return if ($patch->{mpok});
    my $pkg=$patch->pkg($patch->{pkg});
    if (!$sys->padv->pkg_status_sys($sys,$pkg)) {
        # TBD: confirm that package is going to be installed
        if (!Cfg::opt(qw(base_path patch_path hotfix_path))) {
            my $msg=Msg::new("Cannot install patch $patch->{patchname} because $pkg->{pkg} package is not completely installed on $sys->{sys}", 30, 1490, "$patch->{patchname}", "$pkg->{pkg}", "$sys->{sys}");
            #$msg->die();
        }
    }
    return if (Cfg::opt('upgrade_kernelpkgs') && $pkg->{unkernelpkg});
    Msg::log("Patch $patch->{patch_vers} will be installed on $sys->{sys}");
    push(@{$sys->{installpatches}}, $patch->{patch});
    my $cpic=Obj::cpic();
    push(@{$cpic->{installpatches}}, $patch->{patch})
        unless EDRu::inarr($patch->{patch}, @{$cpic->{installpatches}});
    if ($#{$cpic->{padvs}}) {
        push(@{$cpic->{installpatchpkgs}}, $patch->{pkg})
            unless EDRu::inarr($patch->{pkg}, @{$cpic->{installpatchpkgs}});
    }
    # install this patch in round II if {installpatcheslast} is set to 1
    if ($pkg->{installpatcheslast}) {
        Msg::log("Patch $patch->{patch_vers} will be installed in round II on $sys->{sys}");
        push(@{$sys->{installpatcheslast}}, $patch->{patch})
            unless EDRu::inarr($patch->{patch}, @{$sys->{installpatcheslast}});
        push(@{$cpic->{installpatcheslast}}, $patch->{patch})
            unless EDRu::inarr($patch->{patch}, @{$cpic->{installpatcheslast}});
        if ($#{$cpic->{padvs}}) {
            push(@{$cpic->{installpatchpkgslast}}, $patch->{pkg})
                unless EDRu::inarr($patch->{pkg}, @{$cpic->{installpatchpkgslast}});
        }
    }
    return '';
}

sub add_uninstall_sys {
    my ($patch,$sys) = (@_);
    return if ($patch->{mpok});
    my $cpic = Obj::cpic();
    my $pkg = $sys->pkg($patch->{pkg});
    return if (Cfg::opt('upgrade_kernelpkgs') && $pkg->{unkernelpkg});
    Msg::log("Patch $patch->{patch_vers} will be uinstalled on $sys->{sys}");
    push(@{$sys->{uninstallpatches}}, $patch->{patch})
         unless EDRu::inarr($patch->{patch}, @{$sys->{uninstallpatches}});
    push(@{$cpic->{uninstallpatches}}, $patch->{patch})
        unless EDRu::inarr($patch->{patch}, @{$cpic->{uninstallpatches}});
    if ($#{$cpic->{padvs}}) {
        push(@{$cpic->{uninstallpatchpkgs}}, $patch->{pkg})
            unless EDRu::inarr($patch->{pkg}, @{$cpic->{uninstallpatchpkgs}});
    }
    return '';
}

sub copy_sys {
    my ($patch,$sys)= @_;
    return $sys->padv->patch_copy_sys($sys, $patch);
}

sub copy_mark_sys {
    my ($patch,$sys)= @_;
    my $str;
    if ($^O =~ /Win32/i){
        $str = EDR::tmpdir()."\\cp.patch.$sys->{sys}.$patch->{patch_vers}";
    } else {
        $str = EDR::tmpdir()."/cp.patch.$sys->{sys}.$patch->{patch_vers}";
    }
    return $str;
}

sub wait_for_copy_sys {
    my ($patch,$sys)= @_;
    EDRu::wait_for_file($patch->copy_mark_sys($sys));
    return '';
}

sub install_sys {
    my ($patch,$sys)= @_;
    Msg::log("Installing patch $patch->{patch_vers} on $sys->{sys}");
    return $sys->padv->patch_install_sys($sys, $patch);
}

sub remove_sys {
    my ($patch,$sys)= @_;
    Msg::log("Removing $patch->{patch_vers} temporary files on $sys->{sys}");
    return $sys->padv->patch_remove_sys($sys, $patch);
}

sub install_success_sys {
    my ($patch,$sys)= @_;
    return $sys->padv->patch_install_success_sys($sys, $patch);
}

# determine whether a patch is installed on a system
# each platform stores patch info a little differently
# assumes patch install status has been queried and stored in $sys
sub installed_sys {
    my ($patch,$sys)= @_;
    return if ($sys->linux());
    return $sys->padv->patch_installed_sys($sys, $patch);
}

sub preinstall_sys { }

sub postinstall_sys { }

sub postremove_sys { }

sub uninstall_sys {
    my ($patch,$sys)=(@_);
    return $sys->padv->patch_uninstall_sys($sys, $patch);
}

sub uninstall_success_sys {
    my ($patch,$sys)=(@_);
    return $sys->padv->patch_uninstall_success_sys($sys, $patch);
}

1;
