#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#

package Deploy;
use strict;

@Deploy::ISA = qw(CPIP);

sub set_deploy_sfha_menu_msgs {
    my ($deploy)=@_;
    my ($edr,$msg);

    # menu task selection
    $edr=Obj::edr();
    $edr->{msg}{key_manage_repository}='R';
    $edr->{msg}{key_version_check}='V';
    $edr->{msg}{key_upgrade_install}='U';
    $edr->{msg}{key_update_metadata}='M';
    $edr->{msg}{key_set_preferences}='S';
    $edr->{msg}{key_terminology}='T';
    $edr->{msg}{key_deploy_sfha_help}='?';
    $edr->{msg}{key_quit}='Q';

    # set preference key
    $edr->{msg}{key_back}='B';

    # Update Menu
    $msg=Msg::new("Manage Repository Images", 60, 1000);
    $msg->msg('menu_manage_repository');
    $msg=Msg::new("Version Check Systems", 60, 1001);
    $msg->msg('menu_version_check');
    $msg=Msg::new("Upgrade/Install Systems", 60, 1002);
    $msg->msg('menu_upgrade_install');
    $msg=Msg::new("Update Metadata", 60, 1003);
    $msg->msg('menu_update_metadata');
    $msg=Msg::new("Set Preferences", 60, 1004);
    $msg->msg('menu_set_preferences');
    $msg=Msg::new("Terminology and Usage", 60, 1005);
    $msg->msg('menu_terminology');
    $msg=Msg::new("Help", 60, 1006);
    $msg->msg('menu_deploy_sfha_help');
    $msg=Msg::new("Quit", 60, 1007);
    $msg->msg('menu_quit');
    $msg=Msg::new("deploy_sfha script", 60, 1008);
    $msg->msg('deploy_sfha_script');

    # set preference message
    $msg=Msg::new("Back to Main Menu", 60, 1009);
    $msg->msg('menu_back');
    return 1;
}

sub cli_deploy_sfha_menu {
    my $deploy = shift;
    my ($edr,@tasks,$cpic,$task_k);
    $edr = Obj::edr();
    $cpic=Obj::cpic();
    $edr->set_padv_msgs;
    @tasks=qw(manage_repository update_metadata version_check
              set_preferences upgrade_install terminology deploy_sfha_help quit);
    $deploy->set_deploy_sfha_menu_msgs();

    $deploy->create_deploy_sfha_pid_file($edr->{tmpdir},$edr->{script});
    return $deploy->update_metadata() if (Cfg::opt(qw(load_metadata download_metadata)));
    while(1) {
        $deploy->read_deploy_config();
        $deploy->cleanup_stale_deploy_sfha_locks();
        Msg::title();
        $task_k=$deploy->cli_select_task_menu(@tasks);
        EDR::set_value('savelog', 1);
        EDR::set_value('not_display_summary_file', 1);
        if ($task_k eq Msg::get('key_manage_repository')) {
            $deploy->manage_repository();
        } elsif ($task_k eq Msg::get('key_version_check')) {
            $cpic->version_check("deploy_sfha");
        } elsif ($task_k eq Msg::get('key_upgrade_install')) {
            $deploy->upgrade_install();
        } elsif ($task_k eq Msg::get('key_update_metadata')) {
            $deploy->update_metadata();
        } elsif ($task_k eq Msg::get('key_set_preferences')) {
            $deploy->set_preferences();
        } elsif ($task_k eq Msg::get('key_terminology')) {
            $deploy->terminology();
        } elsif ($task_k eq Msg::get('key_deploy_sfha_help')) {
            $deploy->deploy_sfha_help();
        }
        # quit exits via Msg::menu();
    }
    return 1;
}

sub cli_select_task_menu {
    my ($deploy,@tasks)=(@_);
    my(@mk,@mo,$buf,$be,$bs,$cpic,$edr,$m,$mkey,$mm1,$mm2,$msg,$task_k,$tsl);
    $cpic=Obj::cpic();
    $edr=Obj::edr();
    @mo=@tasks;
    $bs=EDR::get2('tput','bs');
    $be=EDR::get2('tput','be');

    $msg=Msg::new("\nTask Menu:\n", 60, 1010);
    $msg->bold;
    while (@mo) {
        $m=shift(@mo);
        $mkey=Msg::get("key_$m");
        push(@mk,$mkey);
        $mm1="$bs$mkey$be) ".Msg::get("menu_$m");
        $m=shift(@mo);
        if ($m) {
            $mkey=Msg::get("key_$m");
            push(@mk,$mkey);
            $mm2="$bs$mkey$be) ".Msg::get("menu_$m");
        } else {
            $mm2='';
        }
        $buf=Msg::string_sprintf('    %-50s %s', $mm1, $mm2);
        $msg=Msg::new_obj($buf);
        $msg->print;
    }
    Msg::n();

    $tsl=join(',',@mk);
    $tsl="[$tsl]";
    while (!EDRu::inarr($task_k,@mk)) {
        $msg=Msg::new("Enter a Task: $tsl", 60, 1011, "$tsl");
        $task_k=$msg->ask;
        $task_k=uc($task_k);
    }
    return $task_k;
}

# _START_Manage_Repository
sub init_repository {
    my ($deploy)=@_;
    my ($part1,$part2,$part3,$part4);
    my ($msg1,$msg2,$msg3);

    # should eventually compute this based on our lowest supported version which is currently 5.1
    $deploy->{repo}{padvs} = [qw(aix53 aix61 aix71 hpux1131 rhel5_x86_64 rhel6_x86_64 sles10_x86_64 sles11_x86_64 sol9_sparc sol10_sparc sol10_x64 sol11_sparc sol11_x64)];
    $deploy->{repo}{levels} = [qw(ga patch hotfix)];
    $msg1=Msg::new("Base", 60, 1012);
    $msg2=Msg::new("Maintenance", 60, 1013);
    $msg3=Msg::new("Hot Fix", 60, 1014);
    $deploy->{repo}{leveltitles} = [$msg1->{msg},$msg2->{msg},$msg3->{msg}];
    $deploy->{repo}{details_to_view}= [qw(release_version release_name release_type file_name release_date downloaded autoinstall install_path upload_location repository_location readme)];
    $deploy->{repo}{details_to_view_obsolete} = [qw(obsoletes obsoleted_by)];
    $part1 = ['release_version',[qw(release_name SORT_release_name)]];
    $part2 = [[qw(downloaded DL)]];
    $part3 = [[qw(obsoleted_repo OBS)],[qw(auto_install AI)]];
    $part4 = [[qw(release_date rel_date)],[qw(upload_size_kb size_KB)]];

    $deploy->{repo}{header_ga_available} = [@$part1,@$part2,@$part4];
    $deploy->{repo}{header_patch_available} = [@$part1,@$part2,@$part3,@$part4];
    $deploy->{repo}{header_hotfix_available}= $deploy->{repo}{header_patch_available};

    $deploy->{repo}{header_ga_local} = [@$part1];
    $deploy->{repo}{header_patch_local} = [@$part1,@$part3];
    $deploy->{repo}{header_hotfix_local}= $deploy->{repo}{header_patch_local};
    $deploy->update_release_matrix();
    return 1;
}

# Modify/Add some values in memory after retrieving release matrix information
sub update_release_matrix {
    my ($deploy)=@_;
    my ($rel,$padvs,$levels,$matrix);
    $rel=Obj::rel();
    # set the obsoleted value to Y
    $padvs = $deploy->{repo}{padvs};
    $levels = $deploy->{repo}{levels};
    for my $padv (@$padvs) {
        for my $level (@$levels) {
            $matrix=$rel->get_release_matrix($padv,$level);
            while (my ($key,$value) = each %$matrix) {
                $value->{obsoleted_repo}='Y' if $value->{obsoleted_by};
                $value->{upload_size_kb}=sprintf("%d",$value->{upload_size} / 1024) if($value->{upload_size});
                $value->{auto_install}='Y' if $value->{install_path};
                # For hot fix version < 6.1.0 we block Deployment Server not allow auto installation.
                undef $value->{auto_install} if (($level eq 'hotfix') && ($value->{vers_level} lt '610'));
            }
        }
    }

    return 1;
}

sub manage_repository {
    my ($deploy) = @_;
    my ($msg,$opt1,$opt2,$opt3,$done,$help,$menuopt,$menu,$rtn);

    return unless ($deploy->repository_set_home);
    $deploy->init_repository;
    $deploy->image_repository_scan($deploy->{preference}{Repository}{value});
    $msg=Msg::new("View/Download Available Releases", 60, 1015);
    $opt1=$msg->{msg};
    $msg=Msg::new("View/Remove Repository Images", 60, 1016);
    $opt2=$msg->{msg};
    $msg=Msg::new("Load a Release Image File", 60, 1017);
    $opt3=$msg->{msg};
    $help=Msg::new("Choose one menu item to manage the release images in your repository.", 60, 1018);
    while (!$done) {
        Msg::title();
        $menuopt=[$opt1,$opt2,$opt3];
        $msg=Msg::new("Select the option you would like to perform", 60, 1019);
        $menu=$msg->menu($menuopt,'',$help,'back');
        return $menu if (EDR::getmsgkey($menu,'back'));

        $rtn = $deploy->repository_view_download_available if ($menu==1);
        $rtn = $deploy->repository_view_remove_image       if ($menu==2);
        $rtn = $deploy->repository_load_image              if ($menu==3);
        # return to main menu only if function return 1.
        $done=1 if ($rtn);
    }

    return 1;
}

sub repository_set_home {
    my ($deploy) = @_;
    my ($msg,$dir,$localsys,$result);
    my $cpic=Obj::cpic();
    my $edr=Obj::edr();
    unless (defined $deploy->{preference}{Repository}{value} && $deploy->{preference}{Repository}{value}) {
        while (1) {
            Msg::n();
            $msg=Msg::new("In what directory would you like to store release images?", 60, 1020);
            $dir=$msg->ask($deploy->{preference}{Repository}{default},0,1);
            return 0 if (EDR::getmsgkey($dir,'back'));
            if ($dir !~ m{^/}) {
                $dir=EDRu::abs_path($dir);
                $msg=Msg::new("Would you like to create repository directory under $dir?", 60, 1021, "$dir");
                next if ($msg->ayny eq 'N');
            }
            $dir=$cpic->path($dir);
            $localsys=$edr->{localsys};
            last if ($deploy->verify_Repository($localsys,$dir));
        }
        # set to $deploy->{preference}
        $deploy->{preference}{Repository}{value}=$dir;
        $deploy->write_deploy_config();
    }
    return 1;
}

sub image_repository_scan {
    my ($deploy,$repository) = @_;
    my ($cfg,$repos,$image_repo,$repo_dir,$repo_info,$rtn,
        $targz,$padv,$rel_id,$install_path,$version);

    $cfg=Obj::cfg();
    $repository=$deploy->get_image_repository_dir();
    return 1 if (!$repository || !-d $repository);
    $repository=~s/\/$//;

    $deploy->repository_reset_matrix();
    # unset $deploy->{image_repository} before setting any value.
    # in case the value still exists during rescan
    undef ($deploy->{image_repository});
    for my $matrix (sort keys %{$deploy->{image_repository_defaults}}) {
        $repo_dir="$repository/$deploy->{image_repository_defaults}{$matrix}";
        $rtn=EDR::cmd_local("_cmd_ls -l $repo_dir/images");
        next if ($?);
        for my $image (split /\n/,$rtn) {
            # all the images should be a directory, not a file
            next unless ($image =~ /^d\w+/);
            $image=(split /\s+/,$image)[-1];
            $targz=$image . ".tar.gz";
            $repo_info=$deploy->image_repository_sanity_check($matrix,$repo_dir,$image,$targz);
            for my $item (@$repo_info) {
                next unless ($item =~ /:/);
                ($padv,$rel_id,$version,$install_path)=(split /:/, $item);
                $deploy->{image_repository}{$padv}{$version}{image} ="$repo_dir/images/$image";
                $deploy->{image_repository}{$padv}{$version}{targz} ="$repo_dir/targz/$targz" if (-f "$repo_dir/targz/$targz");
                $deploy->{image_repository}{$padv}{$version}{type}  ="$deploy->{image_repository_defaults}{$matrix}";
                $deploy->{image_repository}{$padv}{$version}{rel_id}=$rel_id;
                $deploy->{image_repository}{$padv}{$version}{install_path}=$install_path;
            }
        }
    }
    return 1;
}

# basic sanity check of an image repository per release
sub image_repository_sanity_check {
    my ($deploy,$matrix,$repo_dir,$release,$targz) = @_;
    my ($rel,$file,@padv_rel_id);
    my $rel=Obj::rel();
    @padv_rel_id=();
    return \@padv_rel_id unless (-d "$repo_dir/images/$release");
    return \@padv_rel_id unless ($rel->{$matrix});
    for my $padv (sort keys %{$rel->{$matrix}}) {
        for my $rel_id (sort keys %{$rel->{$matrix}{$padv}}) {
            if ($rel->{$matrix}{$padv}{$rel_id}{filename} &&
                $rel->{$matrix}{$padv}{$rel_id}{filename} eq $targz) {
                $deploy->download_cksum_verification($matrix,$padv,$rel_id,"$repo_dir/targz");
                $rel->{$matrix}{$padv}{$rel_id}{downloaded}="Y";
                $rel->{$matrix}{$padv}{$rel_id}{repository_location}="$repo_dir/images/$release/";
                if ($rel->{$matrix}{$padv}{$rel_id}{install_path}) {
                    $file="$repo_dir/images/$release/$rel->{$matrix}{$padv}{$rel_id}{install_path}";
                    $file=$deploy->find_repository_padv_installer($file);
                    next unless (-f $file);
                    push (@padv_rel_id, "$padv:$rel_id:$rel->{$matrix}{$padv}{$rel_id}{release_version}:$file");
                }
            } elsif ( $rel->{$matrix}{$padv}{$rel_id}{release_name} &&
                     ($release =~ /^$rel->{$matrix}{$padv}{$rel_id}{release_name}(|-patches|-rpms)/)) {
                $rel->{$matrix}{$padv}{$rel_id}{downloaded}="Y";
                $rel->{$matrix}{$padv}{$rel_id}{repository_location}="$repo_dir/images/$release/";
                if ($rel->{$matrix}{$padv}{$rel_id}{install_path}) {
                    $file="$repo_dir/images/$release/$rel->{$matrix}{$padv}{$rel_id}{install_path}";
                    $file=$deploy->find_repository_padv_installer($file);
                    next unless (-f $file);
                    push (@padv_rel_id, "$padv:$rel_id:$rel->{$matrix}{$padv}{$rel_id}{release_version}:$file");
                }
            } elsif ( $rel->{$matrix}{$padv}{$rel_id}{upload_location} &&
                     ($rel->{$matrix}{$padv}{$rel_id}{upload_location} =~ /$targz$/)) {
                $rel->{$matrix}{$padv}{$rel_id}{downloaded}="Y";
                $rel->{$matrix}{$padv}{$rel_id}{repository_location}="$repo_dir/images/$release/";
                if ($rel->{$matrix}{$padv}{$rel_id}{install_path}) {
                    $file="$repo_dir/images/$release/$rel->{$matrix}{$padv}{$rel_id}{install_path}";
                    $file=$deploy->find_repository_padv_installer($file);
                    next unless (-f $file);
                    push (@padv_rel_id, "$padv:$rel_id:$rel->{$matrix}{$padv}{$rel_id}{release_version}:$file");
                }
            }
        }
    }
    return \@padv_rel_id;
}

sub download_cksum_verification {
    my ($deploy,$matrix,$padv,$rel_id,$repo_dir) = @_;
    my ($correct_cksum,$filename,@cksum_size,$cksum,$size);

    return;
    $correct_cksum="";
    @cksum_size=($deploy->{$matrix}{$padv}{$rel_id}{upload_cksum},
                 $deploy->{$matrix}{$padv}{$rel_id}{upload_size},
                 $deploy->{$matrix}{$padv}{$rel_id}{gunzip_cksum},
                 $deploy->{$matrix}{$padv}{$rel_id}{gunzip_size});
    $filename=$deploy->{$matrix}{$padv}{$rel_id}{filename};
    $filename="$repo_dir/$filename";

    if ((-f $filename) && ($#cksum_size > 0)) {
        $cksum=EDRu::cksum($filename);
        $size=EDR::cmd_local("_cmd_ls -l $filename");
        $size=(split /\s+/, $size)[4];
        if ($filename =~ /\.tar.gz$/) {
            $correct_cksum="Y" if (($cksum eq $cksum_size[0]) &&
                                   ($size  eq $cksum_size[1]));
        } elsif ($filename =~ /\.tar$/) {
            $correct_cksum="Y" if (($cksum eq $cksum_size[2]) &&
                                   ($size  eq $cksum_size[3]));
        }
    }
    $deploy->{$matrix}{$padv}{$rel_id}{downloaded}="Y" if ($correct_cksum);

    return;
}

sub find_repository_padv_installer {
    my ($deploy,$file)=@_;
    my ($list,$installer);

    if (-d $file) {
        $list=EDR::cmd_local("_cmd_find $file -maxdepth 1 -type f -name \"install*\"");
        $installer=(split /\n/,$list)[0];
    } elsif (-f $file) {
        $installer=$file;
    }

    return $installer;
}

sub repository_reset_matrix {
    my ($deploy) = @_;
    my ($rel,$padvs,$levels,$matrix);
    $rel = Obj::rel();
    $padvs = $deploy->{repo}{padvs};
    $levels = $deploy->{repo}{levels};
    for my $padv (@$padvs) {
        for my $level (@$levels) {
            $matrix=$rel->get_release_matrix($padv,$level);
            while (my ($key,$value) = each %$matrix) {
                delete $value->{downloaded};
            }
        }
    }
    return 1;
}

# In View/Download Available option, all Padvs/Levels will be shown
# In View/Remove Repository option, only Padvs/Levels that have images downloaded will be shown
sub repository_ask_padv_level {
    # if $action eq 'repo' -> Only show Padvs/Levels with images downloaded
    # if $action eq 'all'  -> Show all Padv/Levels retrieved from $padvs/$levels
    # if $padv or $level specified, only $level or $padv will be asked
    my ($deploy,$action,$padvs,$levels,$padv,$level) = @_;
    my ($msg,$menu,$userpadvs,$padvmsg,$repoleveltitles);

    unless ($padv) {
        if($action eq 'repo') {
            $padvs = $deploy->repository_downloaded_padvs($padvs);
            unless(scalar @$padvs) {
                $msg=Msg::new("No downloaded images", 60, 1022);
                $msg->print;
                Msg::prtc();
                return '__back__';
            }
        }
        $userpadvs = [];
        @$userpadvs = map {Msg::get($_)} @$padvs;

        if ($action eq 'repo') {
            $msg=Msg::new("Select the platform of the release to view/remove", 60, 1023);
        } else {
            $msg=Msg::new("Select the platform of the release to view/download", 60, 1024);
        }

        $menu=$msg->menu($userpadvs,scalar(@$userpadvs)==1,'','back',0,0,2);
        return $menu if (EDR::getmsgkey($menu,'back'));
        $padv = $padvs->[$menu - 1];
        $padvmsg=Msg::get($padv);
        Msg::n();
    }

    unless ($level) {
        if($action eq 'repo') {
            $levels = $deploy->repository_downloaded_levels($levels,$padv);
            unless(scalar @$levels) {
                Msg::n();
                $msg=Msg::new("No downloaded images", 60, 1022);
                $msg->print;
                sleep 2;
                return '__back__';
            }
        }
        if ($action eq 'repo') {
            $msg=Msg::new("Select the level of the $padvmsg release to view/remove", 60, 1025, "$padvmsg");
        } else {
            $msg=Msg::new("Select the level of the $padvmsg release to view/download", 60, 1026, "$padvmsg");
        }
        my $help=Msg::new("Base releases are major releases that allow you to perform fresh installations and select products, including new features, and are installable over most prior versions. You can use base releases to install package based updates and perform automated installations. You can obtain base releases from FileConnect (https://fileconnect.symantec.com).\n\nMaintenance releases are patch release updates that include multiple fixes and limited new features that are installable over any product of one specific base release version. Maintenance releases allow you to install patch-based updates (on native-patch based platforms) and perform automated installations. You can obtain maintenance releases from SORT (https://sort.symantec.com).\n\nHot fix releases are product release updates that include fixes only, typically for one specific issue, and are installable over specific products of one specific base or maintenance release version. Hot fix releases allow you to install patch-based updates (on native-patch based platforms). You can obtain hot fix releases from SORT (https://sort.symantec.com).", 60, 1027);

        $repoleveltitles=$deploy->repo_level_titles($levels);
        $menu=$msg->menu($repoleveltitles,scalar(@$repoleveltitles)==1,$help,'back');
        return $menu  if (EDR::getmsgkey($menu,'back'));
        $level = $levels->[$menu - 1];
    }

    # Ensure the $padv,$level matrix file is the latest
    $deploy->repository_update_matrixfile($padv, $level);

    return ($padv,$level);
}

sub repo_level_titles {
    my ($deploy,$levels) = @_;
    my ($leveltitles,$i);

    for my $level (@$levels) {
        for ($i=0; $i<=$#{$deploy->{repo}{levels}};$i++) {
            next unless ($level eq $deploy->{repo}{levels}[$i]);
            push (@{$leveltitles},$deploy->{repo}{leveltitles}[$i]);
        }
    }
    return $leveltitles;
}

sub repository_update_matrixfile {
    my ($deploy,$padv,$level) = @_;
    my ($response,$url,$dir,$jsonhash,$filename,$matrix,$msg);
    my ($time1,$time2,$totaltime,$clocktime);
    my $rel=Obj::rel();
    my $cpic=Obj::cpic();

    return 0 if EDRu::inarr('-noipc', @ARGV);
    # Connecting to SORT, print message.
    $msg=Msg::new("\nRetrieving updated $padv $level release information from SORT", 60, 1028, "$padv", "$level");
    $msg->left();
    $time1 = time;
    $url = CPIP::get_hotfix_sort_url();
    $url .= "?product=SFHA&pretty=1&padv=$padv&level=$level";
    Msg::log( "Downloading content from $url");
    $response = EDRu::download_page($url);
    # No need to adjust the result since 'pretty=1' provided in the $url
    # $response =~ s/\n+|\r+|\t+|\f+//mg;
    # $response =~ s/^.*<pre>//mg;
    # $response =~ s/<\/pre>.*$//mg;
    # $response =~ s/(.*)}.*$/$1}/mg;
    if ($response) {
        # Msg::log("RESULT is :$response");
        $msg->right_done();
        $cpic->{sortready}=1;
        $jsonhash = EDRu::eval_json($response);
        return 0 if($jsonhash == 0);
    } else {
        $msg->right_failed();
        $cpic->{sortready}=0;
        Msg::log("Encounter error during contacting Sort");
        return 0;
    }
    $time2 = time;
    $totaltime = $time2 - $time1;
    $clocktime=EDRu::clocktime($totaltime);
    Msg::log( "Downloading json file for $padv-$level took $totaltime seconds($clocktime)");

    $matrix = (values %$jsonhash)[0];
    if ($level eq 'ga') {
        $rel->{vc_release_matrix_data}{$padv} = $matrix;
    } elsif ($level eq 'patch') {
        $rel->{vc_patch_matrix_data}{$padv}   = $matrix;
    } elsif ($level eq 'hotfix') {
        $rel->{vc_hotfix_matrix_data}{$padv}  = $matrix;
    } else {
        Msg::log("$level is not recongnized");
        return 0;
    }

    $dir = '/opt/VRTS/install/.matrix';
    EDRu::mkdir_local_nosys($dir) unless -d $dir;
    $filename = "$dir/$padv-$level.json";
    EDRu::writefile("$response\n","$filename");

    $dir = "$deploy->{preference}{Repository}{value}/.matrix";
    # if the repository reference is set, also need to update the json file in $dir
    EDRu::mkdir_local_nosys($dir) unless -d $dir;
    $filename = "$dir/$padv-$level.json";
    EDRu::writefile("$response\n","$filename");

    # Todo: Currently image_repository_scan will scan the whole local repository that stores releases.
    # But actually only one json file is changed, we can add some function to only scan specific $level && $padv
    # Same goes to function update_release_matrix.
    $deploy->image_repository_scan($deploy->{preference}{Repository}{value});
    $deploy->update_release_matrix();
    return 1;
}

sub refresh_repository_table_content {
    my ($deploy,$padv,$level,$action)=@_;
    my ($rel,$matrix,$titles,$contents);
    $rel=Obj::rel();
    $matrix=$rel->get_release_matrix($padv,$level);
    $contents = [];
    @$contents = sort {$a->{vers_level} cmp $b->{vers_level}} values %$matrix;
    @$contents = grep {(defined $_->{release_type}) && ($_->{release_type} ne 'HF')} @$contents;
    @$contents = grep {$_->{vers_level} ge '510'} @$contents;
    if ($action eq 'repo') {
        $titles=$deploy->get_titles_local($level);
        @$contents = grep {$_->{downloaded} =~ /y/i} @$contents;
    } else {
        $titles=$deploy->get_titles_available($level);
        @$contents = grep {$_->{in_development} != 1} @$contents;
    }
    return ($contents,$titles);
}

# return 1 if user agree to go to upgrade/install
# Todo: We need to figure out exact when should we prompt this message
# based on release version, padv ...
sub repository_download_to_upgrade_install {
    my ($deploy,$target)=@_;
    my ($msg);
    Msg::n();
    $msg=Msg::new("Would you like to go to the U) option to perform upgrade/install on your systems?", 60, 1029);
    if ($msg->ayny eq 'Y') {
        $deploy->upgrade_install();
        return 1;
    }
    return 0;
}

sub repository_view_download_available {
    my ($deploy) = @_;
    my ($action,$ayn,$cfg,$padvs,$levels,$msg,$done,$padv,$level,$titlemsg,$contents,$titles,$rtn,$details,$details_obs,$targets,$help);
    my $cpic=Obj::cpic();
    $cfg = Obj::cfg();
    $action = 'all';
    $padvs = $deploy->{repo}{padvs};
    $levels = $deploy->{repo}{levels};
    $details= $deploy->{repo}{details_to_view};
    $details_obs = $deploy->{repo}{details_to_view_obsolete};

    $titlemsg = Msg::new("To view or download available releases, the platform type and release level type must be selected.\n", 60, 1030);

    while (!$done) {
        Msg::title();
        $titlemsg->print;
        ($padv,$level) = $deploy->repository_ask_padv_level($action,$padvs,$levels);
        for my $op ($padv,$level) {
            return 0 if (EDR::getmsgkey($op,'back'));
        }

        ($contents,$titles) = $deploy->refresh_repository_table_content($padv,$level,$action);
        $msg=$deploy->available_releases_msg($level,$padv);
        $msg->bold;
        Msg::table($contents,$titles);
        unless (scalar @$contents) {
            Msg::n();
            $msg=Msg::new("Would you like to download another release?", 60, 1031);
            next if ($msg->ayny eq 'Y');
            return 0;
        }

        while (1) {
            Msg::n();
            $msg=Msg::new("Enter the release_version to view the details about a release or press 'Enter' to continue", 60, 1032);
            $help=Msg::new("release_version: The version of the release.\n\nSORT_release_name: The name of the release, used when accessing SORT (https://sort.symantec.com).\n\nDL: An indicator that the release is present in your repository.\n\nOBS: An indicator that the release has been obsoleted by another higher release.\n\nAI: An indicator that the release has scripted install capabilities. All base and maintenance releases have auto install capabilities. Hot Fix releases with auto-install capabilities are available beginning with version 6.1. Otherwise the hot fix will require a manual installation.\n\nrel_date: The date the release is available.\n\nsize_KB: The file size of the release in kilobytes.", 60, 1033);
            $rtn = $msg->ask('',$help,'back','nullok');
            last if (EDR::getmsgkey($rtn,'back'));
            # if user press Enter
            if ($rtn=~/^\s*$/) {
                # $rtn=0 means due to some reason not success, back to manage repository menu
                # $rtn=1 means downloaded, let user press Enter and see the new available release info
                # $rtn=2 means back to Main Menu
                if ($cfg->{opt}{repository}) {
                    $msg=Msg::new("Downloading available releases is not allowed while the -repository option is being used.", 60, 1034);
                    Msg::n();
                    $msg->bold();
                    Msg::prtc();
                    return 0;
                }
                if ($cfg->{opt}{noipc}) {
                    $msg=Msg::new("Downloading available releases is not allowed while the -noipc option is being used.", 60, 1035);
                    Msg::n();
                    $msg->bold();
                    Msg::prtc();
                    return 0;
                }
                unless ($cpic->{sortready}) {
                    $msg=Msg::new("Downloading available releases is not allowed because SORT (https://sort.symantec.com) is not available.", 60, 1036);
                    Msg::n();
                    $msg->bold();
                    Msg::prtc();
                    return 0;
                }
                $rtn=$deploy->repository_download_image($padv,$level);
                if ($rtn == 1) {
                    $msg=$deploy->available_releases_msg($level,$padv);
                    $msg->bold;
                    ($contents,$titles) = $deploy->refresh_repository_table_content($padv,$level,$action);
                    Msg::table($contents,$titles);
                    next;
                } elsif ($rtn == 2) {
                    return 1;
                } else {
                    return 0;
                }
            }
            $targets = [];
            @$targets = grep {$_->{release_version} eq $rtn} @$contents;
            if (scalar @$targets) {
                $msg=$deploy->repository_detail_release_msg($padv,$level,$targets,$details,$details_obs);
                Msg::print($msg);

                if(@$targets[0]->{downloaded} !~ /y/i && $level ne 'ga') {
                    if ($cfg->{opt}{repository}) {
                        $msg=Msg::new("Downloading available release is not allowed while the -repository option is being used.", 60, 1037);
                        $msg->bold();
                        Msg::prtc();
                        return 0;
                    }
                    if ($cfg->{opt}{noipc}) {
                        $msg=Msg::new("Downloading available release is not allowed while the -noipc option is being used.", 60, 1038);
                        $msg->bold();
                        Msg::prtc();
                        return 0;
                    }
                    unless ($cpic->{sortready}) {
                        $msg=Msg::new("Downloading available releases is not allowed because SORT (https://sort.symantec.com) is not available.", 60, 1036);
                        $msg->bold();
                        Msg::prtc();
                        return 0;
                    }
                    $msg = Msg::new("Would you like to download this $level?", 60, 1039, "$level");
                    $ayn=$msg->ayny;
                    if($ayn eq 'Y') {
                        if ($deploy->repository_download($padv,$level,@$targets[0])) {
                            # download successfully
                            #return 1 if((@$targets[0]->{auto_install} eq 'Y') && $deploy->repository_download_to_upgrade_install(\@$targets[0]));

                            $msg=$deploy->available_releases_msg($level,$padv);
                            $msg->bold;
                            ($contents,$titles) = $deploy->refresh_repository_table_content($padv,$level,$action);
                            Msg::table($contents,$titles);
                        } else {
                            Msg::n();
                            $msg=Msg::new("The $targets->[0]{release_name} download did not complete successfully. Please retry", 60, 1040, "$targets->[0]{release_name}");
                            $msg->warning();
                            Msg::prtc();
                            return 0;
                        }
                    }
                }
            } else {
                $msg=Msg::new("No matching release.", 60, 1041);
                $msg->print;
            }
         }
    }
    return 1;
}

sub available_releases_msg {
    my ($deploy,$level,$padv)=(@_);
    if ($level eq 'ga') {
        return Msg::new("\nAvailable Base releases for $padv:\n", 60, 1042, "$padv");
    } elsif ($level eq 'patch') {
        return Msg::new("\nAvailable Maintenance releases for $padv:\n", 60, 1043, "$padv");
    }
    return Msg::new("\nAvailable Hot Fix releases for $padv:\n", 60, 1044, "$padv");
}

sub get_titles_available {
    my ($deploy,$level) = @_;

    return $deploy->{repo}{header_ga_available}     if ($level eq 'ga');
    return $deploy->{repo}{header_patch_available}  if ($level eq 'patch');
    return $deploy->{repo}{header_hotfix_available};
}

sub get_titles_local {
    my ($deploy,$level) = @_;

    return $deploy->{repo}{header_ga_local}     if ($level eq 'ga');
    return $deploy->{repo}{header_patch_local}  if ($level eq 'patch');
    return $deploy->{repo}{header_hotfix_local};
}

sub repository_download_image {
    my ($deploy,$padv,$level) = @_;
    my ($rel,$msg,$ayn,@menu,$matrix,@targets,$all_non,$all_releases,$ret,$download);
    $rel=Obj::rel();
    if ($level eq 'ga') {
        Msg::n();
        $msg=Msg::new("Base level release images must be downloaded from https://fileconnect.symantec.com/ and loaded using the R) 3) Load a Release Image File option", 60, 1045);
        $msg->print();
        Msg::prtc();
        return 0;
    }
    return 0 if ($deploy->{sortIsDown});

    $all_non = "All non-obsoleted releases";
    $all_releases = "All releases";

    # move the following section above the user prompt for Etrack 3086758
    $matrix=$rel->get_release_matrix($padv,$level);

    @targets = sort {$a->{vers_level} cmp $b->{vers_level}} values %$matrix;
    @targets = grep {(defined $_->{release_type}) && ($_->{release_type} ne 'HF')} @targets;
    @targets = grep {$_->{vers_level} ge '510'} @targets;
    return unless scalar @targets;

    @targets = grep {$_->{downloaded} !~ /y/i } @targets;
    # add the following check for Etrack 3086758
    unless (scalar @targets) {
        $msg=Msg::new("\nAll releases are in the repository.", 60, 1046);
        $msg->print;
        Msg::prtc();
        return 0;
    }

    if ($level eq 'ga') {
        $msg=Msg::new("\nWould you like to download a $padv Base Release Image?", 60, 1047, "$padv");
    } elsif ($level eq 'patch') {
        $msg=Msg::new("\nWould you like to download a $padv Maintenance Release Image?", 60, 1048, "$padv");
    } else {
        $msg=Msg::new("\nWould you like to download a $padv Hot Fix Release Image?", 60, 1049, "$padv");
    }

    $ayn=$msg->aynn();
    return 0 unless ($ayn eq 'Y');
    Msg::n();

    @menu = (map({$_->{release_version}} @targets), $all_non, $all_releases);
    $msg=Msg::new("Select the $level release to download, '$all_non' to download all non-obsoleted releases, or '$all_releases' to download all releases", 60, 1050, "$level", "$all_non", "$all_releases");
    $ret=$msg->menu(\@menu,scalar(@menu)==1,'','back');
    return 0 if (EDR::getmsgkey($ret,'back'));
    # $download flag is for user downloading all non-obsolete releases.
    # if $download=0, means no download action take place because all the required images have been downloaded.
    # if $downloaded=1, means at least 1 image have been downloaded.
    $download=0;
    if ($ret < scalar(@menu)-1) {
        # user choose a regular release
        if ($deploy->repository_download($padv,$level,$targets[$ret-1])) {
            #return 2 if (($targets[$ret-1]->{auto_install} ne 'Y') && $deploy->repository_download_to_upgrade_install($targets[$ret-1]));
            return 1;
        } else {
            Msg::n();
            $msg=Msg::new("The $targets[$ret-1]->{release_name} download failed.  Please retry.", 60, 1051, "$targets[$ret-1]->{release_name}");
            $msg->warning();
            Msg::prtc();
            return 0;
        }
    } elsif ($ret == scalar(@menu)-1) {
        # user choose $all_non
        for my $item (@targets) {
            next if ($item->{obsoleted_by});
            $deploy->repository_download($padv,$level,$item);
            $download=1;
        }
    } else {
        # user choose $all_releases
        for my $item (@targets) {
            $deploy->repository_download($padv,$level,$item);
            $download=1;
        }
    }

    unless ($download) {
        # there's no download action. notify user nothing has been dwnloaded.
        # If all releases have been downloaded, the previous code will deal with it.
        # If all non-obsoleted releases have been downloaded, it goes here.
        $msg=Msg::new("\nAll required releases have already been downloaded in the repository.", 60, 1052);
        $msg->print;
    }

    Msg::prtc();
    return 1;
}

sub repository_download {
    my ($deploy,$padv,$level,$item) = @_;
    my ($ayn,$msg,$url,$filename,$targz_dir,$image_dir,$image,$free,$filesize,$conflict);
    my $cpic=Obj::cpic();
    my $edr=Obj::edr();

    unless (defined $item->{upload_location}) {
        $msg=Msg::new("Could not download $item->{release_name} since the upload_location is not set.", 60, 1053, "$item->{release_name}");
        $msg->print();
        return 0;
    }
    $url = $item->{upload_location};
    $image = $filename = (split /\//,$url)[-1];
    $image =~ s/\.tar\.gz\s*$//;
    $targz_dir = "$deploy->{preference}{Repository}{value}/$level/targz";
    # $image is ok here because at the moment we only download Maintenance/Hot Fix
    $image_dir = "$deploy->{preference}{Repository}{value}/$level/images/$image";
    $filename = "$targz_dir/$filename";
    EDR::cmd_local("_cmd_rmr $filename");
    EDR::cmd_local("_cmd_rmr $image_dir");
    EDRu::mkdir_local_nosys($targz_dir) unless -d $targz_dir;

    #Check the available space
    $free = $cpic->volumespace_sys($edr->{localsys},$deploy->{preference}{Repository}{value},1) || $cpic->volumespace_sys($edr->{localsys},'/');
    # need a write check here and alternate message if it fails
    $filesize = $item->{upload_size_kb} * 3;
    if ($free < $filesize) {
        $msg=Msg::new("Downloading and expanding $item->{release_name} require $filesize KB in the repository and only $free is available", 60, 1054, "$item->{release_name}", "$filesize", "$free");
        $msg->print;
        Msg::prtc();
        return 0;
    }
    $conflict=$deploy->add_deploy_sfha_lock($image,"download");
    if (@$conflict) {
        $msg=Msg::new("\nFound the following conflicts for repository download of $image:\n", 60, 1055, "$image");
        $msg->print();
        $deploy->display_deploy_sfha_conflicts($conflict);
        return 0;
    }

    $msg=Msg::new("\nDownloading $item->{release_name} from SORT - https://sort.symantec.com", 60, 1056, "$item->{release_name}");
    $msg->print();
    my $ret = EDRu::download_file($url, $filename, 1, 20);
    if (!$ret) {
        #Error occurred
        my $download_errors=EDR::get('download_errors');
        $msg=Msg::new("Could not download $item->{release_name}\n$download_errors\n", 60, 1057, "$item->{release_name}", "$download_errors");
        $msg->print;
        Msg::prtc();
        $deploy->clear_deploy_sfha_lock($image,"download");
        return 0;
    }
    unless ($deploy->is_cksum_match($filename,$item->{upload_cksum})) {
        $msg=Msg::new("The cksum of file $filename is not correct", 60, 1058, "$filename");
        $msg->print;
        Msg::prtc();
        $deploy->clear_deploy_sfha_lock($image,"download");
        return 0;
    }

    EDRu::mkdir_local_nosys($image_dir) unless -d $image_dir;
    EDR::cmd_local("cd $image_dir; _cmd_gunzip -c $filename | _cmd_tar xvf - 2> /dev/null");
    $msg=Msg::new("Untarring $item->{release_name}", 60, 1059, "$item->{release_name}");
    $msg->left();
    if (EDR::cmdexit()) {
        $msg=Msg::new("Failed to un-zip and un-tar the file $filename", 60, 1060, "$filename");
        $msg->print;
        EDR::cmd_local("_cmd_rmr $image_dir");
        Msg::prtc();
        $deploy->clear_deploy_sfha_lock($image,"download");
        return 0;
    }
    $msg->right_done();

    if ((defined $deploy->{preference}{SaveTar}{value}) && ($deploy->{preference}{SaveTar}{value} eq 'N')) {
        $msg=Msg::new("Deleting $filename", 60, 1061, "$filename");
        $msg->left();
        EDR::cmd_local("_cmd_rmr $filename");
        $msg->right_done();
    }

    $item->{downloaded} = 'Y';

    $msg=Msg::new("$item->{release_name} has been downloaded successfully.", 60, 1062, "$item->{release_name}");
    $msg->nprint;

    $deploy->clear_deploy_sfha_lock($image,"download");
    $deploy->image_repository_scan($deploy->{preference}{Repository}{value});

    return 1;
}

sub repository_view_remove_image {
    my ($deploy) = @_;
    my ($padvs,$levels,$msg,$done,$padv,$level,$titlemsg,$contents,$titles,$rtn,$details,$details_obs,$targets,$help,$ayn,$action,$cfg);

    my $cpic=Obj::cpic();
    $cfg = Obj::cfg();
    $action = 'repo';
    $padvs = $deploy->{repo}{padvs};
    $levels = $deploy->{repo}{levels};
    $details= $deploy->{repo}{details_to_view};
    $details_obs = $deploy->{repo}{details_to_view_obsolete};

    $titlemsg = Msg::new("To view or remove repository images, the platform type and release level type must be selected.\n", 60, 1063);

    while (!$done) {
        Msg::title();
        $titlemsg->print;
        ($padv,$level) = $deploy->repository_ask_padv_level($action,$padvs,$levels);
        for my $op ($padv,$level) {
            return 0 if (EDR::getmsgkey($op,'back'));
        }

        ($contents,$titles) = $deploy->refresh_repository_table_content($padv,$level,$action);
        $msg=Msg::new("\nStored Repository Releases:\n", 60, 1064);
        $msg->bold;
        Msg::table($contents,$titles);
        while (1) {
            Msg::n();
            $msg=Msg::new("Enter the release_version to view details about a release including its stored location, or press 'Enter' to continue", 60, 1065);
            $help=Msg::new("release_version: The version of the release.\n\nSORT_release_name: The name of the release, used when accessing SORT (https://sort.symantec.com).\n\nOBS: An indicator that the release has been obsoleted by another higher release.\n\nAI: An indicator that the release has scripted install capabilities. All base and maintenance releases have auto install capabilities. Hot Fix releases with auto-install capabilities are beginning with version 6.1. Otherwise the Hot Fix will require manual installation.", 60, 1066);
            $rtn = $msg->ask('',$help,'back','nullok');
            last if (EDR::getmsgkey($rtn,'back'));

            if ($rtn =~ /^\s*$/) {
                # $rtn=0 means due to some reason not success, back to manage repository menu
                # $rtn=1 means removed, let user press Enter and see the new available release info
                # $rtn=2 means go back, redo the previous stuff
                if ($cfg->{opt}{repository}) {
                    $msg=Msg::new("Removing repository images is not allowed while the -repository option is being used.", 60, 1067);
                    Msg::n();
                    $msg->bold();
                    Msg::prtc();
                    return 0;
                }
                $rtn=$deploy->repository_remove_image($padv,$level);

                if ($rtn == 1) {
                    Msg::prtc();
                    $msg=Msg::new("\nStored Repository Releases:\n", 60, 1064);
                    $msg->bold;
                    ($contents,$titles) = $deploy->refresh_repository_table_content($padv,$level,$action);
                    Msg::table($contents,$titles);
                    unless(@$contents) {
                        Msg::prtc();
                        return 0;
                    }
                    next;
                } elsif ($rtn == 2) {
                    next;
                }
                else {
                    return 0;
                }
            }

            $targets = [];
            @$targets = grep {$_->{release_version} eq $rtn} @$contents;
            if (scalar @$targets) {
                $msg=$deploy->repository_detail_release_msg($padv,$level,$targets,$details,$details_obs);
                Msg::print($msg);

                if ($cfg->{opt}{repository}) {
                    $msg=Msg::new("Removing repository images is not allowed while the -repository option is being used.", 60, 1067);
                    $msg->bold();
                    Msg::prtc();
                    return 0;
                }

                if (@$targets[0]->{downloaded} =~/y/i) {
                    $msg=Msg::new("Would you like to remove this release?", 60, 1068);
                    # default value is not to remove image.
                    $ayn=$msg->aynn;
                    if($ayn eq 'Y') {
                        # user want to remove this release
                        $deploy->repository_remove($padv,$level,@$targets[0]);
                        Msg::prtc();
                        $msg=Msg::new("\nStored Repository Releases:\n", 60, 1064);
                        $msg->bold;
                        ($contents,$titles) = $deploy->refresh_repository_table_content($padv,$level,$action);
                        Msg::table($contents,$titles);
                        unless(@$contents) {
                            Msg::prtc();
                            return 0;
                        }
                    }
                }
            } else {
                $msg=Msg::new("No matching release.", 60, 1041);
                $msg->print;
            }
        }
    }
    return 0;
}

sub repository_remove {
    my ($deploy,$padv,$level,$item) = @_;
    my ($msg,$url,$filename,$targz_dir,$image_dir,$image,$conflict);
    my (@padvs,$padvlist,$release_version);

    my $cpic=Obj::cpic();

    if ($level eq 'ga') {
        $image = $filename = $item->{filename};
    } else {
        $url = $item->{upload_location};
        $image = $filename = (split /\//,$url)[-1];
    }

    $image =~ s/\.tar\.gz\s*$//;
    $targz_dir = "$deploy->{preference}{Repository}{value}/$level/targz";
    $image_dir = "$deploy->{preference}{Repository}{value}/$level/images/$image";
    $filename = "$targz_dir/$filename";

    $release_version=$item->{release_version};
    # Check if there're other padv share the same image.
    for my $padvi (@{$deploy->{repo}{padvs}}) {
        next if ($padv eq $padvi);
        for my $rel_ver(keys%{$deploy->{image_repository}{$padvi}}) {
            if (EDRu::basename($deploy->{image_repository}{$padvi}{$rel_ver}{image}) eq $image) {
                push(@padvs,$padvi);
            }
        }
    }

    if(@padvs) {
        push(@padvs,$padv);
        $padvlist=join(', ',@padvs);
        Msg::n();
        $msg=Msg::new("The $release_version release image applies to the following platforms:\n$padvlist\nRemoving $release_version release removes it for all platforms.", 60, 1069, "$release_version", "$padvlist", "$release_version");
        $msg->printn();
        $msg=Msg::new("Are you sure you would like to remove the $release_version release?", 60, 1070, "$release_version");
        return if ($msg->ayny eq 'N');
    }

    $conflict=$deploy->add_deploy_sfha_lock($image,"remove");
    if (@$conflict) {
        $msg=Msg::new("\nFound the following conflicts for repository remove of $image:", 60, 1071, "$image");
        $msg->print();
        $deploy->display_deploy_sfha_conflicts($conflict);
        return 0;
    }

    $msg=Msg::new("\nRemoving $image", 60, 1072, "$image");
    $msg->left();
    EDR::cmd_local("_cmd_rmr $filename 2> /dev/null");
    EDR::cmd_local("_cmd_rmr $image_dir 2> /dev/null");

    $msg=Msg::new("Done", 60, 1073);
    $msg->right();

    delete $item->{downloaded};

    $msg=Msg::new("$image has been removed successfully.", 60, 1074, "$image");
    $msg->nprint;
    $deploy->clear_deploy_sfha_lock($image,"remove");
    $deploy->image_repository_scan($deploy->{preference}{Repository}{value});

    return 0;
}

sub is_cksum_match {
    my ($deploy,$filename,$aim) = @_;

    # Consider the file OK if there is no value set in the matrix
    return 1 unless $aim;
    my $cksum=EDRu::cksum($filename);
    return ($cksum eq $aim) ? 1 : 0;
}

sub get_item_by_size_cksum {
    my ($deploy,$padvs,$levels,$filename)=@_;
    my ($rel,$matrix,$local_file_size,$local_file_cksum);
    my ($gzipped_cksum,$gzipped_size,$gunzipped_cksum,$gunzipped_size);
    $rel=Obj::rel();
    $local_file_size=Obj::edr()->{localsys}->filesize($filename);
    $local_file_cksum=EDRu::cksum($filename);

    for my $padv (@$padvs) {
        for my $level (@$levels) {
            $matrix=$rel->get_release_matrix($padv,$level);
            for my $item (values %$matrix) {
                if ($local_file_size == $item->{gunzip_size} && $local_file_cksum == $item->{gunzip_cksum}) {
                    return ($padv,$level,$item,0,1);
                } elsif($local_file_size == $item->{upload_size} && $local_file_cksum == $item->{upload_cksum}) {
                    return ($padv,$level,$item,1,0);
                }
            }
        }
    }
    return 0;
}

sub repository_remove_image {
    my ($deploy,$padv,$level)=@_;
    my ($rel,$matrix,@targets,@menu,$msg,$ret,$ayn);
    $rel=Obj::rel();
    $matrix=$rel->get_release_matrix($padv,$level);
    @targets = sort {$a->{vers_level} cmp $b->{vers_level}} values %$matrix;
    @targets = grep {(defined $_->{release_type}) && ($_->{release_type} ne 'HF')} @targets;
    @targets = grep {$_->{vers_level} ge '510'} @targets;
    return unless scalar @targets;

    @targets = grep {$_->{downloaded} =~ /y/i } @targets;
    @menu = map({$_->{release_version}} @targets);

    Msg::n();
    if ($level eq 'ga') {
        $msg=Msg::new("Would you like to remove a $padv Base Release Image?", 60, 1075, "$padv");
    } elsif ($level eq 'patch') {
        $msg=Msg::new("Would you like to remove a $padv Maintenance Release Image?", 60, 1076, "$padv");
    } else {
        $msg=Msg::new("Would you like to remove a $padv Hot Fix Release Image?", 60, 1077, "$padv");
    }

    $ayn=$msg->aynn();
    return 0 if ($ayn eq 'N');

    Msg::n();
    $msg=Msg::new("Select the $level release to remove", 60, 1078, "$level");
    $ret=$msg->menu(\@menu,scalar(@menu)==1,'','back');
    return 0 if (EDR::getmsgkey($ret,'back'));

    $deploy->repository_remove($padv,$level,$targets[$ret-1]);
    return 1;
}

sub repository_load_image {
    my ($deploy) = @_;
    my ($cpic,$padvs,$levels,$titlemsg,$rtn,$msg,$list,$help,$matrix,$titles,$localsys,$cfg);
    my ($image_padv,$image_level,$image_item,$image,$targz_dir,$image_dir,$filename,$istar,$isgz,$filegzname,$conflict,$free,$filesize);

    $cfg=Obj::cfg();
    $cpic = Obj::cpic();
    $padvs = $deploy->{repo}{padvs};
    $levels = $deploy->{repo}{levels};
    $localsys = Obj::edr()->{localsys};

    Msg::n();
    if ($cfg->{opt}{repository}) {
        $titlemsg = Msg::new("Loading repository is not allowed while the -repository option is being used.", 60, 1079);
        $titlemsg->bold();
        Msg::prtc();
        return 0;
    }
    $titlemsg = Msg::new("Enter the file path of the release image (Example: /tmp/VRTS_SF_HA_Solutions_6.0.1_RHEL.tar.gz)", 60, 1080);

    while (1) {
        $image_padv = $image_level = '';
        $rtn = $titlemsg->ask('','','back','nullok');
        return 0 if (EDR::getmsgkey($rtn,'back'));
        next if ($rtn=~/^\s*$/);

        unless (-f $rtn) {
            $msg=Msg::new("\nThe file $rtn does not exist. Please retry.\n", 60, 1081, "$rtn");
            $msg->print;
            next;
        }

        $isgz = $istar = 0;
        $filegzname = $image = $filename = EDRu::basename($rtn);

        # Even though a file name is changed, it may still be a tar file or gzip file.
        # So we use funtion &get_item_by_size_cksum to determine if it's a tar file or a gzip file.
#        if ($rtn =~ /\.tar\.gz\s*$/) {
#            $isgz = 1;
#            $image =~ s/\.tar\.gz\s*$//;
#        } elsif ($rtn =~ /\.tar\s*$/) {
#            $istar = 1;
#            $image =~ s/\.tar\s*$//;
#        } else {
#            $msg=Msg::new("\nOnly tar file and tar.gz file are allowed. Please retry.\n");
#            $msg->print;
#            next;
#        }

        Msg::n();
        $msg=Msg::new("Verifying the contents of this file", 60, 1082);
        $msg->left;

        ($image_padv,$image_level,$image_item,$isgz,$istar)=$deploy->get_item_by_size_cksum($padvs,$levels,$rtn);

        Msg::log("Padv:$image_padv,Level:$image_level");

        # if image_padv = 0, give user a message indicating matching failed
        unless($image_padv) {
            $msg=Msg::new("Failed", 60, 1083);
            $msg->right;
            $msg=Msg::new("\nCannot match $filegzname to a release.\n", 60, 1084, "$filegzname");
            $msg->print;
            next;
        }

        # It's possible that user renamed the tar.gz file.
        # use $image to track the real release name.
        if ($image_level eq 'ga') {
            $image = $filename = $image_item->{filename};
        } else {
            $image = $filename = (split /\//,($image_item->{upload_location}))[-1];
        }

        unless ($image) {
            $msg=Msg::new("Failed", 60, 1083);
            $msg->right;
            $msg=Msg::new("\nCannot match $filegzname to a release.\n", 60, 1084, "$filegzname");
            $msg->print;
            next;
        }
        Msg::log("Image:$image");

        $msg=Msg::new("Done", 60, 1073);
        $msg->right;

        $image =~ s/\.tar\.gz\s*$//;
        $targz_dir = "$deploy->{preference}{Repository}{value}/$image_level/targz";
        $image_dir = "$deploy->{preference}{Repository}{value}/$image_level/images/$image";
        $filename = "$targz_dir/$filename";

        if ($image_item->{downloaded} =~ /y/i || $localsys->is_dir($image_dir)) {
            # Since the file have already been loaded. Ask user a question if he want to reload
            Msg::n();

            # User may loading it from $targz_dir, if so, only need to remove $image_dir when reloading it.
            # If $filename exists and $filename have the same cksum with $image on SORT. Consider it's a complete file
            if ($localsys->exists($filename) && $deploy->is_cksum_match($filename, $image_item->{upload_cksum})) {
                $list=$image_dir;
            } else {
                $list="$image_dir\n$filename";
            }

            $msg=Msg::new("$image has already been loaded, would you like to reload?", 60, 1085, "$image");
            $help=Msg::new("By reloading $image, the following file or directory will be removed and recreated:\n$list\n\nYou can reload the image when the existing one is potentially corrupted.", 60, 1086, "$image", "$list");
            if ($msg->aynn($help) eq 'N') {
                Msg::n();
                next;
            }
            Msg::n();
            $msg=Msg::new("Removing existing $image", 60, 1087, "$image");
            $msg->left();

            # When the $filename exists and it's cksum not matching the one in SORT, remove this broken file
            EDR::cmd_local("_cmd_rmr $filename") if (($list ne $image_dir) && $localsys->exists($filename));
            EDR::cmd_local("_cmd_rmr $image_dir");

            $msg->right_done();
        } else {
            # if image is not loaded and $image_dir does NOT exist,
            # but $filename exists, there're 2 possible explanation:
            # 1. $filename have the same cksum with the SORT one, we just need to untar it.
            # 2. $filename do NOT have the same cksum with the SORT one, we need to remove it.
            if($localsys->exists($filename)) {
                if($deploy->is_cksum_match($filename, $image_item->{upload_cksum})) {
                    $rtn = $filename;
                } else {
                    EDR::cmd_local("_cmd_rmr $filename");
                }
            }
        }

        # Check free space
        $msg=Msg::new("Checking free space on $localsys->{sys}", 60, 1088, "$localsys->{sys}");
        $msg->left();
        $free = $cpic->volumespace_sys($localsys,$deploy->{preference}{Repository}{value},1) || $cpic->volumespace_sys($localsys,'/');
        $filesize = ($image_item->{upload_size} / 1024) * 3;
        if ($free < $filesize) {
            $msg->right_failed();
            $msg=Msg::new("Loading and expanding $image require $filesize KB in the repository and only $free is available", 60, 1089, "$image", "$filesize", "$free");
            $msg->print;
            Msg::prtc();
            next;
        }
        $msg->right_done();

        $conflict=$deploy->add_deploy_sfha_lock($image,"load");
        if (@$conflict) {
            $msg=Msg::new("\nFound the following conflicts for repository load of $image:\n", 60, 1090, "$image");
            $msg->print();
            $deploy->display_deploy_sfha_conflicts($conflict);
            Msg::prtc();
            return 0;
        }
        $msg=Msg::new("Loading $image", 60, 1091, "$image");
        $msg->left();
        EDRu::mkdir_local_nosys($targz_dir) unless -d $targz_dir;
        EDRu::mkdir_local_nosys($image_dir) unless -d $image_dir;

        ## if user set not to save tar file, do NOT need to copy it to tar file.
        if ($isgz) {
            if ($deploy->{preference}{SaveTar}{value} && ($deploy->{preference}{SaveTar}{value} eq 'N')) {
                EDR::cmd_local("_cmd_rmr $filename") if ($localsys->exists($filename));
            } else {
                EDR::cmd_local("_cmd_cp -rf $rtn $filename") unless ($localsys->exists($filename));
            }
            EDR::cmd_local("cd $image_dir; _cmd_gunzip -c $rtn | _cmd_tar xvf -");
        } elsif ($istar) {
            EDR::cmd_local("cd $image_dir; _cmd_tar -xvf $rtn");
        }
        if (EDR::cmdexit()) {
            $msg=Msg::new("Failed", 60, 1083);
        } else {
            $image_item->{downloaded} = 'Y';
            $msg=Msg::new("Done", 60, 1073);
        }
        $msg->right();
        $msg=Msg::new("\n$image has been loaded successfully.\n", 60, 1092, "$image");
        $msg->print;
        $deploy->clear_deploy_sfha_lock($image,"load");
        $deploy->image_repository_scan($deploy->{preference}{Repository}{value});
    }

    return;
}

sub repository_downloaded_padvs {
    my ($deploy,$padvs) = @_;
    my ($rel,@padvs,$levels,$match,$matrix);
    $rel = Obj::rel();
    $levels = $deploy->{repo}{levels};
    @padvs = ();
    for my $padv (@$padvs) {
        $match = 0;
        for my $level (@$levels) {
            $matrix=$rel->get_release_matrix($padv,$level);
            my @match = grep {$_->{downloaded} =~ /y/i} values %$matrix;
            @match= grep {$_->{vers_level} ge '510'} @match;
            if (scalar @match) {
                $match = 1;
                last;
            }
        }
        push(@padvs,$padv) if $match;
    }
    return \@padvs;
}

sub repository_downloaded_levels {
    my ($deploy,$levels,$padv) = @_;
    my ($rel,@levels,$matrix);
    $rel=Obj::rel();
    for my $level (@$levels) {
        $matrix=$rel->get_release_matrix($padv,$level);
        my @match = grep {$_->{downloaded} =~ /y/i} values %$matrix;
        @match= grep {$_->{vers_level} ge '510'} @match;
        push(@levels,$level) if scalar @match;
    }
    return \@levels;
}

sub repository_detail_release_msg {
    my ($deploy,$padv,$level,$targets,$details,$details_obs)=@_;
    my ($rel,$matrix_ga,$matrix_patch,$matrix_hotfix,$msg,$obs_release,$none);
    $rel=Obj::rel();
    $matrix_ga=$rel->get_release_matrix($padv,'ga');
    $matrix_patch=$rel->get_release_matrix($padv,'patch');
    $matrix_hotfix=$rel->get_release_matrix($padv,'hotfix');
    if (scalar @$targets) {
        $msg = "\n";
        for my $key (@$details) {
            $msg .= "$key: $targets->[0]{$key}\n" if defined $targets->[0]{$key};
        }
        for my $key (@$details_obs) {
            next unless (defined $targets->[0]{$key});
            $msg .= "$key: ";
            $none = 1;
            for my $element (split(/,/,$targets->[0]{$key})) {
                $obs_release="$matrix_ga->{$element}->{release_version}" || "$matrix_patch->{$element}->{release_version}" || "$matrix_hotfix->{$element}->{release_version}";
                $none = 0;
                if($obs_release) {
                    $msg .= $obs_release . ',';
                } else {
                    Msg::log("$padv ga/patch/Hot Fix id $element can not be found in release matrix");
                }
            }
            if ($none) {
                $msg .= Msg::new("None\n", 60, 1093)->{msg} if ($none);
            } else {
                # remove ',' and add '\n'
                chop($msg);
                $msg .= "\n";
            }
        }
    }
    return $msg;
}
# _END_Manage_Repository

# _START_Set_Preference
sub init_preference_hash {
    my $deploy = shift;
    
    $deploy->{preference} = {
        'Repository' => {
            'name' => Msg::new("Repository", 60, 1094),
            'type' => 'edit',
            'default' => '/opt/VRTS/repository',
            'verify_sub'  => \&Deploy::verify_Repository,
            'set_handler' => \&Deploy::set_handler_Repository,
            'sub_dirs'=>["patch","hotfix","ga",".cpihotfix",".matrix",".lock"],
        },
        'Month' => {
            'name' => Msg::new("Month", 60, 1095),
            'type' => 'list',
            'options' => ["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"],
            'default' => 'Jan',
            'verify_sub'  => \&Deploy::verify_Month,
            'set_handler' => \&Deploy::set_handler_Month,
        },
        'Year' => {
            'name' => Msg::new("Year", 60, 1096),
            'type' => 'int',
            'max_value' => 2099,
            'default' => 2014,
            'verify_sub'  => \&Deploy::verify_Year,
            'set_handler' => \&Deploy::set_handler_Year,
        },
        'SaveTar' => {
            'name' => Msg::new("Save Tar Files", 60, 1097),
            'type' => 'yn',
            'default' => 'y',
            'verify_sub'  => \&Deploy::verify_SaveTar,
            'set_handler' => \&Deploy::set_handler_SaveTar,
        },
    };
    $deploy->{pref_elements} = ["Repository","SaveTar"];
    $deploy->{config_file}="/opt/VRTS/.deploy_config";
    $deploy->{image_repository_defaults} = {
        'vc_release_matrix_data'=> "ga",
        'vc_patch_matrix_data'  => "patch",
        'vc_hotfix_matrix_data'=> "hotfix",
    };
        
}

sub set_preferences {
    my ($deploy)=@_;
    my @tasks=qw(set_preferences back);
    while(1) {
        my $task_k=$deploy->cli_set_preference_task_menu(@tasks);
        if ($task_k eq Msg::get('key_set_preferences')) {
            # create a default menu
            $deploy->cli_set_preference_menu();
        } elsif ($task_k eq Msg::get('key_back')) {
            return EDR::get2('msg','back');
        }
    }
    return 1;
}

sub cli_set_preference_menu {
    my ($deploy)=@_;
    my ($msg,$menu,@prefmenu,$conflict);

    # Show preference to user
    $deploy->cli_show_preference();

    $msg=Msg::new("\nPreference List:\n", 60, 1098);
    $msg->bold;

    for my $key(@{$deploy->{pref_elements}}) {
        push(@prefmenu, $deploy->{preference}->{$key}->{name}->{msg});
    }

    $msg=Msg::new("Select a preference to set:", 60, 1099);
    $menu=$msg->menu(\@prefmenu,'','',1);
    if (EDR::getmsgkey($menu,'back')) {
        return 0;
    }

    my $item=$deploy->{pref_elements}[$menu-1];

    # Execute set handler function
    if (&{$deploy->{preference}{$item}{set_handler}}) {
        # Only if set handler function return 1 will preference value be reset
        $deploy->write_deploy_config();
    }
    return $deploy->cli_set_preference_menu();
}

sub cli_set_preference_task_menu {
    my ($deploy,@tasks)=@_;
    my ($msg,@mo,$m,$mm1,$mm2,$bs,$be,$mkey,@mk,$buf,$tsl,$task_k);
    my $cfg=Obj::cfg();
    
    # Show preference to user
    $deploy->cli_show_preference();
    if ($cfg->{opt}{repository}) {
        $deploy->{preference}{Repository}{value} = $cfg->{opt}{repository};
        $msg=Msg::new("Preference values may not be modified while the -repository option is being used.", 60, 1100);
        $msg->bold();
        Msg::prtc();
        return 'B';
    }

    # List task
    $msg=Msg::new("\nTask Menu:\n", 60, 1010);
    $msg->bold;
    @mo=@tasks;
    while (@mo) {
        $m=shift(@mo);
        $mkey=Msg::get("key_$m");
        push(@mk,$mkey);
        $mm1="$bs$mkey$be) ".Msg::get("menu_$m");
        $m=shift(@mo);
        if ($m) {
            $mkey=Msg::get("key_$m");
            push(@mk,$mkey);
            $mm2="$bs$mkey$be) ".Msg::get("menu_$m");
        } else {
            $mm2='';
        }
        $buf=Msg::string_sprintf('    %-39s %s', $mm1, $mm2);
        $msg=Msg::new_obj($buf);
        $msg->print;
    }
    Msg::n();

    $tsl=join(',',@mk);
    $tsl="[$tsl]";
    while (!EDRu::inarr($task_k,@mk)) {
        $msg=Msg::new("Enter a Task: $tsl", 60, 1011, "$tsl");
        $task_k=$msg->ask;
        $task_k=uc($task_k);
    }
    return $task_k;
}

# Perform the following tasks:
# 1. Verify dir
# 2. Create dir
# 3. Change dir permission to 755
sub verify_Repository {
    my ($deploy,$localsys,$dir) = @_;
    my ($result,$msg);
    my ($files,@files);

    if ($localsys->is_dir($dir)) {
        $msg=Msg::new("The directory $dir already exists on $localsys->{sys}", 60, 1101, "$dir", "$localsys->{sys}");
        $msg->log;
        $files=$localsys->lsfile($dir);
        @files=split(/\n/,$files);
        $files=EDRu::arrdel(\@files, @{$deploy->{preference}{Repository}{sub_dirs}});
        if (@$files > 0) {
            # There're irrelevant files in the repository dir
            # print warning message.
            $msg=Msg::new("The directory $dir already exists and contains non-repository files on $localsys->{sys}.  It is recommended to create a new repository directory that does not contain any unrelated files.", 60, 1102, "$dir", "$localsys->{sys}");
            $msg->warning;
            $msg=Msg::new("Do you want to continue?", 60, 1103);
            if ($msg->aynn() eq 'Y') {
                # User still want to use this dir
                $localsys->chmod($dir,'755');
                return 1;
            } else {
                return 0;
            }
        }
    } else {
        $result=$localsys->mkdir($dir) if (!-d $dir);
        if (!$localsys->is_dir($dir)) {
            $msg=Msg::new("Cannot create $dir directory on $localsys->{sys}", 60, 1104, "$dir", "$localsys->{sys}");
            $msg->error();
            return 0;
        }
    }
    $localsys->chmod($dir,'755');
    return 1;
}

sub set_handler_Repository {
    my ($deploy)=@_;
    my ($edr,$msg,$dir,$localsys,$ayn,@files,$files,$conflict);

    $edr=Obj::edr();
    $localsys=$edr->{localsys};
    
    $deploy->cli_show_preference();
    Msg::n();
    
    while (1) {
        $msg=Msg::new("Input a path for the repository directory:", 60, 1105);
        if (defined($deploy->{preference}{Repository}{value})) {
            $dir=$msg->ask('',0,1);
        } else {
            $dir=$msg->ask($deploy->{preference}{Repository}{default},0,1);
        }
        return 0 if (EDR::getmsgkey($dir,'back'));
        unless ($localsys->filename_is_absolute($dir)) {
            $dir=$deploy->path(EDRu::abs_path($dir));
            if ($localsys->is_dir($dir)) { # if the dir already exists, Skip creating the dir
                $msg=Msg::new("Would you like to use repository directory under $dir?", 60, 1106, "$dir");
            } else {
                $msg=Msg::new("Would you like to create repository directory under $dir?", 60, 1021, "$dir");
            }
            next if ($msg->ayny eq 'N');
        }
        $dir=$deploy->path($dir);
        # If user input the same path, skip below steps
        return if ($dir eq $deploy->{preference}{Repository}{value});
        last if ($deploy->verify_Repository($localsys,$dir));
    }

    $conflict=$deploy->add_deploy_sfha_lock("repository","change_repository");
    if (@$conflict) {
        $msg=Msg::new("\nFound the following conflict(s) for repository change:\n", 60, 1107);
        $msg->print();
        $deploy->display_deploy_sfha_conflicts($conflict);
        Msg::prtc();
        return 0;
    }

    # if we already have a repo path, merge the old repo path into the new repo path
    if ($deploy->{preference}{Repository}{value}) {
        $msg=Msg::new("Would you like to merge the old repository files into the new directory?", 60, 1108);
        if ($msg->ayny eq 'Y') {
            for my $repo(@{$deploy->{preference}{Repository}{sub_dirs}}) {
                $deploy->merge_repository($localsys,"$deploy->{preference}{Repository}{value}/$repo", "$dir/$repo");
            }
            # After merge, ask user if he would like to remove the old repository dir
            $msg=Msg::new("Would you like to remove the old repository directory?", 60, 1109);
            if ($msg->ayny() eq 'Y') {
                # remove the old repository
                $files=$localsys->lsfile($deploy->{preference}{Repository}{value});
                @files=split(/\n/,$files);
                $files=EDRu::arrdel(\@files, @{$deploy->{preference}{Repository}{sub_dirs}});
                if (@$files > 0) {
                    # there're irrelevant files in the old repository, only remove @{$deploy->{preference}{Repository}{sub_dirs}}
                    for my $repo(@{$deploy->{preference}{Repository}{sub_dirs}}) {
                        $localsys->rm("$deploy->{preference}{Repository}{value}/$repo");
                    }
                } else {
                    # remove $deploy->{preference}{Repository}{value}
                    $localsys->rm($deploy->{preference}{Repository}{value});
                }
            }
        }
    }
    # Clear the deploy sfha lock first because it clear the lock file under $deploy->{preference}{Repository}{value}
    $deploy->clear_deploy_sfha_lock("repository","change_repository");
    # set to $deploy->{preference}
    $deploy->{preference}{Repository}{value}=$dir;
    return 1;
}

sub verify_Month {}

# Example of an list that allows user to select an item
sub set_handler_Month {
    my ($deploy)=@_;
    my ($msg,$menu);
    
    $deploy->cli_show_preference();
    Msg::n();
    
    $msg=Msg::new("\nMonth List:\n", 60, 1110);
    $msg->bold;
    $msg=Msg::new("Select a month:", 60, 1111);
    $deploy->{preference}{Month}{value}=$deploy->{preference}{Month}{options}[$msg->menu($deploy->{preference}{Month}{options},'1',0,1,0,1,2)-1];
    return 1;
}

# Example of an query that only allows a value < $deploy->{preference}{Year}{max_value}
sub verify_Year {
    my ($deploy,$value)=@_;
    my ($msg);
    
    if ($value >= $deploy->{preference}{Year}{max_value}) {
        $msg=Msg::new("The year you set exceeds the maximum value($deploy->{preference}{Year}{max_value}). Please reset the value.", 60, 1112, "$deploy->{preference}{Year}{max_value}");
        $msg->error();
        return 0;
    } else {
        return 1;
    }
}

# Allow user input a int value
sub set_handler_Year {
    my ($deploy)=@_;
    my ($value,$msg);
    
    $deploy->cli_show_preference();
    Msg::n();
    
    $msg=Msg::new("Set year: ", 60, 1113);
    do {
        $value=$msg->ask($deploy->{preference}{Year}{default},0,1);
        return 0 if (EDR::getmsgkey($value,'back'));
    } while (!$deploy->verify_Year($value));

    # set to $deploy->{preference}
    $deploy->{preference}{Year}{value}=$value;
    return 1;
}

sub verify_SaveTar{}

sub set_handler_SaveTar {
    my ($deploy)=@_;
    my ($msg,$value);
    
    $deploy->cli_show_preference();
    Msg::n();
    
    $msg=Msg::new("Do you want to save the image tarfiles?", 60, 1114);
    $value=$msg->ayn($deploy->{preference}{SaveTar}{default},0,1);
    return 0 if (EDR::getmsgkey($value,'back'));
    $deploy->{preference}{SaveTar}{value}=$value;
    return 1;
}

sub cli_show_preference {
    my ($deploy)=@_;
    my ($msg,$max,$width,$format);
    
    Msg::title();
    $msg=Msg::new("Current Preferences:\n", 60, 1115);
    $msg->bold;
    # Read preference file
    $deploy->read_deploy_config();

    $max = 0;
    for my $key(@{$deploy->{pref_elements}}) {
        $width=Msg::string_width($deploy->{preference}{$key}{name}{msg});
        $max = $max > $width? $max : $width
   }
    # 4 spaces between key and value
    $max += 4;
    $format = "%-${max}s%s";
    for my $key(@{$deploy->{pref_elements}}) {
        if (defined($deploy->{preference}{$key}{value})) {
            $msg=Msg::string_sprintf($format, $deploy->{preference}{$key}{name}{msg}, $deploy->{preference}{$key}{value});
        } else {
            $msg=Msg::string_sprintf($format, $deploy->{preference}{$key}{name}{msg}, "N/A");
        }
        Msg::print($msg);
    }
    return 1;
}

# Deployment Server config file is a json file that contains all the configurations including:
# 1. Preferences
# 2. Install Bundle Definition
sub read_deploy_config {
    my ($deploy) = @_;
    my ($config_file,$content,$localsys);
    my ($deploy_config_hash);
    $config_file = $deploy->{config_file};
    $localsys = Obj::localsys();

    # If the old preference file format has not been converted
    # and the current configure file format does not exist
    # we convert the old configure file to new format.
    if (!$deploy->{config_converted} && !$localsys->exists($config_file)) {
        $deploy->convert_deploy_config();
        $deploy->{config_converted}=1;
    }
    $config_file = $deploy->{config_file};
    $localsys = Obj::localsys();
    $content = $localsys->catfile($config_file);
    return 0 unless ($content);
    $deploy_config_hash = EDRu::eval_json($content);
    return 0 unless ($deploy_config_hash);
    # Read preference file into $deploy object
    for my $key(@{$deploy->{pref_elements}}) {
        if (defined($deploy_config_hash->{preference}{$key})) {
            $deploy->{preference}{$key}{value} = $deploy_config_hash->{preference}{$key};
        } else {
            undef ($deploy->{preference}{$key}{value});
        }
    }
    # Read bundle information into $deploy object
    # Read template information into $deploy object
    return 1;
}

sub write_deploy_config {
    my ($deploy)=@_;
    my ($config_file,$content);
    my (%deploy_config_hash);
    $config_file = $deploy->{config_file};

    # Save preference information
    for my $key(@{$deploy->{pref_elements}}) {
        if (defined($deploy->{preference}{$key}{value})) {
            $deploy_config_hash{preference}{$key} = $deploy->{preference}{$key}{value};
        }
    }
    # Save bundle information
    # Save template information
    $content=JSON::to_json(\%deploy_config_hash,{pretty=>1});
    EDRu::writefile($content,$config_file);
    return 1;
}

# Convert old Deployment Server preference file to current configure file
sub convert_deploy_config {
    my ($deploy)=@_;
    my ($preference_file,$content,$localsys);
    $localsys = Obj::localsys();
    $preference_file = '/opt/VRTS/.install_pref';
    return 1 unless ($localsys->exists($preference_file));
    $deploy->read_default_values($preference_file);
    $localsys->rm($preference_file);
    for my $key(@{$deploy->{pref_elements}}) {
        if (defined($deploy->{default}{$key})) {
            $deploy->{preference}{$key}{value} = $deploy->{default}{$key};
        } else {
            undef ($deploy->{preference}{$key}{value});
        }
    }
    $deploy->write_deploy_config();
    return 1;
}

# Current algorithm:
# If a dir is in $srcdir but not in $destdir, copy it.
# If a dir is in $srcdir and in $destdir, merge it.
# If a file is in $srcdir but not in $destdir, copy it.
# If a file is in $srcdir and in $destdir, ignore it.
sub merge_repository {
    my ($deploy,$localsys,$src,$dest) = @_;
    return unless ($localsys->exists($src));

    if ($localsys->is_dir($src)) {
        if ($localsys->exists($dest)) {
            if ($localsys->is_dir($dest)) {
                for my $repo(split(/\n/,$localsys->lsfile($src))) {
                    $deploy->merge_repository($localsys,"$src/$repo","$dest/$repo");
                }
            }
        } else {
            # Copy the $src to $dest
            $localsys->copyfile($src, $dest);
        }
    } else {
        unless ($localsys->exists($dest)) {
            # Copy the $src to $dest
            $localsys->copyfile($src, $dest);
        }
    }
    return;
}
# _END_Set_Preference

# _START_Upgrade_Install
sub upgrade_install {
    my ($deploy,$padv,$release) = @_;
    my (@padvs,@releases,$repo,$done,$msg,$target);

    $deploy->image_repository_scan();
    unless ($deploy->{image_repository}) {
        # deal with no image repository found on the localhost
        $msg=Msg::new("No downloaded images to install\n", 60, 1116);
        $msg->print();
        sleep 2;
        return;
    }
    @padvs=(sort keys %{$deploy->{image_repository}});

    # @padvs can not be empty from now on
    # and each padv should have at least one release under it

    # first need to validate given $padv against the local image repository
    if ($padv && (!EDRu::inarr($padv,@padvs))) {
        $msg=Msg::new("No releases found in repository for $padv\n", 60, 1117, "$padv");
        $msg->();
        return;
    }
    $padv||=$deploy->upgrade_install_options('platform',\@padvs);
    return if (EDR::getmsgkey($padv,'back'));
    # find the available releases for selected $padv
    @releases=();
    for my $rel_id (sort keys %{$deploy->{image_repository}{$padv}}) {
        push (@releases,$rel_id) if ( $deploy->{image_repository}{$padv}{$rel_id}{install_path}  &&
                                  (-f $deploy->{image_repository}{$padv}{$rel_id}{install_path}) &&
                                 $deploy->deployment_server_supported_installable_hotfix($padv,$rel_id) &&
                                 $deploy->deployment_server_supported_installable_release($padv,$rel_id));
    }

    if ($#releases < 0) {
        # if no releases found for $padv, then we need to point out possibly filtered out releases
        $padv=Msg::get($padv);
        Msg::n();
        $msg=Msg::new("Repository images for $padv must be executed from a $padv system. Releases from $deploy->{deployment_server_base_release} onwards can be installed from any supported platform.", 60, 1118, "$padv", "$padv", "$deploy->{deployment_server_base_release}");
        $msg->bold();
        Msg::prtc();
    } elsif ($release && (!EDRu::inarr($release,@releases))) {
        # also need to validate given $release against the local image repository for $padv
        $padv=Msg::get($padv);
        $msg=Msg::new("No release $release found in repository for $padv\n", 60, 1119, "$release", "$padv");
        $msg->();
        Msg::prtc();
    } else {
        $release||=$deploy->upgrade_install_options('release',\@releases);
        $deploy->upgrade_install() if (EDR::getmsgkey($release,'back'));
        if (EDRu::inarr($release,@releases)) {
            $target=$deploy->upgrade_install_options('hostname');
            if ((!$target) || EDR::getmsgkey($target,'back')) {
                $deploy->upgrade_install();
            } else {
                $deploy->upgrade_install_padv_release($padv,$release,$target);
            }
        }
    }

    return 1;
}

sub deployment_server_supported_base_installable_release {
    my $deploy = shift;

    $deploy->{deployment_server_base_release}||='6.1';
    return $deploy->{deployment_server_base_release};
}

# only 6.1 hotfixes are installable through deployment server
sub deployment_server_supported_installable_hotfix {
    my ($deploy,$padv,$rel_id) = @_;
    my ($path,$base_release);

    $base_release=$deploy->deployment_server_supported_base_installable_release();
    $path = $deploy->{image_repository}{$padv}{$rel_id}{install_path};
    return 1 if (($path !~ /\/hotfix\//) || (EDRu::compvers($rel_id, $base_release) <2));
    return 0;
}

# 6.1 onwards releases support any-to-any
# and older releases on the same platform can be installed on deployment server
sub deployment_server_supported_installable_release {
    my ($deploy,$padv,$rel_id) = @_;
    my ($rel,$localsys,$localpadv,$base_release,$target_release);
    $rel=Obj::rel();
    $localsys=Obj::localsys();
    $localpadv=Padv::cpi_to_sort_padv($localsys->{padv});
    $base_release="610-000";

    $rel_id=$deploy->{image_repository}{$padv}{$rel_id}{'rel_id'};
    $target_release=$rel->{vc_release_matrix_data}{$padv}{$rel_id}{vers_level} ||
                    $rel->{vc_patch_matrix_data}{$padv}{$rel_id}{vers_level} ||
                    $rel->{vc_hotfix_matrix_data}{$padv}{$rel_id}{vers_level};
    return 1 if ($target_release ge $base_release);
    return 1 if ($padv eq $localpadv);
    return 1 if (($padv =~ /^AIX\d+/i) && ($localpadv =~ /^AIX\d+/i));
    return 0;
}

sub upgrade_install_options {
    my ($deploy,$flag,$options) = @_;
    my ($title,$msg,@menu,$option,$help,$ret,$default);

    return unless ($flag);
    return if (($#$options < 0) && ($flag !~ /hostname/i)) ;

    # so far $flag can be platform or release or hostname
    # cannot put $flag in the middle of the sentence due to L10N
    if ($flag eq 'hostname') {
        Msg::n();
        $msg=Msg::new("Enter the target system name(s) separated by spaces: (press [Enter] to go back)", 60, 1120);
        $help=Msg::new("Enter the names of the system(s) where you would like to install or upgrade the selected platform release", 60, 1121);
        $ret=$msg->ask('__back__','','','nullok');
    } else {
        if ($flag eq 'platform') {
            Msg::title();
            $msg=Msg::new("Select the platform of the available release(s) to be upgraded/installed", 60, 1122);
            $msg=Msg::new("Only one platform was found", 60, 1123) if ($#$options==0);
            $help=Msg::new("Select a platform of images within the available repository", 60, 1124);
            $default='';
        } elsif ($flag eq 'release') {
            Msg::n();
            $msg=Msg::new("Select the release image of the available release(s) to be upgraded/installed", 60, 1125);
            if ($#$options==0) {
                $msg=Msg::new("Only one release was found", 60, 1126);
                $default='1';
            }
            $help=Msg::new("Select a release of images within the available repository", 60, 1127);
        } else { # do nothing
            return;
        }
        for $ret (@{$options}) {
            $option=Msg::new("$ret", 60, 1128, "$ret");
            if ($flag eq "platform") {
                push (@menu,Msg::get($option->{msg}));
            } else {
                push (@menu,$option->{msg});
            }
        }
        $ret=$msg->menu(\@menu,$default,$help,'back');
        return $ret if (EDR::getmsgkey($ret,'back'));
        $ret=$options->[$ret-1];
    }

    $ret=~s/\s+/ /g;
    return $ret;
}

sub upgrade_install_padv_release {
    my ($deploy,$padv,$release,$target) = @_;
    my ($rel_id,$msg,$error,$install_path,$key,$basename,$image);
    my ($rel,$target_release);
    $rel=Obj::rel();
    $msg="platform," if (!$padv);
    $msg.="release," if (!$release);
    $msg=~s/,$//g;
    if ($msg) {
        $error=Msg::new("No $msg defined. Cannot proceed without proper release information provided", 60, 1129, "$msg");
        $error->error();
    } elsif (!$deploy->{image_repository}{$padv}{$release}{install_path} ) {
        # should not hit this
        $error=Msg::new("No install_path defined for $release on $padv.", 60, 1130, "$release", "$padv");
        $error->error();
    } else {
        $install_path=$deploy->{image_repository}{$padv}{$release}{install_path};
        if (!-f $install_path) {
            # should not hit this either
            $error=Msg::new("Non-existing install_path directory $install_path found", 60, 1131, "$install_path");
            $error->error();
        } else {
            $install_path=$deploy->{image_repository}{$padv}{$release}{install_path};
            if ($deploy->{image_repository}{$padv}{$release}{rel_id}) {
                $rel_id=$deploy->{image_repository}{$padv}{$release}{rel_id};
                $deploy->get_repository_release_info($padv,$rel_id);
            }
            $basename=EDRu::basename($install_path);
            $install_path=~s/$basename$//;
            $image=(split /\//,$deploy->{image_repository}{$padv}{$release}{image})[-1];
            $error=$deploy->add_deploy_sfha_lock($image,"install",$target);
            if (@$error) {
                $msg=Msg::new("Found the following conflicts for installation of $release on $target:\n", 60, 1132, "$release", "$target");
                $msg->print();
                $deploy->display_deploy_sfha_conflicts($error);
                sleep 3;
                return 0;
            }
            $target_release=$rel->{vc_release_matrix_data}{$padv}{$rel_id}{vers_level} ||
                              $rel->{vc_patch_matrix_data}{$padv}{$rel_id}{vers_level} ||
                              $rel->{vc_hotfix_matrix_data}{$padv}{$rel_id}{vers_level};

            $basename.=" -noipc" if (Cfg::opt('noipc') && $target_release ge "610-000");
            $msg=Msg::new("\nExecuting the following command to start installation of $release on $padv:\n\n\tcd $install_path; ./$basename $target\n", 60, 1133, "$release", "$padv", "$install_path", "$basename", "$target");
            $msg->bold();
            $key=system("cd $install_path; ./$basename $target");
            $error=$deploy->clear_deploy_sfha_lock($image,"install",$target);
        }
    }
    sleep 6;
    return;
}

sub get_repository_release_info {
    my ($deploy,$padv,$rel_id) = @_;

    my $ret="";
    for my $key (sort keys %{$deploy->{image_repository}{$padv}{$rel_id}}) {
        next if (ref($deploy->{image_repository}{$padv}{$rel_id}{$key}) ne 'SCALAR');
        $ret.="\t$key\t: $deploy->{image_repository}{$padv}{$rel_id}{$key}\n";
    }
    return $ret;
}
# _END_Upgrade_Install

# _START_Update_Metadata
sub update_metadata {
    my ($deploy)=@_;
    my ($cpic,$rel,$dir,$edr,$menu,$msg,$response,$ret,$url);
    my ($double_quote,$link);
    $cpic = Obj::cpic();
    $edr  = Obj::edr();
    $rel  = Obj::rel();
    $rel->require_versionmatrix_lib();
    if (!$deploy->{preference}{Repository}{value}) {
        Msg::log("The repository directory is not set. The directory can be set using the 'Set Preferences' task");
    }
    $double_quote="\"";
    $link="https://sort.symantec.com/support/related_links/offline-release-updates";
    $deploy->{install_dir} = '/opt/VRTS/install';
    $deploy->{supported_padvs} = [qw(aix53 aix61 aix71 hpux1123 hpux1131 rhel4_x86_64 rhel5_x86_64 rhel6_x86_64 sles9_x86_64 sles10_x86_64 sles11_x86_64 sol8_sparc sol9_sparc sol10_sparc sol10_x64 sol11_sparc sol11_x64)];
    $deploy->{default_metadata_file} = 'deploy_sfha.tar';
    $deploy->{default_metadata_name} = 'deploy_sfha';
    $dir = "$edr->{tmpdir}/metadata/$deploy->{default_metadata_name}";
    $edr->{localsys}->mkdir($dir) if (!-d $dir);
    $edr->{savelog} = 1;
    $url = CPIP::get_hotfix_sort_url();
    # Need to fill in level,padv,product for a correct page.
    $url.='?level=ga&padv=sol10_sparc&product=SFHA';
    if (Cfg::opt('load_metadata')) {
        # If specified -load_metadata, do NOT check for SORT connectivity, just expand the deploy_sfha.tar file
        my $metadata=Cfg::opt('load_metadata');
        $deploy->expand_and_load_tarfile(EDRu::dirname($metadata), EDRu::basename($metadata));
    } elsif (Cfg::opt('noipc')) {
        $msg=Msg::new("Download release matrix and installer Hot Fixes is not allowed while -noipc option is being used.", 60, 1134);
        Msg::n();
        $msg->bold();
        $ret = $deploy->load_re_matrix_or_install_hotfixes();
        return if EDR::getmsgkey($ret,'back');
        Msg::prtc();
        return;
    } else {
        $msg = Msg::new("\nChecking SORT connectivity", 60, 1135);
        $msg->left();
        $response = EDRu::download_page($url);
        if (!$response) {
            $msg->right_failed();
            $cpic->{sortready}=0;
            Msg::n();
            $msg = Msg::new("Cannot connect with SORT (https://sort.symantec.com) to download update files", 60, 1136);
            $msg->printn;
            return if (Cfg::opt('download_metadata'));
            $msg = Msg::new("When you install the Deployment Server, the installation includes product metadata with up-to-date release matrices. The release matrices include real-time information about the latest Symantec product updates and installer hot fixes available across all the products and platforms. If your system has Internet access, release matrix information is automatically updated from the SORT site with each use. If your system is behind a firewall, these updates are not possible and the release matrices eventually get out of date.", 60, 1137);
            $msg->printn;
            $msg = Msg::new("To update release information on systems without Internet access, you can download a .tar file (deploy_sfha.tar) containing all the latest release matrices. Then, load it on to your Deployment Server.  Go to $link and click on ${double_quote}deploy_sfha.tar [Download]${double_quote}, and save it to your desktop. Then, copy the file to a location accessible to the Deployment Server, and load it by entering the file location at the prompt below:", 60, 1138, "$link", "${double_quote}", "${double_quote}");
            $msg->printn;
            $ret = $deploy->load_re_matrix_or_install_hotfixes();
            return if EDR::getmsgkey($ret,'back');
            Msg::prtc();
            return;
        }
        $msg->right_done();
        $cpic->{sortready}=1;
        $msg=Msg::new("The Update Metadata option is used to load release matrix updates on to systems that do not have an Internet connection with SORT (https://sort.symantec.com).  Your system has a connection with SORT and is able to receive updates.  No action is necessary unless you would like to create a file to update another Deployment Server system.", 60, 1139);
        if (Cfg::opt('download_metadata')) {
            $deploy->download_and_tar_re_matrix_and_install_hotfixes();
        } else {
            while (1) {
                Msg::title();
                $msg->printn();
                $menu = $deploy->metadata_menu();
                last if (EDR::getmsgkey($menu,'back'));
            }
        }
    }
    return;
}

sub metadata_menu {
    my ($deploy)=@_;
    my ($backopt,$def,$help,$menuopts,$msg,$ret,$menu);
    $backopt = 1;
    $help = Msg::new("Release matrix files are reference files used for storing all Storage Foundation release information.", 60, 1140);
    $msg = Msg::new("Download release matrix updates and installer Hot Fixes", 60, 1141);
    push (@$menuopts,$msg->{msg});
    $msg = Msg::new("Load an update tar file", 60, 1142);
    push (@$menuopts,$msg->{msg});
    $msg = Msg::new("Select the option:", 60, 1143);
    $menu = $msg->menu($menuopts, $def, $help, $backopt);
    return $menu if EDR::getmsgkey($menu,'back');
    Msg::n();
    if ($menu == 1) {
        $deploy->download_and_tar_re_matrix_and_install_hotfixes();
        Msg::prtc();
    } elsif ($menu == 2) {
        $ret = $deploy->load_re_matrix_or_install_hotfixes();
        return if EDR::getmsgkey($ret,'back');
        Msg::prtc();
    }
    return;
}

sub download_re_matrix_meta_files {
    my ($deploy)=@_;
    my ($rel,$msg,$noipc,$pseudo_hotfix,$ret,$tmpdir);
    $rel=Obj::rel();
    $tmpdir = EDR::tmpdir();
    $rel->{sort_ready_for_vc} = 1;
    $rel->{vc_matrix_path_update} = "$tmpdir/metadata/$deploy->{default_metadata_name}/matrix/";
    for my $padv(@{$deploy->{supported_padvs}}) {
        $rel->{padv_managed_host_for_vc} = $padv;
        $msg = Msg::new("Downloading release matrix updates for platform $padv", 60, 1144, "$padv");
        $msg->left;
        $ret = $rel->update_matrix_data($pseudo_hotfix, $noipc);
        ($ret) ? $msg->right_failed : $msg->right_done;
    }
    return;
}

sub download_install_hotfixes {
    my ($deploy)=@_;
    my ($content,$cpic,$all_hotfix,$msg,$release,$sort_status,$tmpdir);
    $cpic = Obj::cpic();
    $tmpdir = EDR::tmpdir();

    for my $padv(@{$deploy->{supported_padvs}}) {
        ($sort_status,$all_hotfix) = $deploy->get_all_install_hotfix_via_sort_webservice($padv);
        if (!$sort_status) {
            $msg = Msg::new("Failed to download installer Hot Fixes from the SORT website (https://sort.symantec.com)", 60, 1145);
            $msg->error();
            last;
        }
        if (!@$all_hotfix) {
            Msg::log("Cannot get the install Hot Fixes for padv $padv");
            next;
        }
        $cpic->{hotfixdir} = "$tmpdir/metadata/$deploy->{default_metadata_name}/cpihotfix";
        for my $install_hotfix(@$all_hotfix) {
            $release = $install_hotfix->{release_name};
            $msg = Msg::new("Downloading installer Hot Fix for release $release on platform $padv", 60, 1146, "$release", "$padv");
            $msg->left;
            $content = get_hotfix_content_from_gzfile($install_hotfix);
            if (!EDRu::eval_json($content)) {
                $msg->right_failed;
                next;
            }
            $deploy->save_install_hotfix($install_hotfix,$content,$padv,$release);
            $msg->right_done;
        }
    }
    return;
}

sub get_all_install_hotfix_via_sort_webservice {
    my ($deploy,$padv)=@_;
    my ($rel,$all_hotfix,$ahotfix,$json,$response);
    $rel=Obj::rel();
    $all_hotfix = [];

    ($response,$json) = $rel->get_hotfix_json_via_sort($padv);
    if (($response) && (ref($json->{$padv}) eq 'HASH')) {
        for my $id (keys %{$json->{$padv}}) {
            $ahotfix = $json->{$padv}{$id};
            push (@$all_hotfix,$ahotfix);
        }
    }
    return ($response,$all_hotfix);
}

sub download_and_tar_re_matrix_and_install_hotfixes {
    my ($deploy)=@_;
    my ($msg,$ret,$edr,$tmpdir);
    $edr = Obj::edr();
    $tmpdir = "$edr->{tmpdir}/metadata/$deploy->{default_metadata_name}";
    EDR::cmd_local("_cmd_rmr $tmpdir");
    $deploy->download_re_matrix_meta_files();
    $deploy->download_install_hotfixes();
    $ret = $deploy->tar_metadata_files();
    if ($ret) {
        $msg = Msg::new("File $deploy->{default_metadata_file} was not successfully created.", 60, 1147, "$deploy->{default_metadata_file}");
        $msg->error();
    } else {
        $msg = Msg::new("File $deploy->{default_metadata_file} is saved at $deploy->{install_dir}. It must be copied to a file system accessible by the Deployment Server.", 60, 1148, "$deploy->{default_metadata_file}", "$deploy->{install_dir}");
        $msg->nprint;
    }
    return $ret;
}

sub tar_metadata_files {
    my ($deploy)=@_;
    my($dest_dir,$tarfile,$ret,$tmpdir);
    my $localsys = Obj::localsys();
    $tmpdir = EDR::tmpdir();
    $dest_dir = $deploy->{install_dir};
    $tarfile  = $deploy->{default_metadata_file};
    $localsys->mkdir($dest_dir) if (!-d $dest_dir);
    EDR::cmd_local("cd $tmpdir/metadata; _cmd_tar -cf $tarfile $deploy->{default_metadata_name}");
    $ret ||= EDR::cmdexit();
    EDR::cmd_local("_cmd_cp -pf $tmpdir/metadata/$tarfile $dest_dir");
    $ret ||= EDR::cmdexit();
    return $ret;
}

sub save_install_hotfix {
    my ($deploy,$install_hotfix,$content,$padv,$release) = (@_);
    my ($file,$tmpdir);
    $file = "${release}_$padv.pl";
    $tmpdir = EDR::tmpdir();
    $content = "#HOTFIX_VERSION:$install_hotfix->{release_name}\n$content";
    EDRu::writefile($content,"$tmpdir/metadata/$deploy->{default_metadata_name}/cpihotfix/$file");
    return;
}

sub load_re_matrix_or_install_hotfixes {
    my ($deploy)=@_;
    my ($answer,$backopt,$defopt,$dir,$file,$msg);
    $backopt = 1;
    $defopt = "$deploy->{install_dir}/$deploy->{default_metadata_file}";
    $msg = Msg::new("Enter the location of the meta-data tar file:", 60, 1149);
    $answer = $msg->ask($defopt,'',$backopt);
    return $answer if EDR::getmsgkey($answer,'back');
    if (-f $answer) {
        $dir = EDRu::dirname($answer);
        $file = EDRu::basename($answer);
        $deploy->expand_and_load_tarfile($dir,$file);
    } else {
        $msg =Msg::new("The tar file does not exist. Input again", 60, 1150);
        $msg->print;
        $deploy->load_re_matrix_or_install_hotfixes();
    }
    return;
}

sub expand_and_load_tarfile {
    my ($deploy,$src_dir,$tarfile) = (@_);
    my (@dest_dirs,$cpic,$msg,$repo_dir,$ret,$tmpdir);
    my $localsys = Obj::localsys();
    $cpic = Obj::cpic();
    $repo_dir = $deploy->{preference}{Repository}{value};
    $repo_dir =~ s/\/+$//g;

    push (@dest_dirs,$deploy->{install_dir});
    if ($repo_dir && ($repo_dir ne $deploy->{install_dir})) {
        push (@dest_dirs,$repo_dir);
    }

    $tmpdir = EDR::tmpdir()."/metadata";
    $localsys->mkdir($tmpdir) if (!-d $tmpdir);
    if ($tarfile =~/\.tar$/g) {
        # untar this file
        EDR::cmd_local("_cmd_cp -pf $src_dir/$tarfile $tmpdir; cd $tmpdir; _cmd_tar xf $tarfile 2>/dev/null");
    } elsif ($tarfile =~/\.tar\.gz$/g) {
        # unzip and the untar this file
        EDR::cmd_local("_cmd_cp -pf $src_dir/$tarfile $tmpdir; cd $tmpdir; _cmd_gunzip -c $tarfile | _cmd_tar xf - 2>/dev/null");
    } else {
        $msg = Msg::new("File name must end with .tar or .tar.gz", 60, 1151);
        $msg->print;
        return;
    }
    if (EDR::cmdexit() || (!-d "$tmpdir/$deploy->{default_metadata_name}")) {
        $msg = Msg::new("Failed to expand the tar file $tarfile", 60, 1152, "$tarfile");
        $msg->print;
        return;
    }
    $tmpdir .= "/$deploy->{default_metadata_name}";
    $ret = $deploy->verify_and_update_metadata($tmpdir,@dest_dirs);
    if ($ret) {
        if (@dest_dirs > 1) {
            $msg = Msg::new("Meta-data files in directories $repo_dir and $deploy->{install_dir} are updated", 60, 1153, "$repo_dir", "$deploy->{install_dir}");
        } else {
            $msg = Msg::new("Meta-data files in directory $deploy->{install_dir} are updated", 60, 1154, "$deploy->{install_dir}");
        }
    } else {
        if (@dest_dirs > 1) {
            $msg = Msg::new("None of the meta-data files in the tar file $tarfile needs to be loaded to directories $repo_dir or $deploy->{install_dir}", 60, 1155, "$tarfile", "$repo_dir", "$deploy->{install_dir}");
        } else {
            $msg = Msg::new("None of the meta-data files in the tar file $tarfile needs to be loaded to directory $deploy->{install_dir}", 60, 1156, "$tarfile", "$deploy->{install_dir}");
        }
    }
    (Cfg::opt('load_metadata')) ? $msg->print : $msg->nprint;
    return;
}

sub verify_and_update_metadata {
    my ($deploy,$src_dir,@dest_dirs) = (@_);
    my ($files,$hdest_dir,$hsrc_dir,$mdest_dir,$msrc_dir,$rel,$updated);
    my $localsys = Obj::localsys();
    $rel=Obj::rel();
    $msrc_dir = "$src_dir/matrix";
    $hsrc_dir = "$src_dir/cpihotfix";

    $files = '';
    $files = EDR::cmd_local("cd $msrc_dir 2>/dev/null; _cmd_ls *.json 2>/dev/null") if (-d $msrc_dir);
    for my $dest_dir(@dest_dirs) {
        $mdest_dir = "$dest_dir/.matrix";
        $localsys->mkdir($mdest_dir) if (!-d $mdest_dir);
        for my $file(split(/\n/,$files)) {
            if ($deploy->verify_metadata_file_name($file,$msrc_dir) && $deploy->verify_metadata_contents($file,$msrc_dir,$mdest_dir)) {
                EDR::cmd_local("_cmd_cp -pf $msrc_dir/$file $mdest_dir");
                Msg::log("File $file is loaded to $mdest_dir");
                $updated = 1;
            } else {
                Msg::log("File $file does not have a valid file name or more recent contents, skip to load it to $mdest_dir");
            }
        }
    }
    # Re-init release matrix because lots of matrixes has been updated
    $rel->init_releasematrix($rel->{matrix_path});
    $files = '';
    $files = EDR::cmd_local("cd $hsrc_dir 2>/dev/null; _cmd_ls *.pl 2>/dev/null") if (-d $hsrc_dir);
    for my $dest_dir(@dest_dirs) {
        $hdest_dir = "$dest_dir/.cpihotfix";
        $localsys->mkdir($hdest_dir) if (!-d $hdest_dir);
        for my $file(split(/\n/,$files)) {
            if ($deploy->verify_metadata_file_name($file,$hsrc_dir) && $deploy->verify_metadata_contents($file,$hsrc_dir,$hdest_dir)) {
                EDR::cmd_local("_cmd_cp -pf $hsrc_dir/$file $hdest_dir");
                Msg::log("File $file is loaded to $hdest_dir");
                $updated = 1;
            } else {
                Msg::log("File $file does not have a valid file name or more recent contents, skip to load it to $hdest_dir");
            }
        }
    }
    return $updated;
}

sub verify_metadata_file_name {
    my ($deploy,$file,$src_dir) = (@_);
    my ($filename,$padv,$release,$version);
    chomp($file);
    if ($file =~ /^(\S+)-(ga|patch|hotfix)\.json$/) {
        $padv = $1;
        return 1 if (EDRu::inarr($padv, @{$deploy->{supported_padvs}}));
    } elsif ($file =~ /\.pl$/) {
        $filename = $file;
        $filename =~ s/\.pl$//g;
        ($release,$padv)=split(/_/,$filename,2);
        $version = EDR::cmd_local("_cmd_grep '^#HOTFIX_VERSION' $src_dir/$file 2>/dev/null");
        chomp($version);
        return 1 if ($version =~ /:${release}$/ && (EDRu::inarr($padv, @{$deploy->{supported_padvs}})));
    }
    Msg::log("File $file does not have a valid file name");
    return 0;
}

sub verify_metadata_contents {
    my ($deploy,$file,$src_dir,$dest_dir) = (@_);
    my ($content,$padv,$update,$update_hash);
    my ($is_hotfix,$is_matrix,$dest_file,$matrix,$matrix_hash,$src_file);
    chomp($file);
    return 0 if ($file !~ /\.(pl|json)$/);
    $src_file = "$src_dir/$file";
    $dest_file = "$dest_dir/$file";
    $update = EDRu::readfile($src_file);
    if ($file =~ /\.pl$/) {
        return 0 if (!EDRu::eval_json($update));
        $is_hotfix = 1;
    } elsif ($file =~ /\.json$/) {
        $update_hash = EDRu::eval_json($update);
        return 0 if (!$update_hash);
        $is_matrix = 1;
    }
    return 1 if (!-f "$dest_file");
    # check if update contents if more recent than existing files
    if (EDRu::cksum($src_file) eq EDRu::cksum($dest_file)) {
        Msg::log("$dest_file has the same checksum as $src_file, no need to update");
        return 0;
    }
    if ($is_hotfix) {
        my ($update_vers,$vers);
        $content = EDRu::readfile($dest_file);
        $update_vers = $1 if ($update =~ /^#HOTFIX_VERSION:\S+-\S+P(\d+)$/m);
        $vers = $1 if ($content =~ /^#HOTFIX_VERSION:\S+-\S+P(\d+)$/m);
        return 1 if ($update_vers >= $vers);
    } elsif ($is_matrix) {
        $matrix = EDRu::readfile($dest_file);
        $matrix_hash = EDRu::eval_json($matrix);
        return 1 if (!$matrix_hash);
        $padv = $file;
        $padv =~ s/-\S*$//g;
        return $deploy->verify_matrix_release_date($update_hash,$matrix_hash,$padv);
    }
    return 0;
}

sub verify_matrix_release_date {
    my ($deploy,$update_hash,$matrix_hash,$padv) = (@_);
    my ($max_mdate,$max_mid,$max_udate,$max_uid,$mdate,$mid,$udate,$uid);
    for my $id(keys %{$update_hash->{"$padv"}}) {
        $uid = $update_hash->{"$padv"}{$id}{release_id};
        $udate = $update_hash->{"$padv"}{$id}{release_date};
        $max_uid = $uid if ($uid > $max_uid);
        $max_udate = $udate if ($udate gt $max_udate);
    }
    for my $id(keys %{$matrix_hash->{"$padv"}}) {
        $mid = $matrix_hash->{"$padv"}{$id}{release_id};
        $mdate = $matrix_hash->{"$padv"}{$id}{release_date};
        $max_mid = $mid if ($mid > $max_mid);
        $max_mdate = $mdate if ($mdate gt $max_mdate);
    }
    Msg::log("Update file latest release date: $max_udate, highest release id: $max_uid; Matrix file latest release date: $max_mdate, highest release id: $max_mid.");
    return 1 if (($max_uid >= $max_mid) || ($max_udate ge $max_mdate));
    return 0;
}
# _END_Update_Metadata

# _START_Terminology
sub terminology {
    my ($deploy)=@_;
    Msg::title();
    my $edr=Obj::edr();
    my $title=Msg::new("$edr->{msg}{deploy_sfha_script} Terminology\n", 60, 1157, "$edr->{msg}{deploy_sfha_script}");
    $title->bold;

    my $ri=Msg::new("Release Image", 60, 1158);
    my $rid=Msg::new("A directory that contains the package, patch, documentation, and scripted components of a Storage Foundation release.", 60, 1159);
    $ri->bold_bullet($rid);

    my $rep=Msg::new("Repository", 60, 1094);
    my $repd=Msg::new("A designated directory where release images are stored in an organized manner. Sharing the repository directory makes the stored release images accessible from other systems in your environment.", 60, 1160);
    $rep->bold_bullet($repd);

    my $ds=Msg::new("Deployment Server", 60, 1161);
    my $dsd=Msg::new("The system hosting the repository.  With ssh communication enabled between systems, the Deployment Server can act as a central point of installation and upgrade of other systems in your environment.", 60, 1162);
    $ds->bold_bullet($dsd);

    my $rl=Msg::new("Release Levels", 60, 1163);
    my $rld=Msg::new("Each release image is defined to be one of three release levels:", 60, 1164);
    $rl->bold_bullet($rld);

    my $ga=Msg::new("Base Releases", 60, 1165);
    my $gad=Msg::new("Major releases that:", 60, 1166);
    my $ga1=Msg::new("allow fresh installation and product selection", 60, 1167);
    my $ga2=Msg::new("include new features", 60, 1168);
    my $ga3=Msg::new("are installable over most prior versions", 60, 1169);
    my $ga4=Msg::new("install package-based updates", 60, 1170);
    my $ga5=Msg::new("offer automated installations", 60, 1171);
    my $ga6=Msg::new("are available from FileConnect (https://fileconnect.symantec.com)", 60, 1172);
    $ga->bold_bullet($gad,1);
    $ga1->bullet(2);
    $ga2->bullet(2);
    $ga3->bullet(2);
    $ga4->bullet(2);
    $ga5->bullet(2);
    $ga6->bullet(2);

    my $patch=Msg::new("Maintenance Releases", 60, 1173);
    my $patchd=Msg::new("Patch release updates that:", 60, 1174);
    my $patch1=Msg::new("include multiple fixes and limited new features", 60, 1175);
    my $patch2=Msg::new("are installable over any product of one specific Base Release version", 60, 1176);
    my $patch3=Msg::new("install patch based updates (on native-patch based platforms)", 60, 1177);
    my $patch4=Msg::new("offer automated installations", 60, 1171);
    my $patch5=Msg::new("are available from SORT (https://sort.symantec.com)", 60, 1178);
    $patch->bold_bullet($patchd,1);
    $patch1->bullet(2);
    $patch2->bullet(2);
    $patch3->bullet(2);
    $patch4->bullet(2);
    $patch5->bullet(2);

    my $hf=Msg::new("Hot Fix Releases", 60, 1179);
    my $hfd=Msg::new("Product release updates that:", 60, 1180);

    my $hf1=Msg::new("include fixes only, typically for one specific issue", 60, 1181);
    my $hf2=Msg::new("are installable over specific products of one specific Base or Maintenance release version", 60, 1182);
    my $hf3=Msg::new("multiple Hot Fixes may potentially be installed", 60, 1183);
    my $hf4=Msg::new("install patch based updates (on native-patch based platforms)", 60, 1177);
    my $hf5=Msg::new("offer automated installation from 6.0.1 level forward. Prior levels must be manually installed", 60, 1184);
    my $hf6=Msg::new("are available from SORT (https://sort.symantec.com)", 60, 1178);
    $hf->bold_bullet($hfd,1);
    $hf1->bullet(2);
    $hf2->bullet(2);
    $hf3->bullet(2);
    $hf4->bullet(2);
    $hf5->bullet(2);
    $hf6->bullet(2);

    my $fc=Msg::new("FileConnect", 60, 1185);
    my $fcd=Msg::new("The site where Base images can be downloaded - https://fileconnect.symantec.com/", 60, 1186);
    $fc->bold_bullet($fcd);

    my $sort=Msg::new("SORT", 60, 1187);
    my $sortd=Msg::new("The location where Maintenance and Hot Fix release images can be downloaded - https://sort.symantec.com.  Release images on SORT can be automatically downloaded by the Deployment Script as long as the system has an Internet connection.", 60, 1188);
    $sort->bold_bullet($sortd);

    my $obs=Msg::new("obsoleted", 60, 1189);
    my $obsd=Msg::new("A term describing patches that should typically be avoided because a more recent update exists.", 60, 1190);
    $obs->bold_bullet($obsd);

    my $curr=Msg::new("current", 60, 1191);
    my $currd=Msg::new("A term describing patches that are currently of the most recent version and therefore are not obsoleted.", 60, 1192);
    $curr->bold_bullet($currd);

    my $rm=Msg::new("release matrix", 60, 1193);
    my $rmd=Msg::new("Reference files that store all Storage Foundation release information.  These files are periodically updated in order to provide the most current release information.", 60, 1194);
    $rm->bold_bullet($rmd);

    my $push=Msg::new("push installation", 60, 1195);
    my $pushd=Msg::new("Installing and updating remote systems from the Deployment Server using enabled ssh.", 60, 1196);
    $push->bold_bullet($pushd);

    my $pull=Msg::new("direct installation", 60, 1197);
    my $pulld=Msg::new("Installing and updating systems directly by accessing Release Images stored within a shared repository.", 60, 1198);
    $pull->bold_bullet($pulld);

    Msg::prtc();
}
# _END_Terminology

# _START_Deploy_SFHA_Help
sub deploy_sfha_help {
    my ($deploy)=@_;
    Msg::title();
    my $edr=Obj::edr();
    my $help=Msg::new("$edr->{msg}{deploy_sfha_script} Usage\n", 60, 1199, "$edr->{msg}{deploy_sfha_script}");
    $help->bold;

    my $rmsg=Msg::new("The [$edr->{msg}{key_manage_repository}] Manage Repository Images option is used to:", 60, 1200, "$edr->{msg}{key_manage_repository}");
    my $r1=Msg::new("View All Available Storage Foundation Releases", 60, 1201);
    my $r2=Msg::new("Download a Release Image into the repository", 60, 1202);
    my $r3=Msg::new("View Available Release Images that are currently stored in the repository", 60, 1203);
    my $r4=Msg::new("Remove a Release Image\n", 60, 1204);
    my $r5=Msg::new("Load a previously downloaded Release Image into the repository", 60, 1205);
    $rmsg->print;
    $r1->bullet;
    $r2->bullet;
    $r3->bullet;
    $r4->bullet;
    $r5->bullet;

    my $vmsg=Msg::new("The [$edr->{msg}{key_version_check}] Version Check Systems option is used to:", 60, 1206, "$edr->{msg}{key_version_check}");
    my $v1=Msg::new("Discover what Storage Foundation products are installed on a system including version and Hot Fixes", 60, 1207);
    my $v2=Msg::new("Discover what higher Base, Maintenance, and Hot Fix release versions are available for the system to upgrade to", 60, 1208);
    my $v3=Msg::new("Conveniently download available Maintenance and Hot Fix level Release Images into the repository", 60, 1209);
    my $v4=Msg::new("Install any available release currently stored in the repository that is applicable to the system\n", 60, 1210);
    $vmsg->print;
    $v1->bullet;
    $v2->bullet;
    $v3->bullet;
    $v4->bullet;

    my $umsg=Msg::new("The [$edr->{msg}{key_upgrade_install}] Upgrade/Install Systems option is used to install any available release currently stored in the repository on to the local or remote systems\n", 60, 1211, "$edr->{msg}{key_upgrade_install}");
    $umsg->print;

    my $mmsg=Msg::new("The [$edr->{msg}{key_update_metadata}] Update Meta-Data option is required only when a Deployment Server is behind a firewall and does not have access to the SORT website for receiving updates.  This option performs two tasks:", 60, 1212, "$edr->{msg}{key_update_metadata}");
    my $m1=Msg::new("When run from a system with Internet access, downloads a tar file containing release matrix and Hot Fix updates so that the most up-to-date release information is available", 60, 1213);
    my $m2=Msg::new("Loads the contents of the tar file on to a Deployment Server running behind a firewall\n", 60, 1214);
    $mmsg->print;
    $m1->bullet;
    $m2->bullet;

    my $smsg=Msg::new("The [$edr->{msg}{key_set_preferences}] Set Preferences option is used to view and modify various program settings.\n", 60, 1215, "$edr->{msg}{key_set_preferences}");
    $smsg->print;

    my $tmsg=Msg::new("The [$edr->{msg}{key_terminology}] Terminology and Usage option is used to define frequently referenced terms and describe usage.", 60, 1216, "$edr->{msg}{key_terminology}");
    $tmsg->print;
    Msg::prtc();
}
# _END_Deploy_SFHA_Help

# _START_Deploy_SFHA_Lock
# return 0 if the lock is added successfully
# return 1 otherwise
sub add_deploy_sfha_lock {
    my ($deploy,$image,$action,$target) = @_;
    my $cfg=Obj::cfg();
    return [] if($cfg->{opt}{repository});
    return $deploy->manage_deploy_sfha_lock("add",$image,$action,$target);
}

# return 0 if the lock is cleared successfully
# return 1 otherwise
sub clear_deploy_sfha_lock {
    my ($deploy,$image,$action,$target) = @_;
    my $cfg=Obj::cfg();
    return [] if($cfg->{opt}{repository});
    return $deploy->manage_deploy_sfha_lock("clear",$image,$action,$target);
}

# return 0 if successful
# return 1 if failed
# when writing to the lockfile, the following fields are added in one single line:
# field[0]	hostname that this action is taking place
# field[1]	PID of this process
# field[2]	deploy_sfha script name of the above PID
# field[3]	image to be opterated on
# field[4]	epoch time this action happens, not the PID creating time
# field[5]	action of this entry, right now can be install/delete
# field[6]	target hostname(s) for install only
#
sub manage_deploy_sfha_lock {
    my ($deploy,$action_type,$image,$action,$target) = @_;
    my ($lockfile,$msg,@lines,@fields,$conflict,%seen);

    $deploy->get_deploy_sfha_process_info();
    $conflict=[];

    $lockfile=$deploy->get_deploy_sfha_lockfile($image);
    if (!$lockfile) {
        $deploy->{repo}{read_only}=1;
        return $conflict;
    }
    if ($target) {
        @lines=(split /\s+|,/, $target);
        $target=join ",", @lines;
    }
    $msg=$deploy->read_deploy_sfha_locks();
    @lines=(split /\n/,$msg);
    @lines = grep { ! $seen{ $_ }++ } @lines;
    $deploy->cleanup_stale_deploy_sfha_locks(\@lines,$deploy->{proc_info}{hostname},$deploy->{proc_info}{name});
    # no further action if $action_type is unset, taken as cleanup only.
    return $conflict unless ($action_type);

    if ($action_type eq "add") {
        $conflict=$deploy->find_conflict_deploy_sfha_lock(\@lines,$action_type,$action,$image,$target);
        if ($#$conflict<0) {
            $msg="$deploy->{proc_info}{hostname} $deploy->{proc_info}{pid} $deploy->{proc_info}{name} $image $deploy->{proc_info}{time} $action ";
            $msg.="$target" if ($target);
            push (@lines, $msg);
        }
    } elsif ($action_type eq "clear") {
        for (my $i=$#lines+1; $i>=0; $i--) {
            next if ($lines[$i] =~ /^(#|\s*$)/);
            @fields=(split /\s+/, $lines[$i]);
            next unless (($fields[0] eq $deploy->{proc_info}{hostname}) &&
                         ($fields[1] eq $deploy->{proc_info}{pid}) &&
                         ($fields[2] eq $deploy->{proc_info}{name}) &&
                         ($fields[3] eq $image) &&
                         ($fields[5] eq $action) );
            next unless ($target eq $fields[6]);
            delete $lines[$i];
        }
    }
    %seen = ();
    @lines = grep { ! $seen{ $_ }++ } @lines;
    $msg = join "\n", @lines;
    if ($#$conflict<0) {
        $deploy->write_deploy_sfha_locks($msg,$lockfile);
    }

    return $conflict;
}

sub cleanup_stale_deploy_sfha_locks {
    my ($deploy,$lines,$hostname,$proc_name)=@_;
    my ($pids,@fields,$content,$repo_dir,$files);

    $pids=$deploy->get_deploy_sfha_pids();
    return 0 if ($#$pids < 0);

    $deploy->get_deploy_sfha_process_info() unless ($deploy->{proc_info});
    $hostname ||=$deploy->{proc_info}{hostname};
    $proc_name||=$deploy->{proc_info}{name};
    if ($#$lines <0 ) {
        $repo_dir=$deploy->get_deploy_sfha_lock_dir();
        $files=EDR::cmd_local("_cmd_ls $repo_dir");
        for my $file (split /\n/,$files) {
            next unless (-f "$repo_dir/$file");
            @fields=(split /\n/, EDRu::readfile("$repo_dir/$file"));
            next if ($#fields < 0);
            $deploy->cleanup_stale_deploy_sfha_locks(\@fields,$hostname);
            $content=join "\n",@fields;
            $deploy->write_deploy_sfha_lockfile($content,"$repo_dir/$file");
        }
    } else {
        for (my $i=$#$lines; $i>=0; $i--) {
            if ($lines->[$i] =~ /^(#|\s*$)/) {
                delete $lines->[$i];
                next;
            }
            @fields=(split /\s+/, $lines->[$i]);
            next unless ($fields[0] eq $hostname);
            delete $lines->[$i] unless (EDRu::inarr($fields[1],@{$pids}));
        }
    }
    return 0;
}

sub duplicate_actions_on_host {
    my ($deploy,$targets,$hosts)=@_;
    my ($ret,@existing);

    $ret=0;
    $hosts=~s/,$//g;
    $targets=~s/,$//g;
    if ($hosts && $targets) {
        @existing=(split /,/,$hosts);
        for my $target (split /,/, $targets) {
            next unless (EDRu::inarr($target,@existing));
            $ret=1;
            last;
        }
    }

    return $ret;
}

sub find_conflict_deploy_sfha_lock {
    my ($deploy,$lines,$action_type,$action,$image,$target)=@_;
    my ($conflict,@fields,@conflicts,%seen);

    if ($target) {
        @conflicts=(split /\s+|,/, $target);
        $target=join ",", @conflicts;
    }
    @conflicts=();
    for (my $i=0; $i<=$#$lines; $i++) {
        next if ($lines->[$i] =~ /^(#|\s*$)/);
        @fields=(split /\s+/, $lines->[$i]);
        if ($action eq "change_repository") {
            # for repository change, any exisitng actions for the images are conflicts
            #push (@conflicts, $lines->[$i]);
            push (@conflicts, [@fields]);
        } elsif ($action eq "install") {
            # if the same target being used for another action
            # then we skip this
            #push (@conflicts, $lines->[$i]) if ($deploy->duplicate_actions_on_host($target,$fields[6]));
            push (@conflicts, [@fields]) if ($deploy->duplicate_actions_on_host($target,$fields[6]));
            $conflict=$i+1;
            # if the same image is used for add/delete image, or change_repository
            # then we skip this
            push (@conflicts, [@fields]) if (($fields[5] =~ /change_repository/) ||
                                      (($fields[3] eq $image) && ($fields[5] =~ /(download|load|remove)/)));
        } elsif ($action eq "download") {
            # if the action is download an image
            # then we have to make sure the image is not being downloaded|loaded|removed by another deploy_sfha
            # and no change_repository
            push (@conflicts, [@fields]) if (($fields[5] =~ /change_repository/) ||
                                      (($fields[3] eq $image) && ($fields[5] =~ /(download|load|remove)/)));
        } elsif ($action eq "load") {
            # if the action is load an image
            # then we have to make sure the image is not being downloaded|loaded|installed|removed by another deploy_sfha
            # and no change_repository
            push (@conflicts, [@fields]) if (($fields[5] =~ /change_repository/) ||
                                      (($fields[3] eq $image) && ($fields[5] =~ /(download|install|load|remove)/)));
        } elsif ($action eq "remove") {
            # if the action is delete an image
            # then we have to make sure the image is not being used by anybody else
            # and no change_repository
            push (@conflicts, [@fields]) if (($fields[5] =~ /change_repository/) ||
                                      ($image && ($fields[3] eq $image)));
            $conflict=$i+1;
        } else {
            last;
        }
    }

    @conflicts = grep { ! $seen{ $_ }++ } @conflicts;
    return \@conflicts;
}

sub display_deploy_sfha_conflicts {
    my ($deploy,$conflicts)=@_;
    my ($i,$j,$contents, $titles);

    return '' if ($#$conflicts < 0);
    $titles=['hostname',[qw(pid PID)],[qw(script script)],[qw(image image)],[qw(date_time Starting_time)],[qw(action action)],[qw(targets targets)]];

    for ($i=0; $i <=$#$conflicts; $i++) {
        $contents->[$i]{$titles->[0]} = ($conflicts->[$i][0]) ? $conflicts->[$i][0] : "-";
        for ($j=1; $j <= $#$titles; $j++) {
            $contents->[$i]{$titles->[$j][0]} = ($conflicts->[$i][$j]) ? $conflicts->[$i][$j] : "-";
        }
    }

    return Msg::table($contents,$titles);
}

# get the repository/.lock status file
sub get_deploy_sfha_lockfile {
    my ($deploy,$image)=@_;
    my ($repo,$edr);

    $repo=$deploy->get_deploy_sfha_lock_dir();
    return  unless (-d $repo);
    $repo.="/$image";
    return $repo;
}

# get the repository/.lock dir
sub get_deploy_sfha_lock_dir {
    my $deploy=shift;
    my ($repo,$cpic,$edr);

    $cpic = Obj::cpic();
    $edr = Obj::edr();
    $repo=$deploy->get_image_repository_dir();
    return '' unless (-d $repo);
    $repo=~s/\/$//g;
    $repo.="/.lock";
    $edr->{localsys}->mkdir($repo) if (!-d $repo);

    return $repo;
}

sub get_image_repository_dir {
    my $deploy=shift;
    my ($repository,$cpic,$cfg);

    $cfg=Obj::cfg();
    $cpic=Obj::cpic();
    $repository||=$cfg->{opt}{repository} if (defined $cfg->{opt}{repository});
    $repository||=$deploy->{preference}{Repository}{value} if (defined $deploy->{preference}{Repository}{value});

    return $repository;
}

sub get_deploy_sfha_pids {
    my $deploy=shift;
    my ($edr,$localsys,$file,$files,@pids,$dir,$pid,$sys_pids);

    $edr = Obj::edr();
    $file=$edr->{script};
    $dir=(split /$file/,$edr->{tmpdir}, 2)[0];
    $dir.="/$file-*";
    $files="$dir/$file\.pid";
    $localsys=Obj::localsys();
    $files=$localsys->cmd("_cmd_ls $files");
    $sys_pids=$localsys->proc_pids();
    for $file (split /\n/,$files) {
        next unless (-f $file);
        $pid=$deploy->read_deploy_sfha_pid_file($file);
        $pid=~s/\s+//g;
        next unless ($pid=~/^\d+$/);
        push (@pids, $pid) if (!EDRu::inarr($pid,@pids) && EDRu::inarr($pid,@{$sys_pids}));
    }
    return \@pids;
}

sub read_deploy_sfha_pid_file {
    my ($deploy,$file)=@_;
    my ($pid,$localsys);

    $localsys=Obj::localsys();
    $pid=$localsys->readfile($file);

    return $pid;
}

sub create_deploy_sfha_pid_file {
    my ($deploy,$dir,$file)=@_;
    my ($pid,$localsys);

    $file=$dir . "/". $file . ".pid";
    $pid=$$;
    $localsys=Obj::localsys();
    $localsys->createfile($file);
    $localsys->writefile($pid,$file);

    return 0;
}

# read_deploy_sfha_locks reads the contents of all the lock files
# under <repo>/.lock/
sub read_deploy_sfha_locks {
    my $deploy=shift;
    my ($repo_dir,$locks,$files,$file_content);

    $repo_dir=$deploy->get_deploy_sfha_lock_dir();
    $files=EDR::cmd_local("_cmd_ls $repo_dir");
    for my $file (split /\n/,$files) {
        $file="$repo_dir/$file";
        if (-f $file) {
            $file_content=EDRu::readfile($file);
            $locks.=$file_content ."\n";
        }
    }
    return $locks;
}

sub write_deploy_sfha_lockfile {
    my ($deploy,$content,$file)=@_;

    return unless ($deploy->can_write_deploy_sfha_lockfile($file));
    return EDRu::writefile($content,$file);
}

sub write_deploy_sfha_locks {
    my ($deploy,$locks,$lockfile)=@_;
    my ($repo_dir,$lock,$file,%contents);

    for $lock (split /\n/, $locks) {
        next if ($lock=~/^#|^\s*$/);
        $file=(split /\s+/, $lock)[3];
        push (@{$contents{$file}}, $lock);
    }
    $repo_dir=$deploy->get_deploy_sfha_lock_dir();
    for $file (sort keys %contents) {
        $lock=join "\n",@{$contents{$file}};
        $deploy->write_deploy_sfha_lockfile($lock,"$repo_dir/$file");
    }
    $lockfile=(split /\//, $lockfile)[-1];
    if (!$contents{$lockfile}) {
        $lock="";
        $deploy->write_deploy_sfha_lockfile($lock,"$repo_dir/$lockfile");
    }

    return '';
}

# test the deploy_sfha_lockfile writable or not
sub can_write_deploy_sfha_lockfile {
    my ($deploy,$lock)=@_;
    my ($msg,$ret,$localsys);

    $ret="";
    $localsys=Obj::localsys();
    $msg="# write test from $localsys->{sys}, please ignore";
    $ret=EDR::cmd_local("echo '$msg' >> $lock");
    ($ret) ? $deploy->{repo}{read_only}=1 : $deploy->{repo}{read_only}=0;
    $msg=($ret) ? "No, we can NOT write to $lock" : "Yeah! wecan write to $lock";
    return ($ret) ? 0 : 1;
}

sub get_deploy_sfha_process_info {
    my $deploy=shift;

    $deploy->{proc_info}{pid}=$$;
    $deploy->{proc_info}{name}=$0;
    $deploy->{proc_info}{hostname}=Obj::localsys()->{sys};
    $deploy->{proc_info}{time}=time();
    $deploy->{proc_info}{time}=EDRu::datetime();
    $deploy->{proc_info}{name}=EDRu::app_path($deploy->{proc_info}{name}) ."/". EDRu::basename($deploy->{proc_info}{name});

    return '';
}
# _END_Deploy_SFHA_Lock

1;
