#!/bin/sh
#
# $Copyright: Copyright (c) 2014 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#

# Set PATH
PATH=/usr/bin:/bin:$PATH
export PATH

# Set LANG
LC_ALL=C
export LC_ALL
LANG=C
export LANG

# Get work directory
DIRNAME=`dirname $0`
[ -z "$DIRNAME" ] && DIRNAME="."

OLDPWD=`pwd`
cd $DIRNAME
DIRNAME=`pwd`
cd $OLDPWD

# Get platform information of the running machine 
UNAME_PLATFORM=`(uname -s) 2>/dev/null`  || UNAME_PLATFORM=unknown
UNAME_ARCH=`(uname -m) 2>/dev/null` || UNAME_ARCH=unknown
UNAME_DISTRO=`(uname -r) 2>/dev/null` || UNAME_DISTRO=unknown

case "${UNAME_PLATFORM}" in
    AIX)
        LOCAL_PADV=AIX
        ;;
    SunOS)
        SUNOS_VERS=`echo ${UNAME_DISTRO} | sed 's/5\.//'`
        if [ x"${UNAME_ARCH}" = xi86pc ]; then
            SUNOS_ARCH=x64
        else
            SUNOS_ARCH=sparc
            FALLBACK_PADV=SolSparc
        fi
        LOCAL_PADV="Sol${SUNOS_VERS}${SUNOS_ARCH}"
        VXGETTEXTPADV="sol_${SUNOS_ARCH}"
        ;;
    HP-UX)
        HPUX_VERS=`echo ${UNAME_DISTRO} | sed 's/\.//g;s/B//'`
        LOCAL_PADV="HPUX${HPUX_VERS}"
        ;;
    Linux)
        if [ -f /etc/oracle-release ]; then
            OL_VARIANT=OL
            OL_VERS=`echo ${UNAME_DISTRO} | grep el | sed 's/.*el\([0-9]*\).*/\1/'`
            # set as the latest known OL version if could not get OL version from kernel string
            if [ -z "${OL_VERS}" ]; then
                OL_VERS=7
            fi
            LINUX_DISTRO="${OL_VARIANT}${OL_VERS}"
        elif [ -f /etc/redhat-release ]; then
            RHEL_VARIANT=`grep CentOS /etc/redhat-release 2>/dev/null`
            if [ ! -z "${RHEL_VARIANT}" ]; then
                RHEL_VERS=`echo ${RHEL_VARIANT} | sed 's/.*CentOS release\s*//;s/\([0-9]*\).*/\1/'`
                RHEL_VARIANT=CentOS
            else 
                RHEL_VERS=`echo ${UNAME_DISTRO} | grep el | sed 's/.*el\([0-9]*\).*/\1/'`
                # set as the latest known RHEL version if could not get RHEL version from kernel string
                if [ -z "${RHEL_VERS}" ]; then
                    RHEL_VERS=7
                fi
                RHEL_VARIANT=RHEL
            fi
            LINUX_DISTRO="${RHEL_VARIANT}${RHEL_VERS}"
        elif [ -f /etc/SuSE-release ]; then
            SLES_VARIANT=SLES
            SLES_VERS=`grep '^VERSION' /etc/SuSE-release 2>/dev/null | sed 's/VERSION\s*=\s*//;s/\s.*$//'`
            if [ -z "${SLES_VERS}" ]; then
                KERNEL_VERSION=`echo ${UNAME_DISTRO} | sed 's/-.*$//' | cut -d. -f1`
                if [ ${KERNEL_VERSION} -gt 2 ]; then
                    KERNEL_VERSION=`echo ${UNAME_DISTRO} | sed 's/-.*$//' | cut -d. -f2`
                    if [ ${KERNEL_VERSION} -lt 12 ]; then
                        SLES_VERS=11
                    else
                        SLES_VERS=12
                    fi
                else
                    KERNEL_VERSION=`echo ${UNAME_DISTRO} | sed 's/-.*$//' | cut -d. -f3`
                    if [ ${KERNEL_VERSION} -lt 16 ]; then
                        SLES_VERS=9
                    elif [ ${KERNEL_VERSION} -lt 27 ]; then
                        SLES_VERS=10
                    else
                        SLES_VERS=11
                    fi
                fi
            fi
            LINUX_DISTRO="${SLES_VARIANT}${SLES_VERS}"
        elif [ -f /usr/bin/vmware  ]; then
            LINUX_DISTRO=ESX
        fi
        if [ ! -z "${LINUX_DISTRO}" ]; then
            # change x86_64 to x8664
            LINUX_ARCH=`echo ${UNAME_ARCH} | sed 's/_//'`
            LOCAL_PADV="${LINUX_DISTRO}${LINUX_ARCH}"
            FALLBACK_PADV=SLES10x8664
        fi
        ;;
esac

# Get Perl top directory and EDR scripts directory
if [ -d "$DIRNAME/scripts/EDR" ]; then
    PTOPDIR=$DIRNAME/perl
elif [ -d "$DIRNAME/../scripts/EDR" ]; then
    PTOPDIR=$DIRNAME/../perl
fi

# Get executable Perl binary and Perl version with the following sequence:
# 1.  perl/bin/perl
# 2.  perl/<LOCAL_PADV>/bin/perl
# 3.  perl/<FALLBACK_PADV>/bin/perl
# 4.  /opt/VRTSperl/bin/perl

if [ ! -z "$PTOPDIR" ]; then
    PERLDIRS="$PTOPDIR $PTOPDIR/${LOCAL_PADV}"
    if [ ! -z "${FALLBACK_PADV}" ]; then
        PERLDIRS="$PERLDIRS $PTOPDIR/${FALLBACK_PADV}"
    fi
fi
PERLDIRS="$PERLDIRS /opt/VRTSperl"

for PERLDIR in $PERLDIRS; do
    PERLBIN=$PERLDIR/bin/perl
    if [ -x "$PERLBIN" ]; then
        PERLINFO=`$PERLBIN -v 2>&1`
        if [ "$?" -eq "0" ]; then
            PDIR=$PERLDIR
            PERLVERS=`echo "$PERLINFO" | grep This.is | sed 's/.*v\([0-9.]*\).*/\1/'`
            PERLFOR=`echo "$PERLINFO" | grep This.is | sed 's/.*for \([a-zA-Z0-9._-]*\).*/\1/'`

            # set LC_ALL=C explicitly
            BADLANG=`echo "$PERLINFO" | grep "Setting locale failed" 2>/dev/null`
            if [ ! -z "$BADLANG" ]; then
                LC_ALL=C
                export LC_ALL
                LANG=C
                export LANG
            fi
            break
        fi
    fi
done

if [ -z "$PDIR" ]; then
    echo "Error: Cannot find perl to execute $0"
    exit 1
fi

if [ -z "$PERLVERS" -o -z "$PERLFOR" ]; then
    echo "Error: Cannot get perl version or architecture to execute $0"
    exit 1
fi

PERLDEBUG=""
if [ x"$PWDUTIL_DEBUG" = x1 ]; then
    PERLDEBUG="-d"
fi

IOPT="-I$PDIR/lib/$PERLVERS/$PERLFOR -I$PDIR/lib/$PERLVERS -I$PDIR/lib/site_perl/$PERLVERS/$PERLFOR -I$PDIR/lib/site_perl/$PERLVERS -I$PDIR/lib/site_perl"
IOPT="-I$SDIR/EDR -I$SDIR/CPIC -I$SDIR/CPIP -I$SDIR/Deploy $IOPT"

eval 'exec $PDIR/bin/perl -x $PERLDEBUG -S $IOPT $0 ${1+"$@"}'
        if 0;
#!perl
use strict;
use Getopt::Long qw(:config no_ignore_case);
use File::Basename qw(basename);

use constant {

    # Default ssh port
    SSH_PORT                  => 22,

    # Default rsh port
    RSH_PORT                  => 514,

    # Default Expect prompt
    EXPECT_SYMC_PROMPT        => 'SYMCPrompt>',

    # Exit codes
    OK                        => 0,
    ERR_SYNTAXERROR           => 1,
    ERR_BINARYNOTAVAILABLE    => 2,
    ERR_SERVICENOTAVAILABLE   => 3,
    ERR_CONNECTIONDENIED      => 4,
    ERR_INVALIDPASSWORD       => 5,
    ERR_UNKNOWNERROR          => 255,

};

# Global variables
my $program          = basename($0);
my $g_local_user     = get_local_user() || 'root';

my $g_error_no       = OK;
my $g_error_msg      = '';
my $g_keyfile        = '';
my $g_debug          = 0;

# Default value for connecton action, type, user
my $default_action   = 'check';
my $default_type     = 'ssh';
my $default_user     = $g_local_user;
my $default_password = $default_user;
my $default_port     = SSH_PORT;

# Local variables for main program
my (@host_entries,$ret);

@host_entries=();

process_args(\@host_entries);
$ret=perform_connection_action(\@host_entries);

exit $ret;

sub process_args {
    my ($host_entries) = @_;
    my ($action,$type,$hostname,$user,$password,$port,$hostfile,$errors,$help,$ret,$ret_msg);
    my ($host_entry,$host_uris,@host_uris);

    $help=0;
    $errors=0;
    $hostfile='';

    $action=$ARGV[0];
    if (!$action) {
        $errors++;
    } elsif (is_valid_action($action)) {

        # Simple arguments

        #$ARGV[0]: configure|unconfigure|check
        #$ARGV[1]: ssh|rsh
        #$ARGV[2]: hostname|IP address
        #$ARGV[3]: user, default is local user
        #$ARGV[4]: password
        #$ARGV[5]: port (only for ssh)

        $type=$ARGV[1];
        $hostname=$ARGV[2];
        $user=$ARGV[3];
        $password=$ARGV[4];
        $port=$ARGV[5];

        if (!$type) {
            # type and hostname should be specified with simple format.
            print "Error: connection type and hostname should be specified\n";
            $errors++;
        } elsif (!$hostname) {
            # hostname should be specified for simple format.
            print "Error: hostname should be specified\n";
            $errors++;
        } else {
            ($ret,$ret_msg)=validate_host_entry($action,$type,$hostname,$user,$password,$port);
            if ($ret == ERR_SYNTAXERROR) {
                print "Error: $ret_msg\n";
                $errors++;
            } else {
                $host_entry=create_host_entry($action,$type,$hostname,$user,$password,$port);
                push @{$host_entries}, $host_entry;
            }
        }

     } else {

        # Advanced argurments

        $action='';
        my %options = (
            '-action|a=s' => \$action,
            '-type|t=s' => \$type,
            '-user|u=s' => \$user,
            '-password|p=s' => \$password,
            '-port|P=s' => \$port,
            '-hostfile|f=s' => \$hostfile,
            '-keyfile|k=s' => \$g_keyfile,
            'debug!' => \$g_debug,
            'h|?' => \$help,
        );

        GetOptions(%options);

        if ($help) {
            usage();
            exit OK;
        }

        @host_uris=@ARGV;

        ($ret,$ret_msg)=validate_host_entry($action,$type,'',$user,$password,$port);
        if ($ret == ERR_SYNTAXERROR) {
            print "Error: $ret_msg\n";
            $errors++;
        } else {
            $default_action=$action if ($action);
            $default_type=$type if ($type);
            $default_user=$user if ($user);
            $default_password=$password if ($password);
            $default_port=$port if ($port);
        }

        if ($g_keyfile) {
            if (! -f "$g_keyfile") {
                print "Error: the private key file '$g_keyfile' do not exist\n";
                $errors++;
            } elsif (! -f "$g_keyfile.pub") {
                print "Error: the public key file '$g_keyfile.pub' do not exist\n";
                $errors++;
            }
        }

        if ($hostfile) {
            if (-f $hostfile) {
                $host_uris = run_local_command("grep -v '^#' $hostfile 2>/dev/null");
                for my $host_uri (split(/\n+/, $host_uris)) {
                    $host_uri=~s/^\s*//;
                    $host_uri=~s/\s.*$//;
                    next if (!$host_uri || ($host_uri=~/^#/));

                    push @host_uris, $host_uri;
                }
            } else {
                print "Error: $hostfile do not exist\n";
                $errors++;
            }
        }

        for my $host_uri (@host_uris) {
            ($action,$type,$hostname,$user,$password,$port)=parse_host_uri($host_uri);
            ($ret,$ret_msg)=validate_host_entry($action,$type,$hostname,$user,$password,$port);
            if ($ret == ERR_SYNTAXERROR) {
                print "Error: $host_uri is not a correct host URI, $ret_msg\n";
                $errors++;
            } else {
                $host_entry=create_host_entry($action,$type,$hostname,$user,$password,$port);
                push @{$host_entries}, $host_entry;
            }
        }
    }

    # if no errors and no host specified, then ask to input host names.
    if (!$errors && !@{$host_entries}) {
        print "Error: hostname should be specified\n";
        $errors++;
    }

    if ($errors) {
        usage();
        exit ERR_SYNTAXERROR;
    }

    return OK;
}

sub usage {
    my $usage=<< "_EOF_";

Usage:

Command syntax with simple format:

    $program check|configure|unconfigure ssh|rsh <hostname|IP addr> [<user>] [<password>] [<port>]

Command syntax with advanced format:

    $program [--action|-a 'check|configure|unconfigure']\
               [--type|-t 'ssh|rsh']\
               [--user|-u  '<user>']\
               [--password|-p '<password>']\
               [--port|-P '<port>']\
               [--hostfile|-f '<hostfile>']\
               [--keyfile|-k '<keyfile>']\
               [-debug|-d]
               <host_URI>

    $program -h | -?

Options:

    --action|-a 'check|configure|unconfigure'  Specify action type, default is 'check'
    --type|-t 'ssh|rsh'                        Specify connection type, default is 'ssh'
    --user|-u  '<user>'                        Specify user name on remote machine, default is the current login user
    --password|-p '<password>'                 Specify user password, default is the user name
    --port|-P '<port>'                         Specify port number for ssh connection, default is 22
    --hostfile|-f '<hostfile>'                 Specify the file which list the hosts
    --keyfile|-k '<keyfile>'                   Specify the private key file
    -debug                                     Print debug informations
    -h|-?                                      Print help messages

In which, <host_URI> could be the following formats:
    <hostname>
    <user>:<password>\@<hostname>
    <user>:<password>\@<hostname>:<port>

For example:

1. To check ssh connection for only one host:
    $program check ssh hostname

2. To configure ssh for only one host:
    $program configure ssh hostname user password 

3. To unconfigure rsh for only one host:
    $program unconfigure rsh hostname

4. To configure ssh for multiple hosts with same user ID and password:
    $program -a configure -t ssh -u user -p password hostname1 hostname2 hostname3

5. To configure ssh or rsh for different hosts with different user ID and password
    $program -a configure -t ssh user1:password1\@hostname1 user2:password2\@hostname2

6. To check or configure ssh or rsh for multiple hosts with one host configuration file
    $program -a configure -t ssh --hostfile /hostfile

The content of the configuration file would be like:

# cat /hostfile
user1:password1\@hostname1
user2:password2\@hostname2
user3:password3\@hostname3
user4:password4\@hostname4
hostname5

7. To keep the host configuration file secret, you can use the 3rd party utility to encrypt and decrypt the host file with password. For example:

### run openssl to encrypt the host file in base64 format
# openssl aes-256-cbc -a -salt -in /hostfile -out /hostfile.enc
enter aes-256-cbc encryption password: <password>
Verifying - enter aes-256-cbc encryption password: <password>

### remove the original plain text file
# rm /hostfile

### run openssl to decrypt the encrypted host file
# $program -a configure -t ssh `openssl aes-256-cbc -d -a -in /hostfile.enc`
enter aes-256-cbc decryption password: <password>

8. To use the ssh authentication keys which are not under the default \$HOME/.ssh directory, you can use --keyfile option to specify the ssh keys. For example:

### create a directory to host the key pairs:
# mkdir /keystore

### generate private and public key pair under the directory:
# ssh-keygen -t rsa -f /keystore/id_rsa

### setup ssh connection with the new generated key pair under the directory:
# $program -a configure -t ssh --keyfile /keystore/id_rsa user:password\@hostname
 

The following exit values are returned:

0     Successful completion.
1     Command syntax error.
2     Ssh or rsh binaries do not exist.
3     Ssh or rsh service is down on the remote machine.
4     Ssh or rsh command execution is denied due to password is required.
5     Invalid password is provided.
255   Other unknown error.

_EOF_

    my $undocumented_usage=<< "_EOF_";

9. To configure ssh or rsh for different hosts with different user ID and password
    $program -a configure ssh://user1:password1\@hostname1 rsh://user2:password2\@hostname2

10. To configure ssh on one host, and unconfigure rsh on another host
    $program -- +ssh://user1:password1\@hostname1 -rsh://user2:password2\@hostname2

11. To support the customized ssh/scp binaries:

# SSHPATH=/usr/local/bin/ssh
# export SSHPATH
# SCPPATH=/usr/local/bin/scp
# export SCPPATH
# SSHOPTIONS='-x -o NumberOfPasswordPrompts=0 -o StrictHostKeyChecking=no -p 2222'
# export SSHOPTIONS    
# SSHOPTIONS_PWD="-o 'StrictHostKeyChecking=no' -o 'UserKnownHostsFile=/dev/null' -p 2222"
# export SSHOPTIONS_PWD
# SCPOPTIONS="-o 'StrictHostKeyChecking=no' -o 'UserKnownHostsFile=/dev/null' -P 2222"
# export SCPOPTIONS
# $program -a configure -t ssh user1:password1\@hostname1

Or: 

# SSHPATH=/usr/local/bin/ssh SCPPATH=/usr/local/bin/scp SSHOPTIONS='-x -o NumberOfPasswordPrompts=0 -o StrictHostKeyChecking=no -p 2222' SSHOPTIONS_PWD="-o 'StrictHostKeyChecking=no' -o 'UserKnownHostsFile=/dev/null' -p 2222" SCPOPTIONS="-o 'StrictHostKeyChecking=no' -o 'UserKnownHostsFile=/dev/null' -P 2222" ./pwdutil.pl -a configure -t ssh user1:password1\@hostname1

_EOF_

    print $usage;
}

sub parse_host_uri {
    my ($host_uri)=@_;
    my ($schemapart,$userpart,$hostpart);
    my ($action,$type,$hostname,$user,$password,$port)=('','','','','','');

    # trim the leading and trailing spaces.
    $host_uri=~s/^\s*//;
    $host_uri=~s/\s*$//;

    if ($host_uri=~m|^(?:(.*)://+)?(?:([^@]*)\@)?(.*)|) {
        $schemapart=$1;
        $userpart=$2;
        $hostpart=$3;

        if ($schemapart) {
            $type=$schemapart;
            if ($schemapart=~/^([^A-Za-z0-9]*)(\w*)$/) {
                $action=$1;
                $type=$2;
                if ($action=~/^\+/) {
                    $action='configure';
                } elsif ($action=~/^\-/) {
                    $action='unconfigure';
                } elsif ($action=~/^\?/) {
                    $action='check';
                }
            }
        }

        if ($userpart) {
            $user=$userpart;
            if ($userpart=~/^([^:]*):(.*)$/) {
                $user=$1;
                $password=$2;
            }
        }

        if ($hostpart) {
            $hostname=$hostpart;
            if ($hostname =~ /^[^:]*:\d*$/ || $hostname =~ /]:\d*$/) {
                $hostname =~ s/:(\d*)$//;
                $port = $1;
            }
        }
    }

    printx("host_uri: $host_uri\n");
    printx("==>");
    printx(" action: $action,") if ($action);
    printx(" type: $type,") if ($type);
    printx(" hostname: $hostname,") if ($hostname);
    printx(" user: $user,") if ($user);
    printx(" password: $password,") if ($password);
    printx(" port: $port") if ($port);
    printx("\n");

    return ($action,$type,$hostname,$user,$password,$port);
}

sub validate_host_entry {
    my ($action,$type,$hostname,$user,$password,$port)=@_;
    my ($ret,$ret_msg,@errors);

    @errors=();
    if ($action && !is_valid_action($action)) {
        push @errors, "action '$action'";
    }

    if ($type && !is_valid_type($type)) {
        push @errors, "type '$type'";
    }

    if ($hostname && !is_valid_hostname($hostname)) {
        push @errors, "hostname '$hostname'";
    }

    if ($user && !is_valid_user($user)) {
        push @errors, "user '$user'";
    }

    if ($password && !is_valid_password($password)) {
        push @errors, "password '$password'";
    }

    if ($port && !is_valid_port($port)) {
        push @errors, "port '$port'";
    }

    $ret=OK;
    $ret_msg='';
    if (@errors) {
        $ret=ERR_SYNTAXERROR;
        $ret_msg="the ";
        $ret_msg.=join(', ',@errors);
        if (@errors > 1) {
            $ret_msg.=" are not correct";
        } else {
            $ret_msg.=" is not correct";
        }
    }

    if (wantarray) {
        return ($ret,$ret_msg);
    }
    return $ret;
}

sub create_host_entry {
    my ($action,$type,$hostname,$user,$password,$port)=@_;

    my $host_entry={};

    $host_entry->{action} = $action || $default_action;
    $host_entry->{type} = $type || $default_type;
    if ($hostname && $hostname=~/^\[(.*)\]$/) {
        $hostname="$1";
    }
    $host_entry->{hostname} = $hostname;
    $host_entry->{user} = $user || $default_user;
    $host_entry->{password} = $password || $default_password;

    # only set port for ssh
    if ($host_entry->{type} eq 'ssh') {
        if ($port) {
            if ($port != SSH_PORT) {
                $host_entry->{port} = $port;
            }
        } else {
            if ($default_port != SSH_PORT) {
                $host_entry->{port} = $default_port;
            }
        }
    }

    return $host_entry;
}

sub perform_connection_action_for_host {
    my ($host_entry)=@_;
    my ($action,$type,$user,$password,$hostname,$port,$ret);

    $action=$host_entry->{action};
    $type=$host_entry->{type};
    $user=$host_entry->{user};
    $password=$host_entry->{password};
    $hostname=$host_entry->{hostname};
    $port=$host_entry->{port};

    # clear error message
    set_error_sys($hostname, OK, '');

    if ( $action eq "check" ) {
        if ( $type eq "ssh" ) {
            $ret=check_ssh_sys($hostname,$user,$port);
        } elsif ( $type eq "rsh" ) {
            $ret=check_rsh_sys($hostname,$user);
        }
    } elsif ( $action eq "configure" ) {
        if ( $type eq "ssh" ) {
            $ret=setup_ssh_sys($hostname,$user,$password,$port);
        } elsif ( $type eq "rsh" ) {
            $ret=setup_rsh_sys($hostname,$user,$password);
        }
    } elsif ( $action eq "unconfigure" ) {
        if ( $type eq "ssh" ) {
            $ret=cleanup_ssh_sys($hostname,$user,$port);
        } elsif ( $type eq "rsh" ) {
            $ret=cleanup_rsh_sys($hostname,$user);
        }
    }

    return $ret;
}

sub perform_connection_action {
    my ($host_entries)=@_;
    my ($msg,$ret,$final_ret,$action,$type,$hostname,$user,$port);

    $final_ret=OK;
    for my $host_entry (@{$host_entries}) {
        $action=$host_entry->{action};
        $hostname=$host_entry->{hostname};
        $type=$host_entry->{type};
        $user=$host_entry->{user};
        $port='';
        $port=$host_entry->{port} || '' if ($type eq 'ssh');
        print "$hostname: $action $type $user $port\n";

        $ret=perform_connection_action_for_host($host_entry);
        $msg=get_error_sys($host_entry->{hostname});
        if ($msg) {
            print "$msg\n";
        } elsif ($ret == OK) {
            if ($action eq 'check') {
                print "$type connection with $hostname is already ready\n"; 
            } else {
                print "successfully $action $type for $hostname\n"; 
            }
        } elsif ($ret == ERR_SYNTAXERROR) {
            print "Error: the command syntax is not correct\n"; 
        }
        print "return code: $ret\n\n";
        $final_ret=$ret || $final_ret;
    }

    return $final_ret;
}

sub is_valid_action {
    my ($action)=@_;
    return 1 if ($action &&
                 (($action eq "configure") ||
                  ($action eq "unconfigure") ||
                  ($action eq "check")));
    return 0;
}

sub is_valid_type {
    my ($type)=@_;
    return 1 if ($type &&
                 (($type eq "ssh") ||
                  ($type eq "rsh")));
    return 0;
}

sub is_valid_hostname {
    my ($hostname)=@_;
    return 0 if (!$hostname);

    if ($hostname=~/^\[(.*)\]$/) {
        $hostname=$1;
    }
    return 1 if ((isip($hostname)) ||
                 ($hostname=~/^\w[\w\-\.]*$/));
    return 0;
}

sub is_valid_port {
    my ($port)=@_;
    return 1 if ($port && $port=~/^\d+$/);
    return 0;
}

sub is_valid_user {
    my ($user)=@_;
    return 1 if ($user && $user=~/^[\w\+\-\.]+$/);
    return 0;
}

sub is_valid_password {
    my ($password)=@_;
    return 1 if ($password && $password=~/^\S+$/);
    return 0;
}

#
# To support ssh
#

sub get_sshpath {
    my $sshpath=$ENV{SSHPATH};
    $sshpath=get_command_path('ssh') if (!$sshpath);
    return $sshpath;
}

# SSHOPTIONS:      the ssh options do not require password input
# SSHOPTIONS_PWD:  the ssh options require password input
sub get_sshoptions {
    my ($port,$askpwd)=@_;
    my ($sshoptions,$sshkeyfile);

    $sshoptions=$ENV{SSHOPTIONS};
    $sshoptions=$ENV{SSHOPTIONS_PWD} || $sshoptions if ($askpwd);

    if (!$sshoptions) {
        if ($askpwd) {
            # ask password
            $sshoptions="-o 'StrictHostKeyChecking=no' -o 'UserKnownHostsFile=/dev/null'";
        } else {
            # do not ask password
            $sshoptions='-x -o NumberOfPasswordPrompts=0 -o StrictHostKeyChecking=no';
        }

        if ($port && ($port != SSH_PORT)) {
             $sshoptions.=" -p $port";
        }

        $sshkeyfile=get_sshkeyfile();
        $sshoptions.=" -i $sshkeyfile" if ($sshkeyfile);
    }
    return $sshoptions;
}

sub get_scppath {
    my $scppath=$ENV{SCPPATH};
    $scppath=get_command_path('scp') if (!$scppath);
    return $scppath;
}

# For the customized scp whose options with password input work well for ssh connection already set up,
# Then only need specify SCPOPTIONS which require password input,
# For example, for openssl,
# SCPOPTIONS="-o 'StrictHostKeyChecking=no' -o 'UserKnownHostsFile=/dev/null'
# Otherwise, specify the following 2 environment variables:
# SCPOPTIONS:      the scp options do not require password input
# SCPOPTIONS_PWD:  the scp options require password input
sub get_scpoptions {
    my ($port,$askpwd)=@_;
    my ($scpoptions,$sshkeyfile);

    $scpoptions=$ENV{SCPOPTIONS} || $ENV{SSHOPTIONS} ;
    $scpoptions=$ENV{SCPOPTIONS_PWD} || $scpoptions if ($askpwd);

    if (!$scpoptions) {
        if ($askpwd) {
            # ask password
            $scpoptions="-o 'StrictHostKeyChecking=no' -o 'UserKnownHostsFile=/dev/null'";
        } else {
            # do not ask password
            $scpoptions='-o NumberOfPasswordPrompts=0 -o StrictHostKeyChecking=no';
        }

        if ($port && ($port != SSH_PORT)) {
             $scpoptions.=" -P $port";
        }

        $sshkeyfile=get_sshkeyfile();
        $scpoptions.=" -i $sshkeyfile" if ($sshkeyfile);
    }
    return $scpoptions;
}

sub get_sshkeyfile {
    return $g_keyfile || '';
}

sub get_sshkeygen {
    my $sshkeygen=$ENV{SSHKEYGEN};
    $sshkeygen=get_command_path('ssh-keygen') if (!$sshkeygen);
    return $sshkeygen;
}

# Check ssh connection on system
sub check_ssh_sys {
    my ($sys,$user,$port)=@_;
    my ($sshpath,$sshoptions,$cmd,$banner,$rc,$error_no,$error_msg);

    # check ssh
    $sshpath=get_sshpath();
    if (!$sshpath || ! -f $sshpath) {
        $error_no=ERR_BINARYNOTAVAILABLE;
        if (!$sshpath) {
            $error_msg="could not find ssh command on the local system";
        } else {
            $error_msg="ssh binary '$sshpath' does not exist on the local system";
        }
        set_error_sys($sys,$error_no,$error_msg);
        return $error_no;
    }

    $user||=$g_local_user;

    $sshoptions=get_sshoptions($port);
    $cmd = "LC_ALL=C LANG=C $sshpath $sshoptions $user\@$sys echo";

    ($banner,$rc)=run_local_command("$cmd 2>&1");
    if ($rc) {
        # SSH connection is not configured
        if ($banner=~/Connection refused/mi) {
            $error_no=ERR_SERVICENOTAVAILABLE;
            $error_msg="ssh service is not started on $sys";
            if ($port && ($port != SSH_PORT)) {
                $error_msg="ssh service is not started on $sys with port $port";
            }
        } elsif ($banner=~/Permission denied/mi) {
            $error_no=ERR_CONNECTIONDENIED;
            $error_msg="ssh permission was denied on $sys";
        } else {
            $error_no=ERR_UNKNOWNERROR;
            $error_msg="ssh command to $sys exit with $rc";
        }
        set_error_sys($sys,$error_no,$error_msg);
        return $error_no;
    }

    # SSH connection is well
    return OK;
}

# Setup ssh connection between local system and the remote system
sub setup_ssh_sys {
    my ($sys,$user,$passwd,$port,$timeout)=@_;
    my ($sshpath,$sshoptions,$scppath,$scpoptions,$cmd,$ret,$error,$error_no,$error_msg);
    my (@keyfiles,$keyfile,$keyfile_public,$keyfile_tmp,$sshkeygen);
    my ($exp,@exp_params,$prompt,$connected,@cmds);
    my ($localsys,$local_home,$remote_home,$ssh_dir,$authorized_keys_file);

    $user||=$g_local_user;
    $ret=check_ssh_sys($sys,$user,$port);
    if ($ret == OK) {
        set_error_sys($sys, OK, "ssh connection with $sys is already ready");
        return $ret;
    } elsif ($ret != ERR_CONNECTIONDENIED) {
        return $ret;
    }

    # clear error message
    set_error_sys($sys, OK, '');

    $local_home = get_local_home($user);

    # Create ssh-key file if no key/pub exist already, specified by "-keyfile" or default.
    @keyfiles=();
    $keyfile = get_sshkeyfile();
    push(@keyfiles, $keyfile) if ($keyfile);
    push(@keyfiles, "$local_home/.ssh/id_rsa");
    push(@keyfiles, "$local_home/.ssh/id_dsa");

    $keyfile='';
    $keyfile_public='';
    for my $file (@keyfiles) {
        if (-f "$file" && -f "$file".'.pub') {
            $keyfile=$file;
            $keyfile_public="$file".'.pub';
            last;
        }
    }
    if (!$keyfile) {
        $sshkeygen=get_sshkeygen();
        if (!$sshkeygen || ! -f $sshkeygen) {
            $error_no=ERR_BINARYNOTAVAILABLE;
            if (!$sshkeygen) {
                $error_msg="could not find ssh-keygen command on the local system";
            } else {
                $error_msg="ssh-keygen binary '$sshkeygen' does not exist on the local system";
            }
            set_error_sys($sys,$error_no,$error_msg);
            return $error_no;
        }

        $keyfile="$local_home/.ssh/id_rsa";
        $keyfile_public=$keyfile.'.pub';
        run_local_command("$sshkeygen -q -t rsa -N \'\' -f $keyfile 2>/dev/null");
    }

    if (! -f "$keyfile") {
        $error_no=ERR_UNKNOWNERROR;
        $error_msg="could not generate ssh key file on the local system";
        set_error_sys($sys,$error_no,$error_msg);
        return $error_no;
    }

    # Copy ssh public key file to remote
    $scppath=get_scppath();
    $scpoptions=get_scpoptions($port,1);
    if (!$scppath || ! -f $scppath) {
        $error_no=ERR_BINARYNOTAVAILABLE;
        if (!$scppath) {
            $error_msg="could not find scp command on the local system";
        } else {
            $error_msg="scp binary '$scppath' does not exist on the local system";
        }
        set_error_sys($sys,$error_no,$error_msg);
        return $error_no;
    }

    $passwd||='';
    $timeout||=5;

    $connected=1;
    $localsys=get_localsys();
    $keyfile_tmp='/var/tmp/symc_auto_keyfile.' . $localsys;
    $cmd = "LC_ALL=C LANG=C $scppath $scpoptions $keyfile_public $user\@[$sys]:$keyfile_tmp 2>&1";
    @exp_params = (
        [ qr/\(yes\/no\)\?\s*$/ => sub { my $self=shift;
                                         $self->send("yes\n");
                                         $self->exp_continue; } ],
        #[ '-re', qr/ermission denied/mi => sub { $connected=0 } ],
        [ qr/(ermission denied|many authentication failures)/i
                                => sub { $connected=0 } ],
        [ qr/assword:\s*$/      => sub { my $self=shift;
                                         $self->send("$passwd\n");
                                         $self->exp_continue;  } ],
    );

    ($exp,undef,$error) = setup_expect($sys,$user,$passwd,$cmd,$timeout,@exp_params);
    if (!$exp){
        $error_no=ERR_UNKNOWNERROR;
        $error_msg="could not set up scp connection with the Expect module";
        set_error_sys($sys,$error_no,$error_msg);
        return $error_no;
    }
    close_expect($exp);

    if (!$connected) {
        $error_no=ERR_INVALIDPASSWORD;
        $error_msg="could not scp the public key file to $sys due to invalid password";
        set_error_sys($sys,$error_no,$error_msg);
        return $error_no;
    }

    # if $error begin with 1: timeout.
    # if $error begin with 4: unknown error.
    # if $error begin with 3 and not end with 'status 0': scp command finished with some errors
    # if $error begin with 3 and end with 'status 0': scp command finished successfully
    # if $error begin with 2: scp command finished successfully
    if ($error) {
        if ($error=~/^1/m) {
            $error_no=ERR_UNKNOWNERROR;
            $error_msg="could not scp the public key file to $sys due to timeout";
            set_error_sys($sys,$error_no,$error_msg);
            return $error_no;
        } elsif ($error=~/^4/m) {
            $error_no=ERR_UNKNOWNERROR;
            $error_msg="could not scp the public key file to $sys due to unknown issues";
            set_error_sys($sys,$error_no,$error_msg);
            return $error_no;
        } elsif ($error=~/^3/m && $error!~/status 0$/mi) {
            $error_no=ERR_UNKNOWNERROR;
            $error_msg="could not scp the public key file to $sys due to scp command failed";
            set_error_sys($sys,$error_no,$error_msg);
            return $error_no;
        }
    }

    # Setup Expect object with ssh
    $sshpath=get_sshpath();
    $sshoptions=get_sshoptions($port,1);
    $cmd="LC_ALL=C LANG=C $sshpath $sshoptions $user\@$sys 2>&1";

    # 0: means not connected
    # 1: means connected and password is inputed
    # 2: means connected and PS1 prompt is inputed
    # 3: means connected and PS1 prompt is confirmed
    $connected=0;
    $prompt=EXPECT_SYMC_PROMPT;

    # SSH login
    @exp_params=(
        [ qr/\(yes\/no\)\?\s*$/ => sub { my $self=shift;
                                         $connected=1;
                                         $self->send("yes\n");
                                         $self->exp_continue; } ],
        [ qr/ogin:\s*$/         => sub { my $self=shift;
                                         $connected=1;
                                         $self->send("$user\n");
                                         $self->exp_continue; } ],
        [ qr/ermission denied/i => sub { $connected=0 } ],
        [ qr/assword.*?:/       => sub { my $self=shift;
                                         $connected=1;
                                         $self->send("$passwd\n");
                                         $self->exp_continue; } ],
        [ qr/[\]\$\>\#]\s/      => sub { my $self=shift;
                                         $connected=2;
                                         $self->send("PS1='$prompt'\n");
                                         $self->exp_continue; } ],
        [ '-re', qr/$prompt$/   => sub { $connected=3 } ],
    );
    $exp = setup_expect($sys,$user,$passwd,$cmd,$timeout,@exp_params);
    if (!$exp || !$connected){
        $error_no=ERR_INVALIDPASSWORD;
        $error_msg="could not setup ssh connection to $sys due to invalid password";
        set_error_sys($sys,$error_no,$error_msg);
        return $error_no;
    }

    # For some machine, the PS1 is customized and do not has default characters: ], $, #
    if ($connected != 3) {
        $cmd="PS1='$prompt'";
        exec_expect($exp,$cmd,$timeout,"^$prompt\$");
    }

    # get home dir on remote system.
    $cmd="echo HOMEDIR:\$HOME";
    $remote_home=exec_expect($exp,$cmd,$timeout,'^HOMEDIR:.*$');
    $remote_home=~s/HOMEDIR://;
    $remote_home=~s/\r+$//;
    if ( $remote_home ne "/") {
        $remote_home=~s/\/+$//;
    }

    # mkdir $remotehome/.ssh, set its ownership/mode, and add pub key file as authorized key
    $ssh_dir="$remote_home/.ssh";
    $authorized_keys_file="$ssh_dir/authorized_keys";

    @cmds=(
        "mkdir -p $ssh_dir",
        "chmod 0700 $ssh_dir",
        "chown $user $ssh_dir",
        "cat $keyfile_tmp >> $authorized_keys_file",
        "chmod 0600 $authorized_keys_file",
        "chown $user $authorized_keys_file",
        "rm -rf $keyfile_tmp",
        "/sbin/restorecon -v $authorized_keys_file",
    );

    for my $cmd (@cmds) {
        exec_expect($exp,$cmd,$timeout);
    }

    close_expect($exp);
    return OK;
}

sub cleanup_ssh_sys {
    my ($sys,$user,$port)=@_;
    my ($local_home,$remote_home,$sshkey,@sshkeys,$keyfile,@keyfiles,$authorized_keys_file);
    my ($sshpath,$sshoptions,$sshcmd,$cmd,@cmds,$ret,$error_no,$error_msg);

    $user||=$g_local_user;
    $ret=check_ssh_sys($sys,$user,$port);
    if ($ret != OK) {
        # SSH connection is not ready, need not cleanup.
        set_error_sys($sys, OK, "ssh with $sys is not connected, need not clean up");
        return OK;
    }

    # clear error message
    set_error_sys($sys, OK, '');

    $local_home=get_local_home();

    @keyfiles=();
    $keyfile = get_sshkeyfile();
    push(@keyfiles, $keyfile) if ($keyfile);
    push(@keyfiles, "$local_home/.ssh/id_rsa.pub");
    push(@keyfiles, "$local_home/.ssh/id_dsa.pub");

    for my $keyfile (@keyfiles) {
        if (-f "$keyfile") {
            $sshkey=`cat $keyfile`;
            chomp $sshkey;
            push @sshkeys, $sshkey;
        }
    }

    return OK unless (@sshkeys);

    $sshpath=get_sshpath();
    $sshoptions=get_sshoptions($port);
    $sshcmd="LC_ALL=C LANG=C $sshpath $sshoptions $user\@$sys ";

    $cmd="$sshcmd 'echo HOMEDIR:\$HOME' 2>&1";
    $remote_home=run_local_command($cmd);
    if ($remote_home=~/HOMEDIR:(.*)$/mx) {
        $remote_home=$1;
    }
    if (!$remote_home) {
        $error_no=ERR_UNKNOWNERROR;
        $error_msg="could not get HOME directory on $sys";
        set_error_sys($sys,$error_no,$error_msg);
        return $error_no;
    }

    $authorized_keys_file="$remote_home/.ssh/authorized_keys";

    push @cmds, "cp -f $authorized_keys_file $authorized_keys_file.bak 2>/dev/null";
    $cmd="cat $authorized_keys_file.bak 2>/dev/null";
    for my $sshkey (@sshkeys) {
        $cmd.=" | grep -v \"$sshkey\"";
    }
    $cmd="$cmd > $authorized_keys_file";
    $cmd.=";chmod 0600 $authorized_keys_file 2>/dev/null";
    push @cmds, "$cmd";

    for my $cmd (@cmds) {
        run_local_command("$sshcmd '$cmd' 2>&1");
    }

    return OK;
}


#
# To support rsh
#

sub get_rshpath {
    my $rshpath=$ENV{RSHPATH};
    if (!$rshpath) {
        my $os=`uname -s`;
        chomp $os;
        if ($os eq "HP-UX") {
            $rshpath=get_command_path('remsh');
        } else {
            $rshpath=get_command_path('rsh');
        }
    }
    return $rshpath;
}

# Check rsh connection on system
sub check_rsh_sys {
    my ($sys,$user)=@_;
    my ($rshpath,$cmd,$banner,$rc,$error_no,$error_msg);

    # check rsh
    $rshpath=get_rshpath();
    if (!$rshpath || ! -f $rshpath) {
        $error_no=ERR_BINARYNOTAVAILABLE;
        if (!$rshpath) {
            $error_msg="could not find rsh command on the local system";
        } else {
            $error_msg="rsh binary '$rshpath' do not exist on local system";
        }
        set_error_sys($sys,$error_no,$error_msg);
        return $error_no;
    }

    # Check whether rshd ready on remote machine, rshd use 514 port.
    $rc=is_port_connectable($sys, RSH_PORT);
    if (!$rc) {
        $error_no=ERR_SERVICENOTAVAILABLE;
        $error_msg="rsh service is not started on $sys";
        set_error_sys($sys,$error_no,$error_msg);
        return $error_no;
    }

    $user||=$g_local_user;
    $cmd = "LC_ALL=C LANG=C $rshpath $sys -l $user echo 2>&1";
    ($banner,$rc)=run_local_command($cmd);
    if ($rc) {
        # RSH connection is not configured
        if ($banner=~/permission denied|permission is denied|Host name for your address unknown|login incorrect/mi) {
            $error_no=ERR_CONNECTIONDENIED;
            $error_msg="rsh permission was denied on $sys";
        } else {
            $error_no=ERR_UNKNOWNERROR;
            $error_msg="rsh command to $sys exit with $rc";
        }
        set_error_sys($sys,$error_no,$error_msg);
        return $error_no;
    }

    # RSH connection is well
    return OK;
}

# Setup rsh connection between local system and the remote system
sub setup_rsh_sys {
    my ($sys,$user,$passwd,$timeout)=@_;
    my ($rshpath,@cmds,$cmd,$prompt,$connected,$ret,$error_no,$error_msg,@fields);
    my ($exp,@exp_params,$remote_home,$remote_os,$host,$hostname,$aliasname,@rsh_entries,$rsh_entries);

    $user||=$g_local_user;
    $ret=check_rsh_sys($sys,$user);
    if ($ret == OK) {
        set_error_sys($sys, OK, "rsh connection with $sys is already ready");
        return $ret;
    } elsif ($ret != ERR_CONNECTIONDENIED) {
        return $ret;
    }

    # clear error message
    set_error_sys($sys, OK, '');

    # Begin to setup rsh with Expect
    $passwd||='';
    $timeout||=5;

    # Setup Expect object with rsh
    $rshpath=get_rshpath();
    $cmd = "LC_ALL=C LANG=C $rshpath $sys -l $user 2>&1";

    # 0: means not connected
    # 1: means connected and password is inputed
    # 2: means connected and PS1 prompt is inputed
    # 3: means connected and PS1 prompt is confirmed
    $connected=0;
    $prompt=EXPECT_SYMC_PROMPT;

    # RSH login
    @exp_params=(
        [ qr/password:/i         => sub { my $self=shift;
                                         $connected=1;
                                         $self->send("$passwd\r");
                                         $self->exp_continue; } ],
        [ qr/(ogin incorrect|invalid login name or password)/i
                                => sub { $connected=0 } ],
        [ qr/[\]\$\>\#]\s/      => sub { my $self=shift;
                                         $connected=2;
                                         $self->send("PS1='$prompt'\n");
                                         $self->exp_continue; } ],
        [ '-re', qr/$prompt$/   => sub { $connected=3 } ],
    );

    $exp = setup_expect($sys,$user,$passwd,$cmd,$timeout,@exp_params);
    if (!$exp || !$connected){
        $error_no=ERR_INVALIDPASSWORD;
        $error_msg="could not set up rsh connection due to invalid password";
        set_error_sys($sys,$error_no,$error_msg);
        return $error_no;
    }

    # For some machine, the PS1 is customized and do not has default characters: ], $, #
    if ($connected != 3) {
        $cmd="PS1='$prompt'";
        exec_expect($exp,$cmd,$timeout,"^$prompt\$");
    }

    # Get home dir on remote system.
    $cmd="echo HOMEDIR:\$HOME";
    $remote_home=exec_expect($exp,$cmd,$timeout,'^HOMEDIR:.*$');
    $remote_home=~s/HOMEDIR://;
    $remote_home=~s/\r+$//;
    if ($remote_home ne "/") {
        $remote_home=~s/\/+$//;
    }

    # Get OS type on remote system.
    $cmd="echo OS:\`uname -s\`";
    $remote_os=exec_expect($exp,$cmd,$timeout,'^OS:.*$');
    $remote_os=~s/OS://;
    $remote_os=~s/\r+$//;

    # Get the hostname or ip address of local node recognied by remote system.
    if ($remote_os eq 'HP-UX') {
        $cmd="who -u am i";
    } else {
        $cmd="who am i";
    }
    $host=exec_expect($exp,$cmd,$timeout,"^$user\\s.*\$");
    @fields=split /\s+/, $host;
    $host=$fields[-1];
    if ($host=~/\(([^)]*)\)/sx) {
        $host=$1;
    }

    # Add entry in /etc/hosts on remote system if needed.
    @cmds=();

    $aliasname = '';
    if (isip($host)) {
        $hostname = get_localsys();
        $aliasname = $hostname;
        $aliasname =~ s/\..*//;
        if ($aliasname ne $hostname) {
            push @cmds, "echo \"$host $hostname $aliasname\" >> /etc/hosts";
        } else {
            push @cmds, "echo \"$host $hostname\" >> /etc/hosts";
        }
        $host=$hostname;
    }

    # Add entry in ~/.rhosts on remote system
    @rsh_entries=();
    push @rsh_entries, "$host $g_local_user";
    push @rsh_entries, "$aliasname $g_local_user" if ($aliasname && $aliasname ne $host);
    for my $rsh_entry (@rsh_entries) {
        push @cmds, "echo \"$rsh_entry\" >> $remote_home/.rhosts";
    }
    $rsh_entries=join "\n", @rsh_entries;

    # Set permission mode on ~/.rhosts
    push @cmds, "chmod 0600 $remote_home/.rhosts";

    # Set ownership on ~/.rhosts
    push @cmds, "chown $user $remote_home/.rhosts";

    # Execute the commands with Expect
    for my $cmd (@cmds) {
        exec_expect($exp,$cmd,$timeout);
    }

    close_expect($exp);
    return OK;
}

sub cleanup_rsh_sys {
    my ($sys,$user)=@_;
    my ($rshpath,$rshcmd,$cmd,$remote_home,$remote_os,$host,$rhosts_file,$matched);
    my ($rsh_entries,@rsh_entries,$remote_rsh_entries,@remote_rsh_entries,@new_rsh_entries);
    my ($connected,$prompt,$timeout,@exp_params,$exp,@fields,$ret,$error_no,$error_msg);

    $user||=$g_local_user;
    $ret=check_rsh_sys($sys,$user);
    if ($ret != OK) {
        # RSH connection is not ready, need not cleanup
        set_error_sys($sys, OK, "rsh with $sys is not connected, need not clean up");
        return OK;
    }

    # clear error message
    set_error_sys($sys, OK, '');

    # remove the local host entries in ~/.rhosts file.
    $rshpath=get_rshpath();
    $rshcmd="LC_ALL=C LANG=C $rshpath $sys -l $user ";

    $cmd="$rshcmd 'echo OS:\`uname -s\`' 2>/dev/null";
    $remote_os=run_local_command($cmd);
    if ($remote_os=~/OS:(.*)$/mx) {
        $remote_os=$1;
    }
    if (!$remote_os) {
        $error_no=ERR_UNKNOWNERROR;
        $error_msg="could not get OS type on $sys";
        set_error_sys($sys,$error_no,$error_msg);
        return $error_no;
    }

    # Get the hostname or ip address of local node recognied by remote system.

    # Run "rsh <remotemachine> 'who am i'", it return "Must be attached to terminal for 'am I' option"
    # Use Expect to get result of "who am i"

    # 0: means not connected
    # 1: means connected and password is inputed
    # 2: means connected and PS1 prompt is inputed
    # 3: means connected and PS1 prompt is confirmed
    $connected=0;
    $timeout=5;
    $prompt=EXPECT_SYMC_PROMPT;

    # RSH login
    @exp_params=(
        [ qr/[\]\$\>\#]\s/      => sub { my $self=shift;
                                         $connected=2;
                                         $self->send("PS1='$prompt'\n");
                                         $self->exp_continue; } ],
        [ '-re', qr/$prompt$/   => sub { $connected=3 } ],
    );

    $exp = setup_expect($sys,$user,'',$rshcmd,$timeout,@exp_params);
    if (!$exp || !$connected){
        $error_no=ERR_INVALIDPASSWORD;
        $error_msg="could not set up rsh connection with $sys";
        set_error_sys($sys,$error_no,$error_msg);
        return $error_no;
    }

    # For some machine, the PS1 is customized and do not has default characters: ], $, #
    if ($connected != 3) {
        $cmd="PS1='$prompt'";
        exec_expect($exp,$cmd,$timeout,"^$prompt\$");
    }

    if ($remote_os eq 'HP-UX') {
        $cmd="who -u am i";
    } else {
        $cmd="who am i";
    }

    $host=exec_expect($exp,$cmd,$timeout,"^$user\\s.*\$");
    @fields=split /\s+/, $host;
    $host=$fields[-1];
    if ($host=~/\(([^)]*)\)/sx) {
        $host=$1;
    }
    if (!$host || isip($host)) {
        $host=get_localsys();
    }

    close_expect($exp);

    # Get HOME directory
    $cmd="$rshcmd 'echo HOMEDIR:\$HOME' 2>/dev/null";
    $remote_home=run_local_command($cmd);
    if ($remote_home=~/HOMEDIR:(.*)$/mx) {
        $remote_home=$1;
    }
    if (!$remote_home) {
        $error_no=ERR_UNKNOWNERROR;
        $error_msg="could not get HOME directory on $sys";
        set_error_sys($sys,$error_no,$error_msg);
        return $error_no;
    }

    $rhosts_file="$remote_home/.rhosts";

    $cmd="$rshcmd 'cat $rhosts_file' 2>/dev/null";
    $remote_rsh_entries=run_local_command($cmd);
    chomp $remote_rsh_entries;
    @remote_rsh_entries=split(/\n/, $remote_rsh_entries);

    # kick off those rsh entries for the local host in ~/.rhosts file.
    push @rsh_entries, $host.'\s+'.$g_local_user;
    push @rsh_entries, $host.'\..*\s+'.$g_local_user;

    @new_rsh_entries=();
    for my $remote_rsh_entry (@remote_rsh_entries) {
        $matched=0;
        for my $rsh_entry (@rsh_entries) {
            if ($remote_rsh_entry =~ /^$rsh_entry$/m) {
                $matched=1;
                last;
            }
        }
        next if ($matched);
        push @new_rsh_entries, $remote_rsh_entry;
    }

    if (@new_rsh_entries) {
        # return 0 if no rsh entries are removed
        return 0 if (scalar @new_rsh_entries == scalar @remote_rsh_entries);

        $rsh_entries=join "\n", @new_rsh_entries;
        $cmd="$rshcmd 'echo \"$rsh_entries\" > $rhosts_file' 2>/dev/null";
        run_local_command($cmd);
    } else {
        $cmd="$rshcmd 'rm -rf $rhosts_file' 2>/dev/null";
        run_local_command($cmd);
    }

    return OK;
}

#
# Command sub routines
#

# Save error_no and error_msg
sub set_error_sys {
    my ($sys,$error_no,$error_msg)=@_;
    $g_error_no=$error_no || OK;
    $g_error_msg=$error_msg || '';
    return;
}

# Get the latest error_no and error_msg
sub get_error_sys {
    my ($sys)=@_;
    if (wantarray) {
        return ($g_error_no, $g_error_msg);
    }
    return $g_error_msg;
}

# Get current local user name
sub get_local_user {
    my $username=$ENV{LOGNAME};
    chomp $username;
    return $username;
}

# Get HOME directory for local user
sub get_local_home {
    my $homedir=$ENV{HOME};
    chomp $homedir;
    return $homedir;
}

# Get current local user name
sub get_localsys {
    my $localsys=`uname -n`;
    chomp $localsys;
    return $localsys;
}

# Get full path for command
sub get_command_path {
    my ($cmd)=@_;

    my ($cmdpath,$rc)=run_local_command("which $cmd 2>/dev/null");
    return if ($rc || $cmdpath=~/no $cmd/);
    return $cmdpath;
}

# Run command on local system
sub run_local_command {
    my $cmd=shift;
    my ($stdout,$exitcode);

    # Get stdout
    $stdout=`$cmd`;
    chomp $stdout;

    # Get exit code
    $exitcode=$?;
    $exitcode >>= 8 if ($exitcode > 0);
    $exitcode -= 256 if ($exitcode > 127);

    printx("cmd $cmd\n");
    printx("cmd stdout=\n$stdout\n") if ($stdout);
    printx("cmd exit=$exitcode\n\n");

    if (wantarray) {
        return ($stdout,$exitcode);
    }
    return $stdout;
}

sub printx {
    my ($msg)=@_;
    print "$msg" if ($g_debug);
    return;
}

sub isip {
    my ($ip) = (@_);
    return 1 if ((ip_is_ipv4($ip)) || (ip_is_ipv6($ip)));
    return 0;
}

sub ip_is_ipv4 {
    my $ip = shift;

    # Check for invalid chars
    unless ($ip =~ m/^[\d\.]+$/) {
        #$ERROR = "Invalid chars in IP $ip";
        #$ERRNO = 107;
        return 0;
    }

    if ($ip =~ m/^\./) {
        #$ERROR = "Invalid IP $ip - starts with a dot";
        #$ERRNO = 103;
        return 0;
    }

    if ($ip =~ m/\.$/) {
        #$ERROR = "Invalid IP $ip - ends with a dot";
        #$ERRNO = 104;
        return 0;
    }

    # Single Numbers are considered to be IPv4
    if ($ip =~ m/^(\d+)$/ and $1 < 256) { return 1 }

    # Count quads
    my $n = ($ip =~ tr/\./\./);

    # IPv4 must have from 1 to 4 quads
    unless ($n >= 0 and $n < 4) {
        #$ERROR = "Invalid IP address $ip";
        #$ERRNO = 105;
        return 0;
    }

    # Check for empty quads
    if ($ip =~ m/\.\./) {
        #$ERROR = "Empty quad in IP address $ip";
        #$ERRNO = 106;
        return 0;
    }

    foreach (split /\./, $ip) {

        # Check for invalid quads
        unless ($_ >= 0 and $_ < 256) {
            #$ERROR = "Invalid quad in IP address $ip - $_";
            #$ERRNO = 107;
            return 0;
        }
    }
    return 1;
}

sub ip_is_ipv6 {
    my $ip = shift;

    # Count octets
    my $n = ($ip =~ tr/:/:/);
    return (0) unless ($n > 0 and $n < 8);

    # $k is a counter
    my $k;

    foreach (split /:/, $ip) {
        $k++;

        # Empty octet ?
        next if ($_ eq '');

        # Normal v6 octet ?
        next if (/^[a-f\d]{1,4}$/i);

        # Last octet - is it IPv4 ?
        if ($k == $n + 1) {
            next if (ip_is_ipv4($_));
        }

        #$ERROR = "Invalid IP address $ip";
        #$ERRNO = 108;
        return 0;
    }

    # Does the IP address start with : ?
    if ($ip =~ m/^:[^:]/) {
        #$ERROR = "Invalid address $ip (starts with :)";
        #$ERRNO = 109;
        return 0;
    }

    # Does the IP address finish with : ?
    if ($ip =~ m/[^:]:$/) {
        #$ERROR = "Invalid address $ip (ends with :)";
        #$ERRNO = 110;
        return 0;
    }

    # Does the IP address have more than one '::' pattern ?
    if ($ip =~ s/:(?=:)//g > 1) {
        #$ERROR = "Invalid address $ip (More than one :: pattern)";
        #$ERRNO = 111;
        return 0;
    }

    return 1;
}

# Check if port of addr is available or not. (only TCP)
sub is_port_connectable {
    my ($addr,$port,$timeout) = @_;
    my (@classes,$ipv6,$sock);

    $timeout ||= 5;
    require IO::Socket::INET;
    $ipv6=1;
    eval { require IO::Socket::INET6 };
    $ipv6=0 if ($@);

    @classes=();
    if (ip_is_ipv6($addr)) {
        push @classes, 'IO::Socket::INET6' if ($ipv6);
    } else {
        push @classes, 'IO::Socket::INET';
        push @classes, 'IO::Socket::INET6' if ($ipv6);
    }

    for my $class (@classes) {
        if ($sock = $class->new(
            PeerAddr => $addr,
            PeerPort => $port,
            Timeout  => $timeout,
            Proto    => 'tcp'))
        {
            close $sock;
            return 1;
        }
    }
    return 0;
}

# Setup Expect and execute the command
sub setup_expect {
    my ($sys,$user,$passwd,$cmd,$timeout,@exp_params) = @_;
    my ($exp,$debug,$pos,$error,$match,$before,$after);

    # Check arguments
    return unless ($cmd);
    return unless (@exp_params);

    # Check if Expect module is ready.
    eval { require Expect };
    return if ($@);

    # Creating the Expect object
    $exp = Expect->spawn($cmd);
    return if (!$exp);

    # Configuring the expect object
    $exp->log_stdout(0);

    # To print debug information, set $debug to 1
    $debug= $g_debug || 0;
    $exp->debug($debug);
    $exp->exp_internal($debug);

    # Execute the command with Expect
    ($pos,$error,$match,$before,$after)=$exp->expect($timeout, @exp_params);

    # If $error begin with 1, means TIMEOUT
    # If $error begin with 2, means EOF, the connection is aborted or scp command finished
    # If $error begin with 3, means the ssh or rsh process exit with error.
    # If $error begin with 4, means unknown error

    if (wantarray) {
        return ($exp,$pos,$error,$match,$before,$after);
    }
    return $exp;
}

# Execute command with the Expect object
sub exec_expect {
    my ($exp,$cmd,$timeout,$match_str) = @_;
    my ($begin,$stdout,$stderr,$exitcode);

    $exp->expect(1, EXPECT_SYMC_PROMPT);
    if ($match_str) {
        $exp->send("$cmd\n");
        $exp->expect($timeout,'-re',"$match_str");
        $stdout=$exp->match() || '';
        chomp $stdout;
        $exitcode=0;
    } else {
        $exp->send("echo __BEGIN__; $cmd; echo __END__=\$?\n");
        $exp->expect($timeout,'-re', EXPECT_SYMC_PROMPT);
        $match_str=$exp->before() || '';
        $begin=0;
        $stdout='';
        $exitcode='0';
        for my $line (split /\r?\n/, $match_str) {
            if ($line eq '__BEGIN__') {
                $begin=1;
                next;
            }
            next if (!$begin);

            if ($line =~ /^(.*)__END__=\s*(-?\d+)$/mxs) {
                $stdout.="$1";
                $exitcode="$2";
                last;
            }
            $stdout.="$line\n";
        }
        chomp $stdout;
        chomp $exitcode;
    }
    $stderr='';

    if (wantarray) {
        return ($stdout,$stderr,$exitcode);
    }
    return $stdout;
}

# Close the expect object
sub close_expect {
    my $exp = shift;
    $exp->hard_close();
    return 1;
}
